/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2014. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.ui.internal.view.common.CustomAttributesEditor");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.InlineEditor");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.layout.FormDateField");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.layout.FormTextAreaField");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.layout.AbstractFormField");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.layout.FormTextField");
dojo.require("com.ibm.rqm.planning.web.client.internal.CustomAttributesClient");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.asq.common.web.client.internal.Constants");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

(function()
{
	var AttributeClient 			   = com.ibm.rqm.planning.web.client.internal.CustomAttributesClient;
	var Utility 					   = com.ibm.asq.common.web.ui.internal.util.Utility;
	var _textContent 				   = dojo.isIE?"innerText":"textContent";
	var Constants 					   = com.ibm.asq.common.web.client.internal.Constants;
	var ServiceResponseHandler 		   = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	
	dojo.declare("com.ibm.asq.common.web.ui.internal.view.common.CustomAttributesEditor",
	[dijit._Widget, dijit._Templated],{
		
		templateString:	'<div dojoAttachPoint="mainContainer" class="com-ibm-common-custom-attributes-editor">'+
							'<div dojoAttachPoint="customAttributesTitleNode" >'+
								'<span dojoAttachPoint="spanTitleNode" class="section-title">${titleLabel}</span>'+
							'</div>'+
							'<div dojoAttachPoint="mainAttributesContainer" class="form-fields">'+
							'</div>'+							
					   	'</div>',
						
		titleLabel: Constants.getMessage("labelAttributes"),		
		scope: "",
		readOnly: false,
		requiredOnly: false, 
	    // if we are in a standard form (true) we should not validate and 
	    // mark fields in error till user has passed through them
		formValidationStyle: false,
		hideTitle:false,
		onChange:null,
		
		constructor: function(){
			this.attributes = [];
			this.existingCustomAttributes = [];
		},
		
		postCreate: function(){
			if(this.hideTitle){
				dojo.style(this.customAttributesTitleNode,"display","none");
			}
			
			if (this.parent && this.parent._compositeDTO) {
				this.getData(this.parent._compositeDTO.customAttributes);
			} else {
				var args = {
		 				scope:this.scope
		 		};
				var responseHandler = new ServiceResponseHandler(this, "getData", "_initError");
		 		AttributeClient.getCustomAttributesDTO(responseHandler, args);
			}            
            // We are using a field group, our dom node isn't added
            // to the dom tree. Place ourself under the group's management.
            if(this.fieldGroup) {
                this.fieldGroup.addMangedWidget(this);
            }
		},
		
		isAttributeAlreadyStored: function(itemId){
			var length = this.existingCustomAttributes?this.existingCustomAttributes.length:0;
			for(var i=0; i<length; i++){
				var item  = this.existingCustomAttributes[i];
				var localAttributeItemId = item.itemId;
				if(localAttributeItemId && localAttributeItemId == itemId)
					return item;				
			}
			return null;
		},
		
		getData: function(items){
            if (!items) {
                dojox.xml.parser.removeChildren(this.mainContainer); //if we don't have attribs don't show anything 
            }
            else {
            	var hasRequiredAttributes = false;
                var length = items.length;
                for (var i = 0; i < length; i++) {
                    var currentItem = items[i];
                    hasRequiredAttributes = hasRequiredAttributes || currentItem.required;
                    var isExistingItem = this.isAttributeAlreadyStored(currentItem.itemId);
                    if (isExistingItem) {
                        currentItem.value = isExistingItem.value;
                    }
                    this.attributes.push(currentItem);
                }
                
                // WI 68980 - In OSLC Context, only required attributes should appear. If there is none, do not show the Attributes title.
    			if(this.requiredOnly && !hasRequiredAttributes) {
    				dojo.empty(this.mainContainer);
    			}
    			
                this._createTable();
            }
            if (this.onInitializeComplete) {
                this.onInitializeComplete();
            }
        },
		_createTable: function(){
            // we are being managed by the field group, add a title 
            // to it.
            if (this.fieldGroup && !this.hideTitle) {
                this.fieldGroup.addSeparator();
                this.fieldGroup.addContent({
                    label: this.titleLabel,
                    labelClass: "section-title"
                });
            }             
			var length = this.attributes.length;			
			for(var i=0; i<length; i++){				
				var currentAttribute = this.attributes[i];
				if(this.requiredOnly && !currentAttribute.required){
					continue;
				}					
				var widget = this.getEditWidget(currentAttribute);
                
                // not group managed, add the field to our container. 
                if (!this.fieldGroup) {
                    this.mainAttributesContainer.appendChild(widget.domNode);
                }
			}
		},
		
		setDisabled: function(disable) {
			var length = this.attributes.length;			
			for(var i=0; i<length; i++) {
				var attribute = this.attributes[i];				
				var widget = attribute.widget;
				if(widget)
					widget.setDisabled(disable);
			}
		},
								
		getEditWidget:function(attribute)
		{
			var type = attribute.type;
			var internalValue = attribute.value;
			var attributeName =  Constants.getMessage("labelIdLabel",attribute.name);
			var required = attribute.required?true:false;			
			var widget = null;
			if (this.readOnly) { //no need choose another widget  if this is a read only artifact				
				if(type == "TIMESTAMP") {
                    if(internalValue) {
                        var date = Utility.formatDateFromLong(internalValue);
                        var time = Utility.getTimeString(internalValue);
                        internalValue = date + "  " + time;
                    }
				}	
				widget = new com.ibm.asq.common.web.ui.internal.widgets.layout.AbstractFormField({
                    fieldGroup: this.fieldGroup,
					fieldLabel:attributeName,
					formValidationStyle:this.formValidationStyle,
					isRequired:required
				});   
				var span = document.createElement("span");
				span.innerHTML = internalValue?internalValue:"";
				widget.getValue = dojo.hitch(this, function(){ 
					return  attribute.value;
				});
				widget.fieldHolderNode.appendChild(span);
				attribute.widget = widget;
			} else {
				if(type == "SMALL_STRING" || type == "MEDIUM_STRING"){				
				    widget = new com.ibm.asq.common.web.ui.internal.widgets.layout.FormTextAreaField({
	                    fieldGroup: this.fieldGroup,
				    	fieldLabel:attributeName,
	                    formValidationStyle:this.formValidationStyle,
		                name: "myarea",	                
		                value: internalValue,	        	                
		                maxLength: (type=="SMALL_STRING")?250:1000,
		                isRequired:required
		            });
				    widget.onValueChanged = dojo.hitch(this, this._setDirty);
				    attribute.widget = widget;
				} else if(type == "INTEGER") {
					widget =  new com.ibm.asq.common.web.ui.internal.widgets.layout.FormNumericField({
	                    fieldGroup: this.fieldGroup,
						fieldLabel: attributeName,		           		            
	                    formValidationStyle:this.formValidationStyle,
			            currentValue: internalValue,
		                isRequired:required			            
			            });
									
					widget.onValueChanged = dojo.hitch(this, this._setDirty);
					widget.setErrorMessage(Constants.getMessage("msgInvalidNumber"));
					attribute.widget = widget;
				} else if(type == "TIMESTAMP") {
					var myDate = "";
					if(internalValue && internalValue!= null && internalValue!= ""){
						myDate = new Date();
						myDate.setTime(internalValue);					
					}
					widget = new com.ibm.asq.common.web.ui.internal.widgets.layout.SimpleDateTimePickerFormField({
						label: "",
						fieldGroup: this.fieldGroup,
						fieldLabel: attributeName,
						formValidationStyle:this.formValidationStyle,
						currentValue: myDate,
						time: myDate,
						noValidationDisplay: true,
						isRequired: required
					});
					dojo.style(widget.dateField.timePicker.domNode, "width", "8em");
					widget.onValueChanged = dojo.hitch(this, this._setDirty);
					widget.setErrorMessage(Constants.getMessage("dtpInvalidDate"));
	                attribute.widget = widget;
				} else {
					widget = document.createElement("span");
					widget[_textContent] = Constants.getMessage("labelErrorWidgetNotfound");				
				}
			}
			return widget;
		},
		
        _initError: function(error) {
            this._error();
            if (this.onInitializeComplete) {
                this.onInitializeComplete();
            }
        },
        
		_error: function(error){
			console.error("CustomAttributesEditor- Error:"+ error.messages);
		},
		
		_setDirty: function(){
			if (this.parent && dojo.isFunction(this.parent.setDirty)){
				this.parent.setDirty();
			}
			else if(this.parent && dojo.isFunction(this.parent._setDirty))
				this.parent._setDirty(true);
			if(dojo.isFunction(this.onChange))
				this.onChange();
		},
		
		getAttributes: function(){
			var formattedAttributes = [];
			var length = this.attributes.length;			
			for(var i =0; i< length; i++){
				var widget = this.attributes[i].widget;				
				var value = "";
				if(widget && widget.getValue()){
					value = widget.getValue();
				}
				if(value && value!= ""){
					formattedAttributes.push({"attributeId":this.attributes[i].itemId, "attributeValue": ""+value });
				}
			}
			return formattedAttributes;
		},
		
		setAttributes: function(existingAttributes){
			var length = this.attributes ? this.attributes.length : 0;
			var existingAttributeLength = existingAttributes ? existingAttributes.length : 0;
			for(var j = 0; j < existingAttributeLength; j++){
				var existingAttribute = existingAttributes[j];
				for(var i = 0; i< length; i++){
					if(this.attributes[i].itemId == existingAttribute.itemId){
						var attribute = this.attributes[i];
						var widget = attribute.widget;
						if(widget){
							var type = attribute.type;
							if(type == "TIMESTAMP"){
								var myDate = "";
								if(existingAttribute.value && existingAttribute.value!= null && existingAttribute.value!= ""){
									myDate = new Date();
									myDate.setTime(existingAttribute.value);
									widget.setValue(myDate);
								}
							}else{
								widget.setValue(existingAttribute.value);
							}
						}
					}
				}
			}
		},
		
		validate: function(){
			var length = this.attributes ?  this.attributes.length : 0;	
			var msgs = [];
			for(var i = 0; i< length; i++){
				var attribute = this.attributes[i];				
				var widget = attribute.widget;				
				if(attribute.required && (!widget.getValue() || widget.getValue() == "")){			    
					msgs.push(Constants.getMessage("msgCustomAttributeIsRequired",attribute.name));					
				}else if(widget && widget.hasError){
					var errorMessage = widget.errorMessage;
					errorMessage = errorMessage ? "<br />" + errorMessage : "";
					msgs.push(Constants.getMessage("msgInvalidAttribute", attribute.name) + errorMessage);
				}
			}
			 msgs = (msgs.length > 1) ? msgs.join("<br/>") : (msgs.length > 0 ? msgs[0] : null);
			 return {
				 status: msgs == null,
	            msg: msgs
	        };
		},
		uninitialize: function(){
			this.attributes = null;
			this.existingCustomAttributes = null;
		}
    });
})();
