/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2014. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function(){
dojo.provide("com.ibm.rdm.web.external.ImportWizard");

dojo.require("dojo.i18n");
dojo.require("jazz.ui.Wizard");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.csv.CsvImportClient");
dojo.require("com.ibm.rdm.web.csv.CsvImportDocumentStep");
dojo.require("com.ibm.rdm.web.external.ImportExportWizard");
dojo.require("com.ibm.rdm.web.external.ImportTypeStep");
dojo.require("com.ibm.rdm.web.external.WordImportStep");
dojo.require("com.ibm.rdm.web.parsereq.model.Configuration");
dojo.require("com.ibm.rdm.web.parsereq.ReqParsingClient");
dojo.require("com.ibm.rdm.web.parsereq.ReqParsingCriteriaStep");
dojo.require("com.ibm.rdm.web.parsereq.ReqParsingDocumentStep");
dojo.require("com.ibm.rdm.web.parsereq.ReqParsingSummaryStep");
dojo.require("com.ibm.rdm.web.reqif.ReqIFImportPackageStep");
dojo.require("com.ibm.rdm.web.reqif.ReqIFImportDefinitionStep");
dojo.require("com.ibm.rdm.web.reqif.ReqIFImportUploadStep");
dojo.require("com.ibm.rdm.web.reqif.ReqIFImportPackageSelectStep");
dojo.require("com.ibm.rdm.web.reqif.ReqIFImportPreviewStep");

dojo.requireLocalization("com.ibm.rdm.web.external", "ImportWizard");

//Wizard clients
var ReqParsingClient = com.ibm.rdm.web.parsereq.ReqParsingClient;
var CsvImportClient = com.ibm.rdm.web.csv.CsvImportClient;

//Home page wizard step
var ImportTypeStep = com.ibm.rdm.web.external.ImportTypeStep;

//Requirements Parsing wizard steps
var ReqParsingCriteriaStep = com.ibm.rdm.web.parsereq.ReqParsingCriteriaStep;
var ReqParsingDocumentStep = com.ibm.rdm.web.parsereq.ReqParsingDocumentStep;
var ReqParsingSummaryStep = com.ibm.rdm.web.parsereq.ReqParsingSummaryStep;

//CSV Import wizard steps
var CsvImportDocumentStep = com.ibm.rdm.web.csv.CsvImportDocumentStep;

//ReqIF Import wizard steps
var ReqIFImportPackageStep = com.ibm.rdm.web.reqif.ReqIFImportPackageStep;
var ReqIFImportDefinitionStep = com.ibm.rdm.web.reqif.ReqIFImportDefinitionStep;
var ReqIFImportUploadStep = com.ibm.rdm.web.reqif.ReqIFImportUploadStep;
var ReqIFImportPackageSelectStep = com.ibm.rdm.web.reqif.ReqIFImportPackageSelectStep;
var ReqIFImportPreviewStep = com.ibm.rdm.web.reqif.ReqIFImportPreviewStep;

//Word Import wizard steps
var WordImportStep = com.ibm.rdm.web.external.WordImportStep;

dojo.declare(
		"com.ibm.rdm.web.external.ImportWizard",
		[dijit._Widget, com.ibm.rdm.web.external.ImportExportWizard],
{
	
	//Wizard client
	_wizardClient: null,
	_importTypeStep: null,
	_permissionsDeferred: null,
	
	//Requirements Parsing
	_reqParsingSummaryStep: null,

	constructor: function(args){
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.external", "ImportWizard");
		
		this._setWizardTitle(this._messages.importLabel);
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// com.ibm.rdm.web.external.ImportExportWizard API
	// 		The following methods are implementing those defined as part of 
	//		ImportExportWizard, which this is based on.
	///////////////////////////////////////////////////////////////////////////////////

	show: function(/*Callback?*/onWizardClose) {
		// summary:
		//		Create and show the jazz.ui.Wizard.
		
		var wizardArgs = {
			steps: this.steps,				  
			primaryTitle: this._wizardTitle,
			showStepSummary: true,
			destroyContents: true,
			width: "705px",
			processFinish: function() {
				// Override default behavior
			}
		};
		this.wizard = new jazz.ui.Wizard(wizardArgs);
		// Note: We're overriding 'uninitialize' here in order to make sure that
		// onWizardClose has a chance to be called (we passed destroyContents as
		// true when constructing the wizard, so we know we can rely on this
		// uninitialize being called).
		this.wizard.uninitialize = dojo.hitch(this, function(){
			// Call the superclass version of 'uninitialize'.
			this.wizard.inherited("uninitialize", arguments);
			// Now call onWizardClose if needed.
			if (onWizardClose) {
				onWizardClose();
			}
		});
		
		if (dojo.isIE && this.wizard.contentWidget && this.wizard.contentWidget.embedPanelArea) {
			this.wizard.contentWidget.embedPanelArea.style.display = "none";
		}
	},
	
	// summary:
	//		Creates the wizard steps for the given wizard type. The argument
	// 		contextArgs will be defined if the wizard is being called from a
	// 		context menu.
	//
	// returns:
	//		A deferred containing the wizard steps to add
	getWizardSteps: function(wizardType, /*Optional Object*/contextArgs) {
		var deferred = new dojo.Deferred();
		var wizardSteps = [];
		var step2 = null, step3 = null, step4 = null;
		var folderURI = null;
		if (this.folderURI) {
			folderURI = this.folderURI;
		}
		switch(wizardType)
		{
		case com.ibm.rdm.web.external.ImportWizard.wizardType.HOME_PAGE:
		    this._permissionsDeferred = com.ibm.rdm.web.common.ProjectCache.getProjectPermissions(this.projectURI);
		    
		    this._permissionsDeferred.addCallback(this, function(projectOperations) {		
    			this._importTypeStep = new ImportTypeStep({
    				importWizard: this,
    				projectURI: this.projectURI,
    				stepTitle: this._messages.importLabel,
    				canImportReqIF: projectOperations.item(com.ibm.rdm.web.common.ProjectCache.IMPORT_REQIF)
    			});
    			wizardSteps.push(this._importTypeStep);
    			deferred.callback(wizardSteps);
    			
    			return projectOperations;
		    });
			break;
		case com.ibm.rdm.web.external.ImportWizard.wizardType.REQ_PARSING:
			this.setWizardClient(ReqParsingClient);
			var configParams, sourceURIs, docStepTitle = null;
			this.getWizardClient().createOperationInstance(this.projectURI).addBoth(this, function(response) {
				var configurationUri =
					this.getWizardClient().getOperationUri(com.ibm.rdm.web.parsereq.ReqParsingClient.OPERATIONS.CONFIG);
				this.getWizardClient().getResource({uri: configurationUri}).addBoth(this, function(modelData) {
					var configurationModel = new com.ibm.rdm.web.parsereq.model.Configuration();
					configurationModel.load(modelData.rawData, modelData.uri);
					
					if (contextArgs) {
						sourceURIs = contextArgs.sourceURIs;
						docStepTitle = this._messages.specifyDetails;
						this._setWizardTitle(this._messages.parseRequirements);
					} else {
						docStepTitle = this._messages.selectTextDocument;
					}
					
					var docStep = new ReqParsingDocumentStep({
						importWizard: this,
						projectURI: this.projectURI,
						configurationModel: configurationModel,
						sourceURIs: sourceURIs,
						stepTitle: docStepTitle,
						folderURI: folderURI
					});
					docStep.loadObjectTypes().addCallback(this, function(objectTypesStore) {
						var criteriaStep = new ReqParsingCriteriaStep({
							importWizard: this,
							projectURI: this.projectURI,
							configurationModel: configurationModel,
							objectTypesStore: objectTypesStore,
							stepTitle: this._messages.identifyRequirements
						});
						this._reqParsingSummaryStep = new ReqParsingSummaryStep({
							importWizard: this,
							projectURI: this.projectURI,
							objectTypesStore: objectTypesStore,
							configurationModel: configurationModel,
							stepTitle: this._messages.parseRequirements
						});
						wizardSteps.push(docStep, criteriaStep, this._reqParsingSummaryStep);
						deferred.callback(wizardSteps);
						
						return objectTypesStore;
					});
					return configParams;
				});
				return response;
			});
			break;
		case com.ibm.rdm.web.external.ImportWizard.wizardType.CSV_IMPORT:
			this.setWizardClient(CsvImportClient);
			var docStep = new CsvImportDocumentStep({
				importWizard: this,
				projectURI: this.projectURI,
				stepTitle: this._messages.csvImportDocumentStepTitle,
				folderURI: folderURI
			});
			wizardSteps.push(docStep);
			deferred.callback(wizardSteps);
			break;
		case com.ibm.rdm.web.external.ImportWizard.wizardType.REQ_IF_IMPORT:
	        step2 = new ReqIFImportPackageStep({
	            importWizard: this,
	            projectURI: this.projectURI,
	            stepTitle: this._messages.reqIFStepOneTitle
	        });
	        step3 = new ReqIFImportDefinitionStep({
	            importWizard: this,
	            projectURI: this.projectURI,
	            stepTitle: this._messages.reqIFImportDefinitionStepTitle,
	            folderURI: folderURI
	        });
	        step4 = new ReqIFImportPreviewStep({
	               importWizard: this,
	               projectURI: this.projectURI,
	               stepTitle: this._messages.reqIFImportPreviewStepTitle
	            });
	        wizardSteps.push(step2, step3, step4);
	        deferred.callback(wizardSteps);
			break;
		case com.ibm.rdm.web.external.ImportWizard.wizardType.REQ_IF_IMPORT_CONTINUED:
		    step2 = new ReqIFImportPackageSelectStep({
		        importWizard: this,
		        projectURI: this.projectURI,
		        stepTitle: this._messages.reqIFPackageSelectStepTitle
		    });
	        step3 = new ReqIFImportDefinitionStep({
	            importWizard: this,
	            projectURI: this.projectURI,
	            stepTitle: this._messages.reqIFImportDefinitionStepTitle,
	            folderURI: folderURI
	        });
		    step4 = new ReqIFImportPreviewStep({
		       importWizard: this,
		       projectURI: this.projectURI,
		       stepTitle: this._messages.reqIFImportPreviewStepTitle
		    });
		    wizardSteps.push(step2, step3, step4);
		    deferred.callback(wizardSteps);
		    break;
		case com.ibm.rdm.web.external.ImportWizard.wizardType.WORD_IMPORT:
			var sourceURI, resource, stepTitle = null;
			if (contextArgs) {
				sourceURI = contextArgs.sourceURI;
				resource = contextArgs.resource;
				stepTitle = this._messages.specifyDetails;
				this._setWizardTitle(this._messages.importTextDocument);
			}
			else {
				stepTitle = this._messages.selectTextDocument;
			}
			step2 = new WordImportStep({
				importWizard: this,
				stepTitle: stepTitle,
				projectURI: this.projectURI,
				sourceURI: sourceURI,
				resource: resource,
				folderURI: this.folderURI
			});
			wizardSteps.push(step2);
			deferred.callback(wizardSteps);
			break;
		default:
			deferred.callback(wizardSteps);
		}
		
		return deferred;
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// Public API
	///////////////////////////////////////////////////////////////////////////////////
	
	selectFirstOption: function() {
		if (!this._importTypeStep) {
			this._permissionsDeferred.addCallback(this, function(projectOperations) {
				this._importTypeStep && this._importTypeStep.selectFirstOption();
				return projectOperations;
			});
		} else {
			this._importTypeStep.selectFirstOption();
		}
	},
	
	setWizardClient: function(client) {
		this._wizardClient = client;
	},
	
	getWizardClient: function() {
		return this._wizardClient;
	},
	
	getReqParsingSummaryStep: function() {
		return this._reqParsingSummaryStep;
	},
	
	onWizardPageCancel: function(/*Boolean*/deleteOperationInstance) {
		this.inherited("onWizardPageCancel", arguments);
		if (deleteOperationInstance) {
			if (this.getWizardClient() && 
					this.getWizardClient().deleteOperationInstance &&
						dojo.isFunction(this.getWizardClient().deleteOperationInstance)) {
				this.getWizardClient().deleteOperationInstance();
			}
		}
	}
	
});

com.ibm.rdm.web.external.ImportWizard.wizardType = {
		HOME_PAGE: 					"homePageWizardType",
		REQ_PARSING: 				"requirementsParsingWizardType",
		CSV_IMPORT: 				"csvImportWizardType",
		REQ_IF_IMPORT: 				"reqIfImportType",
		REQ_IF_IMPORT_CONTINUED:	"reqIfImportContinuedType",
		WORD_IMPORT: 				"wordImportWizardType"
};

})();
