/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012, 2014.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/

(function(){
dojo.provide("com.ibm.rdm.web.history.ChangeHistoryWidget");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");

dojo.require("com.ibm.rdm.web.history.AuditHistoryRetriever");
dojo.require("com.ibm.rdm.web.server.client.utils.UserCache");
dojo.require("com.ibm.rdm.web.common.ArtifactIcons");
dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");
dojo.require("com.ibm.rdm.web.common.DateTimeHelper");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("dojo.i18n");
dojo.require("dojo.date");

dojo.requireLocalization("com.ibm.rdm.web.history", "HistoryHover");
dojo.requireLocalization("com.ibm.rdm.web.common", "common");

var ArtifactIcons = com.ibm.rdm.web.common.ArtifactIcons;
var DateTimeHelper = com.ibm.rdm.web.common.DateTimeHelper;
var domutils = com.ibm.rdm.web.util.domutils;

// This widget shows the changes made in the last NUM_DAYS days
var NUM_DAYS = 14;

dojo.declare("com.ibm.rdm.web.history.ChangeHistoryWidget", [dijit._Widget, dijit._Templated], {
	
	// summary:
	//		This is a widget that displays the audit history of an artifact. It displays one 
	//		change entry at a time, and uses a paging mechanism to navigate between change entries.
	//		This was originally written for use in the Recent Changes viewlet.
	
	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "history/templates/ChangeHistoryWidget.html"),
	
	// Public fields (passed upon instantiation)
	projectURI: null,
	resourceURI: null,
	resourceName: null,
	artifactTypeURI: null,
	usedInModuleArgs: null,
	
	// Private fields
	_messages: null,
	_commonMessages: null,
	_auditHistoryParser: null,
	_currentPage: null,
	_totalPages: null,
	
	// _pages: Array of DOM nodes
	//		This array contains pages that have already been displayed. 
	//		Page 1 in the widget is recorded as this._pages[0]
	_pages: null,
	
	constructor: function(){
		this.inherited(arguments);
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.history", "HistoryHover");
		this._commonMessages = dojo.i18n.getLocalization("com.ibm.rdm.web.common", "common");
		this._pages = [];
		this._currentPage = 1;
	},
	
	postCreate: function(){
		this.inherited(arguments);
		
		// Get the artifact history
		var retriever = new com.ibm.rdm.web.history.AuditHistoryRetriever();
		var startDate = dojo.date.stamp.toISOString(dojo.date.add(new Date(), "day", -NUM_DAYS));
		var uri = "?uri=" + this.resourceURI + "&startDate=" + encodeURIComponent(startDate);
		retriever.getArtifactHistory(uri).addCallback(this, function(artifactHistoryResponse){
			this._auditHistoryParser = artifactHistoryResponse.doc;
			this._totalPages = this._auditHistoryParser.getChangeLogs().length;
			this._loadContents();
			dojo.style(this._loadingDiv, "display", "none");
			dojo.style(this._containerDiv, "display", "inline-block");
		});
		
		// Set up connections for the paging mechanism
		this.connect(this._pagingPrev, "onclick", dojo.hitch(this, function(event){
			dojo.stopEvent(event);
			this._changePage(false);
		}));
		this.connect(this._pagingNext, "onclick", dojo.hitch(this, function(event){
			dojo.stopEvent(event);
			this._changePage(true);
		}));
		
		// Display the title
		if (this.resourceName && this.projectURI){
			// title vulnerable to XSS if xml chars not escaped
			var titleText = com.ibm.rdm.web.util.domutils.getXMLEncodedText(this.resourceName);
			var titleLink = dojo.create("a", {href: this.resourceURI, innerHTML: titleText});
			dojo.style(titleLink, "paddingLeft", "6px");
			this.connect(titleLink, "onclick", function(event){
				com.ibm.rdm.web.common.ResourceLinksHandler.handleAnchorClicked(event);
			});
			if (this.artifactTypeURI){
				this._titleDiv.appendChild(ArtifactIcons.getSmallImage(this.projectURI, this.artifactTypeURI));
			}
			this._titleDiv.appendChild(titleLink);
		} else {
			dojo.style(this._titleDiv, "display", "none");
		}
	},
	
	uninitialize: function(){
		this.inherited(arguments);
		dojo.forEach(this._pages, function(page){
			dojo.empty(page);
			dojo.destroy(page);
		});
		this._pages = null;
	},
	
	_loadContents: function(){
		this._updatePaging();
		// If the page isn't in this._pages, then create the page
		if (!this._pages[this._currentPage-1]){
			this._pages[this._currentPage-1] = this._createPage();
		}
		domutils.removeChildNodes(this._contentNode);
		this._contentNode.appendChild(this._pages[this._currentPage-1]);
	},
	
	_createPage: function(){
		var page = dojo.create("div");
		var changeEntryId = this._auditHistoryParser.getChangeLogs()[this._currentPage-1];
		var changeEntries = this._auditHistoryParser.getChangeLog(changeEntryId).changeEntries;
		var modifiedDate = this._auditHistoryParser.getChangeLogDate(changeEntryId);
		var authorURI = this._auditHistoryParser.getChangeLogContributor(changeEntryId);
		// Date
		var divDate = dojo.create("div", {"class": "dateNode"}, page);
		var formattedDate = DateTimeHelper.getDateTimeDisplayStringFromString(modifiedDate);
		var formattedDifference = DateTimeHelper.getFormattedDifference(modifiedDate);
		divDate.appendChild(document.createTextNode(dojo.string.substitute(this._messages.dateDescription, [formattedDate, formattedDifference])));
		// User
		if (authorURI) {
			var divUser = dojo.create("div", {"class": "userModifiedBy"}, page);
			com.ibm.rdm.web.server.client.utils.UserCache.getUser(authorURI).addCallback(this, function(user) {
				var userChangedString = dojo.string.substitute(this._messages["changed"], [user.name]);
				divUser.innerHTML = userChangedString;
			});
		}
		// Table of changes
		var tableDiv = dojo.create("div", {}, page);
		tableDiv.appendChild(this._auditHistoryParser.buildContentTable(changeEntries, changeEntryId));
		// Makes a request to the server to fetch comments.
		this._auditHistoryParser.parseComplete();
		
		return page;
	},
	
	_changePage: function(/*Boolean*/ next){
		var pageNumber = this._currentPage + (next ? 1 : -1);
		if (pageNumber >= 1 && pageNumber <= this._totalPages){
			this._currentPage = pageNumber;
			this._loadContents();
		}
	},
	
	_updatePaging: function(){
		dojo.empty(this._pagingSpan);
		var pagingString = "" + this._currentPage + "/" + this._totalPages;
		this._pagingSpan.appendChild(document.createTextNode(pagingString));
		dojo.removeClass(this._pagingPrev, "prev-disabled");
		dojo.removeClass(this._pagingNext, "next-disabled");
		if (this._currentPage === 1) {
			dojo.addClass(this._pagingPrev, "prev-disabled");
		}
		if (this._currentPage === this._totalPages) {
			dojo.addClass(this._pagingNext, "next-disabled");
		}
	}
});
})();
