/*
 * Copyright (c) 2010 - 2014. IBM Corporation
 * All rights reserved.
 */
(function(){
	dojo.provide("com.ibm.rdm.web.links.LinkItemWidget");
	
	dojo.require("dijit._Templated");
	dojo.require("dijit._Widget");
	dojo.require("dijit._base");
	dojo.require("dojo.i18n");
	
	dojo.require("com.ibm.rdm.web.common.MimeTypes");
	dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");
	dojo.require("com.ibm.rdm.web.viewer.viewerutils");
	dojo.require("jazz.ui.ResourceLink");
	dojo.require("com.ibm.rdm.web.links.LinkTypeUtils");
	dojo.require("com.ibm.rdm.web.review.ReviewHoverPopup");
	dojo.require("com.ibm.rdm.web.util.queryutils");
	dojo.require("com.ibm.rdm.web.util.domutils");
	dojo.require("com.ibm.rdm.web.common.ArtifactIcons");
	dojo.require("com.ibm.rdm.web.util.bidiutils"); //bidi
	
	dojo.requireLocalization("com.ibm.rdm.web.links", "LinksListWidget");
	
	var ResourceLinksHandler = com.ibm.rdm.web.common.ResourceLinksHandler;
	var LinkTypeUtils = com.ibm.rdm.web.links.LinkTypeUtils;
	var viewerutils = com.ibm.rdm.web.viewer.viewerutils;
	var ReviewHoverPopup = com.ibm.rdm.web.review.ReviewHoverPopup;
	var BidiUtils = com.ibm.rdm.web.util.bidiutils;  //bidi
	
	dojo.declare("com.ibm.rdm.web.links.LinkItemWidget", [dijit._Widget, dijit._Templated], {
	
		templateString: "<tr dojoAttachPoint=\"linkRow\"><td><span dojoAttachPoint=\"_img\"></span><span class=\"link-result\"><span class=\"linked-resource-result\" dojoAttachPoint=\"linkInfo\"></span></span></td></tr>",
		
		status: null,
		// resource: Object
		//		Information about the RM resource that is the target of the link, not supplied for an external 
		//		link
		//
		// 		resource attributes:
		// 			projectURI: String
		// 			ofType:
		// 			contentType
		// 			type:
		// 			name: String
		// 			identifier: String 
		//				required value
		// 			resourceURI: String 
		//				required value, uri of link target
		// 			usedInModuleArgs: Object 
		//				optional value
		// 			revisionURI: String
		//				optional value
		// 			isNotInModuleBaseline: boolean 
		//				optional value, set to true for a link in a module baseline where link target 
		//				is not in the module baseline
		resource: null,
		resourceViewer: null,
		isViewerInHistoryMode: false,
		linkType: null,
		remoteLinkInfo: null, 
		uri: null,
		label: null,
		elementDescription: null,
		link: null,
		doNavigateSelection: null,
		_resourceLinks:[],
		displayResourceIdOnly: false,
		moduleHostURI: null,
		_anchorHref: null,
		isForbidden: false,
		
		_messages: null,

		postCreate: function(){
			this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.links", "LinksListWidget");
			this.inherited("postCreate", arguments);
			this.status = com.ibm.rdm.web.links.LinkItemWidget.SHOWING;
			this._resourceLinks = [];
			
			var anchor = null;
			if (this.resource) {
				//Create the image type for the link
				var imageClass;
				var resourceLabel;
				var artifactIcon = com.ibm.rdm.web.common.ArtifactIcons.getSmallImage(this.resource.projectURI, this.resource.ofType);
				if((artifactIcon && artifactIcon.className)) {
					imageClass = artifactIcon.className;
				}
				else {
					imageClass = com.ibm.rdm.web.common.MimeTypeRegistry.getImageClass(this.resource.contentType, this.resource.projectURI);
				}
				
				this._createLinkImage(imageClass);
				
				var resourceTitle = this.resource.name;
				resourceTitle = BidiUtils.setTextDirUCC (resourceTitle);//bidi

				var rrcId = this.resource.identifier;
				if (rrcId != null) {
					if(this.displayResourceIdOnly){
						resourceTitle = dojo.string.substitute(this._messages.linkWithoutId, [rrcId]);						
					}else{
						resourceTitle = dojo.string.substitute(this._messages.linkWithId, [rrcId, resourceTitle]);
					}
					
				}
				if (this.elementDescription != null && this.elementDescription != "" && !this.displayResourceIdOnly) {
					resourceTitle = dojo.string.substitute(this._messages.linkWithLocationDescription, [resourceTitle, this.elementDescription]);
				}
				
				if(this.resource.resourceURI.indexOf("review") != -1) {
					var reviewHover = new ReviewHoverPopup({
						viewType: ReviewHoverPopup
							.VIEW_TYPE.OVERALL_REVIEW_SUMMARY_STATUS,
						orientation: "L" // Popup to the left of the target node.
					});

					//	TODO:  This won't work properly with Open in new tab/window (i.e., want to show resource
					//	in context of the review).  Try to use a variation of ResourceLink here and don't do programmatic
					//	onclick behavior.
					anchor = dojo.create("a", {
						href: this.resource.resourceURI,
						innerHTML: resourceTitle
					});
					dojo.connect(anchor, "onclick", this, function(event){
						dojo.stopEvent(event);
						ResourceLinksHandler
							.showInternalResource(this.resource.resourceURI);
					});
					reviewHover.createHoverPopup(anchor,
						com.ibm.rdm.web.util.queryutils.getLastPathSegment(
							this.resource.resourceURI));
							
					this._addResourceLink(reviewHover);
				}
				else {
					var reviewId = (this.resourceViewer && this.resourceViewer.reviewId);
					if (reviewId || this._isNodeLinkItemWidget()) {
						// 	If we have a reviewId, we need to do special programmatic navigation to calculate what revision
						// 	of the artifact to show (vs. just navigating via an href), so create a ResourceLink and use an 
						//	onclick handler. 
						//
						//	If the link item is a linksexplorer node, special programmatic processing needs to be done to close LinksExplorer
						//	dialog window.
						//
						// 	TODO: Navigating to resource links in the context of a review/history will not work like other resource links.  
						//	ctrl+click and shift+click will not work since we are supplying our own click behavior for the ResourceLink 
						//	and open in new tab/new window from the link context menu will not open the artifact in the correct context 
						//	(i.e., in the context of the review or history).
						anchor = ResourceLinksHandler.createLink({
							uri: this.resource.resourceURI,
							presentationURI: this.resource.resourceURI, 
							label: resourceTitle, 
							usedInModuleArgs: this.resource.usedInModuleArgs
						});
						this._addResourceLink(dijit.byNode(anchor));
						this.connect(anchor, "onclick", this._handleLinkClicked);					
					} else {
						//	Otherwise, let ResourceLinksHandler calculate the presentationURI and let the ResourceLink handle the click 
						//	navigation.
						var linkBaseline = (this.resourceViewer) ? this.resourceViewer.baselineURI : null;
						var	linkRevision = this.resource.revisionURI;
						var linkURI = null;
						if (this.doNavigateSelection) {
							linkURI = this.link && this.link.getResourceURI();
						}
						if (this.resource.isNotInModuleBaseline) {
							// if the link is for a resource in a module baseline and the link target is not
							// in the module baseline, isNotInModuleBaseline will be true.  In this instance
							// we want to navigate to the HEAD version of the link target, so set linkBaseline
							// and linkRevision to null
							linkBaseline = null;
							linkRevision = null;
						} 
						anchor = ResourceLinksHandler.createLink({
							uri: this.resource.resourceURI,
							label: resourceTitle, 
							linkURI: linkURI,
							baselineURI: linkBaseline,
							revisionURI: linkRevision,
							usedInModuleArgs: this.resource.usedInModuleArgs
						});
						this._addResourceLink(dijit.byNode(anchor));
					}
				}
			}
			else {
				//Check if this is a CALM link 
				if (LinkTypeUtils.isCalmLink(this.linkType.getSameAs())) {
					var resourceLinkNode = null;
					var CALMIconImage = dojo.create("span");
					dojo.addClass(CALMIconImage, "img-div");
					dojo.addClass(this._img, "link-img");
					this._img.appendChild(CALMIconImage);
					var resourceLink = ResourceLinksHandler.createLink({
						uri: this.uri,
						label: BidiUtils.setTextDirComposedText(this.label, ": "),
						iconNode: CALMIconImage,
						lazyFetch: false,
						returnWidget: true
					});
					anchor = resourceLink.domNode;
					dojo.connect(resourceLink, "onLoad", this, this._updateLinkTitle);
				}
				else {
					//Decode the external URL
					var decodedURI = com.ibm.rdm.web.util.domutils.getXMLDecodedText(this.uri);
					var isLinkInternal = ResourceLinksHandler.isLinkInternal(decodedURI);
					//Check if link is external i.e: http://www.ibm.com
					if (!isLinkInternal) {
						anchor = this._createExternalLink(this.label, decodedURI);
					}
					//At this point we have a link to somewhere in the same server context. If the link is pointing to some artifact, then the artifact
					//either no longer exists or the user does not have permission to access it. Otherwise, create an external link out of whatever we have.					
					else {
						//If we made it this far, the link is pointing to an internal resource (NOT CLM resource or external resource such as
						//http://www.ibm.com yet for some reason the resourceInfo is undefined.
						
						//Check if the URL is of the form https://<server>:<port>/rm/web?debug=true#action=com.ibm.rdm.web.pages.showArtifact&artifactURI=<path to artifactURI>,
						//pointing to an internal resource.	
						var artifactURIfromExternalLink = ResourceLinksHandler.getArtifactUriFromExternalLink(decodedURI);
						if (artifactURIfromExternalLink != null) {
							anchor = this._createExternalLink(this.label, artifactURIfromExternalLink);
							
						}else{
							
							if (ResourceLinksHandler.isInternalResourceLink(decodedURI)) {
								// Resource linked to has been deleted or user does not have the permission to access. 											
								var imageClass = "error";
								
								if (this.isForbidden) {
									imageClass = "warning";	
								}
								
								this._createLinkImage(imageClass, this.label);
								anchor = dojo.create("div");
								dojo.attr(anchor, "title", this.isForbidden ? this._messages.forbiddenArtifactTooltip : this._messages.deletedArtifactTooltip);
								var linkLabel = dojo.string.substitute(this._messages.deletedArtifactID, [this.label]);
								anchor.appendChild(document.createTextNode(linkLabel));
							}else {
								anchor = this._createExternalLink(this.label, decodedURI);
							}
						}					
					}
				}
			}
			anchor.setAttribute("tabIndex", "-1");
			this._anchorHref = dojo.attr(anchor, "href");
			this.linkInfo.appendChild(anchor);
		},
		
		_updateLinkTitle: function() {
			if (this.savedViewLabel) {
				var resourceLinkNode = this.linkInfo.children[0];
				if (resourceLinkNode) {
					resourceLinkNode.innerHTML += " " + dojo.string.substitute(this._messages.usingView, [this.savedViewLabel]);
					var imageNode = this._img && this._img.children[0];
					if (imageNode) {
						var overlayImage = this._createOverlayImage();
						imageNode.appendChild(overlayImage);
					}
				}
			}
		},
		
		_createOverlayImage: function() {
			var overlayURL = this._blankGif;
			var overlayImage = document.createElement("img");
			overlayImage.src = overlayURL;
			overlayImage.className = "content-overlay-image";
			return overlayImage;
		},
		
		_createLinkImage: function(imageClass, resourceLabel) {
			var span = document.createElement("span");
			dojo.addClass(span, "img-div");
			this._img.appendChild(span);
			dojo.addClass(this._img, "link-img");
			this._createLinkImageIcon(span, imageClass, resourceLabel);
			return span;
		},
		
		_createLinkImageIcon: function(parentNode, imageClass, label) {
			if (!label) {
				label = "";
			}
			var img = document.createElement("img");
			img.src = this._blankGif;
			
			parentNode.appendChild(img);
			
			if (this._isContentLink()) {
				// visually indicate contentlinks with an overlay icon
				var overlayImage = this._createOverlayImage();
				img.setAttribute("alt", label);
				img.setAttribute("title", label);
				parentNode.appendChild(overlayImage);
			} 
			var foundBaselineURI = null;
			var foundDiscoveredLink = false;
			var foundInHistoryMode = (!this.isViewerInHistoryMode)?false:this.isViewerInHistoryMode;
			
			if(this.resourceViewer && this.link){
				foundBaselineURI = this.resourceViewer.baselineURI;
				foundDiscoveredLink = this.link.isDiscoveredLink;			
			}
			if(this.remoteLinkInfo){
				foundBaselineURI = this.remoteLinkInfo.baselineURI;
				foundDiscoveredLink = this.remoteLinkInfo.isDiscoveredLink;
			}
			var isDiscovered = (foundInHistoryMode || foundBaselineURI) && foundDiscoveredLink;
			if (isDiscovered || (this.resource && this.resource.isNotInModuleBaseline)) {
				// visually indicate discovered links within a baseline and module links that are not in the baseline
				// with an overlay icon.
				dojo.addClass(parentNode, "img-baseline-overlay-div");
				var overlayImg = document.createElement("img");
				overlayImg.src = this._blankGif;
				dojo.addClass(overlayImg, "discovered-overlay-image");
				var imageLabel = null;
				if (isDiscovered) {
					imageLabel = this._messages.inaccurateLinks;
				} else {
					imageLabel = this._messages.notInModuleBaseline;
				}
				img.setAttribute("alt", imageLabel);
				img.setAttribute("title", imageLabel);
				if (this._isContentLink()) {
					dojo.addClass(overlayImg, "baseline-content-overlay-image");
				} else {
					dojo.addClass(overlayImg, "baseline-overlay-image");
				}
				parentNode.appendChild(overlayImg);
			} else if (!this._isContentLink()) {
				img.className = "content-type-entry-image";
				img.setAttribute("alt", label);
				img.setAttribute("title", label);
			}
			if(this.link && this.link.isBaseArtifactLink){
				var iconImg = document.createElement("img");
				iconImg.src = this._blankGif;
				dojo.addClass(iconImg, "baseartifact-link-overlay-image");
				parentNode.appendChild(iconImg);
				dojo.addClass(parentNode, "img-baseartifact-overlay-div");
				if (this._isContentLink()) {
					if (isDiscovered || (this.resource && this.resource.isNotInModuleBaseline)) {
						dojo.removeClass(parentNode, "img-baseline-overlay-div");
						dojo.addClass(iconImg, "baseArtifactContentLink-notInBaseline");							
					} else {
						dojo.addClass(iconImg, "baseArtifactContentLink");							
					}
				} else {
					if (isDiscovered || (this.resource && this.resource.isNotInModuleBaseline)) {
						dojo.removeClass(parentNode, "img-baseline-overlay-div");
						dojo.addClass(iconImg, "baseArtifactLink-notInBaseline");							
					} else {
						dojo.addClass(iconImg, "baseArtifactLink");
					}
				}
			}
			dojo.addClass(img, imageClass);
			return img;
		},
		
		_isContentLink: function() {
			if (!this.link) {
				return false;
			}
			return this.link.isElementLink();
		},
		
		//Collect all resourcelinks created. We will need to destroy them on unitialize
		_addResourceLink: function(resourceLink) {
			this._resourceLinks.push(resourceLink);
		},
		
		_createExternalLink: function(label, target){
			//Create an external link, use the default external link icon to display the link
			var anchor = dojo.create("span");
			dojo.addClass(anchor, "img-div");
			this._img.appendChild(anchor);
			dojo.addClass(this._img, "link-img");
			var linkLabel = com.ibm.rdm.web.util.domutils.getXMLDecodedText(label);
			var imageClass = "externalLinkIcon";
			var img = this._createLinkImageIcon(anchor, imageClass);
			var linkAnchor = ResourceLinksHandler.createLink({
				uri: target,
				label: linkLabel,
				lazyFetch: true
			});
			return linkAnchor;
		},
		
		startup: function(){
			this.inherited("startup", arguments);
		},
		
		uninitialize: function(){
			//Destroy any resource links created.
			for(var i=0; i < this._resourceLinks.length; i++){
				this._resourceLinks[i].destroyRecursive();
				this._resourceLinks[i] = null;
			}
			this._resourceLinks = [];
		},
		
		destroy: function(){
			this.inherited(arguments);
		},
		
		hide: function(){
			this.status = com.ibm.rdm.web.links.LinkItemWidget.HIDDEN;
			this.linkRow.style.display = "none";
		},
		
		unhide: function(){
			this.status = com.ibm.rdm.web.links.LinkItemWidget.SHOWING;
			this.linkRow.style.display = "";
		},
		
		handleLinkSelectedByKeyPress: function(){
			// summary:
			//		Called by LinksListWidget.js when enter/space key is pressed on the 
			//		cell containing this LinkItemWidget.
			this._selectLink(this._anchorHref);
		},
		
		_handleLinkClicked: function(event){
			dojo.stopEvent(event);
			var clickedResourceURI = event.target.attributes.getNamedItem("href").value;
			this._selectLink(clickedResourceURI);
		},
		
		_selectLink: function(/*String*/resourceURI) {
			var reviewId = (this.resourceViewer && this.resourceViewer.reviewId);
			var projectSnapshotURI = (this.resourceViewer && this.resourceViewer.projectSnapshotURI);
			var linkURI = null;
			if (this.doNavigateSelection) {
				linkURI = this.link.getResourceURI();
			}
			//test if this artifact is hosted in a module, if so, then pass the modulefact URL to be selected
			var artifactInModule = null;
			if(this.moduleHostURI){
				artifactInModule = this.moduleHostURI;
				linkURI = null;
			}
			viewerutils.openResourceUsingAppropriateViewer(resourceURI, reviewId, projectSnapshotURI, linkURI, artifactInModule);
		},
		
		_isNodeLinkItemWidget: function(){
			try{
				if(this instanceof com.ibm.rdm.web.linksexplorer.NodeLinkItemWidget){
					return true;
				}else{
					return false;
				}
			}catch(e){}
			return false;
		}
	});
	
	com.ibm.rdm.web.links.LinkItemWidget.HIDDEN = "hidden";
	com.ibm.rdm.web.links.LinkItemWidget.SHOWING = "showing";
	
})();
