/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.viewer.RecentlyViewedSection");

dojo.require("com.ibm.rdm.web.viewer.RecentArtifactList");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("dojo.i18n");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.viewer.ComponentMenu");
dojo.require("com.ibm.rdm.web.model.types.RRMTypes");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");
dojo.require("com.ibm.rdm.web.external.ImportWizard");
dojo.require("com.ibm.rdm.web.common.ArtifactIcons");
dojo.require("com.ibm.rdm.web.util.bidiutils"); //bidi

dojo.requireLocalization("com.ibm.rdm.web.project", "DashboardTabPage");
dojo.requireLocalization("com.ibm.rdm.web.pages", "ResourcesPage");


var UriHelper = com.ibm.rdm.web.common.UriHelper;
var domutils = com.ibm.rdm.web.util.domutils;
var messages = dojo.i18n.getLocalization("com.ibm.rdm.web.pages", "ResourcesPage");
var messages3 = dojo.i18n.getLocalization("com.ibm.rdm.web.project", "DashboardTabPage");
var MimeTypes = com.ibm.rdm.web.common.MimeTypes;
var RRMTypes = com.ibm.rdm.web.model.types.RRMTypes;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var RecentArtifactList = com.ibm.rdm.web.viewer.RecentArtifactList;
var ResourceLinksHandler = com.ibm.rdm.web.common.ResourceLinksHandler;
var ArtifactIcons = com.ibm.rdm.web.common.ArtifactIcons;
var BidiUtils = com.ibm.rdm.web.util.bidiutils;  //bidi

var MAX_RECENTLY_VIEWED_ITEMS = 10;
var mapProjectUriToProjectRecentlyViewedInfo = {};

// TODO Consider adding "Show More" "Show Less" links to expand/shrink the
// number of items displayed in this section ... as it stands right now
// this class will display up to MAX_RECENTLY_VIEWED_ITEMS and when an item
// is added that would exceed this amount, the oldest item is removed from
// the listing.


com.ibm.rdm.web.viewer.RecentlyViewedSection = {

	// If the recently viewed item is in a module, we need to get the module
	// to present it's title. Keep a map of Module URI -> Module Objects here
	URI_TO_MODULE_MAP: {},

	VIEW_TYPE: {
		Artifacts: "artifacts",
		Modules: "modules",
		Collections: "collections",
		Collection: "collection"
	},

	addNewResource: function(resourceURI, reviewId, revision, /* Full resource object */ resourceObject) {
		var projectURI = resourceObject.projectURI;
		if (!(reviewId && 0 == resourceURI.indexOf(com.ibm.rdm.web.FRONTING_SERVICE_REVIEW_AND_APPROVAL))) {
			var projectRecentlyViewedInfo =
				this._getOrCreateProjectRecentlyViewedInfo(projectURI);
			// menuArtifactList will be set the the RecentArtifactList associated
			// with the specific menu for which this resource should be categorized
			// (Artifacts, Collections or Modules).
			var menuArtifactList = null;
			var artifactFormat = resourceObject.getMimeType();
			if (artifactFormat ===
				MimeTypes.getArtifactFormat(MimeTypes.COLLECTION_NAME, projectURI)
				) {
				// It's a Collection based artifact.
				menuArtifactList =
					projectRecentlyViewedInfo.collectionsRecentArtifactList;
			}
			else if (
				artifactFormat ===
				MimeTypes.getArtifactFormat(MimeTypes.MODULE_NAME, projectURI)
				) {
				// It's a Module based artifact.
				menuArtifactList =
					projectRecentlyViewedInfo.modulesRecentArtifactList;
			}
			else {
				// It's a normal artifact (not a collection or module).
				menuArtifactList =
					projectRecentlyViewedInfo.artifactsRecentArtifactList;
			}
			// Add this entry to the RecentArtifactList that's associated with a
			// specific menu (Artifacts, Collections or Modules).
			if (menuArtifactList) {
				menuArtifactList.addArtifactAccessed(resourceURI, revision, resourceObject);
			}
			// Add this entry to the RecentArtifactList that's associated with
			// all menus, so that the artifact picker can call
			// getRecentArtifacts() to get an appropriate "recently viewed"
			// artifact listing for ALL artifact types (regardless of whether
			// they're collections or modules.
			projectRecentlyViewedInfo.allMenusRecentArtifactList.addArtifactAccessed(
				resourceURI, revision, resourceObject
			);
			
			// Load the module and keep a track of it's name
			if (resourceObject.isInModule()) {
				com.ibm.rdm.web.model.resources.ResourceManager.get({
					resourceURI: resourceObject.getModuleURI(),
					skipCache: false
				}).then(dojo.hitch(this, function(moduleResource){
					this.URI_TO_MODULE_MAP[moduleResource.resourceURI] = moduleResource;						
				}), function(err){
					console.debug("Failed to load module ", err);
				});
			}
		}
	},
	
	getRecentArtifacts: function(
		/*com.ibm.rdm.web.viewer.RecentlyViewedSection.VIEW_TYPE?*/viewType)
	{
		// summary:
		//		Returns an array of artifact properties, each element of which
		//		represents a "recently viewed artifact".
		// description:
		//		If the viewType is passed the returned list will represent only
		//		those recently viewed artifacts pertaining to a particular menu
		//		(e.g. viewType.Collections will return only recently viewed
		//		collections.  If not passed, the returned list will represent
		//		recently viewed artifacts from all the Artifacts, Collections
		//		and Module MegaMenu drop downs.
		// returns:
		//		An array in which each element is of the form:
		//		{
		//		    resourceURI: <resourceURI>,
		//		    revision: <revision>,
		//		    resourceObject: <resourceObject>
		//		}
		// viewType:
		//		Optional, but if specified, it must be one of the values found in:
		//		    com.ibm.rdm.web.viewer.RecentlyViewedSection.VIEW_TYPE
		//		If not specified the returned array will represent the "recently
		//		viewed" artifacts without regard to what kind of artifacts they
		//		are (e.g. collection, module or otherwise).  Note: in practice,
		//		only the the artifact picker calls this function without passing
		//		this parameter.
		var projectURI = com.ibm.rdm.web.CURRENT_PROJECT;
		var projectRecentlyViewedInfo = this._getOrCreateProjectRecentlyViewedInfo(projectURI);
		var recentArtifactList = null;
		if (viewType) {
			switch (viewType) {
			case com.ibm.rdm.web.viewer.RecentlyViewedSection.VIEW_TYPE.Artifacts:
				recentArtifactList = projectRecentlyViewedInfo.artifactsRecentArtifactList;
				break;
			case com.ibm.rdm.web.viewer.RecentlyViewedSection.VIEW_TYPE.Collections:
				recentArtifactList = projectRecentlyViewedInfo.collectionsRecentArtifactList;
				break;
			case com.ibm.rdm.web.viewer.RecentlyViewedSection.VIEW_TYPE.Modules:
				recentArtifactList = projectRecentlyViewedInfo.modulesRecentArtifactList;
				break;
			}
			return recentArtifactList.getRecentArtifacts();
		} else {
			return projectRecentlyViewedInfo.allMenusRecentArtifactList.getRecentArtifacts();
		}
	},
	
	removeResource: function(resourceURI) {
		var projectURI = com.ibm.rdm.web.CURRENT_PROJECT;
		var projectRecentlyViewedInfo = this._getOrCreateProjectRecentlyViewedInfo(projectURI);

		// Note: All we have to go on is the resourceURI, so we don't know which of
		// the three RecentArtifactList instances (associated with this project)
		// needs to remove this artifact, so we just ask each of them to remove
		// the entry if it's present ...
		projectRecentlyViewedInfo.artifactsRecentArtifactList.removeArtifact(resourceURI);
		projectRecentlyViewedInfo.collectionsRecentArtifactList.removeArtifact(resourceURI);
		projectRecentlyViewedInfo.modulesRecentArtifactList.removeArtifact(resourceURI);

		projectRecentlyViewedInfo.allMenusRecentArtifactList.removeArtifact(resourceURI);
	},


	_getOrCreateProjectRecentlyViewedInfo: function(projectURI) {
		var projectRecentlyViewedInfo =
			mapProjectUriToProjectRecentlyViewedInfo[projectURI];
		if (!projectRecentlyViewedInfo) {
			projectRecentlyViewedInfo = {
				artifactsRecentArtifactList:
					new RecentArtifactList(MAX_RECENTLY_VIEWED_ITEMS),
				collectionsRecentArtifactList:
					new RecentArtifactList(MAX_RECENTLY_VIEWED_ITEMS),
				modulesRecentArtifactList:
					new RecentArtifactList(MAX_RECENTLY_VIEWED_ITEMS),
				allMenusRecentArtifactList:
					new RecentArtifactList(3 * MAX_RECENTLY_VIEWED_ITEMS)
			};
			mapProjectUriToProjectRecentlyViewedInfo[projectURI] =
				projectRecentlyViewedInfo;
		}
		return projectRecentlyViewedInfo;
	},

	_menuProvider: function() {
		var openHandler = dojo.hitch(this, this._populateArtifactsMenu);
		var menu = new com.ibm.rdm.web.viewer.ComponentMenu({
			onMenuOpen: function() {
				this.clear(); 
				openHandler(this);
			},
			
			getSelected: function() {
				var items = this.getItems();
				
				//check if browse artifacts should be selected
				if (items.length != 0
					&& com.ibm.rdm.web.LAST_ARGS.action == "com.ibm.rdm.web.pages.showProjectDashboard"
					&& items[0].label == messages3.browseArtifacts 
					&& ((com.ibm.rdm.web.LAST_ARGS.viewType != com.ibm.rdm.web.viewer.RecentlyViewedSection.VIEW_TYPE.Modules 
							&& com.ibm.rdm.web.LAST_ARGS.viewType != com.ibm.rdm.web.viewer.RecentlyViewedSection.VIEW_TYPE.Collection))
					) {
					return items[0];
				}
	
				//check recently viewed 
				for(var i = 0; i < items.length; i++) {
					var item = items[i];
					if (item.resourceUri&&com.ibm.rdm.web.LAST_ARGS.artifactURI
						&& item.resourceUri==com.ibm.rdm.web.LAST_ARGS.artifactURI
						) {
						return item;
					}
				}
				
				return null;
			}
			
		});
		return menu;
	},
	
	_getBrowseArtifactsHref: function() {
		return com.ibm.rdm.web.pages.WebUIinitializer.getNavigateToProjectDashboardURL(this.projectURI, null, com.ibm.rdm.web.CURRENT_PROJECT_BASELINE);
	},
	
	_removeAndDestroyLoadingMenuItem: function(menu, loadingMenuItem) {
		menu.remove(loadingMenuItem);
		loadingMenuItem.destroy();
	},

	_populateArtifactsMenu: function(menu) {
		if (com.ibm.rdm.web.CURRENT_PROJECT) {
			this.projectURI = com.ibm.rdm.web.CURRENT_PROJECT;
			var linkHref = this._getBrowseArtifactsHref();
			menu.addItemByArgs({
				label: messages3.browseArtifacts,
				href: linkHref,
				iconClass: "browseArtifactsIcon"
			});
			var loadingMenuItem = menu.addLoading(messages.loading, 2);
			
			ProjectCache.getProjectPermissions(this.projectURI).then(
				dojo.hitch(this, function(projectOperations){
					this._removeAndDestroyLoadingMenuItem(menu, loadingMenuItem);
					var isModifyResourceAllowed = projectOperations.item(ProjectCache.MODIFY_RESOURCE);
					var loadingItem = null;
					var recentIndex = 1;
					this._addCreateArtifactsRecentlyViewedMenuSections(
						menu, recentIndex, isModifyResourceAllowed
					);
				})
			);
		}
	},

	_addCreateArtifactsRecentlyViewedMenuSections: function(menu, recentIndex, isModifyResourceAllowed) {
		var LinksExplorerDialog = dojo.getObject("com.ibm.rdm.web.linksexplorer.LinksExplorerDialog"),
		currentExplorerDialog = LinksExplorerDialog && LinksExplorerDialog.returnInstance();

		var loadingItem = null;
		if (isModifyResourceAllowed) {
			menu.addTitle(messages.createArtifactMenu, 1);
			loadingItem = menu.addLoading(messages.loading, 2);
			recentIndex += 2;
		}

		//Add a section for Recent Artifacts.
		var recentArtifacts = this.getRecentArtifacts(
			com.ibm.rdm.web.viewer.RecentlyViewedSection.VIEW_TYPE.Artifacts
		);
		
		if (recentArtifacts.length === 0 && currentExplorerDialog) {
			var explorerArgs = {};
			explorerArgs.insertIndex = recentIndex;
			explorerArgs.wrapHint = 2;//FORCE_WRAP constant
				
			menu.addTitle(messages3.linksExplorerTitle, explorerArgs);
			menu.addItemByArgs({
				label: messages3.linksExplorerLabel + currentExplorerDialog._dialogTitle, 
				icon: "com.ibm.rdm.web/links/images/icons/etool16/links_exp.gif",
				onClick: dojo.hitch(this, this._openCurrentExplorer)
			});
		}
		
		if (recentArtifacts.length > 0) {
			var args = {};
			args.insertIndex = recentIndex;
			args.wrapHint = 2;//FORCE_WRAP constant
			menu.addTitle(dojo.string.substitute(messages.recentlyViewedArtifacts, [recentArtifacts.length]), args);
			recentIndex = args.insertIndex;
			
			var counter = recentIndex + 1;
			
			for (var i = 0; i < recentArtifacts.length; i++) {
				var recentArtifact = recentArtifacts[i];
				for (var typeURI in recentArtifact.resourceObject.getTypes()) {};
				var imageClass = ArtifactIcons.getSmallImageClass(this.projectURI, typeURI);
				var label = domutils.getXMLEncodedText(recentArtifact.resourceObject.getID() + ": " + BidiUtils.setTextDirUCC(recentArtifact.resourceObject.getDisplayTitle(), true));
				if (recentArtifact.resourceObject.isInModule()) {
					var moduleURI = recentArtifact.resourceObject.getModuleURI(),
						module = this.URI_TO_MODULE_MAP[moduleURI],
						title = module ? BidiUtils.setTextDirUCC(module.getTitle(), true) : "";
					
					menu.addItemByArgs({
						label: label + " "+dojo.string.substitute(messages["inModule"], [title]),
						href: this._getLinkHref(recentArtifact),
						iconClass: imageClass
					}, counter);
						
					
				}
				else {
				
					menu.addItemByArgs({
						label: label,
						href: this._getLinkHref(recentArtifact),
						iconClass: imageClass
					}, counter++);
				}
			}
			if (currentExplorerDialog) {
				menu.addTitle(messages3.linksExplorerTitle);
				menu.addItemByArgs({
					label: messages3.linksExplorerLabel + currentExplorerDialog._dialogTitle, 
					icon: "com.ibm.rdm.web/links/images/icons/etool16/links_exp.gif",
					onClick: dojo.hitch(this, this._openCurrentExplorer)					
				});
			}
		}
		
		if (com.ibm.rdm.web.CURRENT_PROJECT && isModifyResourceAllowed) {
			//Add the details under the loading item
			var objectTypesDeferred = com.ibm.rdm.web.model.types.TypesManager.getObjectTypesForProject({projectURI: com.ibm.rdm.web.CURRENT_PROJECT});
			
			objectTypesDeferred.addCallback(this, function(objectTypesInfo){
				var index = loadingItem.getIndexInParent();
				this._removeAndDestroyLoadingMenuItem(menu, loadingItem);
				
				this._addUploadArtifactMenuItem(menu, index);
				var importMenuItem = this._addImportMenuItem(menu, index);
				
				if (objectTypesInfo.length > 0) {
					// The top section of the menu holds artifact types that have
					// anything except "Glossary", "Term" and "Collection" as their default format
					var objectTypes = dojo.filter(objectTypesInfo, function(item){
						return item.getDefaultFormat() !== MimeTypes.getArtifactFormat(MimeTypes.GLOSSARY_NAME, com.ibm.rdm.web.CURRENT_PROJECT) &&
						item.getDefaultFormat() !== MimeTypes.getArtifactFormat(MimeTypes.TERM_NAME, com.ibm.rdm.web.CURRENT_PROJECT) &&
						item.getDefaultFormat() !== MimeTypes.getArtifactFormat(MimeTypes.COLLECTION_NAME, com.ibm.rdm.web.CURRENT_PROJECT) &&
						item.getDefaultFormat() !== MimeTypes.getArtifactFormat(MimeTypes.WRAPPER_RESOURCE_NAME, com.ibm.rdm.web.CURRENT_PROJECT);
					});
										
					var MAX_OBJECT_TYPE_MENU_ITEMS = 9;
					var start = Math.min(MAX_OBJECT_TYPE_MENU_ITEMS, objectTypes.length);
					//Add backwards as insertion reverses them.
					for (var i = start-1; i >= 0; i--) {
						this._addCreateArtifactTypeMenuItem(menu, objectTypes[i], index);
					}
					if (objectTypes.length > MAX_OBJECT_TYPE_MENU_ITEMS) {
						this._addArtifactTypeMoreMenuItem(menu, importMenuItem.getIndexInParent());						
					}
		
					// The artifact formats section of the menu lets you choose
					// any of the other formats, such as "Collection" and
					// "Glossary" to create artifacts with.
					if (!com.ibm.rdm.web.release.HIDE_2011_FEATURES) {
						var artifactFormats = dojo.filter(com.ibm.rdm.web.common.MimeTypeRegistry.getArtifactsOrdered(com.ibm.rdm.web.CURRENT_PROJECT), function(item){
							return item !==  com.ibm.rdm.web.common.MimeTypeRegistry.getMimeType(
												MimeTypes.getArtifactFormat(MimeTypes.REQUIREMENT_NEW_NAME, com.ibm.rdm.web.CURRENT_PROJECT),com.ibm.rdm.web.CURRENT_PROJECT);
						});
						for (var i = 0; i < artifactFormats.length; i++) {
							menu.add(this._createArtifactFormatMenuItem(artifactFormats[i]), insertIndex);
						}
					}
					
					// Add a menu separator to separate the Import and Upload Artifact menu items from the artifact types.
					menu.addSeparator(start+3);
				}								
				
				return objectTypesInfo;
			});
			objectTypesDeferred.addErrback(this, function(errback){
				// TODO:
				//		handle this error
				return errback;
			});
		} 
	},
	
	_openCurrentExplorer: function() {
		dojo.publish('linksexplorer/restore_explorer');
	},
	
	_addUploadArtifactMenuItem: function(menu, index){
		menu.addItemByArgs(
			{
				label: messages["uploadArtifact"],
				onClick: dojo.hitch(this, function(){
					this._uploadArtifact();
				}),
				iconClass: "uploadIcon"
			},
			index
		);
	},
	
	_addImportMenuItem: function(menu, index){
		return menu.addItemByArgs(
			{
				label: messages["import"],
				onClick: dojo.hitch(this, function(){
					this._invokeImport();
				}),
				iconClass: "importIcon"
			},
			index
		);
	},
	
	_uploadArtifact: function() {
		var projectURI = com.ibm.rdm.web.CURRENT_PROJECT;
		var selectedFolder = this.setFolderURI();	
		var uploadArtifactDialog = new com.ibm.rdm.web.project.UploadArtifactForm({
			artifactFormat: MimeTypes.getArtifactFormat(MimeTypes.WRAPPER_RESOURCE_NAME, this.projectURI),
			projectURI: projectURI,
			folderURI: selectedFolder
		});
		uploadArtifactDialog.show();
		var con = dojo.connect(uploadArtifactDialog, "onCreate", this, function (artifact, openArtifactInEditMode){
			dojo.disconnect(con);
			if (openArtifactInEditMode) {
				var artifactToOpen = dojo.isArray(artifact) ? artifact[0] : artifact;
				ResourceLinksHandler.showArtifactInEditMode(artifactToOpen.getResourceURI());
			}
		});
	},
	
	_invokeImport: function() {
		var projectURI = com.ibm.rdm.web.CURRENT_PROJECT;
		var selectedFolder = this.setFolderURI();
		var wizard = new com.ibm.rdm.web.external.ImportWizard({
			projectURI: projectURI,
			folderURI: selectedFolder
		});
		wizard.show();
		wizard.addWizardSteps(com.ibm.rdm.web.external.ImportWizard.wizardType.HOME_PAGE);
		wizard.selectFirstOption();
	},

	_addCreateArtifactTypeMenuItem: function(menu, objectType, index) {
		menu.addItemByArgs(
			{
				label: domutils.getXMLEncodedText(objectType.getTitle()),
				onClick: dojo.hitch(this, function(){
					this._createArtifact({
						typeURI: objectType.getResourceURI()
					});
				}),
				iconClass: ArtifactIcons.getSmallImageClass(this.projectURI, objectType.getResourceURI())
			},
			index
		);
	},

	_addArtifactTypeMoreMenuItem: function(menu, index){
		menu.addItemByArgs(
			{
				label: messages3.more,
				onClick: dojo.hitch(this, function(){
					this._createArtifact({
						artifactFormat: RRMTypes.getArtifactFormats(this.projectURI).Text,
						isMoreOption: true
					});
				})
			},
			index
		);
	},

	_createArtifactFormatMenuItem: function(artifactFormat){
		var menuItem = new dijit.MenuItem({
			label: artifactFormat.name,
			onClick: dojo.hitch(this, function(){
				this._createArtifact({
					artifactFormat: artifactFormat.contentType
				});
			})
		});
		menuItem.iconNode.src = artifactFormat.imageURI;
		return menuItem;
	},
	
	_createArtifact: function(args) {
		var projectURI = com.ibm.rdm.web.CURRENT_PROJECT;
		var selectedFolder = this.setFolderURI();	
		var newArtifactDialog = new com.ibm.rdm.web.project.NewArtifactForm({
			artifactFormat: args && args.artifactFormat ? args.artifactFormat : null,
			typeURI: args && args.typeURI ? args.typeURI : null,		
			projectURI: projectURI,
			folderURI: selectedFolder,
			isMoreOption: args.isMoreOption
		});
		newArtifactDialog.show();
		var con = dojo.connect(newArtifactDialog, "onCreate", this, function (artifact, openArtifactInEditMode){
			dojo.disconnect(con);
			if (openArtifactInEditMode) {
				ResourceLinksHandler.showArtifactInEditMode(artifact.getResourceURI());
			}
		});
	},

	_getLinkHref: function(resource) {
		var href;
		if (resource.resourceObject.reviewId) {
			// The resource is either a review or an artifact under review.
			if (0 === resource.resourceURI.indexOf(com.ibm.rdm.web.FRONTING_SERVICE_REVIEW_AND_APPROVAL)) {
				// It's a review.
				href = UriHelper.getResourceJafLink(resource.resourceObject.getResourceURI());
			} else {
				// It's an artifact under review
				href = UriHelper.getResourceInReviewContextJafLink(resource.resourceObject.getResourceURI(), resource.revision, resource.resourceObject.reviewId);
			}
		} else {
			// The resource is NOT a review or an artifact under review.
			href = UriHelper.getResourceJafLink(resource.resourceObject.getResourceURI());
		}
		return href;		
	},
	// Uses either the foldersSection or breadcrumbs to find the correct folderURI
	// if folderURI isn't set it will default to the root folder.
	setFolderURI: function() {
		var selectedFolder = null;
		try {
			if(this._foldersSection) {
				// HACK
				// Querys the DOM with the css selector :visible to see if the folder 
				// widget is visible. If not, it uses breadcrumbs to set folderURI
				if (dojo.query('#'+this._foldersSection.id+':visible').length > 0){
						selectedFolder = this._foldersSection.getSelectedFolders()[0];			
				} 
			}
			// We have a breadcrumb reference and selected folder hasn't been set yet
			if (this._foldersBreadcrumb && !selectedFolder){
					selectedFolder = this._foldersBreadcrumb.uri;
			}
			return selectedFolder;
		}
		// Just to be safe
		catch (error) {
		if (djConfig.isDebug) {
			console.log("Couldn't find selected folder ", error);
			console.log("folderSection: ", this._foldersSection, " breadCrumb: " , this._foldersBreadcrumb);
		}	
			return null;
		}
		
	},
	// sets the foldersection reference to the currently viewed grid folder filter
	setFoldersSection: function(foldersSection) {	
		this._foldersSection = foldersSection;
	},
	// Updates the breadcrumb to the currently viewed artifact
	setFolderBreadcrumb: function(folderBreadcrumb) {
		// gets the last folder in the breadcrumb widget
		this._foldersBreadcrumb = null;
		if (folderBreadcrumb && folderBreadcrumb.length > 1) {
			this._foldersBreadcrumb = folderBreadcrumb[folderBreadcrumb.length - 1];
		} 
		if (this._foldersBreadcrumb && this._foldersBreadcrumb.uri && this._foldersBreadcrumb.uri.indexOf("/folders/") === -1) {
			// We found a module URI. Lets try one up.
			this._foldersBreadcrumb = (folderBreadcrumb && folderBreadcrumb.length > 2) ?
			folderBreadcrumb[folderBreadcrumb.length - 2] : null;
		}
	},
	// Used when the dashboard is loaded. If this didn't happen the user dashboard would 
	// use the last used breadcrumbs last folderURI. If these are both null the folderURI would be null,
	// which makes the default folder the rootURI
	resetFolderReferences: function(){
		this._foldersBreadcrumb = null;
		this._foldersSection = null;
	}
};

})();
