/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2014. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rqm.defects.web.client.internal.ResourceLinkService");

dojo.require("com.ibm.rqm.defects.web.util.XMLUtil");
dojo.require("com.ibm.rqm.defects.web.util.ArtifactURIUtil");
dojo.require("com.ibm.asq.common.web.client.internal.Constants");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("dojox.xml.parser");
dojo.require("com.ibm.rqm.defects.web.client.internal.ServiceProviderProxyClient");

(function(){
    var XMLUtil = com.ibm.rqm.defects.web.util.XMLUtil;
    var Constants = com.ibm.asq.common.web.client.internal.Constants;
    var Utility = com.ibm.asq.common.web.ui.internal.util.Utility;
    var ArtifactURIUtil = com.ibm.rqm.defects.web.util.ArtifactURIUtil;
    var ServiceProviderProxyClient = com.ibm.rqm.defects.web.client.internal.ServiceProviderProxyClient;
    // XML resource types
    
    // turn off 1.0 header for now... Need a better handling to support both v1 & v2 headers...
	var HEADER_ACCEPT_REQ_V1 = "application/x-oslc-rm-requirement-1.0+xml";
	var HEADER_ACCEPT_REQ_COL_V1 = "application/x-oslc-rm-requirement-collection-1.0+xml";
    var HEADER_ACCEPT_XML = "application/xml";
    var HEADER_ACCEPT_XML_V2 = "application/rdf+xml";
    var RESOURCE_TYPE_REQ = "rm-req";
    var RESOURCE_TYPE_REQ_COL = "rm-req-col";    
    var NAMESPACE_TAG_V1 = "xmlns";
    var RESOURCE_TYPE_BUILD_REC = "qm_bld";
    
    // XML element tags
    var TAG_RELATED_CHANGE_REQUEST = "relatedChangeRequest";
    var TAG_VALIDATES_REQ = "validatesRequirement";
    var TAG_VALIDATES_REQ_SET = "validatesRequirementSet";
	var TAG_VALIDATES_REQ_SET_V2 = "validatesRequirementCollection";
    var TAG_LABEL = "label";
    var TAG_RESOURCE = "resource";
    var TAG_LOCATION = "location";
    var TAG_STATUS = "status";
    var TAG_VALIDATED_BY = "validatedBy";
	var TAG_AFFECTED_BY = "affectedBy";
    var TAG_LINK_COL = "LinkCollection";
    var TAG_RDF = "RDF";
    var TAG_REQ = "Requirement";
    var TAG_REQ_COL = "RequirementCollection";
    var TAG_LI = "li";
    var TAG_DESCRIPTION = "Description";
    var TAG_MEMBER= "member";
	var TAG_ABOUT = "about";
	var TAG_TITLE = "title";
	var TAG_ID = "ID";
	var TAG_PREX_RDF = "rdf";
	var TAG_PREX_OSLC_QM = "oslc_qm";
	var TAG_PREX_RQM_QM = "rqm_qm";
	var TAG_PREX_RDFS = "rdfs";
	var TAG_PREX_OSLC_RM = "oslc_rm";
	var TAG_PREX_CALM = "calm";
	var TAG_PREX_BASE = "prefix_";
	var TAG_TESTCASE = "TestCase";
    var TAG_TESTPLAN = "TestPlan";
	var TAG_TESTSCRIPT = "TestScript";
	var TAG_TCRESULT = "TestResult";
	var TAG_TSRESULT = "TestSuiteResult";
	var TAG_BUILDRESULT = "AutomationResult";
	var TAG_DCTERMS = "dcterms";
	var TAG_DC = "dc";
	var TAG_RDF_PARSETYPE = "parseType";
	var TAG_LITERAL = "Literal";
	var TAG_SERVICE_PROVIDER = "serviceProvider";
	var TAG_EXECUTED_ON = "executedOnBuild";
	var TAG_TESTCASE_QUERY = "TestCaseQuery";
	var TAG_TESTCASE_2 = "testCase";
	var TAG_WEBID = "webId";
	var TAG_SOFT_DELETED="softDeleted";
	var TAG_MODIFIED="modified";
	var TAG_EXECUTIONRECORD = "TestExecutionRecord";
	var TAG_TESTRESULT = "TestResult";
	var TAG_EXECUTED_ON_DEPLOYMENT = "executedOnDeployment";
	var TAG_RUNS_ON_DEPLOYMENT = "runsOnDeployment";
	var TAG_CHG_REQUEST = "ChangeRequest";
	
	// collection V2
	 var TAG_USES = "uses";
	
	
    // Link collection id
    var ID_VALIDATES_COL = "links/validatesRequirement";
	var ID_VALIDATES_REQ_SET_COL_V2 = "links/validatesRequirementCollection";
    var ID_VALIDATED_BY_COL = "validatedByCollection";
	
	// Link types
	var LINK_TYPE_TESTES_PI = "calm:testsPlanItem";
	
	//newAdded Reqt Links
	var NEW_ADDED_LINKS = "newAdded";
	var NEW_ADDED_LINKS_TAG = "#newAdded";

 	var IS_IE_OR_CHROME = (dojo.isIE && dojo.isIE < 9) || dojo.isChrome || dojo.isSafari;
 	
 	var QUERY_MAX_PAGE_SIZE = 512;
 	
	//Summary:
	//		A global map for deffered _getLinksDetail requests
	//		it is for the purpuse of cancel request
	var onGoingRequests = {};
    
    // Summary:
    // 		A client that provides utils to handle resource request, 
    //		parse resource representation and so on
    //
    
    com.ibm.rqm.defects.web.client.internal.ResourceLinkService = {
        /***
         * Retrieve a qm resource's 'validates' link collection.
         * @param resourceUri
         * @return - callback([{summary, uri}])
         */
        getValidatesLinks: function(resourceType, resourceUri, callback){
            _getValidatesLinks(resourceType, resourceUri, callback);
        },
		
        getValidatesLinksV2: function(resourceType, resourceUri, callback){
            _getValidatesLinksV2(resourceType, resourceUri, callback);
        },
        
        getRemoteRRCRequirement: function(resourceUri, callback, oslcVersion){
        	_getReqResource(resourceUri, callback, oslcVersion);
        },
        
         /***
         * Check whether the RM resource is requirement collection
         * @param {Object} resourceUri
         * @param {Object} callback [{uri, isCollection}]
         * @param {Object} oslcVersion
         */
        checkIsRmCollection: function(resourceUri, callback, oslcVersion){
        	_getIsReqCollectionResource(resourceUri, oslcVersion, 
        		dojo.hitch(this, function(result){
        			var callbackResult = {
        			uri: resourceUri,
        			isCollection : result
        			};
        			callback(callbackResult);
        		}), 
        		dojo.hitch(this, function(error){
        			console.log(error);
        			var callbackResult = {
        			uri: resourceUri,
        			isCollection : "false"
        			};
        			callback(callbackResult);
        		}));
        },
        
        /***
         * Update a qm resource's 'validates' link collection
         * @param {Object} resourceUri
         * @param {Object} addedLinks [{summary, uri}]
         * @param {Object} removedLinks [{summary, uri}]
         */
        updateValidatesLinks: function(oslcVersion, resourceType, resourceUri, addedLinks, removedLinks, callback, errorCallback){
            // 1. get the resource doc 
            _getValidatesDocV2(resourceType, resourceUri, dojo.hitch(this, function(doc){
                //2. modify links in the doc according to user's request
                _modifyValidatesLinks(resourceType, doc, addedLinks, removedLinks);
                //3. update the resource doc
                _updateValidatesDoc(oslcVersion, resourceType, resourceUri, doc, callback, errorCallback);
            }));
            
        },
        
        removeRelatedChangeRequestLinks: function(resourceUri, removedLinks, callback, errorCallback){
        	
        	if(!resourceUri || !removedLinks || removedLinks.length == 0){	
        		if(dojo.isFunction(errorCallback)){
        			errorCallback();
        		}else{
        			return;
        		}
        	}
        	
        	//1.Get test case resource xml first
        	_getRelatedChangeRequestDocV2(resourceUri, dojo.hitch(this, function(doc){
        		//2.Insert the relatedChangeRequest link into xml doc
        		_removeRelatedChangeRequestLinks(doc, removedLinks);
        		//3.Save the XML doc
        		_updateRelateChangeRequestDoc(resourceUri, doc, callback, errorCallback);    		
        	}));
        	
        	
        },
        
        getRelatedChangeRequestLinks: function(resourceUri, callback){
        	_getRelatedChangeRequestLinksV2(resourceUri, callback);
        },
        
        addRelatedChangeRequestLinks: function(resourceUri, newLinks, callback, errorCallback){
        	
        	if(!resourceUri || !newLinks || newLinks.length == 0){
        		
        		if(dojo.isFunction(errorCallback)){
        			errorCallback();
        		}else{
        			return;
        		}
        		
        	}
        	
        	//1.Get test case resource xml first
        	_getRelatedChangeRequestDocV2(resourceUri, dojo.hitch(this, function(doc){
        		//2.Insert the relatedChangeRequest link into xml doc
        		_addRelatedChangeRequestLinks(doc, newLinks);
        		//3.Save the XML doc
        		_updateRelateChangeRequestDoc(resourceUri, doc, callback, errorCallback);    		
        	}));
        },
        
        /***
         * Retrieve a requirement collection's linked requirements
         * @param {Object} collectionUri
         * @param {Object} callback
         * @return array of uri string
         */
        getLinksFromReqCollection: function(collectionUri, callback, errorCallback){
            _getLinksFromReqCollection(collectionUri, callback, errorCallback);
        },
        
		/**
		 * @deprecated it is only used by CollectionLinksUtil.getAllContainedLinks
		 * that method is deprecated
		 *
		 * @param {Object} collectionUri
		 * @param {Object} callback
		 * @param {Object} errorCallback
		 */
        getLinksFromReqCollectionV2: function(collectionUri, callback, errorCallback){
            _getLinksFromReqCollectionV2(collectionUri, callback, errorCallback);
        },
		
		/***
		 * Given the Requirement URI, get requirement details {uri, summary, oslcVersion, lastModified} from RM side
		 * @param {Object} uris
		 * @param {Object} callback
		 * @param {Object} errorCallback
		 */
		getReqtDetailsForReqtUrisV2: function(uris, callback, errorCallback){
			if (uris.length > 0) {
            	_getLinksDetail(uris, callback);
            } else {
            	callback([]);
            }
		},
		
		/**
		 * Given the collection URI, get all its requirements' urils
		 * 
		 * @param {Object} collectionUri
		 * @param {Object} callback
		 * @param {Object} errorCallback
		 */
		getReqtURIsWithoutDetailsFromReqCollV2: function(collectionUri, callback, errorCallback){
            _getReqtURIsWithoutDetailsFromReqCollV2(collectionUri, callback, errorCallback);
        },
		
		/**
		 * 
		 * @param {Object} collectionUri
		 * @param {Object} callback
		 * @param {Object} errorCallback
		 */
		getReqtURIsWithoutDetailsFromReqColl: function(collectionUri, callback, errorCallback){
            _getReqtURIsWithoutDetailsFromReqColl(collectionUri, callback, errorCallback);
        },
        
        testRequirementResource: function(resourceType, resourceUri, callback, resultCallbacks){
        	_getRequirementResourceV1(resourceType, resourceUri, callback, resultCallbacks);
        },
        
        
        /***
         * Retrieve a ccm resource's link collection of specified link type
         * @param {string} linkType
         * @param {string} workitemUri
         * @param {Object} callback
         * @param {Object} errorCallback
         * @return array of uri string
         */
        getLinksFromWorkItem: function(linkType, workitemUri, callback, errorCallback){
            _getLinksFromWorkItem(linkType, workitemUri, callback, errorCallback);
        },
        
        /***
         * Remove an existing link from rm resource's 'validatedBy' link collection
         * @param {Object} resourceUri
         * @param {Object} linkDesc [{linkComment, rqmUri}]
         */
        removeValidatedByLink: function(oslcVersion, resourceType, resourceUri, linkDesc){
            var removedLinks = [];
            removedLinks.push({
                'uri': linkDesc.rqmUri,
                'summary': linkDesc.linkComment
            });
            _getValidatedByDoc(oslcVersion, resourceType, resourceUri, function(doc){
                _modifyValidatedByLinks(oslcVersion, doc, [], removedLinks);
                _updateValidatedByDoc(oslcVersion, resourceType, resourceUri, doc);
            });
        },
        
        /***
         * add a new link in rm resource's 'validatedBy' link collection
         * @param {Object} resourceUri
         * @param {Object} linkDesc [{linkComment, rqmUri}]
         */
        addValidatedByLink: function(oslcVersion, resourceType, resourceUri, linkDesc){
            var addedLinks = [];
            addedLinks.push({
                'uri': linkDesc.rqmUri,
                'summary': linkDesc.linkComment
            });
            _getValidatedByDoc(oslcVersion, resourceType, resourceUri, function(doc){
                _modifyValidatedByLinks(oslcVersion, doc, addedLinks, []);
                _updateValidatedByDoc(oslcVersion, resourceType, resourceUri, doc);
            });
        },
		
		
		/**
		 * get validated by links for the specific resource, Requirement Link or Requirement Collection
         * @param {Object} resourceType
         * @param {Object} resourceUri
         * @param {Object} oslcVersion
         * @param {Object} callback
         * @param {Object} errorCallback
		 */
		getValidatedByLink: function(resourceType, resourceUri, oslcVersion, callback, errorCallback){
            return _getValidatedByDoc(oslcVersion, resourceType, resourceUri, function(doc){
                var links = _readValidatedByLinks(oslcVersion, doc);
				callback(links);
            }, errorCallback);
		},
        
        
        /***
         * Update one or many new links in rm resource's 'validatedBy' link collection. This will be done
         * in a synchronous way to be able to handle errors. It will process added links first, then removed ones.
         * @param oslcVersion
         * @param resourceType
         * @param [string] addedResourceUris
         * @param [string] removedResourceUris
         * @param {Object} linkDesc [{linkComment, rqmUri}]
         * @param {Object} callbacks {success, error}
         */
        updateValidatedByLinks: function(oslcVersion, resourceType, addedResourceUris, removedResourceUris, linkDesc, callbacks){
			var _linkDesc = dojo.isArray(linkDesc) ? linkDesc : [linkDesc];
			
        	_updateValidatedByLinks(oslcVersion, resourceType, addedResourceUris, removedResourceUris, _linkDesc, callbacks);
        	
        },
        
        /***
         * remove a 'affectedBy' link from rm resource. This link type is only defined in OSLC_RM v2.
         * @param {Object} resourceUri
         * @param {Object} linkDesc [{linkComment, rqmUri}]
         */
        removeAffectedByLink: function(oslcVersion, resourceType, resourceUri, linkDesc){
            var removedLinks = [];
            removedLinks.push({
                'uri': linkDesc.defectUri,
                'summary': linkDesc.linkComment
            });
			var callback = function(doc){
				if (oslcVersion == null  || oslcVersion == "1.0") {
					_modifyRequirementResourceV1(doc, [], removedLinks, TAG_AFFECTED_BY);
					_updateRequirementResourceV1(resourceType, resourceUri, doc);
				}
				else 
					if (oslcVersion == "2.0") {
						_modifyRequirementResourceV2(doc, [], removedLinks, TAG_AFFECTED_BY);
						_updateRequirementResourceV2(resourceType, resourceUri, doc);
					}
			};
			if(oslcVersion == null  || oslcVersion == "1.0"){
				_getRequirementResourceV1(resourceType, resourceUri, callback);
			}else if(oslcVersion == "2.0"){
				_getRequirementResourceV2(resourceType, resourceUri, callback);
			}
        },
		
		/***
         * add new 'affectedBy' link to requirement resource. This link type is only defined in OSLC_RM v2.
         * @param {Object} resourceUri
         * @param {Object} linkDesc [{linkComment, rqmUri}]
         */
        addAffectedByLink: function(oslcVersion, resourceType, resourceUri, linkDesc, extCallbacks){
            var addedLinks = [];
            var linkDescs = [];
			if(dojo.isArray(linkDesc)){
				linkDescs = linkDesc;
			} else {
				linkDescs.push(linkDesc);
			}
            dojo.forEach(linkDescs, function(link){
				addedLinks.push({
	                'uri': link.defectUri,
	                'summary': link.linkComment
	            });
			});
            //console.debug("Linking artifact: " + resourceUri + " affected by: " + linkDesc.linkComment);
			var callback = function(doc, etag){
                if(oslcVersion == null || oslcVersion == "1.0"){
					_modifyRequirementResourceV1(doc, addedLinks, [], TAG_AFFECTED_BY);
                	_updateRequirementResourceV1Sync(resourceType, resourceUri, doc, extCallbacks.next);
				} else if(oslcVersion == "2.0"){
					_modifyRequirementResourceV2(doc, addedLinks, [], TAG_AFFECTED_BY);
                	_updateRequirementResourceV2Sync(resourceType, resourceUri, doc, etag, extCallbacks.next);
				}
            };
			if(oslcVersion == null || oslcVersion == "1.0"){
				_getRequirementResourceV1(resourceType, resourceUri, callback, extCallbacks);
			}else if(oslcVersion == "2.0"){
				_getRequirementResourceV2(resourceType, resourceUri, callback, extCallbacks);
			}
        },
		
		/***
         * Retrieve a qm resource's 'affectsPlanItem' link collection.
         * @param resourceUri
         * @return - callback([{summary, uri}])
         */
        getTestsPlanItemLinks: function(resourceUri, callback){
            this.getLinks(LINK_TYPE_TESTES_PI, resourceUri, callback);
        },
        
        /***
         * Get links for a resource
         * @param {Object} linkType
         * @param {Object} resourceUri
         * @param {Object} callback
         * @return - callback([{summary, uri}])
         */
        getLinks: function(linkType, resourceUri, callback){
            jazz.client.xhrGet({
                url: resourceUri + "?oslc_qm.properties="+linkType,
                headers: {
                    'Accept': HEADER_ACCEPT_XML
                },
                handleAs: 'xml',
                load: function(doc){
                    var qname = XMLUtil.resolveQName(linkType);
                    var rootNode = doc.documentElement;
                    var linkNodes = XMLUtil.getNodes(rootNode.childNodes, qname["localPart"], qname["uri"]);
                    var links = [];
                    dojo.forEach(linkNodes, function(linkNode){
                        var summaryAttrNode = XMLUtil.getAttributeNode(linkNode, TAG_LABEL, XMLUtil.NAMESPACE_OSLC_QM);
                        var summary = XMLUtil.getAttributeNodeValue(summaryAttrNode);
                        var uriAttrNode = XMLUtil.getAttributeNode(linkNode, TAG_RESOURCE, XMLUtil.NAMESPACE_RDF);
                        var uri = XMLUtil.getAttributeNodeValue(uriAttrNode);
                        links.push({
                            'summary': summary,
                            'uri': uri
                        });
                    });
                    callback(links);
                }, 
                error: function(result) {
                    callback(null);
                }
            });
        },

        /***
         * Update the links for a resource
         * @param {Object} linkType
         * @param {Object} resourceUri
         * @param {Object} newItems
         * @param {Object} removedItems
         * @param {Object} callback
         * @return - callback(xhrPut response)
         */
        updateLinks: function(linkType, resourceUri, newItems, removedItems, callback) {
            var deferredGet = jazz.client.xhrGet({
                url: resourceUri + "?oslc_qm.properties="+linkType,
                headers: {
                    'Accept': HEADER_ACCEPT_XML
                },
                error: function(result) {
                    callback(null);
                },
                handleAs: 'xml'
            });
            
            deferredGet.addCallback(function(doc){
                if (doc instanceof Error) {
                    var status = new Status(Status.OK);
                    callback(status, doc);
                }
                
                var etag = deferredGet.ioArgs.xhr.getResponseHeader('ETag');
                var rootNode = doc.documentElement;

                var qname = XMLUtil.resolveQName(linkType);
                
                var linkNodes = XMLUtil.getNodes(rootNode.childNodes, qname.localPart, qname.uri);
                var existingLinks = [];
                dojo.forEach(linkNodes, function(linkNode){
                    var uriAttrNode = XMLUtil.getAttributeNode(linkNode, TAG_RESOURCE, XMLUtil.NAMESPACE_RDF);
                    var uri = XMLUtil.getAttributeNodeValue(uriAttrNode);
                    existingLinks.push(uri);
                });

                // remove the deleted links
                dojo.forEach(removedItems, function(removedItem){
                    var linkNodes = XMLUtil.getNodes(rootNode.childNodes, qname.localPart, qname.uri);
                    dojo.forEach(linkNodes, function(linkNode){
                        var uriAttrNode = XMLUtil.getAttributeNode(linkNode, TAG_RESOURCE, XMLUtil.NAMESPACE_RDF);
                        var uri = XMLUtil.getAttributeNodeValue(uriAttrNode);
                        if (removedItem.uri == uri) {
                            rootNode.removeChild(linkNode);
                        }
                    });
                });

                // add the new links
                dojo.forEach(newItems, function(newItem){
                    if (dojo.indexOf(existingLinks, newItem.uri) == -1) {
                        var linkNode = null;
                        if (IS_IE_OR_CHROME) { // handle IE & Chrome xml attribute manipulation 
                            linkNode = doc.createElement(qname.prefix + ":" + qname.localPart);
                            linkNode.setAttribute(TAG_PREX_RDF + ":" + TAG_RESOURCE, newItem.url);
                            linkNode.setAttribute(TAG_PREX_OSLC_QM + ":" + TAG_LABEL, newItem.summary);
                        }
                        else {
                            linkNode = doc.createElementNS(qname.uri, qname.localPart);
                            linkNode.setAttributeNS(XMLUtil.NAMESPACE_RDF, TAG_RESOURCE, newItem.url);
                            linkNode.setAttributeNS(XMLUtil.NAMESPACE_OSLC_QM, TAG_LABEL, newItem.summary);
                        }
                        rootNode.appendChild(linkNode);
                    }
                });

                // send the updated XML to the RQM server
                jazz.client.xhrPut({
                    url: resourceUri + "?oslc_qm.properties="+linkType,
                    headers: {
                        "Content-Type": HEADER_ACCEPT_XML,
                        "If-Match": etag
                    },
                    postData: dojox.xml.parser.innerXML(doc),
                    load: function(result){
                        if (callback && dojo.isFunction(callback)) {
                            callback(result);
                        }
                    }
                });
            });
        },
        //Similar method with "findCollectionContentChangedSince", used to retrieve a single requirement status
        //Requirement status includes Updated, Deleted and Others which can be treated as New.
        findReqContentChangedSince: function(serviceUrl, reqUrl, changedSince, callback, errCallback) {
            var args = "?item="+reqUrl+"&after="+changedSince;
            var url = serviceUrl + args;
            jazz.client.xhrGet({
                url:url,
                
                headers: {
                    'Accept': HEADER_ACCEPT_XML,
                    'DoorsRP-Request-Type' : 'private'
                },
                
                handleAs: 'xml',
                load: function(doc){
                    var rootNode = doc.documentElement;
                    var linkNodes = XMLUtil.getNodes(rootNode.childNodes, TAG_REQ, XMLUtil.NAMESPACE_OSLC_RDM);
                    var links = [];
                    dojo.forEach(linkNodes, function(linkNode){
                        var locNode = XMLUtil.getNode(linkNode.childNodes, TAG_LOCATION, null);
                       
                        var statusNode = XMLUtil.getNode(linkNode.childNodes, TAG_STATUS, null);
                        var locAttr = XMLUtil.getAttributeNode(locNode, TAG_RESOURCE, XMLUtil.NAMESPACE_RDF);
                        var uri = XMLUtil.getAttributeNodeValue(locAttr);
                        
                        //the attribute name is different between IE and FF
                        var status =  statusNode.textContent;
                        
                        if(!status){status = statusNode.text;}
                        
                        links.push({
                            'uri': uri,
                            'status': status
                        });
                    });
                    callback(links);
                }, 
                error: function(result) {
                    errCallback(result);
                }
            });     
        },
        
          /**
         * Returns a req's updated and soft deleted information
         * @param {Object} ReqUrl - the req url
         * @param {Object} changedSince - a timestamp in zulu format
         * @param {Object} callback - callback to call with the results
         * @param {Object} errCallback
         */
   
        findReqContentSoftDeletedAndUpdated: function(reqUrl, changedSince, callback, errCallback){
        	
        	_getRequirementResourceV2(com.ibm.rqm.defects.web.client.internal.ResourceLinkService.RESOURCE_TYPE_REQ,reqUrl, function(doc){
        		var links = [];
             	var rdfNode = XMLUtil.getNode(doc.childNodes, TAG_RDF, XMLUtil.NAMESPACE_RDF);
             	
             	var rootNode = XMLUtil.getNode(rdfNode.childNodes, TAG_REQ, XMLUtil.NAMESPACE_OSLC_RM_V2);
             	if (rootNode != undefined) {
             		var softDeletedNode = XMLUtil.getNode(rootNode.childNodes, TAG_SOFT_DELETED, XMLUtil.NAMESPACE_JRM);
             		
             		if (softDeletedNode) {
             			var softDeleted = dojox.xml.parser.textContent(softDeletedNode);
             			
             			if (softDeleted=="true"){
             				var linkTitleNode = XMLUtil.getNodes(rootNode.childNodes, TAG_TITLE, XMLUtil.NAMESPACE_DC);			
            				//IE uses text property
    						var summary = linkTitleNode[0].textContent || linkTitleNode[0].text;
                 			
        					links.push({
            					'uri': reqUrl,
            					'summary': summary,
            					'status': 'removed'
        					});
        					callback(links);
        					return;
        					
                 		};
             		}
             		
             		 var lastModifiedNode = XMLUtil.getNode(rootNode.childNodes, TAG_MODIFIED, XMLUtil.NAMESPACE_DCTERMS);
                     
                     if(lastModifiedNode){
                     	var lastModified = dojox.xml.parser.textContent(lastModifiedNode);
                     	
                     	//var lastModifiedDate = dojo.date.stamp.fromISOString(lastModified);
                     	
                     	if (lastModified>changedSince){
             				var linkTitleNode = XMLUtil.getNodes(rootNode.childNodes, TAG_TITLE, XMLUtil.NAMESPACE_DC);			
            				//IE uses text property
    						var summary = linkTitleNode[0].textContent || linkTitleNode[0].text;
                 			
        					links.push({
            					'uri': reqUrl,
            					'summary': summary,
            					'status': 'updated'
        					});
                 		};
                     }
             	}
             	callback(links);
        	},  function(error) {
    		 	console.log(error);
    		 	errCallback(error);});           
        },


        /**
         * Returns a collection's updated requirements.
         * @param {Object} serviceUrl - the filtering service url
         * @param {Object} collectionUrl - the collection url
         * @param {Object} changedSince - a timestamp in zulu format
         * @param {Object} callback - callback to call with the results
         * @param {Object} errCallback
         */
        findCollectionContentChangedSince: function(serviceUrl, collectionUrl, changedSince, callback, errCallback) {
            var args = "?collection="+collectionUrl+"&after="+changedSince;
            var url = serviceUrl + args;
            jazz.client.xhrGet({
                url:url,
                
                headers: {
                    'Accept': HEADER_ACCEPT_XML,
                    'DoorsRP-Request-Type' : 'private'
                },
                
                handleAs: 'xml',
                load: function(doc){
                    var rootNode = doc.documentElement;
                    var linkNodes = XMLUtil.getNodes(rootNode.childNodes, TAG_REQ, XMLUtil.NAMESPACE_OSLC_RDM);
                    var links = [];
                    dojo.forEach(linkNodes, function(linkNode){
                        var locNode = XMLUtil.getNode(linkNode.childNodes, TAG_LOCATION, null);
                        var statusNode = XMLUtil.getNode(linkNode.childNodes, TAG_STATUS, null);
                        var locAttr = XMLUtil.getAttributeNode(locNode, TAG_RESOURCE, XMLUtil.NAMESPACE_RDF);
                        var uri = XMLUtil.getAttributeNodeValue(locAttr);
						//for IE the attribute is text not textContent
                        var status = statusNode.textContent ? statusNode.textContent : statusNode.text;
                        
                        links.push({
                            'uri': uri,
                            'status': status
                        });
                    });
                    callback(links);
                }, 
                error: function(result) {
                    errCallback(result);
                }
            });
        },
        
          /**
         * Returns a collection's updated requirements.
         * @param {Object} ReqUrl - the collection url
         * @param {Object} changedSince - a timestamp in zulu format
         * @param {Object} callback - callback to call with the results
         * @param {Object} errCallback
         */
        findCollectionContentModifiedByQueryService: function(collectionUrl, changedSince, callback, errCallback) {
        
        	var serviceUrl = {};
          	var args = "?oslc.where=dcterms:modified>"+ '"' + changedSince + '"';
        	//var args = "?oslc.where=dcterms:modified>"+ '"' + "2012-02-10T09:22:07Z" + '"';
            
            var querybaseUrl = {};
            
            this._getQueryServiceFromReqCollectionV2(collectionUrl, 
            function(providerURL){
            	if (providerURL) {
     				
     			com.ibm.rqm.defects.web.client.internal.ResourceLinkService._getProviderServiceDocV2 (providerURL, 
     				function(result){
                		var services = com.ibm.rqm.defects.web.util.OSLCServicesDocUtil.readServiceDescription(result, "validates", "2.0");
                		// TODO - This is a walkaroud for DOORS - as the View provider will return the reqQureyService
                		// For the Repostiory Service, it will retrun collectionQueryService
                		if (services.collectionQueryService || services.reqQueryService){
                			var queryService = services.collectionQueryService || services.reqQueryService;
                			querybaseUrl = queryService.url;
                		} else {
                			var result = {};
                			result.message = "No Query Service for the Provider of the collection.";
                			result.uri = collectionUrl;
                			errCallback(result);
                		}
                			
                		if (querybaseUrl)
            				serviceUrl = querybaseUrl;
          
            				var url = encodeURI(serviceUrl + args);
            				jazz.client.xhrGet({
                				url:url,
                
                				headers: {
                    				'Accept': HEADER_ACCEPT_XML_V2,
                    				"OSLC-Core-Version": "2.0"    
                				},
                
                				handleAs: 'xml',
                				load: function(doc){
                    				var rdfNode = XMLUtil.getNode(doc.childNodes, TAG_RDF, XMLUtil.NAMESPACE_RDF);
                    				var links = [];
                    				var decNode = XMLUtil.getNode(rdfNode.childNodes, TAG_DESCRIPTION, XMLUtil.NAMESPACE_RDF);
									if (decNode == null) {
										callback(links);
									} else {
									var memberNodes = XMLUtil.getNodes(decNode.childNodes, TAG_MEMBER, XMLUtil.NAMESPACE_RDFS);
								
                    				
                    				dojo.forEach(memberNodes, function(linkNode){
                       					var reqNode = XMLUtil.getNode(linkNode.childNodes, TAG_REQ, XMLUtil.NAMESPACE_OSLC_RM_V2);
                       					var locAttr = XMLUtil.getAttributeNode(reqNode, TAG_ABOUT, XMLUtil.NAMESPACE_RDF);
                       					var uri = XMLUtil.getAttributeNodeValue(locAttr);					
										var linkTitleNode = XMLUtil.getNodes(reqNode.childNodes, TAG_TITLE, XMLUtil.NAMESPACE_DC);			
                        				//IE uses text property
										var summary = linkTitleNode[0].textContent || linkTitleNode[0].text;
                        
                        				links.push({
                            				'uri': uri,
                            				'status' : "modified",
                            				'summary': summary
                        				});
                    				});
                    				callback(links);
                    				}
                				}, 
                				error: function(result) {
                    				errCallback(result);
                				}});
             			}, 
                		function(error) {
                		 	console.log(error);
                	});
                }
            }, errCallback);
            
        },
        
		/**
		 * Cancel the Request Calls
		 * @param {Object} uris
		 */
		cancelGetReqLinksDetailRequests: function(uris){
			if(onGoingRequests){
		        dojo.forEach(uris, function(uri){
		            if(onGoingRequests[uri]){
						var deferred = onGoingRequests[uri];
						deferred.cancel();
					}				
		        });							
			}
		},
		
		/***
         * Get the test cases in the tcUris set.  The query also returns all validatesRequirement links of each test case
         * @param {Object} tcUris
         * @param {Object} callback
         * @return - callback([{uri, validatedRequirements[uri,summary]},...])
         */
        getRequirementsCoverageByTestCases: function(tcUris, callback, errorCallback){
        	if(dojo.isArray(tcUris) && tcUris.length){
        		var queryCondition = 'oslc.select=dcterms:title,oslc_qm:validatesRequirement&oslc.where=dcterms:identifier in [' + tcUris.toString() + ']';
        		this._getTestCasesWithValidatedRequirements(queryCondition, callback, errorCallback);
        	} else {
        		callback([]);
        	}
        },
		
		/***
         * Get the test cases which have validatesRequirement links to the provided resources
         * @param {Object} rmReqLinks
         * @param {Object} callback
         * @return - callback([{uri, validatedRequirements[uri,summary]},...])
         */
        getRequirementsCoverageByRequirement: function(rmReqLinks, callback, errorCallback){
        	if(dojo.isArray(rmReqLinks) && rmReqLinks.length){
        		var queryCondition = 'oslc.select=dcterms:title,oslc_qm:validatesRequirement&oslc.where=oslc_qm:validatesRequirement in [' + rmReqLinks.toString() + ']';
        		this._getTestCasesWithValidatedRequirements(queryCondition, callback, errorCallback);
        	} else {
        		callback([]);
        	}
        },
        
		_getTestCasesWithValidatedRequirements: function(queryCondition, callback, errorCallback){
				jazz.client.xhrPost({
		            url: ArtifactURIUtil.getTestCaseQueryRootURI(), 
		            headers: {
					    'Accept': HEADER_ACCEPT_XML,
					    'OSLC-Core-Version': '2.0'
		            },
		            postData: queryCondition + '&oslc.pageSize=' + QUERY_MAX_PAGE_SIZE,
		            handleAs: 'xml',
		            load: function(doc){
	                    var rootNode = doc.documentElement;
	                    
	                    var descNodes = XMLUtil.getNodes(rootNode.childNodes, TAG_DESCRIPTION, XMLUtil.NAMESPACE_RDF);
	                    var descMap = {};
	                    dojo.forEach(descNodes, function(descNode){
	                    	var descAboutNode = XMLUtil.getAttributeNode(descNode, TAG_ABOUT, XMLUtil.NAMESPACE_RDF);
                    		var descAbout = XMLUtil.getAttributeNodeValue(descAboutNode);
                    		
                    		var titleNode = XMLUtil.getNode(descNode.childNodes, TAG_TITLE, XMLUtil.NAMESPACE_DCTERMS);
                    		var title = titleNode.textContent || titleNode.text;
                    		
                    		descMap[descAbout] = title;
	                    });
	                    
	                    var queryNode = XMLUtil.getNode(rootNode.childNodes, TAG_TESTCASE_QUERY, XMLUtil.NAMESPACE_OSLC_QM_V2);
	                    var testCases = [];
                    	var testCaseOuterNodes = XMLUtil.getNodes(queryNode.childNodes, TAG_TESTCASE_2, XMLUtil.NAMESPACE_OSLC_QM_V2);
                    	dojo.forEach(testCaseOuterNodes, function(testCaseOuterNode){
                    		var testCase = {
                    			uri: '',
                    			validatedRequirements: []
                    		};
                    		var testCaseNode = XMLUtil.getNode(testCaseOuterNode.childNodes, TAG_TESTCASE, XMLUtil.NAMESPACE_OSLC_QM_V2);
                    		
                    		var testCaseUriNode = XMLUtil.getAttributeNode(testCaseNode, TAG_ABOUT, XMLUtil.NAMESPACE_RDF);
                    		testCase.uri = XMLUtil.getAttributeNodeValue(testCaseUriNode);
                    		
                    		var validatesNodes = XMLUtil.getNodes(testCaseNode.childNodes, TAG_VALIDATES_REQ, XMLUtil.NAMESPACE_OSLC_QM_V2);
                    		
                    		dojo.forEach(validatesNodes, function(validatesNode){
								var requirementUriNode = XMLUtil.getAttributeNode(validatesNode, TAG_RESOURCE, XMLUtil.NAMESPACE_RDF);
                    			var requirementUri = XMLUtil.getAttributeNodeValue(requirementUriNode);
                    			
                    			var requirementIdNode = XMLUtil.getAttributeNode(validatesNode, TAG_ID, XMLUtil.NAMESPACE_RDF);
                    			var requirementId = XMLUtil.getAttributeNodeValue(requirementIdNode);
                    			
                    			var summary = descMap['#' + requirementId];
                    			testCase.validatedRequirements.push({uri:requirementUri, summary:summary});                    			
                    		});
                    		
                    		testCases.push(testCase);
                    	});
	                    callback(testCases);
		            }, 
					error: function(result) {
						console.debug(result);
						errorCallback(result);
					}
		        });
		},        
        
        _getProviderServiceDocV2 : function(providerURL, callback, errorCallback){
        	var acceptHeader = HEADER_ACCEPT_XML;
       		jazz.client.xhrGet({
            	url: providerURL,
            	headers: {
                	'Accept': acceptHeader,
                	'OSLC-Core-Version':'2.0'
               
            	},
            	handleAs: 'xml',
            	load: function(doc){
                	callback(doc);
            	},
            	error: function(error, ioArgs){
                errorCallback(error, ioArgs);
            	}
            
        	});
    	},
    	
    	_getProviderUrl: function(doc, tag, namespace) {
    		var rdfNode = XMLUtil.getNode(doc.childNodes, TAG_RDF, XMLUtil.NAMESPACE_RDF);
         	
         	var rootNode = XMLUtil.getNode(rdfNode.childNodes, tag, namespace);
         	if (rootNode != undefined) {
         		var providerNode = XMLUtil.getNode(rootNode.childNodes, TAG_SERVICE_PROVIDER, XMLUtil.NAMESPACE_OSLC_CORE);
         		var providerAttrNode = XMLUtil.getAttributeNode(providerNode, TAG_RESOURCE, XMLUtil.NAMESPACE_RDF);
	            var providerUrl = XMLUtil.getAttributeNodeValue(providerAttrNode);
	            return providerUrl;
         	}
         	return null;
    	},
    	
    	_getProviderUrlFromChangeRequest: function(doc) {
    		return this._getProviderUrl(doc, TAG_CHG_REQUEST, XMLUtil.NAMESPACE_OSLC_CM_V2);
    	},
        
        _getQueryServiceFromReqCollectionV2: function(collectionUri, callback, errorCallback){
            var self=this;
            
    		_getReqCollectionDocV2(collectionUri, function(doc){
         		var providerUrl = {};
         	
         		providerUrl = self._getProviderUrl(doc,  TAG_REQ_COL, XMLUtil.NAMESPACE_OSLC_RM_V2);
         	
            	if (providerUrl) {
            		callback(providerUrl);
           		 }
        	}, errorCallback);
    	},
    	
    	_getQueryServiceFromReqV2: function(uri, callback, errorCallback){
       		var self=this;
       		
    		_getRequirementResourceV2(com.ibm.rqm.defects.web.client.internal.ResourceLinkService.RESOURCE_TYPE_REQ,uri, function(doc){
         	var providerUrl = {};
         	
         	providerUrl = self._getProviderUrl(doc, TAG_REQ, XMLUtil.NAMESPACE_OSLC_RM_V2);

            if (providerUrl) {
            	callback(providerUrl);
            }
        }, errorCallback);
    	},
        
        
        getTestPlanDefectsRelated: function(artifactType, scope, callback, errCallback) {
        	var queryBase = Utility.getQmResourceQueryUri("TestCaseExecutionResult");
        	queryBase = queryBase.substr(0,(queryBase.length-1));
        	
        	//Check if scope involves child test plans
        	var inCondition = [];
			if(dojo.isArray(scope)) {
				for(var i=0;scope[i];i++) {
					inCondition.push('"' + Utility.getQmResourceLinkUri(artifactType, scope[i].versionableItemId + '"'));
				}
				inCondition = inCondition.join();
			} else
				inCondition = '"' + Utility.getQmResourceLinkUri(artifactType, scope) + '"';
        	
        	var queryURL = queryBase + "?oslc.select=" + 
        		"oslc_qm%3AaffectedByChangeRequest&oslc.where=" + encodeURIComponent('oslc_qm:reportsOnTestPlan in[' + 
        		inCondition + ']');
        		
        	// The defects are from 3 sources:
        	// 1. Defects linking with test case execution results that are in the test plans.
        	// 2. Blocking defects linking with test case execution records that are in the test plans.
        	// 3. Defects linking with test plans directly.
        	jazz.client.xhrGet({
                url: queryURL,
                headers: {
	        		"OSLC-Core-Version": "2.0",
	                'Accept': HEADER_ACCEPT_XML
                },
                handleAs: 'xml',
                load: dojo.hitch(this, function(doc) {
                    // The array to store defects related with test plan
                	var links = [];
                	// Get defects from test case execution results that are in the test plans, then add the defects to defects links array
                	this._addTestPlanRelatedDefectsArray(doc, links, "TestResultQuery", "TestResult", "affectedByChangeRequest");
                	
                	// Get blocking defects from test case execution records that are in the test plans
                	this._getBlockingDefectsFromTCER(links, inCondition, callback, errCallback);
                }),
                error: function(result) {
                	errCallback(result);
                }
        	});
        },
        
        _getBlockingDefectsFromTCER: function(links, inCondition, callback, errCallback) {
            var queryBase = Utility.getQmResourceQueryUri(Constants.TYPE_EWI);
            queryBase = queryBase.substr(0,(queryBase.length-1));
            
            var queryURL = queryBase + "?oslc.select=" + 
                "oslc_qm%3AblockedByChangeRequest&oslc.where=" + encodeURIComponent('oslc_qm:reportsOnTestPlan in[' + 
                inCondition + ']');
                            
            jazz.client.xhrGet({
                url: queryURL,
                headers: {
                    "OSLC-Core-Version": "2.0",
                    'Accept': HEADER_ACCEPT_XML
                },
                handleAs: 'xml',
                load: dojo.hitch(this, function(doc) {
                    // Get blocking defects from test case execution records that are in the test plans, then add the defects to defects links array
                    this._addTestPlanRelatedDefectsArray(doc, links, "TestExecutionRecordQuery", "TestExecutionRecord", "blockedByChangeRequest");
                    
                    // Get defects directly linking with the test plans
                    this._getDefectsFromTestPlan(links, inCondition, callback, errCallback);
                }),
                error: function(result) {
                    errCallback(result);
                }
             });
        },
        
        _getDefectsFromTestPlan: function(links, inCondition, callback, errCallback) {
            var queryBase = Utility.getQmResourceQueryUri(Constants.TYPE_PLAN);
            queryBase = queryBase.substr(0,(queryBase.length-1));
            
            var queryURL = queryBase + "?oslc.select=" + 
                "oslc_qm%3ArelatedChangeRequest&oslc.where=" + encodeURIComponent('dcterms:identifier in[' + 
                inCondition + ']');
                            
            jazz.client.xhrGet({
                url: queryURL,
                headers: {
                    "OSLC-Core-Version": "2.0",
                    'Accept': HEADER_ACCEPT_XML
                },
                handleAs: 'xml',
                load: dojo.hitch(this, function(doc) {
                    // Get defects directly linking with the test plans, then add the defects to defects links array
                    this._addTestPlanRelatedDefectsArray(doc, links, "TestPlanQuery", "TestPlan", "relatedChangeRequest");
                    
                    // Get the status and severity of the Defects, according to the provider (CCM or CQ)
                    this.getWorkItemStatusAndSeverity(links, callback, errCallback);
                }),
                error: function(result) {
                    errCallback(result);
                }
             });
        },
        
        _addTestPlanRelatedDefectsArray: function(doc, links, queryNodeName, artifactNodeName, linkNodeName) {
             var uriAttrNode = null;
             var linkNodes = null;
             var uri = null;
             if(doc) {
                var rootNode = doc.documentElement;
                var queryNode = XMLUtil.getNode(rootNode.childNodes, queryNodeName, XMLUtil.NAMESPACE_OSLC_QM_V2);
                if(queryNode) {
                    // cycle through oslc_qm:testResult, oslc_qm:testExecutionRecord or oslc_qm:testPlan nodes
                    dojo.forEach(queryNode.childNodes, function(node){
                        // Playing safe: looking for TestResult, TestExecutionRecord or TestPlan node ("T" capitalized) even when seems to be only child
                        var artifactNode = XMLUtil.getNode(node.childNodes, artifactNodeName, XMLUtil.NAMESPACE_OSLC_QM_V2);
                        if(artifactNode) {
                            linkNodes = XMLUtil.getNodes(artifactNode.childNodes, linkNodeName, XMLUtil.NAMESPACE_OSLC_QM_V2);
                            //Get Uris
                            dojo.forEach(linkNodes, function(linkNode){
                                var link = {};
                                uriAttrNode = XMLUtil.getAttributeNode(linkNode, TAG_RESOURCE, XMLUtil.NAMESPACE_RDF);
                                uri = XMLUtil.getAttributeNodeValue(uriAttrNode);
                                link.uri = uri;
                                link.isCQ = uri.indexOf('/oslc/repo/') >= 0;
                                links.push(link);
                            });
                        }
                    });
                }
            }
        },
        
        getWorkItemStatusAndSeverity: function(links, callback, errorCallback) {
        	var length = links.length;
			var index = 0;
			if(length){
				var _getWorkItemStatusAndSeverity = dojo.hitch(this, function(){
					var itemURL = links[index].uri + "?oslc.properties=oslc_cm%3Aclosed," + (links[index].isCQ ? 'cq%3ASeverity' : 'oslc_cmx%3Aseverity');
		        	index++;
					jazz.client.xhrGet({
		                url: itemURL,
		                headers: {
		    	    		"Accept": "application/json",
		    				"OSLC-Core-Version": "2.0"
		                },
		                preventCache: true,
		                handleAs: 'json',
		                load: dojo.hitch(this, function(doc){
		                	if (doc instanceof Error) {
		                        // Ignore errors and continue
		                        console.debug("Attempt to get status & severity failed. Skipped.");
		                        if(index == length) {
		                    		callback(links);
		                    	} else {
		                    		_getWorkItemStatusAndSeverity();
		                    	}
		                    }
		                    else {
		                    	// Extract Status(we only need to know if it's closed or not)
		                    	var isClosed = doc['oslc_cm:closed'];
		                    	if(dojo.isString(isClosed)) // CQ stores a string instead of a boolean
		                    		isClosed = isClosed == 'true';
		                    	links[index-1].closed = isClosed;
		                    	// and Severity from JSON response
		                    	if(doc['oslc_cmx:severity']) {
		                    		var literal = doc['oslc_cmx:severity']['rdf:resource'];
		                    		links[index-1].severity = literal.substr(-2) || '';
		                    	} else { // From CQ
		                    		links[index-1].severity = doc['cq:Severity'];
		                    	}
		                    	if(index == length) {
		                    		callback(links);
		                    	} else {
		                    		_getWorkItemStatusAndSeverity();
		                    	}
		                    }
		                }),
		                error: function(error, ioArgs){
		                	// Ignore errors and continue
	                        console.debug("Attempt to get status & severity failed. Skipped.");
	                        if(index == length) {
	                    		callback(links);
	                    	} else {
	                    		_getWorkItemStatusAndSeverity();
	                    	}
		                }
		                
		            });
				});
				_getWorkItemStatusAndSeverity();
			} else {
				callback([]);
			}
        },
        
        /**
         * Get OSLC resource's content by leveraging remote OSLC service call
         */
        getOSLCResouceContent: function(resourceUrl, callback){
    		var args = {
						uri: resourceUrl,
						properties: ["oslc:serviceProvider"]
					};
    		ServiceProviderProxyClient.getResource(args, callback);
        },
        
        /**
         * Get completed OSLC automation result by corresponding automation request 
         */
        getAutomationResultByRequest: function(providerUrl, autoRequestUrl, callback){
        	var args = {
						serviceProviderUri: providerUrl,
						serviceUsageUri: "http://open-services.net/ns/auto#Deploy",
						resourceTypeUri: "http://open-services.net/ns/auto#AutomationResult",
						where: encodeURI("oslc_auto:producedByAutomationRequest=<"+autoRequestUrl+
								"> and oslc_auto:state=<http://open-services.net/ns/auto#complete>"),
						pollingAttempts: 5,
						pollingInterval: 3000
					};
			ServiceProviderProxyClient.postQuery(args, callback);
        },
        
        /**
         * Get RQM<->RTVS integration feature enablement switch value
         */
        getOSLCAutomationConsumerEnabled: function(callback){
        	ServiceProviderProxyClient.getOSLCAutomationConsumerEnabled({}, callback);
        },
        
        /**
         * Get associated executedOnDeployment automation plan link of a specified resource
         */
        getExecutedOnDeploymentLink: function(oslcVersion, resourceType, resourceUri, callback){
        	this.getLinkByProperty(oslcVersion, resourceType, resourceUri, XMLUtil.NAMESPACE_QM_RQM, TAG_EXECUTED_ON_DEPLOYMENT, callback);
        },
        
        /**
         * Get associated runsOnDeployement automation plan link of a specified resource
         * @param {Object} resourceUri
         * @return  {Object} link [{summary, uri}]
         */
        getRunsOnDeploymentLink: function(oslcVersion, resourceType, resourceUri, callback){
        	this.getLinkByProperty(oslcVersion, resourceType, resourceUri, XMLUtil.NAMESPACE_QM_RQM, TAG_RUNS_ON_DEPLOYMENT, callback);
        },
        
        /***
         * Get a qm resource's link according to property's QName
         * @param {String} propNS property namespace
         * @param {String} propName property name
         * @param {Object} resourceUri
         * @return {Object} links [{summary, uri}]
         */
        getLinkByProperty: function(oslcVersion, resourceType, resourceUri, propNS, propName, callback){
        	// 1. get the resource doc 
            _getValidatesDocV2(resourceType, resourceUri, dojo.hitch(this, function(doc){
            	//2. extract the links, and return in the form of [{summary, uri}]
                var rootNode = XMLUtil.getNode(doc.childNodes, TAG_RDF, XMLUtil.NAMESPACE_RDF);
        		var rootNodeTR = XMLUtil.getNode(rootNode.childNodes, resourceType, XMLUtil.NAMESPACE_OSLC_QM_V2);
        		var descNodes = XMLUtil.getNodes(rootNode.childNodes, TAG_DESCRIPTION, XMLUtil.NAMESPACE_RDF);
        		
                var linkNodes = XMLUtil.getNodes(rootNodeTR.childNodes, propName, propNS);
                var links = [];
                dojo.forEach(linkNodes, function(linkNode){
                	var linkRecord = {};
                	var urlNode = XMLUtil.getAttributeNode(linkNode, TAG_RESOURCE, XMLUtil.NAMESPACE_RDF);
                	linkRecord.url = XMLUtil.getAttributeNodeValue(urlNode);
                	var idNode = XMLUtil.getAttributeNode(linkNode, TAG_ID, XMLUtil.NAMESPACE_RDF);
                    var id = XMLUtil.getAttributeNodeValue(idNode); 
                    
                    var summary;
						
    				dojo.every(descNodes, function(descNode){
    					var idNode = XMLUtil.getAttributeNode(descNode, TAG_ABOUT, XMLUtil.NAMESPACE_RDF);
    					var nodeId = XMLUtil.getAttributeNodeValue(idNode); 
    					if(nodeId && nodeId==('#'+id)){
    						var titleNode = XMLUtil.getNode(descNode.childNodes, TAG_TITLE, XMLUtil.NAMESPACE_DC);
    						if(titleNode && titleNode.childNodes.length >0){
    							if(titleNode.childNodes[0].nodeValue && titleNode.childNodes[0].nodeValue!=""){
    								linkRecord.summary = titleNode.childNodes[0].nodeValue;
    								return false;
    							}
    						}       						
    					}
    				});
                    
					if(linkRecord.url)
						links.push(linkRecord);
                    
                });
                
                callback(links);
            }));
        },
        
        /***
         * Update a qm resource's 'executedOn' link.
         * @param {Object} resourceUri
         * @param {Object} addedLink [{summary, uri}]
         */
        updateExecutedOnLink: function(oslcVersion, resourceType, resourceUri, addedLinks, removedLinks, callback, errCallback){
            this._updateLinkByProperty(oslcVersion, resourceType, resourceUri, XMLUtil.NAMESPACE_QM_RQM, TAG_EXECUTED_ON, addedLinks, removedLinks, callback, errCallback);
        },
        
        /***
         * Update a qm resource's 'executedOnDeployment' link.
         * @param {Object} resourceUri
         * @param {Object} addedLink [{summary, uri}]
         */
        updateExecutedOnDeploymentLink: function(oslcVersion, resourceType, resourceUri, addedLinks, removedLinks, callback, errCallback){
            this._updateLinkByProperty(oslcVersion, resourceType, resourceUri, XMLUtil.NAMESPACE_QM_RQM, TAG_EXECUTED_ON_DEPLOYMENT, addedLinks, removedLinks, callback, errCallback);
        },
        
        /***
         * Update a qm resource's 'runsOnDeployment' link.
         * @param {Object} resourceUri
         * @param {Object} addedLink [{summary, uri}]
         */
        updateRunsOnDeploymentLink: function(oslcVersion, resourceType, resourceUri, addedLinks, removedLinks, callback, errCallback){
            this._updateLinkByProperty(oslcVersion, resourceType, resourceUri, XMLUtil.NAMESPACE_QM_RQM, TAG_RUNS_ON_DEPLOYMENT, addedLinks, removedLinks, callback, errCallback);
        },
        
        /***
         * Update a qm resource's link according to property's QName
         * @param {String} propNS property namespace
         * @param {String} propName property name
         * @param {Object} resourceUri
         * @param {Object} addedLink [{summary, uri}]
         */
        _updateLinkByProperty: function(oslcVersion, resourceType, resourceUri, propNS, propName, addedLinks, removedLinks, callback, errCallback){
            // 1. get the resource doc 
            _getValidatesDocV2(resourceType, resourceUri, dojo.hitch(this, function(doc){
            	//2. remove previous link, then add new one in the doc according to user's request
                var rootNode = XMLUtil.getNode(doc.childNodes, TAG_RDF, XMLUtil.NAMESPACE_RDF);
        		var rootNodeTR = XMLUtil.getNode(rootNode.childNodes, resourceType, (resourceType == TAG_TSRESULT ? XMLUtil.NAMESPACE_IBM_RQM : XMLUtil.NAMESPACE_OSLC_QM_V2));
        		
                var linkNodes = XMLUtil.getNodes(rootNodeTR.childNodes, propName, propNS);
                dojo.forEach(linkNodes, function(linkNode){
                    rootNodeTR.removeChild(linkNode);
                });
        		
        		var i = 0; //generate ID for Reqt Links
                dojo.forEach(addedLinks, function(link){
                    var linkNode = null;
                    if (IS_IE_OR_CHROME) { // handle IE xml attribute manipulation
                    	var newPrefixIndex =0;
						var prefixElem = rootNode.getAttribute(TAG_PREX_BASE+newPrefixIndex);
						while(prefixElem){
							newPrefixIndex ++;
							prefixElem = rootNode.getAttribute(TAG_PREX_BASE+newPrefixIndex);
						}
						rootNode.setAttribute("xmlns:"+TAG_PREX_BASE+newPrefixIndex, propNS);
                        linkNode = doc.createElement(TAG_PREX_BASE+newPrefixIndex + ":" + propName);
                        linkNode.setAttribute(TAG_PREX_RDF + ":" + TAG_RESOURCE, link.uri || link.url);				
        				//IE is using different codes comparing to FireFox
						//assign an ID for the rqt url, the nubmer will be used by rootNodeDec
        				linkNode.setAttribute(TAG_ID,  NEW_ADDED_LINKS + i);

        				rootNodeTR.appendChild(linkNode);
        				
        				var rootNodeDec = doc.createElement(TAG_PREX_RDF + ":" + TAG_DESCRIPTION);
        				rootNodeDec.setAttribute(TAG_PREX_RDF + ":" + TAG_ABOUT, NEW_ADDED_LINKS_TAG + i);
        				rootNode.appendChild(rootNodeDec);
        				
                        var rootNodeTitle = doc.createElement(TAG_DCTERMS + ":" + TAG_TITLE);
                        if(!link.summary && !link.comment) console.debug("link summary empty");
        				rootNodeTitle.text = link.summary || link.comment;
						if(rootNodeTitle.textContent != null)
							rootNodeTitle.textContent = link.summary || link.comment;
        				rootNodeDec.appendChild(rootNodeTitle);
                    }
                    else {
        				linkNode = document.createElementNS(propNS, propName);
                        linkNode.setAttributeNS(XMLUtil.NAMESPACE_RDF, TAG_RESOURCE, link.uri || link.url);
        				//assign an ID for the rqt url, the nubmer will be used by rootNodeDec
        				linkNode.setAttributeNS(XMLUtil.NAMESPACE_RDF, TAG_ID,  NEW_ADDED_LINKS + i);

        				rootNodeTR.appendChild(linkNode);
        				
        				var rootNodeDec = document.createElementNS(XMLUtil.NAMESPACE_RDF, TAG_DESCRIPTION);
        				rootNodeDec.setAttributeNS(XMLUtil.NAMESPACE_RDF, TAG_ABOUT, NEW_ADDED_LINKS_TAG + i);
        				rootNode.appendChild(rootNodeDec);
        				
        				var rootNodeTitle = document.createElementNS(XMLUtil.NAMESPACE_DC, TAG_TITLE);
        				rootNodeTitle.textContent = link.summary || link.comment;
        				rootNodeDec.appendChild(rootNodeTitle);   
                    }
					i++;
                    
                });
                
                //3. update the resource doc
                _updateValidatesDoc(oslcVersion, resourceType, resourceUri, doc, callback, errCallback);
            }));
            
        },
        
        getBuildRecordFromResult: function(resourceType, resourceUri, callback) {
        	var queryURL = resourceUri + "?oslc.properties=" + TAG_PREX_RQM_QM + "%3A" + TAG_EXECUTED_ON;
        	_getValidatesDocV2(RESOURCE_TYPE_BUILD_REC, queryURL, dojo.hitch(this, function(doc){
        		var result = {};
        		if (doc) {        			
	        		var rootNode = XMLUtil.getNode(doc.childNodes, TAG_RDF, XMLUtil.NAMESPACE_RDF);
	        		var rootNodeTR = XMLUtil.getNode(rootNode.childNodes, resourceType, (resourceType == TAG_TSRESULT ? XMLUtil.NAMESPACE_IBM_RQM : XMLUtil.NAMESPACE_OSLC_QM_V2));
	        		var executedOnNode = XMLUtil.getNode(rootNodeTR.childNodes, TAG_EXECUTED_ON, XMLUtil.NAMESPACE_QM_RQM);
	        		
	        		if(executedOnNode) {
		        		//Get Uri						 	
						var uriAttrNode = XMLUtil.getAttributeNode(executedOnNode, TAG_RESOURCE, XMLUtil.NAMESPACE_RDF);
						result.uri = XMLUtil.getAttributeNodeValue(uriAttrNode);
						
		        		var decNode = XMLUtil.getNode(rootNode.childNodes, TAG_DESCRIPTION, XMLUtil.NAMESPACE_RDF);
		        		var summaryNode = XMLUtil.getNode(decNode.childNodes, TAG_TITLE, XMLUtil.NAMESPACE_DCTERMS);
		        		result.summary = summaryNode.textContent || summaryNode.text;
		        		
		                callback(result);
	        		} else {
	        			callback(null);
	        		}
        		} else { // ! doc
        			callback(null);
        		}
        	}));
        },
        
        // WI 72466: Looks for a node of the form <webId> which will be used  to redirect to the right build view
        redirectToWebURI: function(buildUri, callback) {
        	_getValidatesDocV2(RESOURCE_TYPE_BUILD_REC, buildUri, dojo.hitch(this, function(doc){
        		var rootNode = XMLUtil.getNode(doc.childNodes, TAG_RDF, XMLUtil.NAMESPACE_RDF);
        		var buildNode = XMLUtil.getNode(rootNode.childNodes, TAG_BUILDRESULT, XMLUtil.NAMESPACE_OSLC_AUTO);
        		var webIdNode = XMLUtil.getNode(buildNode.childNodes, TAG_WEBID, XMLUtil.NAMESPACE_AUTO_RQM);
        		
        		// In case the node is not present, it means it is a external link; the external app will handle the redirect
        		var webUIURL = buildUri;
        		
        		if(webIdNode) {
	        		// In case the node is present, use it					 	
					webUIURL = callback(webIdNode.textContent || webIdNode.text);
        		}
        		
        		// redirect to Web UI URL
        		window.location.href = webUIURL;
        	}));
        },
        
        // Obtain all links of the referred link Types that belong to the artifact. They are read from the OSLC RDF representation. 
        getAllLinksRelated: function(artifactURI, artifactData, linkTypes, callback) {
        	_getValidatesDocV2(null, artifactURI, dojo.hitch(this, function(doc) {
        		var allLinksData = [];
        		if (doc) {
	        		var typesLength = linkTypes.length;
	        		for(var i = 0;i < typesLength; i++) {
	        			var type = linkTypes[i];
	        			var typeTag = type.tagName;
	        			var links = [];
	        			var rdfNode = XMLUtil.getNode(doc.childNodes, TAG_RDF, XMLUtil.NAMESPACE_RDF);
	        			var artifactNode = XMLUtil.getNode(rdfNode.childNodes, artifactData.tagName, artifactData.namespace);
	        			var linkNodes = XMLUtil.getNodes(artifactNode.childNodes, typeTag, type.namespace);
	        			var decNodes = XMLUtil.getNodes(rdfNode.childNodes, TAG_DESCRIPTION, XMLUtil.NAMESPACE_RDF);
	        			
	    				dojo.forEach(linkNodes, function(linkNode){
	    					var uriAttrNode = XMLUtil.getAttributeNode(linkNode, TAG_RESOURCE, XMLUtil.NAMESPACE_RDF);
	    					var uri = XMLUtil.getAttributeNodeValue(uriAttrNode);
	    					try {
		    					var idAttrNode = XMLUtil.getAttributeNode(linkNode, TAG_ID, XMLUtil.NAMESPACE_RDF);
		    					var id = XMLUtil.getAttributeNodeValue(idAttrNode);
	    					} catch(Error) {}
	    					var summary = getDescriptionFromNodes(decNodes, uri, id);
	    					links.push({
	    						'summary': summary,
	    						'uri': uri
	    					});
	    				});
	    				
	    				if(links.length) {
		    				allLinksData.push({
		    					"titleLabel": type.label,
		    					"links": links
		    				});
	    				}
	        		}
        		}
        		callback(allLinksData.length ? allLinksData : null);
        	}));
        },
		
		getLinksFromTestScript:function(artifactURI, artifactData, linkTypes, callback){
			var baseUrl = artifactURI+"?oslc.properties=";
			var linkTypeStr = "";
			dojo.forEach(linkTypes,function(linkType){
				linkTypeStr = linkTypeStr + linkType.tagPrex+":"+linkType.tagName+",";
			});
			var url = baseUrl+linkTypeStr.substring(0,linkTypeStr.length-1);
			jazz.client.xhrGet({
				url: url,
				headers: {
					'Accept': HEADER_ACCEPT_XML
				},
				handleAs: 'xml',
				load: function(doc){
					var allLinksData = [];
					if (doc) {
						var typesLength = linkTypes.length;
						for (var i = 0; i < typesLength; i++) {
							var type = linkTypes[i];
							var typeTag = type.tagName;
							var links = [];
							var rdfNode = XMLUtil.getNode(doc.childNodes, TAG_RDF, XMLUtil.NAMESPACE_RDF);
							var artifactNode = XMLUtil.getNode(rdfNode.childNodes, artifactData.tagName, artifactData.namespace);
							var linkNodes = XMLUtil.getNodes(artifactNode.childNodes, typeTag, type.namespace);
							var decNodes = XMLUtil.getNodes(rdfNode.childNodes, TAG_DESCRIPTION, XMLUtil.NAMESPACE_RDF);
							
							dojo.forEach(linkNodes, function(linkNode){
								var uriAttrNode = XMLUtil.getAttributeNode(linkNode, TAG_RESOURCE, XMLUtil.NAMESPACE_RDF);
								var uri = XMLUtil.getAttributeNodeValue(uriAttrNode);
								try {
									var idAttrNode = XMLUtil.getAttributeNode(linkNode, TAG_ID, XMLUtil.NAMESPACE_RDF);
									var id = XMLUtil.getAttributeNodeValue(idAttrNode);
								} 
								catch (Error) {
								}
								var summary = getDescriptionFromNodes(decNodes, uri, id);
								links.push({
									'summary': summary,
									'uri': uri
								});
							});
							
							if (links.length) {
								allLinksData.push({
									"titleLabel": type.label,
									"links": links
								});
							}
						}
					}
					callback(allLinksData.length ? allLinksData : null);
				}
			});
		},
        
         postUpdateBackLinks: function(linkType, resourceUrl, removedLinks, addedLinks, addedLinkLabels, success, errorCallback){
    		var args = {
						uri: resourceUrl,
						linkType: linkType,
						removedlinks: removedLinks,
						addedLinks: addedLinks,
						addedLinkLabels: addedLinkLabels
					};
    		ServiceProviderProxyClient.postUpdateBackLinks(args, success, errorCallback);
        },
        
        __sentinel: null // terminates this class definition
    };
    
	function _getValidatesLinks(resourceType, resourceUri, callback){
        var validatesTagName = _getValidatesTagNameV1(resourceType);
        _getValidatesDocV1(resourceType, resourceUri, function(doc){
			var links = [];
			if (doc) {
				var rootNode = XMLUtil.getNode(doc.childNodes, TAG_TESTCASE.toLowerCase(), XMLUtil.NAMESPACE_OSLC_QM);
				if(rootNode == null)
					rootNode = XMLUtil.getNode(doc.childNodes, TAG_TESTPLAN.toLowerCase(), XMLUtil.NAMESPACE_OSLC_QM);
				var linkNodes = XMLUtil.getNodes(rootNode.childNodes, validatesTagName, XMLUtil.NAMESPACE_CALM);
				
				dojo.forEach(linkNodes, function(linkNode){
					var summaryAttrNode = XMLUtil.getAttributeNode(linkNode, TAG_LABEL, XMLUtil.NAMESPACE_OSLC_QM);
					var summary = XMLUtil.getAttributeNodeValue(summaryAttrNode);
					var uriAttrNode = XMLUtil.getAttributeNode(linkNode, TAG_RESOURCE, XMLUtil.NAMESPACE_RDF);
					var uri = XMLUtil.getAttributeNodeValue(uriAttrNode);
					links.push({
						'summary': summary,
						'uri': uri
					});
				});
			}
            callback(links);
        });
    }
    
    function _getRelatedChangeRequestLinksV2(resourceUri, callback){
        var relatedChangeRequestTagName = TAG_RELATED_CHANGE_REQUEST;
        _getRelatedChangeRequestDocV2(resourceUri, function(doc){
			var links = [];
			if (doc) {
				var rootNodes = XMLUtil.getNode(doc.childNodes, TAG_RDF, XMLUtil.NAMESPACE_RDF);
				var rootNode = XMLUtil.getNode(rootNodes.childNodes, TAG_TESTCASE, XMLUtil.NAMESPACE_OSLC_QM_V2);
				if (rootNode == null) {
					rootNode = XMLUtil.getNode(rootNodes.childNodes, TAG_TESTPLAN, XMLUtil.NAMESPACE_OSLC_QM_V2);
					if (rootNode == null) {
						rootNode = XMLUtil.getNode(rootNodes.childNodes, TAG_TESTSCRIPT, XMLUtil.NAMESPACE_OSLC_QM_V2);
					}
				}
				var linkNodes = XMLUtil.getNodes(rootNode.childNodes, relatedChangeRequestTagName, XMLUtil.NAMESPACE_OSLC_QM_V2);
				var decNodes = XMLUtil.getNodes(rootNodes.childNodes, TAG_DESCRIPTION, XMLUtil.NAMESPACE_RDF);
				//Initial Parameters
				var uriAttrNode = null;
				var uri = null;
				var linkIdentifier = null;
				var linkIdentVal = null;
				var linkTitleNode = null;
				var summary = null;
				dojo.forEach(linkNodes, function(linkNode){
					//Get Uri						 	
					uriAttrNode = XMLUtil.getAttributeNode(linkNode, TAG_RESOURCE, XMLUtil.NAMESPACE_RDF);
					uri = XMLUtil.getAttributeNodeValue(uriAttrNode);
					var idAttrNode = XMLUtil.getAttributeNode(linkNode, TAG_ID, XMLUtil.NAMESPACE_RDF);
					var id = XMLUtil.getAttributeNodeValue(idAttrNode);
					//Get Summary, rdf:ID, rdf:about equals uri
					dojo.forEach(decNodes, function(decNode){
						//If rdf:about equals "#"+uri, then matches
						linkIdentifier = XMLUtil.getAttributeNode(decNode, TAG_ABOUT, XMLUtil.NAMESPACE_RDF);
						linkIdentVal = XMLUtil.getAttributeNodeValue(linkIdentifier);
						if((linkIdentVal == ("#" + uri)) || (linkIdentVal == ("#" + id))){
							linkTitleNode = XMLUtil.getNodes(decNodes.childNodes, TAG_TITLE, XMLUtil.NAMESPACE_DC);
							if(linkTitleNode.length == 0){
								linkTitleNode = XMLUtil.getNodes(decNode.childNodes, TAG_TITLE, XMLUtil.NAMESPACE_DCTERMS);
								//IE uses text property
								summary = linkTitleNode[0].textContent || linkTitleNode[0].text;
							}else{
								summary = linkTitleNode.textContent || linkTitleNode.text;
							}
						}
					});
					links.push({
						'summary': summary,
						'uri': uri
					});
				});
			}
            callback(links);
        });
    }
		
    function _getValidatesLinksV2(resourceType, resourceUri, callback){
        var validatesTagName = _getValidatesTagName(resourceType);
        _getValidatesDocV2(resourceType, resourceUri, function(doc){
			var links = [];
			if (doc) {
				var rootNodes = XMLUtil.getNode(doc.childNodes, TAG_RDF, XMLUtil.NAMESPACE_RDF);
				var rootNode = XMLUtil.getNode(rootNodes.childNodes, TAG_TESTCASE, XMLUtil.NAMESPACE_OSLC_QM_V2);
				if (rootNode == null) {
					rootNode = XMLUtil.getNode(rootNodes.childNodes, TAG_TESTPLAN, XMLUtil.NAMESPACE_OSLC_QM_V2);
					if (rootNode == null) {
						rootNode = XMLUtil.getNode(rootNodes.childNodes, TAG_TESTSCRIPT, XMLUtil.NAMESPACE_OSLC_QM_V2);
					}
				}
				var linkNodes = XMLUtil.getNodes(rootNode.childNodes, validatesTagName, XMLUtil.NAMESPACE_OSLC_QM_V2);
				var decNodes = XMLUtil.getNodes(rootNodes.childNodes, TAG_DESCRIPTION, XMLUtil.NAMESPACE_RDF);
				//Initial Parameters
				var uriAttrNode = null;
				var uri = null;
				var summary = null;
				dojo.forEach(linkNodes, function(linkNode){
					//Get Uri						 	
					uriAttrNode = XMLUtil.getAttributeNode(linkNode, TAG_RESOURCE, XMLUtil.NAMESPACE_RDF);
					uri = XMLUtil.getAttributeNodeValue(uriAttrNode);
					var idAttrNode = XMLUtil.getAttributeNode(linkNode, TAG_ID, XMLUtil.NAMESPACE_RDF);
					var id = XMLUtil.getAttributeNodeValue(idAttrNode);
					summary = getDescriptionFromNodes(decNodes, uri, id);
					links.push({
						'summary': summary,
						'uri': uri
					});
				});
			}
            callback(links);
        });
    }
    
    function getDescriptionFromNodes(decNodes, uri, id) {
    	//Get Summary, rdf:ID, rdf:about equals uri
    	var linkIdentifier = null;
		var linkIdentVal = null;
		var linkTitleNode = null;
		var summary = null;
    	dojo.forEach(decNodes, function(decNode){
			//If rdf:about equals "#"+uri, then matches
			linkIdentifier = XMLUtil.getAttributeNode(decNode, TAG_ABOUT, XMLUtil.NAMESPACE_RDF);
			linkIdentVal = XMLUtil.getAttributeNodeValue(linkIdentifier);
			if((linkIdentVal == ("#" + uri)) || (linkIdentVal == ("#" + id))){
				linkTitleNode = XMLUtil.getNodes(decNodes.childNodes, TAG_TITLE, XMLUtil.NAMESPACE_DC);
				if(linkTitleNode.length == 0){
					linkTitleNode = XMLUtil.getNodes(decNode.childNodes, TAG_TITLE, XMLUtil.NAMESPACE_DCTERMS);
					//IE uses text property
					summary = linkTitleNode[0].textContent || linkTitleNode[0].text;
				}else{
					summary = linkTitleNode.textContent || linkTitleNode.text;
				}
			}
		});
    	return summary;
    }
    
    function _getValidatesDocV1(resourceType, resourceUri, callback){
        jazz.client.xhrGet({
            url: resourceUri, 
            headers: {
			    'Accept': HEADER_ACCEPT_XML
            },
            handleAs: 'xml',
            load: function(doc){
                callback(doc);
            }, 
			error: function(result) {
				callback(null);
			}
        });
    }
    
	function _getValidatesDocV2(resourceType, resourceUri, callback){
        jazz.client.xhrGet({
            url: resourceUri, 
            headers: {
				"OSLC-Core-Version": "2.0",
                'Accept': HEADER_ACCEPT_XML
            },
            handleAs: 'xml',
            load: function(doc){
            	//console.debug("Success on _getValidatesDocV2.");
                callback(doc);
            }, 
			error: function(result) {
            	console.debug("Error: " + result.message || result);
				callback(null);
			}
        });
    }
    
    function _getRelatedChangeRequestDocV2(resourceUri, callback){
       jazz.client.xhrGet({
            url: resourceUri, 
            headers: {
				"OSLC-Core-Version": "2.0",
                'Accept': HEADER_ACCEPT_XML
            },
            handleAs: 'xml',
            load: function(doc){
            	console.debug("Success on _getRelatedChangeRequestDocV2.");
                callback(doc);
            }, 
			error: function(result) {
            	console.debug("Error: " + result.message || result);
				callback(null);
			}
        });
    }
    
    function _updateValidatesDoc(oslcVersion, resourceType, resourceUri, doc, callback, errorCallback){
        jazz.client.xhrPut({
            url: resourceUri,
			headers: {
                 "OSLC-Core-Version": "2.0",
				 'Accept': HEADER_ACCEPT_XML
            },
			contentType: HEADER_ACCEPT_XML_V2,
            postData: dojox.xml.parser.innerXML(doc),
            load: function(result){
            	if(callback && dojo.isFunction(callback))
            		callback( result);
            },
            error: function(result, ioArgs) {
            	if(errorCallback && dojo.isFunction(errorCallback)) {
	            	errorCallback(result);
            	}else {
					console.debug("There is no errorCallback function.");
				}
            }
        });  
    }
    
    function _updateRelateChangeRequestDoc(resourceUri, doc, callback, errorCallback){
        jazz.client.xhrPut({
            url: resourceUri,
			headers: {
                 "OSLC-Core-Version": "2.0",
				 'Accept': HEADER_ACCEPT_XML
            },
			contentType: HEADER_ACCEPT_XML_V2,
            postData: dojox.xml.parser.innerXML(doc),
            load: function(result){
            	if(callback && dojo.isFunction(callback))
            		callback( result);
            },
            error: function(result, ioArgs) {
            	if(errorCallback && dojo.isFunction(errorCallback)) {
	            	errorCallback(result);
            	}else {
					console.debug("There is no errorCallback function.");
				}
            }
        });  
    }
    
    function _updateValidatedByLinks(oslcVersion, resourceType, addedResourceUris, removedResourceUris, linkDesc, callbacks){
        var qmLinks = [];
		dojo.forEach(linkDesc, function(desc){
			 qmLinks.push({
	            'uri': desc.rqmUri,
	            'summary': desc.linkComment
	         });
		});
		
		var qmLinkUris = [];
		dojo.forEach(linkDesc, function(desc){
			 qmLinkUris.push(
	            desc.rqmUri
	         );
		});
		
		var qmLinkLabels = [];
		dojo.forEach(linkDesc, function(desc){
			 qmLinkLabels.push(
	            desc.linkComment
	         );
		});
        if(!callbacks)
        	callbacks = {success:null, error:null};
        
        // Defining contention function to handle OSLC pre-V1 service providers.
        if(!callbacks.handleRequirementsPreV1)
	        callbacks.handleRequirementsPreV1 = function(retry, removing) {
	        	if(removing)
	        		removedResourceUris.unshift(retry);
	        		
	        	else
	        		addedResourceUris.unshift(retry);
	        	_updateValidatedByLinks(oslcVersion, resourceType, addedResourceUris, removedResourceUris, linkDesc, callbacks);
	        };
        
        var resourceUri;
        var addingLinks = false;
        if(addedResourceUris.length > 0) {
        	resourceUri = addedResourceUris.shift();
        	addingLinks = true;
        }
        else if(removedResourceUris.length > 0) {
        	resourceUri = removedResourceUris.shift();
        }
        var success;
        if(addedResourceUris.length == 0 && removedResourceUris.length == 0)
        	success = callbacks.success;
        else
        	success = function() {
        		_updateValidatedByLinks(oslcVersion, resourceType, addedResourceUris, removedResourceUris, linkDesc, callbacks);
        	};
        
        if(addingLinks)
        	if (oslcVersion == 1.0) {
			_getValidatedByDoc(oslcVersion, resourceType, resourceUri, function(doc, etag){
			    _modifyValidatedByLinks(oslcVersion, doc, qmLinks, []);
			    _updateValidatedByDocSync(oslcVersion, resourceType, resourceUri, doc, etag, success, callbacks);
            }, callbacks);
         	} else {
         		com.ibm.rqm.defects.web.client.internal.ResourceLinkService.postUpdateBackLinks(XMLUtil.NAMESPACE_OSLC_RM_V2 + TAG_VALIDATED_BY, resourceUri, [], qmLinkUris, qmLinkLabels, success, callbacks.error);
         	}	
           
        else { // When deleting links, if GET request status is 410 (Gone), it means the requirement is not on RM anymore.
        	   // Proceed to delete the local links. Otherwise error will be notified.
        	callbacks.nextLink = success;
        	_getValidatedByDoc(oslcVersion, resourceType, resourceUri, function(doc, etag){
        		if (oslcVersion == 1.0) {
        			_modifyValidatedByLinks(oslcVersion, doc, [], qmLinks);
                	_updateValidatedByDocSync(oslcVersion, resourceType, resourceUri, doc, etag, success, callbacks, 
                	(callbacks.errorMessages ? callbacks.errorMessages.join("\n") : null));
        		}
        		else {
        			com.ibm.rqm.defects.web.client.internal.ResourceLinkService.postUpdateBackLinks(XMLUtil.NAMESPACE_OSLC_RM_V2 + TAG_VALIDATED_BY, resourceUri, qmLinkUris, [], [], success, callbacks.error);
        		}
                
            }, callbacks);
        }
    }
    
    function _removeRelatedChangeRequestLinks(doc, removedLinks){
    	
    	var relatedChangeRequestTagName = TAG_RELATED_CHANGE_REQUEST;
        var rootNode = XMLUtil.getNode(doc.childNodes, TAG_RDF, XMLUtil.NAMESPACE_RDF);
		var rootNodeTC = XMLUtil.getNode(rootNode.childNodes, TAG_TESTCASE, XMLUtil.NAMESPACE_OSLC_QM_V2);
		if(rootNodeTC == null){
			rootNodeTC = XMLUtil.getNode(rootNode.childNodes, TAG_TESTPLAN, XMLUtil.NAMESPACE_OSLC_QM_V2);
			if(rootNodeTC == null)
				rootNodeTC = XMLUtil.getNode(rootNode.childNodes, TAG_TESTSCRIPT, XMLUtil.NAMESPACE_OSLC_QM_V2);
		}
    	
    	dojo.forEach(removedLinks, function(link){
            var linkNodes = XMLUtil.getNodes(rootNodeTC.childNodes, relatedChangeRequestTagName, XMLUtil.NAMESPACE_OSLC_QM_V2);
            dojo.forEach(linkNodes, function(linkNode){
                var uriAttrNode = XMLUtil.getAttributeNode(linkNode, TAG_RESOURCE, XMLUtil.NAMESPACE_RDF);
                var uri = XMLUtil.getAttributeNodeValue(uriAttrNode);
                if (link.uri == uri || link.url == uri) {
                    rootNodeTC.removeChild(linkNode);
                }
            });
        });
    	
    	
    }
    
    function _addRelatedChangeRequestLinks(doc, addedLinks){
        
        var relatedChangeRequestTagName = TAG_RELATED_CHANGE_REQUEST;
        var rootNode = XMLUtil.getNode(doc.childNodes, TAG_RDF, XMLUtil.NAMESPACE_RDF);
		var rootNodeTC = XMLUtil.getNode(rootNode.childNodes, TAG_TESTCASE, XMLUtil.NAMESPACE_OSLC_QM_V2);
		if(rootNodeTC == null){
			rootNodeTC = XMLUtil.getNode(rootNode.childNodes, TAG_TESTPLAN, XMLUtil.NAMESPACE_OSLC_QM_V2);
			if(rootNodeTC == null)
				rootNodeTC = XMLUtil.getNode(rootNode.childNodes, TAG_TESTSCRIPT, XMLUtil.NAMESPACE_OSLC_QM_V2);
		}
		
		var i = 0; //generate ID for Reqt Links
        dojo.forEach(addedLinks, function(link){
            var linkNode = null;
            if (IS_IE_OR_CHROME) { // handle IE xml attribute manipulation 
                linkNode = doc.createElement(TAG_PREX_OSLC_QM + ":" + relatedChangeRequestTagName);
                linkNode.setAttribute(TAG_PREX_RDF + ":" + TAG_RESOURCE, link.uri || link.url);								
				//IE is using different codes comparing to FireFox
				//assign an ID for the rqt url, the nubmer will be used by rootNodeDec
				linkNode.setAttribute(TAG_ID,  "n" + i);

				rootNodeTC.appendChild(linkNode);
				
				var rootNodeDec = doc.createElement(TAG_PREX_RDF + ":" + TAG_DESCRIPTION);
				rootNodeDec.setAttribute(TAG_PREX_RDF + ":" + TAG_ABOUT, "#n" + i);
				rootNode.appendChild(rootNodeDec);
				
                var rootNodeTitle = doc.createElement(TAG_DCTERMS + ":" + TAG_TITLE);
				rootNodeTitle.text = link.summary;
				rootNodeDec.appendChild(rootNodeTitle);
            }
            else {
				linkNode = document.createElementNS(XMLUtil.NAMESPACE_OSLC_QM_V2, relatedChangeRequestTagName);
                linkNode.setAttributeNS(XMLUtil.NAMESPACE_RDF, TAG_RESOURCE, link.uri || link.url);
				//assign an ID for the rqt url, the nubmer will be used by rootNodeDec
				linkNode.setAttributeNS(XMLUtil.NAMESPACE_RDF, TAG_ID, "n" + i);

				rootNodeTC.appendChild(linkNode);
				
				var rootNodeDec = document.createElementNS(XMLUtil.NAMESPACE_RDF, TAG_DESCRIPTION);
				//description definition for the reqt url
				rootNodeDec.setAttributeNS(XMLUtil.NAMESPACE_RDF, TAG_ABOUT, "#n" + i);
				rootNode.appendChild(rootNodeDec);
				
				var rootNodeTitle = document.createElementNS(XMLUtil.NAMESPACE_DC, TAG_TITLE);
				rootNodeTitle.textContent = link.summary;
				rootNodeDec.appendChild(rootNodeTitle);   
            }
			i++;            
        });
        
    }
    
    function _modifyValidatesLinks(resourceType, doc, addedLinks, removedLinks){
        var validatesTagName = _getValidatesTagName(resourceType);
        var rootNode = XMLUtil.getNode(doc.childNodes, TAG_RDF, XMLUtil.NAMESPACE_RDF);
		var rootNodeTC = XMLUtil.getNode(rootNode.childNodes, TAG_TESTCASE, XMLUtil.NAMESPACE_OSLC_QM_V2);
		if(rootNodeTC == null){
			rootNodeTC = XMLUtil.getNode(rootNode.childNodes, TAG_TESTPLAN, XMLUtil.NAMESPACE_OSLC_QM_V2);
			if(rootNodeTC == null)
				rootNodeTC = XMLUtil.getNode(rootNode.childNodes, TAG_TESTSCRIPT, XMLUtil.NAMESPACE_OSLC_QM_V2);
		}
		
		var i = 0; //generate ID for Reqt Links
        dojo.forEach(addedLinks, function(link){
            var linkNode = null;
            if (IS_IE_OR_CHROME) { // handle IE xml attribute manipulation 
                linkNode = doc.createElement(TAG_PREX_OSLC_QM + ":" + validatesTagName);
                linkNode.setAttribute(TAG_PREX_RDF + ":" + TAG_RESOURCE, link.uri);								
				//IE is using different codes comparing to FireFox
				//assign an ID for the rqt url, the nubmer will be used by rootNodeDec
				linkNode.setAttribute(TAG_PREX_RDF + ":" + TAG_ID,  NEW_ADDED_LINKS + i); //WI 82406: Add "rdf:" to use qualified "rdf:ID".

				rootNodeTC.appendChild(linkNode);
				
				var rootNodeDec = doc.createElement(TAG_PREX_RDF + ":" + TAG_DESCRIPTION);
				rootNodeDec.setAttribute(TAG_PREX_RDF + ":" + TAG_ABOUT, NEW_ADDED_LINKS_TAG + i);
				rootNode.appendChild(rootNodeDec);
				
                var rootNodeTitle = doc.createElement(TAG_DCTERMS + ":" + TAG_TITLE);
				rootNodeTitle.appendChild(doc.createTextNode(link.summary));//WI 82406: Add the link label by appending the text node.
				rootNodeDec.appendChild(rootNodeTitle);
            }
            else {
				linkNode = document.createElementNS(XMLUtil.NAMESPACE_OSLC_QM_V2, validatesTagName);
                linkNode.setAttributeNS(XMLUtil.NAMESPACE_RDF, TAG_RESOURCE, link.uri);
				//assign an ID for the rqt url, the nubmer will be used by rootNodeDec
				linkNode.setAttributeNS(XMLUtil.NAMESPACE_RDF, TAG_ID, NEW_ADDED_LINKS + i);

				rootNodeTC.appendChild(linkNode);
				
				var rootNodeDec = document.createElementNS(XMLUtil.NAMESPACE_RDF, TAG_DESCRIPTION);
				//description definition for the reqt url
				rootNodeDec.setAttributeNS(XMLUtil.NAMESPACE_RDF, TAG_ABOUT, NEW_ADDED_LINKS_TAG + i);
				rootNode.appendChild(rootNodeDec);
				
				var rootNodeTitle = document.createElementNS(XMLUtil.NAMESPACE_DC, TAG_TITLE);
				rootNodeTitle.textContent = link.summary;
				rootNodeDec.appendChild(rootNodeTitle);   
            }
			i++;            
        });
        dojo.forEach(removedLinks, function(link){
            var linkNodes = XMLUtil.getNodes(rootNodeTC.childNodes, validatesTagName, XMLUtil.NAMESPACE_OSLC_QM_V2);
            dojo.forEach(linkNodes, function(linkNode){
                var uriAttrNode = XMLUtil.getAttributeNode(linkNode, TAG_RESOURCE, XMLUtil.NAMESPACE_RDF);
                var uri = XMLUtil.getAttributeNodeValue(uriAttrNode);
                if (link.uri == uri) {
                    rootNodeTC.removeChild(linkNode);
                }
            });
        });
    }
    
    function _getLinksFromReqCollection(collectionUri, callback, errorCallback){
        _getReqCollectionDoc(collectionUri, function(doc){
       //     var rdfNode = XMLUtil.getNode(doc.childNodes, TAG_RDF, XMLUtil.NAMESPACE_RDF);
            var rootNode = XMLUtil.getNode(doc.childNodes, TAG_REQ_COL, XMLUtil.NAMESPACE_OSLC_RDM);
            var linkNodes = XMLUtil.getNodes(rootNode.childNodes, TAG_LI, XMLUtil.NAMESPACE_RDF);
            
            var uris = [];
            dojo.forEach(linkNodes, function(linkNode){
                var uriAttrNode = XMLUtil.getAttributeNode(linkNode, TAG_RESOURCE, XMLUtil.NAMESPACE_RDF);
                var uri = XMLUtil.getAttributeNodeValue(uriAttrNode);
                uris.push(uri);
            });
            if (uris.length > 0) {
            	_getLinksDetail(uris, callback);
            } else {
            	callback([]);
            }
        }, errorCallback);
    }
	
	/**
	 * Instead of get all the details for all requirement (as _getLinksFromReqCollectionV2, deprecating it),
	 * this method only get requirement uris for the given collection uri
	 * 
	 * @param {Object} collectionUri
	 * @param {Object} callback
	 * @param {Object} errorCallback
	 */
    function _getReqtURIsWithoutDetailsFromReqColl(collectionUri, callback, errorCallback){
        _getReqCollectionDoc(collectionUri, function(doc){
            var rootNode = XMLUtil.getNode(doc.childNodes, TAG_REQ_COL, XMLUtil.NAMESPACE_OSLC_RDM);
            var linkNodes = XMLUtil.getNodes(rootNode.childNodes, TAG_LI, XMLUtil.NAMESPACE_RDF);
            
            var uris = [];
            dojo.forEach(linkNodes, function(linkNode){
                var uriAttrNode = XMLUtil.getAttributeNode(linkNode, TAG_RESOURCE, XMLUtil.NAMESPACE_RDF);
                var uri = XMLUtil.getAttributeNodeValue(uriAttrNode);
                uris.push(uri);
            });
            if (uris.length > 0) {
            	callback(uris);
            } else {
            	callback([]);
            }
        }, errorCallback);
    }
	
	/**
	 * Instead of get all the details for all requirement (as _getLinksFromReqCollectionV2, deprecating it),
	 * this method only get requirement uris for the given collection uri
	 * 
	 * @param {Object} collectionUri
	 * @param {Object} callback
	 * @param {Object} errorCallback
	 */
	function _getReqtURIsWithoutDetailsFromReqCollV2(collectionUri, callback, errorCallback){
        _getReqCollectionDocV2(collectionUri, function(doc){
         	var uris = [];
         	var rdfNode = XMLUtil.getNode(doc.childNodes, TAG_RDF, XMLUtil.NAMESPACE_RDF);
         	
         	var rootNode = XMLUtil.getNode(rdfNode.childNodes, TAG_REQ_COL, XMLUtil.NAMESPACE_OSLC_RM_V2);
         	if (rootNode != undefined) {
         		var linkNodes = XMLUtil.getNodes(rootNode.childNodes, TAG_USES, XMLUtil.NAMESPACE_OSLC_RM_V2);
         		dojo.forEach(linkNodes, function(linkNode){
         			var uriAttrNode = XMLUtil.getAttributeNode(linkNode, TAG_RESOURCE, XMLUtil.NAMESPACE_RDF);
	            	if (uriAttrNode){
	            		var uri = XMLUtil.getAttributeNodeValue(uriAttrNode);
	           			if (uri){
	           				uris.push(uri);}
	            	}
	            	
	        	});
         	}

            if (uris.length > 0) {
            	callback(uris);
            } else {
            	callback([]);
            }
        }, errorCallback);
    }
    
	/**
	 * @deprecated
	 * @param {Object} collectionUri
	 * @param {Object} callback
	 * @param {Object} errorCallback
	 */
    function _getLinksFromReqCollectionV2(collectionUri, callback, errorCallback){
        _getReqCollectionDocV2(collectionUri, function(doc){
         	var uris = [];
         	var rdfNode = XMLUtil.getNode(doc.childNodes, TAG_RDF, XMLUtil.NAMESPACE_RDF);
         	
         	var rootNode = XMLUtil.getNode(rdfNode.childNodes, TAG_REQ_COL, XMLUtil.NAMESPACE_OSLC_RM_V2);
         	if (rootNode != undefined) {
         		var linkNodes = XMLUtil.getNodes(rootNode.childNodes, TAG_USES, XMLUtil.NAMESPACE_OSLC_RM_V2);
         		dojo.forEach(linkNodes, function(linkNode){
         			var uriAttrNode = XMLUtil.getAttributeNode(linkNode, TAG_RESOURCE, XMLUtil.NAMESPACE_RDF);
	            	if (uriAttrNode){
	            		var uri = XMLUtil.getAttributeNodeValue(uriAttrNode);
	           			if (uri){
	           				uris.push(uri);}
	            	}
	            	
	        	});
         	}

            if (uris.length > 0) {
            	_getLinksDetail(uris, callback);
            } else {
            	callback([]);
            }
        }, errorCallback);
    }
	
	/**
	 * give the Requirement URIs get the requirement details from remote RM server
	 * after the call was made (got return), empty the deffer map
	 * 
	 * @param {Object} uris
	 * @param {Object} callback
	 */
    function _getLinksDetail(uris, callback){
        var links = [];
        var index = 0;
		var paramsClause = "oslc.prefix=" + "dcterms=" + encodeURIComponent("<") + "http://purl.org/dc/terms/" + encodeURIComponent(">") + "&oslc.properties=" + "dcterms:title,dcterms:modified";
        dojo.forEach(uris, function(uri){
            _getReqResource(uri, function(req){		
				//if the call had been made, clear it from the onGoingRequests		
				if (onGoingRequests[uri]) {delete onGoingRequests[uri];} 
				
            	if (req) {
                	links.push(req);
                }
                if (++index == uris.length) 
                    callback(links);
                
            }, null, paramsClause);
        });		
    }
    
    // Defect 54513: Ask for OSLC V2 resource as default first. If it fails, ask for V1.
    function _getReqResource(uri, callback, oslcVersion, paramsClause){
		var url = uri;
    	var version = !oslcVersion ? '2.0' : oslcVersion;
        var acceptHeader = version == '2.0' ? HEADER_ACCEPT_XML : HEADER_ACCEPT_REQ_V1;
        var currentHeaders = {'Accept': acceptHeader};
        if(version == '2.0'){
			currentHeaders['OSLC-Core-Version'] = '2.0';
			//Append the paramsClause for selective retrieval if paramsClause exists
			url = paramsClause ? (uri + "?" + paramsClause) : uri;
		}
        var deferred = jazz.client.xhrGet({
            url: url,
            headers: currentHeaders,
            oslcVersion: version,
            handleAs: 'xml',
            load: function(doc){
                var req = {};
				//Use uri as the identifier for the resource  
                req.uri = uri;
                var rootNode = null;
                if(version == '2.0') {
                	var rdfNode = XMLUtil.getNode(doc.childNodes, TAG_RDF, XMLUtil.NAMESPACE_RDF);
                	// Defect 56886: RRC returns several Description nodes with different Titles. Let's look for the right one.
                	var attribute = {value : uri,
                					 baseName: 'about',
                					 namespace: XMLUtil.NAMESPACE_RDF};
                	rootNode = XMLUtil.getNode(rdfNode.childNodes, TAG_DESCRIPTION, XMLUtil.NAMESPACE_RDF, attribute);
                	if (rootNode == null)
                		rootNode = XMLUtil.getNode(rdfNode.childNodes, TAG_REQ, XMLUtil.NAMESPACE_OSLC_RM_V2);
						
					req.oslcVersion = "2.0";
                }
                else if (version == '1.0'){
                  rootNode = XMLUtil.getNode(doc.childNodes, TAG_REQ, XMLUtil.NAMESPACE_OSLC_RDM);
				  req.oslcVersion = "1.0";
                }  
                var summaryNode = XMLUtil.getNode(rootNode.childNodes, TAG_TITLE, XMLUtil.NAMESPACE_DCTERMS);
                if (summaryNode) {
                    req.summary = dojox.xml.parser.textContent(summaryNode);
                }
                else {
                    req.summary = "_blank";
                }
                
                var lastModifiedNode = XMLUtil.getNode(rootNode.childNodes, "modified", XMLUtil.NAMESPACE_DCTERMS);
                
                if(lastModifiedNode){
                	req.lastModified = dojox.xml.parser.textContent(lastModifiedNode);
                }
                
                callback(req);
            },
            error: function(result) {
            	// Trying as 1.0
            	if(version == '2.0' && paramsClause){
            		//Fix defect 89393, try to not get the selected properties of the requirement specified by paramsClause 
            		_getReqResource(uri, callback, '2.0');
            	}else if(version == '2.0'){
            		_getReqResource(uri, callback, '1.0');	
            	}else
            		callback(null);// For 22129, if resource doesn't exist any more, the process should proceed
            }
        });
		
		//put this deferred in the onGoingRequests map, if the user made a cancel request, will cancel the deferred.
		onGoingRequests[uri]=deferred;
    }
    
    function _getReqCollectionDoc(collectionUri, callback, errorCallback){
        var acceptHeader = HEADER_ACCEPT_REQ_COL_V1;
        jazz.client.xhrGet({
            url: collectionUri,
            headers: {
                'Accept': acceptHeader
            },
            handleAs: 'xml',
            load: function(doc){
                callback(doc);
            },
            error: function(error, ioArgs){
                errorCallback(error, ioArgs);
            }
            
        });
    }
    
    function _getReqCollectionDocV2(collectionUri, callback, errorCallback){
        var acceptHeader = HEADER_ACCEPT_XML;
        jazz.client.xhrGet({
            url: collectionUri,
            headers: {
                'Accept': acceptHeader,
                'OSLC-Core-Version':'2.0'
            },
            handleAs: 'xml',
            load: function(doc){
                callback(doc);
            },
            error: function(error, ioArgs){
                errorCallback(error, ioArgs);
            }
            
        });
    }
    
    // Check whether the collectionUri is pointed to a RM collection, if it's, callback "true", otherwise, callback "false"
    function _getIsReqCollectionResource(collectionUri, oslcVersion, callback, errorCallback){
    	if (oslcVersion == "2.0")
			_getReqCollectionDocV2(collectionUri, function(doc){
         	var rdfNode = XMLUtil.getNode(doc.childNodes, TAG_RDF, XMLUtil.NAMESPACE_RDF);
         	
         	var rootNode = XMLUtil.getNode(rdfNode.childNodes, TAG_REQ_COL, XMLUtil.NAMESPACE_OSLC_RM_V2);
         	if (rootNode != undefined) {
         		callback("true");
         	}
         	else {
         		callback("false");
         	}
 
        }, errorCallback);
		else if (oslcVersion == null || oslcVersion == "1.0")
			_getReqCollectionDoc(collectionUri, function(doc){
       
            var rootNode = XMLUtil.getNode(doc.childNodes, TAG_REQ_COL, XMLUtil.NAMESPACE_OSLC_RDM);
           	if (rootNode != undefined) {
         		callback("true");
         	}
         	else {
         		callback("false");
         	}
        }, errorCallback);
    }
    
	function _getValidatedByDoc(oslcVersion,resourceType, resourceUri, callback, resultCallbacks){ //Default is V2
		if (oslcVersion == "2.0")
			return _getValidatedByDocV2(resourceType, resourceUri, callback, resultCallbacks);
		else if (oslcVersion == null || oslcVersion == "1.0")
			return _getValidatedByDocV1(resourceType, resourceUri, callback, resultCallbacks);
	}
	
	function _getValidatedByDocV1(resourceType, resourceUri, callback, resultCallbacks){//for V1
    	return _getRequirementResourceV1(resourceType, resourceUri, callback, resultCallbacks);
    }
	
    function _getRequirementResourceV1(resourceType, resourceUri, callback, resultCallbacks){//for V1   	
        var acceptHeader = null;
        if (resourceType == RESOURCE_TYPE_REQ) {
            acceptHeader = HEADER_ACCEPT_REQ_V1;
        }
        else {
            acceptHeader = HEADER_ACCEPT_REQ_COL_V1;
        }        
        
        var deferred = jazz.client.xhrGet({
            url: resourceUri,
            headers: {
                'Accept': acceptHeader
            },
            handleAs: 'xml',
            resultCallbacks: resultCallbacks,
            load: function(doc, ioargs){
				var etag = ioargs.xhr.getResponseHeader('ETag');
            	callback(doc, etag);
            },
            error: function(result, ioargs) {
            	if(this.resultCallbacks) {
            		// XHR status 410 equals "Gone". Allow local deletion.
            		if(this.resultCallbacks.nextLink && (ioargs.xhr.status == 410 || ioargs.xhr.status == 404)) {
            			this.resultCallbacks.errorMessages = this.resultCallbacks.errorMessages || new Array();
            			this.resultCallbacks.errorMessages.push(result);
            			this.resultCallbacks.nextLink(this.resultCallbacks.errorMessages.join("\n"));
            		}
            		else {
            			if (this.resultCallbacks.error)
            				this.resultCallbacks.error(result);
            			else {
            				this.resultCallbacks(result);
            			}
            		}
            	}
            }
        });
        
        return deferred;
    }

    function _getValidatedByDocV2(resourceType, resourceUri, callback, resultCallbacks){//for V2
    	return _getRequirementResourceV2(resourceType, resourceUri, callback, resultCallbacks);
    }
    
    function _getRequirementResourceV2(resourceType, resourceUri, callback, resultCallbacks){//for V2    	       
        var deferred = jazz.client.xhrGet({
            url: resourceUri, 
            headers: {
				"OSLC-Core-Version": "2.0",
                'Accept': HEADER_ACCEPT_XML
            },
            handleAs: 'xml',
			contentType: HEADER_ACCEPT_XML,
			resultCallbacks: resultCallbacks,
            load: function(doc, ioargs){
				var etag = ioargs.xhr.getResponseHeader('ETag');
            	callback(doc, etag);
            },
            error: function(result, ioargs) {
            	if(this.resultCallbacks) {
            		// XHR status 410 equals "Gone". Allow local deletion.
            		if(this.resultCallbacks.nextLink && (ioargs.xhr.status == 410 || ioargs.xhr.status == 404)) {
            			this.resultCallbacks.errorMessages = this.resultCallbacks.errorMessages || new Array();
            			this.resultCallbacks.errorMessages.push(result);
            			this.resultCallbacks.nextLink(this.resultCallbacks.errorMessages.join("\n"));
            		}
            		else {
            			if (this.resultCallbacks.error)
            				this.resultCallbacks.error(result);
            			else {
            				this.resultCallbacks(result);
            			}
            		}
            	}
            }
        });
        
        return deferred;
    }
    
	function _updateValidatedByDoc(oslcVersion, resourceType, resourceUri, doc){//Default is V2
		if (oslcVersion == "2.0")
			_updateValidatedByDocV2(resourceType, resourceUri, doc);
		else if (oslcVersion == null || oslcVersion == "1.0")
			_updateValidatedByDocV1(resourceType, resourceUri, doc);
	}
	
	// Synchronous version of _updateValidatedByDoc with error handling
	function _updateValidatedByDocSync(oslcVersion, resourceType, resourceUri, doc, etag, success, errorCllbcks, errorMessage){//Default is V2
		if (oslcVersion == "2.0")
			
			_updateRequirementResourceV2Sync(resourceType, resourceUri, doc, etag, success, errorCllbcks, errorMessage);
		else if (oslcVersion == null || oslcVersion == "1.0")
			_updateRequirementResourceV1Sync(resourceType, resourceUri, doc, etag, success, errorCllbcks, errorMessage);
	}
	
    function _updateRequirementResourceV1(resourceType, resourceUri, doc){//for V1
        var contentType = null;
        if (resourceType == RESOURCE_TYPE_REQ) {
            contentType = HEADER_ACCEPT_REQ_V1;
        }
        else {
            contentType = HEADER_ACCEPT_REQ_COL_V1;
        }
        jazz.client.xhrPut({
            url: resourceUri,
            headers: {
                'Accept': contentType
            },
            handleAs: 'xml',            
            contentType: contentType,
            postData: dojox.xml.parser.innerXML(doc)
        });
    }
    
    function _updateRequirementResourceV1Sync(resourceType, resourceUri, doc, etag, success, errorCllbcks, errorMessage){//for V1
        var contentType = null;
        if (resourceType == RESOURCE_TYPE_REQ) {
            contentType = HEADER_ACCEPT_REQ_V1;
        }
        else {
            contentType = HEADER_ACCEPT_REQ_COL_V1;
        }
        jazz.client.xhrPut({
            url: resourceUri,
            headers: {
                'Accept': contentType,
                "If-Match": etag
            },
            handleAs: 'xml',            
            contentType: contentType,
            postData: dojox.xml.parser.innerXML(doc),
            successCallback: success,
            errorCallbacks: errorCllbcks,
            errorMessage: errorMessage,
            load: function(result){
            	//console.debug("Affected by link added V1.");
            	if(this.successCallback && dojo.isFunction(this.successCallback))
            		this.successCallback(this.errorMessage || result);
            },
            error: function(result, ioArgs) {
            	if(this.errorCallbacks) {
	            	if(ioArgs.xhr.status == 400 && this.errorCallbacks.error) {
	            		// WI 51476: Chances are the Requirement provider implements pre-V1 OSLC. Look for special URI.
	            		this.errorCallbacks.handleRequirementsPreV1(this.url + "/" + ID_VALIDATED_BY_COL, this.errorCallbacks.nextLink);
	            	}
	            	else if(this.errorCallbacks.error && dojo.isFunction(this.errorCallbacks.error))
	            		this.errorCallbacks.error(result);
            	}
            }
        });
    }
	
	function _updateValidatedByDocV1(resourceType, resourceUri, doc){
    	_updateRequirementResourceV1(resourceType, resourceUri, doc);
    }
    
    function _updateValidatedByDocV2(resourceType, resourceUri, doc){//for V2
    	_updateRequirementResourceV2(resourceType, resourceUri, doc);
    }
    
    function _updateRequirementResourceV2(resourceType, resourceUri, doc){//for V2
        jazz.client.xhrPut({
            url: resourceUri,
            headers: {
				"OSLC-Core-Version": "2.0",
                'Accept': HEADER_ACCEPT_XML
            },
            handleAs: 'xml',            
			contentType: HEADER_ACCEPT_XML,
            postData: dojox.xml.parser.innerXML(doc)
        });
    }
    
    function _updateRequirementResourceV2Sync(resourceType, resourceUri, doc, etag, success, errorCllbcks, errorMessage){//for V2
        jazz.client.xhrPut({
            url: resourceUri,
            headers: {
				"OSLC-Core-Version": "2.0",
                'Accept': HEADER_ACCEPT_XML,
				"If-Match": etag
            },
            handleAs: 'xml',            
			contentType: HEADER_ACCEPT_XML,
            postData: dojox.xml.parser.innerXML(doc),
            successCallback: success,
            errorCallback: errorCllbcks ? errorCllbcks.error : null,
            errorMessage: errorMessage,
            load: function(result){
            	//console.debug("Affected by link added V2.");
            	if(this.successCallback && dojo.isFunction(this.successCallback))
            		this.successCallback(this.errorMessage || result);
            },
            error: function(result) {
            	if(this.errorCallback && dojo.isFunction(this.errorCallback))
            		this.errorCallback(result);
            }
        });
    }
    
	function _modifyValidatedByLinks(oslcVersion, doc, addedLinks, removedLinks){//Default is V2
		if (oslcVersion == "2.0")
			_modifyValidatedByLinksV2(doc, addedLinks, removedLinks);
		else if (oslcVersion == null || oslcVersion == "1.0")
			_modifyValidatedByLinksV1(doc, addedLinks, removedLinks);
	}
	
	function _modifyValidatedByLinksV1(doc, addedLinks, removedLinks){
		_modifyRequirementResourceV1(doc, addedLinks, removedLinks, TAG_VALIDATED_BY);	    
    }
	
    function _modifyRequirementResourceV1(doc, addedLinks, removedLinks, linkType){//for V1
        var rootNode = XMLUtil.getNode(doc.childNodes, TAG_REQ, XMLUtil.NAMESPACE_OSLC_RDM);
        if(rootNode == null)
        	rootNode = XMLUtil.getNode(doc.childNodes, TAG_REQ_COL, XMLUtil.NAMESPACE_OSLC_RDM);
        // WI 51476: In case the root node is still null, we are modifying a RRC 2.0.0.3 requirement,
        // which does not implement OSLC V1 correctly. Handle it properly.
        if(rootNode == null) { // WI 45102 - Adding links to right node for pre V1.
        	var rdfNode = XMLUtil.getNode(doc.childNodes, TAG_RDF, XMLUtil.NAMESPACE_RDF);
        	rootNode = XMLUtil.getNode(rdfNode.childNodes, TAG_LINK_COL, XMLUtil.NAMESPACE_OSLC_RDM);
        }
        	
        dojo.forEach(addedLinks, function(link){
            var linkNode = null;
            if (IS_IE_OR_CHROME) { // handle IE xml attribute manipulation
                linkNode = doc.createElement(TAG_PREX_CALM + ":" + linkType);
//                linkNode.namespaceURI = XMLUtil.NAMESPACE_CALM;
                linkNode.setAttribute(TAG_PREX_RDF + ":" + TAG_RESOURCE, link.uri);
                linkNode.setAttribute(TAG_PREX_RDFS + ":" + TAG_LABEL, link.summary);
                linkNode.setAttribute(NAMESPACE_TAG_V1 + ":" + TAG_PREX_RDFS, XMLUtil.NAMESPACE_RDFS);
            }
            else {
                linkNode = document.createElementNS(XMLUtil.NAMESPACE_CALM, linkType);
                linkNode.setAttributeNS(XMLUtil.NAMESPACE_RDF, TAG_RESOURCE, link.uri);
                linkNode.setAttributeNS(XMLUtil.NAMESPACE_RDFS, TAG_LABEL, link.summary);
            }
            rootNode.appendChild(linkNode);
        });
        dojo.forEach(removedLinks, function(link){
            var linkNodes = XMLUtil.getNodes(rootNode.childNodes, linkType, XMLUtil.NAMESPACE_CALM);
            dojo.forEach(linkNodes, function(linkNode){
                var uriAttrNode = XMLUtil.getAttributeNode(linkNode, TAG_RESOURCE, XMLUtil.NAMESPACE_RDF);
                var uri = XMLUtil.getAttributeNodeValue(uriAttrNode);
                if (link.uri == uri) {
                    rootNode.removeChild(linkNode);
                }
            });
        });
    }
    
    function _modifyValidatedByLinksV2(doc, addedLinks, removedLinks){//for v2
		_modifyRequirementResourceV2(doc, addedLinks, removedLinks, TAG_VALIDATED_BY);	    
    }
    
    function _modifyRequirementResourceV2(doc, addedLinks, removedLinks, linkType){//for v2
        var rdfNode = XMLUtil.getNode(doc.childNodes, TAG_RDF, XMLUtil.NAMESPACE_RDF);
        var rootNode = XMLUtil.getNode(rdfNode.childNodes, TAG_REQ, XMLUtil.NAMESPACE_OSLC_RM_V2);
        if(rootNode == null)
        	rootNode = XMLUtil.getNode(rdfNode.childNodes, TAG_REQ_COL, XMLUtil.NAMESPACE_OSLC_RM_V2);
        
		var i=0; //generate ID for Reqt links
        dojo.forEach(addedLinks, function(link){
            var linkNode = null;
            if (IS_IE_OR_CHROME) { // handle IE/Chrome xml attribute manipulation
                linkNode = doc.createElement(TAG_PREX_OSLC_RM + ":" + linkType);
                linkNode.setAttribute(TAG_PREX_RDF + ":" + TAG_RESOURCE, link.uri);
				
				//assign an ID for the rqt url, the nubmer will be used by rootNodeDec
				linkNode.setAttribute(TAG_ID,  NEW_ADDED_LINKS + i);
				rootNode.appendChild(linkNode);
				
				var rootNodeDec = doc.createElement(TAG_PREX_RDF + ":" + TAG_DESCRIPTION);//rdf:Description
				rootNodeDec.setAttribute(TAG_PREX_RDF + ":" + TAG_ABOUT, NEW_ADDED_LINKS_TAG + i);//rdf:about
				rdfNode.appendChild(rootNodeDec);

                // workaround for RRC using "dc" as dcterms prefix.  See WI 63092.			
                var rootNodeTitle;			
                var isDcPrefix = rdfNode.getAttribute("xmlns:" + TAG_DC);
                if (isDcPrefix && (isDcPrefix == XMLUtil.NAMESPACE_DCTERMS)) {
				    rootNodeTitle = doc.createElement(TAG_DC + ":" + TAG_TITLE);//dc:title
                }
                else {
				    rootNodeTitle = doc.createElement(TAG_DCTERMS + ":" + TAG_TITLE);//dcterms:title
                }
				
				rootNodeTitle.text = link.summary;
				rootNodeDec.appendChild(rootNodeTitle);  
            }
            else {
                linkNode = document.createElementNS(XMLUtil.NAMESPACE_OSLC_RM_V2, linkType);
                linkNode.setAttributeNS(XMLUtil.NAMESPACE_RDF, TAG_RESOURCE, link.uri);
				
				//assign an ID for the rqt url, the nubmer will be used by rootNodeDec
        		linkNode.setAttributeNS(XMLUtil.NAMESPACE_RDF, TAG_ID,  NEW_ADDED_LINKS + i);
				
				rootNode.appendChild(linkNode);
				
				var rootNodeDec = document.createElementNS(XMLUtil.NAMESPACE_RDF, TAG_DESCRIPTION);//rdf:Description
				rootNodeDec.setAttributeNS(XMLUtil.NAMESPACE_RDF, TAG_ABOUT, NEW_ADDED_LINKS_TAG + i);//rdf:about
				rdfNode.appendChild(rootNodeDec);
				
				var rootNodeTitle = document.createElementNS(XMLUtil.NAMESPACE_DC, TAG_TITLE);//dcterms:title
				rootNodeTitle.textContent = link.summary;
				rootNodeTitle.setAttributeNS(XMLUtil.NAMESPACE_RDF, TAG_RDF_PARSETYPE, TAG_LITERAL);//rdf:parseType
				rootNodeDec.appendChild(rootNodeTitle);  
				
				//linkNode.setAttributeNS(XMLUtil.NAMESPACE_RDFS, TAG_LABEL, link.summary);
            }
			i++;            
        });
        dojo.forEach(removedLinks, function(link){
            var linkNodes = XMLUtil.getNodes(rootNode.childNodes, linkType, XMLUtil.NAMESPACE_OSLC_RM_V2);
            dojo.forEach(linkNodes, function(linkNode){
                var uriAttrNode = XMLUtil.getAttributeNode(linkNode, TAG_RESOURCE, XMLUtil.NAMESPACE_RDF);
                var uri = XMLUtil.getAttributeNodeValue(uriAttrNode);
                if (link.uri == uri) {
                    rootNode.removeChild(linkNode);
                }
            });
        });
    }
	
	function _readValidatedByLinks(oslcVersion, doc, linkType){
		if (oslcVersion == "2.0"){
			var rdfNode = XMLUtil.getNode(doc.childNodes, TAG_RDF, XMLUtil.NAMESPACE_RDF);
	        var rootNode = XMLUtil.getNode(rdfNode.childNodes, TAG_REQ, XMLUtil.NAMESPACE_OSLC_RM_V2);
	        if(rootNode == null)
	        	rootNode = XMLUtil.getNode(rdfNode.childNodes, TAG_REQ_COL, XMLUtil.NAMESPACE_OSLC_RM_V2);
				
			var validatedByNodes = [];
			var validatedByLinks = [];	
			if(rootNode){
				validatedByNodes = XMLUtil.getNodes(rootNode.childNodes, TAG_VALIDATED_BY, XMLUtil.NAMESPACE_OSLC_RM_V2);
				dojo.forEach(validatedByNodes, function(node){
					var link = XMLUtil.getAttributeNode(node, TAG_RESOURCE, XMLUtil.NAMESPACE_RDF);
					if(link){
						validatedByLinks.push(XMLUtil.getAttributeNodeValue(link));
					}
				});
			}
			return validatedByLinks;
		} else if (oslcVersion == null || oslcVersion == "1.0") {
			var rootNode = XMLUtil.getNode(doc.childNodes, TAG_REQ, XMLUtil.NAMESPACE_OSLC_RDM);
	        if(rootNode == null)
	        	rootNode = XMLUtil.getNode(doc.childNodes, TAG_REQ_COL, XMLUtil.NAMESPACE_OSLC_RDM);
	        if(rootNode == null) { 
	        	var rdfNode = XMLUtil.getNode(doc.childNodes, TAG_RDF, XMLUtil.NAMESPACE_RDF);
	        	rootNode = XMLUtil.getNode(rdfNode.childNodes, TAG_LINK_COL, XMLUtil.NAMESPACE_OSLC_RDM);
	        }
	        var validatedByNodes = [];
			var validatedByLinks = [];	
			if(rootNode){
				validatedByNodes = XMLUtil.getNodes(rootNode.childNodes, TAG_VALIDATED_BY, XMLUtil.NAMESPACE_CALM);
				dojo.forEach(validatedByNodes, function(node){
					var link = XMLUtil.getAttributeNode(node, TAG_RESOURCE, XMLUtil.NAMESPACE_RDF);
					if(link){
						validatedByLinks.push(XMLUtil.getAttributeNodeValue(link));
					}
				});
			}
			return validatedByLinks;
		}
    }
    
    function _getValidatesLinkType(resourceType){
        var linkType = null;
        if (resourceType == RESOURCE_TYPE_REQ) {
            linkType = ID_VALIDATES_COL;
        }
        else {
            linkType = ID_VALIDATES_REQ_SET_COL_V2;
        }
        return linkType;
    }
    
	function _getValidatesTagNameV1(resourceType){
        var tagName = null;
        if (resourceType == RESOURCE_TYPE_REQ) {
            tagName = TAG_VALIDATES_REQ;
        }
        else {
            tagName = TAG_VALIDATES_REQ_SET;
        }
        return tagName;
    }
	
    function _getValidatesTagName(resourceType){
        var tagName = null;
        if (resourceType == RESOURCE_TYPE_REQ) {
            tagName = TAG_VALIDATES_REQ;
        }
        else {
            tagName = TAG_VALIDATES_REQ_SET_V2;
        }
        return tagName;
    }
    
    function _getLinksFromWorkItem(linkType, workitemUri, callback, errorCallback) {
    	var itemURL = workitemUri + '?oslc.properties=' + escape(linkType);
    	jazz.client.xhrGet({
            url: itemURL,
            headers: {
	    		"Accept": "application/json",
				"OSLC-Core-Version": "2.0"
            },
            preventCache: true,
            handleAs: 'json',
            load: function(doc){
            	if (doc instanceof Error) {
                    callback([]);
                }
                else {
                	var linkCollection = doc[linkType] || [];
                	callback(linkCollection);
                }
            },
            error: function(error, ioArgs){
                errorCallback(error, ioArgs);
            }
            
        });
    }
    
    com.ibm.rqm.defects.web.client.internal.ResourceLinkService.RESOURCE_TYPE_REQ = RESOURCE_TYPE_REQ;
    com.ibm.rqm.defects.web.client.internal.ResourceLinkService.RESOURCE_TYPE_REQ_COL = RESOURCE_TYPE_REQ_COL;
    com.ibm.rqm.defects.web.client.internal.ResourceLinkService.TAG_TCRESULT = TAG_TCRESULT;
    com.ibm.rqm.defects.web.client.internal.ResourceLinkService.TAG_TSRESULT = TAG_TSRESULT;
    com.ibm.rqm.defects.web.client.internal.ResourceLinkService.RESOURCE_TYPE_BUILD_REC = RESOURCE_TYPE_BUILD_REC;
    com.ibm.rqm.defects.web.client.internal.ResourceLinkService.QUERY_MAX_PAGE_SIZE = QUERY_MAX_PAGE_SIZE;
    com.ibm.rqm.defects.web.client.internal.ResourceLinkService.TAG_EXECUTED_ON_DEPLOYMENT = TAG_EXECUTED_ON_DEPLOYMENT;
    com.ibm.rqm.defects.web.client.internal.ResourceLinkService.TAG_RUNS_ON_DEPLOYMENT = TAG_RUNS_ON_DEPLOYMENT;
    com.ibm.rqm.defects.web.client.internal.ResourceLinkService.TAG_EXECUTIONRECORD = TAG_EXECUTIONRECORD;
    com.ibm.rqm.defects.web.client.internal.ResourceLinkService.TAG_TESTRESULT = TAG_TESTRESULT;
})();
