<?xml version="1.0" encoding="utf-8"?>
<!-- 
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2014. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:prep="http://com.ibm.rqm.print/prep"
    xmlns:rqm="http://jazz.net/xmlns/alm/qm/v0.1/"
    xmlns:dc="http://purl.org/dc/elements/1.1/"
    xmlns:alm="http://jazz.net/xmlns/alm/v0.1/"
    xmlns:exslt="http://exslt.org/common"
    xmlns:HttpClientExtension="com.ibm.rqm.print.render.fop.xml.xslt.extension.http.HttpClientExtension"
    xmlns:log="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.LoggingExtension"
    xmlns:utils="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.UtilsExtension"
    exclude-result-prefixes="xsl HttpClientExtension log">
    
    <!-- templates -->
    <xsl:template match="table-list/rqm:testcase">
        <xsl:param name="columns"/>
        <xsl:variable name="tpHref" select="../@parent"/>
        
        <xsl:variable name="fields">
            <xsl:variable name="tmp-fields">
                <xsl:text>identifier|</xsl:text>
                <xsl:for-each select="$columns">
                    <xsl:apply-templates mode="tc-param" select="prep:name"/>
                </xsl:for-each>
            </xsl:variable>
            
            <xsl:value-of select="substring($tmp-fields, 0, string-length($tmp-fields))"/>
        </xsl:variable>
        
        <xsl:if test="log:isDebugEnabled()">
            <xsl:value-of select="log:debug(concat('testcase $fields=', $fields))"/></xsl:if>
        
        <xsl:apply-templates mode="tc-copy" select="HttpClientExtension:get($restClient, @href, concat('fields=feed/entry/content/testcase/(', $fields, ')'))">
            <xsl:with-param name="columns" select="$columns"/>
            <xsl:with-param name="tpHref" select="$tpHref"/>
        </xsl:apply-templates>
    </xsl:template>
    
    <xsl:template mode="tc-param" match="prep:column/prep:name">
        <!-- intentionally left blank -->
        <!-- ignores all columns that are not otherwise matched by a template -->
    </xsl:template>
    
    <xsl:template mode="tc-param" match="prep:column/prep:name[.='id']">
        <xsl:text>webId|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="tc-param" match="prep:column/prep:name[.='priority']">
        <xsl:text>priority|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="tc-param" match="prep:column/prep:name[.='name']">
        <xsl:text>title|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="tc-param" match="prep:column/prep:name[.='modified']">
        <xsl:text>updated|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="tc-param" match="prep:column/prep:name[.='state']">
        <xsl:text>state|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="tc-param" match="prep:column/prep:name[.='owner']">
        <xsl:text>owner|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="tc-param" match="prep:column/prep:name[.='originator']">
        <xsl:text>creator|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="tc-param" match="prep:column/prep:name[.='weight']">
        <xsl:text>weight|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="tc-param" match="prep:column/prep:name[.='calculatedRisk' or .='communityRisk' or .='systemRisk']">
        <xsl:text>risk|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="tc-param" match="prep:column/prep:name[.='suspect']">
        <xsl:text>suspect|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="tc-param" match="prep:column/prep:name[.='estimate']">
        <xsl:text>estimate|</xsl:text>
    </xsl:template>

    <xsl:template mode="tc-param" match="prep:column/prep:name[.='testScripts']">
        <xsl:text>testscript|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="tc-param" match="prep:column/prep:name[.='reqlLink']">
        <xsl:text>requirement|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="tc-param" match="prep:column/prep:name[.='devItemLink']">
        <xsl:text>workitem|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="tc-param" match="prep:column/prep:name[starts-with(., 'isCategoryColumn')]">
        <xsl:text>category|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="tc-param" match="prep:column/prep:name[starts-with(., 'isCustomColumn')]">
        <xsl:text>customAttributes|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="tc-param" match="prep:column/prep:name[.='copiedFrom']">
        <xsl:text>copiedFrom|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="tc-param" match="prep:column/prep:name[.='copiedRoot']">
        <xsl:text>copiedRoot|</xsl:text>
    </xsl:template>
    
    <!-- additional processing -->    
    <xsl:template mode="tc-copy" match="@*|node()">
        <xsl:param name="columns"/>
        <xsl:param name="tpHref"/>
        
        <xsl:copy>
            <xsl:apply-templates mode="tc-copy" select="@*|node()">
                <xsl:with-param name="columns" select="$columns"/>
                <xsl:with-param name="tpHref" select="$tpHref"/>
            </xsl:apply-templates>
        </xsl:copy>
    </xsl:template>
    
    <xsl:template mode="tc-copy" match="dc:creator|alm:owner">
        <xsl:apply-templates select="."/>
    </xsl:template>
    
    <xsl:template mode="tc-copy" match="rqm:testcase/rqm:testscript">
        <!-- we only need the test script's url and name -->
        <xsl:copy-of select="HttpClientExtension:get($restClient, @href, 'fields=feed/entry/content/testscript/(identifier|webId|title)')"/>
    </xsl:template>
    
    <xsl:template mode="tc-copy" match="rqm:testcase">
        <xsl:param name="columns"/>
        <xsl:param name="tpHref"/>
        
        <xsl:copy>
            <xsl:apply-templates mode="tc-copy" select="@*|node()"/>
            
            <xsl:if test="$tpHref">
                <prep:terCount>
                    <xsl:variable name="terFeed">
                        <xsl:variable name="tcParam">
                            <xsl:text>testcase[@href='</xsl:text>
                            <xsl:value-of select="utils:encodeUri(dc:identifier)"/>
                            <xsl:text>']</xsl:text>
                        </xsl:variable>
                        
                        <xsl:variable name="tpParam">
                            <xsl:text>testplan[@href='</xsl:text>
                            <xsl:value-of select="utils:encodeUri($tpHref)"/>
                            <xsl:text>']</xsl:text>
                        </xsl:variable>
                        
                        <xsl:call-template name="restGET">
                            <xsl:with-param name="path"  select="'../executionworkitem'"/>
                            <xsl:with-param name="query" select="concat('fields=feed/entry/content/executionworkitem/(testplan|testcase|', $tcParam, '|', $tpParam, ')')"/>
                        </xsl:call-template>
                    </xsl:variable>
                    
                    <!-- count them -->
                    <xsl:value-of select="count(exslt:node-set($terFeed)//rqm:executionworkitem)"/>
                </prep:terCount>
            </xsl:if>
        	
            <xsl:if test="$columns//prep:name[.='parentVTestPlans']">
                <!-- copies necessary information for test plans associated with this test case if needed -->
                <xsl:variable name="testPlanFeed">
                    <xsl:variable name="query">
                        <xsl:variable name="whereQuery">
                            <xsl:text>testcase/@href='</xsl:text>
                            <xsl:value-of select="utils:encodeUri(dc:identifier)"/>
                            <xsl:text>'</xsl:text>
                        </xsl:variable>
                        
                        <xsl:text>fields=feed/entry/content/testplan[</xsl:text>
                        <xsl:value-of select="$whereQuery"/>
                        <xsl:text>]/(identifier|webId|title)</xsl:text>
                    </xsl:variable>
                    
                    <xsl:call-template name="restGET">
                        <xsl:with-param name="path"  select="'../testplan'"/>
                        <xsl:with-param name="query" select="$query"/>
                    </xsl:call-template>
                </xsl:variable>
                
                <xsl:copy-of select="exslt:node-set($testPlanFeed)//rqm:testplan"/>
            </xsl:if>
        </xsl:copy>
    </xsl:template>
    
    <xsl:template mode="tc-copy" match="rqm:testcase/rqm:copiedFrom">
	     <!-- we only need the copied from test case's url and webId -->
	     <xsl:apply-templates mode="copiedFrom" select="HttpClientExtension:get($restClient, @href, 'fields=feed/entry/content/testcase/(identifier|webId)')"/>
    </xsl:template>
    
    <xsl:template mode="copiedFrom" match="rqm:testcase">
        <prep:copiedFrom>
        	<xsl:apply-templates select="dc:identifier"/>
            <xsl:apply-templates select="rqm:webId"/>
        </prep:copiedFrom>
    </xsl:template>
    
    <xsl:template mode="tc-copy" match="rqm:testcase/rqm:copiedRoot">
	     <!-- we only need the copied root test case's url and webId -->
	     <xsl:apply-templates mode="copiedRoot" select="HttpClientExtension:get($restClient, @href, 'fields=feed/entry/content/testcase/(identifier|webId)')"/>
    </xsl:template>
    
    <xsl:template mode="copiedRoot" match="rqm:testcase">
        <prep:copiedRoot>
        	<xsl:apply-templates select="dc:identifier"/>
            <xsl:apply-templates select="rqm:webId"/>
        </prep:copiedRoot>
    </xsl:template>
     
</xsl:stylesheet>