/*******************************************************************************
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2012, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
// NOTE: THIS IS A GENERATED FILE. DO NOT EDIT DIRECTLY!
dojo.provide("com.ibm.team.apt.web.ui.internal.widgets.TreeParser");

dojo.require("com.ibm.team.apt.api.common.ItemType");
dojo.require("jazz.ui.ModeledTree");

(function() {
var ItemType= com.ibm.team.apt.api.common.ItemType;
var ModeledTree= jazz.ui.ModeledTree;

dojo.declare("com.ibm.team.apt.web.ui.internal.widgets.TreeParser", null, {
	"-chains-": { constructor: "manual" },

	//fTree: null,

	//fDataStore: null,

	//fInput: null,

	//fFilterValue: null,

	//fTreeNodeParams: null,

	constructor: function(input, filterValue, treeNodeParams) {
		if (filterValue != null) {
			this.fFilterValue= filterValue;
		} else {
			this.fFilterValue= new RegExp("^");
		}
		this.fTreeNodeParams= treeNodeParams;
		this.parseContent(input);
	},

	/**
	 *  summary:
	 * 		Returns the current ModeledTree object.
	 *  description:
	 * 		Returns the jazz.ui.ModeledTree object that has been created by current TreeParser Object
	 * 		on which it is being called.
	 */
	getModeledTree: function() {
		return this.fTree;
	},

	/**
	 *  summary:
	 * 		Used by this class's constructor to generate the ModeledTree.
	 *  description:
	 * 		Generates the jazz.ui.ModeledTree object based on the data in 
	 * 		the DataModel given by the input parameter. It creates a data 
	 * 		store from the input, parses each data item in the ItemHierarchy 
	 * 		to create its respective tree nodes and build the Tree.
	 * @param input
	 * 		The input data required to generate the tree items present in IUIItemHierarchy object.
	 */
	parseContent: function(input) {
		this.fInput= input;
		this.fDataStore= this.createDataStore();
		if (this.fInput != null) {
			var $subject= this.fInput;
			var $length= $subject.length;
			for (var $count= 0; $count < $length; $count++){
				var itemHierarchy= $subject[$count];
				this.parseTreeRoots(itemHierarchy);
			}
			this.fTree= this.createTree();
		}
	},

	/**
	 *  summary:
	 * 		Used to parse the data to create the ModeledTree's root nodes.
	 *  description:
	 * 		Parses the data in the itemHierarchy object to identify the 
	 * 		root nodes and push them into the Data Store so that the 
	 * 		jazz.ui.ModeledTree can be created based on the data in 
	 * 		the Data Store.
	 * @param itemHierarchy
	 * 		The itemHierarchy data required to generate the tree's root nodes present in UIItemHierarchy object.
	 */
	parseTreeRoots: function(itemHierarchy) {
		if (itemHierarchy != null) {
			var items= itemHierarchy.getRoots();
			if (this.fTreeNodeParams.sort) {
				items= this.sort(items);
			}
			var $subject2= items;
			var $length2= $subject2.length;
			for (var $count2= 0; $count2 < $length2; $count2++){
				var item= $subject2[$count2];
				if (item != null) {
					if ((item.getItemType() === ItemType.Timeline) && (itemHierarchy.getChildren(item).length === 0)) {
						continue;
					}
					var rootNode= {};
					rootNode.children= this.createTreeNode();
					this.parseTreeChildren(itemHierarchy, item, rootNode);
					rootNode.id= item.getItemId();
					rootNode.label= jazz.util.html.escape(item.getLabel());
					if (item.getItemType() === ItemType.ProjectArea) {
						rootNode.iconClass= (!item.isArchived()) ? "projectImage" : "projectArchivedImage";
					} else if (item.getItemType() === ItemType.Timeline) {
						rootNode.iconClass= (!item.isArchived()) ? "timelineImage" : "timelineArchivedImage";
						rootNode.selectable= false;
						rootNode.label= "<span class=\"nonselectable\">" + rootNode.label + "</span>";
					}
					if (this.fFilterValue.test(rootNode.label.toLowerCase()) || rootNode.children.length > 0) 
						this.fDataStore.push(rootNode);
				}
			}
		}
	},

	/**
	 *  summary:
	 * 		Used to parse the data to create the ModeledTree's child nodes.
	 *  description:
	 * 		Parses the data in the itemHierarchy object to identify the 
	 * 		child nodes of a particular parent IUIItem and push them into
	 * 		an array used by the data store to locate its children.
	 * @param itemHierarchy
	 * 		The itemHierarchy data required to generate the root's children nodes present in UIItemHierarchy object.
	 * @param item
	 * 		The reference to the parent node's data item.
	 * @param parentNode
	 * 		The reference to the parent node.
	 */
	parseTreeChildren: function(itemHierarchy, item, parentNode) {
		var children= itemHierarchy.getChildren(item);
		if (this.fTreeNodeParams.sort) {
			children= this.sort(children);
		}
		var $subject3= children;
		var $length3= $subject3.length;
		for (var $count3= 0; $count3 < $length3; $count3++){
			var child= $subject3[$count3];
			if ((child != null) && (!child.isArchived())) {
				var childNode= {};
				childNode.children= this.createTreeNode();
				this.parseTreeChildren(itemHierarchy, child, childNode);
				childNode.id= child.getItemId();
				childNode.label= jazz.util.html.escape(child.getLabel());
				if (child.getItemType() === ItemType.TeamArea) {
					childNode.iconClass= (!child.isArchived()) ? "teamAreaImage" : "teamAreaArchivedImage";
				} else if (child.getItemType() === ItemType.Iteration) {
					if (this.excludeNonDeliverable(child)) {
						childNode.selectable= false;
						childNode.label= "<span class=\"nonselectable\">" + childNode.label + "</span>";
					}
					childNode.iconClass= (!child.isArchived()) ? "iterationImage" : "iterationArchivedImage";
				}
				if (this.fFilterValue.test(childNode.label.toLowerCase()) || childNode.children.length > 0) {
					parentNode.children.push(childNode);
				}
			}
		}
	},

	/**
	 *  summary:
	 * 		Returns the ModeledTree object using the data in the fDataStore object.
	 *  description:
	 * 		Generates and returns the ModeledTree object created using the data 
	 * 		in the fDataStore object.
	 */
	createTree: function() {
		var fTreeParams= {};
		fTreeParams.model= this.fDataStore;
		return (new ModeledTree(fTreeParams));
	},

	/**
	 *  summary:
	 * 		Returns an empty IconItem[] array used by fDataStore object.
	 *  description:
	 * 		Generates and returns an empty IconItem[] array used by 
	 * 		fDataStore object which is populated later and used to create 
	 * 		the ModeledTree.
	 */
	createDataStore: function() {
		var dataStore= [];
		return dataStore;
	},

	/**
	 *  summary:
	 * 		Returns an empty TreeNode[] array used to store the child nodes.
	 *  description:
	 * 		Generates and returns an empty TreeNode[] array used to store 
	 * 		the child nodes of their respective parent node.
	 */
	createTreeNode: function() {
		var newNode= [];
		return newNode;
	},

	excludeNonDeliverable: function(item) {
		if (this.fTreeNodeParams != null) {
			return ((item.getItemType() === ItemType.Iteration) && (this.fTreeNodeParams.excludeNonDeliverable === true) && (item.hasDeliverable() === false));
		}
		return false;
	},

	sort: function(items) {
		if (items == null || items.length < 1) {
			return items;
		}
		var sortedItems= items.sort(function(u1, u2) {
			return u1.getLabel().toLowerCase().localeCompare(u2.getLabel().toLowerCase());
		});
		return sortedItems;
	}
});

})();
