/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2014. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.cognos.web.ui.internal.NewTemplateDialog");

dojo.require("com.ibm.team.reports.cognos.web.client.internal.CognosReportsClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.reports.cognos.web.ui.internal.CognosReportsBrowseDialog");
dojo.require("com.ibm.team.reports.web.ui.internal.utils.StringUtils");
dojo.require("com.ibm.team.repository.web.client.session.Session");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Tree");
dojo.require("dojo.data.ItemFileReadStore");
dojo.require("jazz.app.context");
dojo.require("jazz.ui.Dialog");

dojo.requireLocalization("com.ibm.team.reports.cognos.web.ui.internal", "messages");

var CognosReportsClient = com.ibm.team.reports.cognos.web.client.internal.CognosReportsClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var NewTemplateDialog = com.ibm.team.reports.cognos.web.ui.internal.NewTemplateDialog;
var CognosReportsBrowseDialog = com.ibm.team.reports.cognos.web.ui.internal.CognosReportsBrowseDialog;
var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
var session = com.ibm.team.repository.web.client.session;
var StringUtils = com.ibm.team.reports.web.ui.internal.utils.StringUtils;	

var SHARED = 1 << 0;
var PRIVATE = 1 << 2;
var SYSTEM = 1 << 3;

(function() { // module container

/**
 * Displays a dialog to upload a Cognos report to RTC server
 */
dojo.declare(
	"com.ibm.team.reports.cognos.web.ui.internal.NewTemplateDialog",
	[dijit._Widget, dijit._Templated],	{
	
	templatePath: dojo.moduleUrl("com.ibm.team.reports.cognos.web", "ui/internal/templates/NewTemplateDialog.html"),
	
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.cognos.web.ui.internal", "messages");

		this._responseCount = 0;
		this._allFolders = [];
							
		// Strings for the dialog
		this.labelOK = this.messages.labelOK;
		this.labelCancel = this.messages.labelCancel;
		this.labelFolder = this.messages.labelFolder;
		this.labelBrowse = this.messages.labelBrowse;
		this.labelProperties = this.messages.labelProperties;
		this.labelName = this.messages.labelName;
		this.labelDescription = this.messages.labelDescription;
		this.labelShared = this.messages.labelShared;
		this.labelReportDetails = this.messages.labelReportDetails;
		this.labelCognosReport = this.messages.labelCognosReport;
	},
	
	uninitialize: function() {
		if (this.cognosReportsBrowseDialog){
			this.cognosReportsBrowseDialog.destroy();
		}
	},
	
	postCreate: function() {
		this.dialog = new jazz.ui.Dialog({
			contentNode: this.contentNode,
			primaryTitle: this.messages.labelImportReport,
			width: "390px",
			isClosable: true
		});
		this.connect(this._browseButton, "onclick", this._onBrowse);
		this.connect(this._okButton, "onclick", this._onOk);
		this.connect(this._cancelButton, "onclick", this._onCancel);

		this._createFolderTree();			

		var args = {
			ownerUUID: session.getAuthenticatedContributor().itemId,
			flags: PRIVATE
		};
		responseHandler = new ServiceResponseHandler(
				this,
				"_addFolders",
				"_FolderError");
		ReportsClient.getFolderDescriptors(responseHandler, args);

		args = {
			flags: (SHARED | SYSTEM)
		};
		responseHandler = new ServiceResponseHandler(
				this,
				"_addFolders",
				"_FolderError");
		ReportsClient.getFolderDescriptors(responseHandler, args);		
	},
	
	_onBrowse: function() {
	  var args = {
	  	viewer: this,
		elementsToBeDisplayed: "ReportTemplate"
	  }	;
	  var currentProjectArea = jazz.app.context.get();

	  this.cognosReportsBrowseDialog = new CognosReportsBrowseDialog(args);
	  this.connect(this.cognosReportsBrowseDialog._okButton, "onclick", this._onOkClick);	  
	},
	
	_onOkClick: function() {
		this.reportNode = this.cognosReportsBrowseDialog.node;
		this.reportPath = this.cognosReportsBrowseDialog.reportPath;
		this._IDInput.value = this.cognosReportsBrowseDialog.reportId;
		var fileName = this.reportNode.label;
		this._nameInput.value = fileName;
		this._reportInput.value = fileName;
		this._okButton.disabled = false;
		this.cognosReportsBrowseDialog.close();
	},
	
	_onCancel: function() {
		this.dialog.close();
	},
	
	_handleError: function(){
		alert(this.messages.errorCantCommunicate);
	},
	
	_Error: function(){
		alert(this.messages.errorCantCommunicate);
	},
	
	_FolderError: function(){
		alert(this.messages.errorCantCommunicate);
	},
	
	_onOk: function(){
		this.dialog.close();
	},
	
	_addFolders: function(folders) {
		this._responseCount++;
		if (!folders) folders = [];
		for (var i = 0; i < folders.length; i++) {
			this._allFolders.push(folders[i]);
		}
		//If we haven't handled both the shared and private folders return
		if (this._responseCount == 1) {
			return;
		}
		this._addFoldersToTree();			
	},

	_addFoldersToTree: function() {
		// First, find the two root folders
		var sharedRoot = null;
		var privateRoot = null;
		for (var i = 0; i < this._allFolders.length; i++) {
			var folder = this._allFolders[i];
			if (folder.system) {
				if (folder.shared) {
					sharedRoot = folder;
				} else {
					privateRoot = folder;
				}
			}
		}

		if (privateRoot) {
			var rootItem = new jazz.ui.tree.IconItem({label: privateRoot.name, iconClass: "iconFolder"});
			privateRoot.treeItemID = rootItem.getId();  //Save the item ID in the folders array in case we need to make tree calls based on the id
			rootItem.folderUUID = null;
			rootItem.isSharedRoot = false;
			this._folderTreeWidget.addChild(rootItem);
			this._addChildrenForFolder(rootItem, privateRoot, this._allFolders, true, false);
			rootItem.setExpanded(true);			
		}
		if (sharedRoot) {
			var rootItem = new jazz.ui.tree.IconItem({label: sharedRoot.name, iconClass: "iconFolder"});
			sharedRoot.treeItemID = rootItem.getId();  //Save the item ID in the folders array in case we need to make tree calls based on the id
			rootItem.folderUUID = null;
			rootItem.isSharedRoot = true;
			this._folderTreeWidget.addChild(rootItem);
			this._addChildrenForFolder(rootItem, sharedRoot, this._allFolders, true, true);
			this._folderTreeWidget.setSelection(rootItem);
			rootItem.setExpanded(true);			
		}
	},
	
	/**
	 * From the list of folders, find any that are children of parentFolder, and
	 * add them to the tree. Then recurse for each of the children.
	 */
	_addChildrenForFolder: function(parentFolderNode, parentFolder, folders, parentIsRoot, parentIsShared) {
		var foldersToAdd = [];
		for (var i = 0; i < folders.length; i++) {
			// Special case where the parent is a root system folder
			if (parentIsRoot) {
				if (!folders[i].parentUUID && (folders[i].shared == parentIsShared) && (folders[i].name != parentFolder.name)) {
					foldersToAdd.push(folders[i]);
				}
			} else {
				if (folders[i].parentUUID == parentFolder.folderUUID) {
					foldersToAdd.push(folders[i]);
				}
			}
		}
		foldersToAdd = foldersToAdd.sort(function(a, b) {return a.name > b.name;});
		for (var i = 0; i < foldersToAdd.length; i++) {
			var childNode = new jazz.ui.tree.IconItem({label: StringUtils.escapeAndCheckForNull(foldersToAdd[i].name), iconClass: "iconFolder"});
			childNode.folderUUID = foldersToAdd[i].folderUUID;
			foldersToAdd[i].treeItemID = childNode.getId();  //Save the item ID in the folders array in case we need to make tree calls based on the id
			parentFolderNode.addChild(childNode);
			this._addChildrenForFolder(childNode, foldersToAdd[i], folders, false, parentIsShared);
		}
	},

	_createFolderTree: function() {
		this._folderTreeWidget = new jazz.ui.Tree();
		this._folderTree.appendChild(this._folderTreeWidget.domNode);
	}
		
});

})();