/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.process.ui.internal.view.type.AddAttributeContent");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dojo.data.util.sorter");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.ButtonDialog");
dojo.require("com.ibm.team.workitem.web.process.ui.internal.view.common.utils");
dojo.require("com.ibm.team.workitem.web.model.types.AttributeTypes");
dojo.requireLocalization("com.ibm.team.workitem.web.process.ui.internal", "TypeMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

var messages;
var comparator = dojo.data.util.sorter.basicComparator;
var AttributeTypes= com.ibm.team.workitem.web.model.types.AttributeTypes;
var utils = com.ibm.team.workitem.web.process.ui.internal.view.common.utils;

dojo.declare(
	"com.ibm.team.workitem.web.process.ui.internal.view.type.AddAttributeContent",
	[dijit._Widget, dijit._Templated],
{
	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web.process", "ui/internal/view/type/templates/AddAttributeContent.html"),
	
	name: "",
	attrId: "",
	type: "",
	model: null,
	workItemType: null,
	_existingAttributes: null,

	postMixInProperties: function(){
		messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.process.ui.internal", "TypeMessages");
		this.messages = messages;
		this.inherited(arguments);
	},
	
	postCreate: function(){
		this.inherited(arguments);
		
		// set "id" & "type" fields to readonly for edit attributes
		if(this.attrId && this.attrId.length>0 && this.type && this.type.length>0){
			this.idField.readOnly = true;
			this.typeSelect.disabled = true;
		}

		this.createRadio.checked = true;
		this._setCreateAttributeMode(this._isCreateMode());
		 
		// populate attribute types dropdown
		var sel = this.typeSelect;
		sel.innerHTML = "";
		var selectedType = this.type;
		this.model.getAttributeTypes(function(configData){
			var enumerationTypes = configData.enumerationTypes || [];
			var types = configData.basicTypes.concat(enumerationTypes);
			
			dojo.forEach(enumerationTypes, function(enumType){
				var typeId= AttributeTypes.getContainerType(enumType.id);
				var typeDisplayName= dojo.string.substitute(messages.EnumListType, [enumType.displayName || enumType.id]);
				var containsId= dojo.some(types, function(type){
					return type.id === typeId;
				});
				if (!containsId) {
					types.push({id: typeId, displayName: typeDisplayName});					
				}
			});
			
			types.sort(function(o1, o2){
				// 111126: Script error occurs when adding custom attributes
				// displayName may be null
				var name1 = (o1.displayName) ? o1.displayName : o1.id;
				var name2 = (o2.displayName) ? o2.displayName : o2.id;
				return comparator(name1.toLowerCase(), name2.toLowerCase());
			});
			dojo.forEach(types, function(attrType){
				var displayName = attrType.displayName || attrType.id;
				var opt = dojo.create("option", {
					value: attrType.id,
					innerHTML: displayName
				}, sel);
				
				if(selectedType && selectedType.length > 0){
					if(attrType.id == selectedType){
						opt.selected = true;
					}
				}else if(attrType.id == "string"){
					// make string default
					opt.selected = true;
				}
			});
		});

		var self= this;
		this._existingAttributes= [];
		var attributeSelect= this.attributeSelect;
		var allattributes= this.model.getAllCustomAttributes();
		allattributes= allattributes.concat(this.model.getArchivedAttributes());
		allattributes.sort(function(o1, o2) {
			return comparator(o1.name.toLowerCase(), o2.name.toLowerCase());
		});

		var workItemTypeCustomAttributes= this.model.getCustomAttributes(this.workItemType);
   		customAttributes= {};
   		dojo.forEach(workItemTypeCustomAttributes, function(att) {
            customAttributes[att.id]= true;
        });
        
   		dojo.forEach(allattributes, function(attrType) {
            if (!customAttributes[attrType.id]) {
               var opt = dojo.create("option", {
               value: attrType.id
               }, attributeSelect);
               opt.innerHTML= dojo.string.substitute(messages.AttributeFormat, [attrType.name, attrType.typeName]);
               self._existingAttributes.push(attrType);
            }
        });
	},

	_isCreateMode: function() {
		return this.createRadio.checked;
	},

	_handleRadioChanged: function(e) {
		this._setCreateAttributeMode(this._isCreateMode());
		dijit.focus(e.target);
	},

	_handleNameChanged: function(e) {
		var id = this.idField.value;
		if(!id || id.length === 0){
			this.idField.value = this.nameField.value;
		}
	},

	_setCreateAttributeMode: function(isCreate) {
		this.nameField.disabled= !isCreate;
		this.idField.disabled= !isCreate;
		this.typeSelect.disabled= !isCreate;
		this.attributeSelect.disabled= isCreate;
		if (isCreate) {
			dojo.removeClass(this.createGroup, "disabled");
			dojo.addClass(this.reuseGroup, "disabled");
		} else {
			dojo.removeClass(this.reuseGroup, "disabled");
			dojo.addClass(this.createGroup, "disabled");
		}
	},

	validate: function(values, callback){
		if (!values.name || values.name.length === 0) {
			alert(messages.errAttrNameEmpty);
		} else if (!values.typeId || values.typeId.length === 0) {
			alert(messages.errAttrTypeEmpty);
		} else if (!values.id || values.id.length === 0) {
			alert(messages.errIDEmpty);
		} else if (!utils.isValidIdentifier(values.id)) {
			alert(messages.errIDHasInvalidCharacters);
		} else {
			var id = values.id;
			if(!id || id.length === 0){
				alert(messages.errAttrIDEmpty);
			}else{
				var isAttrIDUsed = function(attr){
					return (
						(attr.id == id) ||
						(attr.stringIdentifier == id)
					);
				};
				
				// verify that the attribute id doesn't collide with an permission action name
				var isActionID = function(attr){
					var intPrefix= "internal";
					var prefixLength= intPrefix.length;
					var attrId= attr.id;
					if (attrId && attrId.length > prefixLength && attrId.substring(0, prefixLength) == intPrefix) {
						var actionName= attrId.substring(prefixLength, prefixLength + 1).toLowerCase() + attrId.substring(prefixLength + 1);
						return id == actionName;
					}
					return false;	
				};
				
				var isCreateMode= this._isCreateMode();
				// 111584: Creation of identical custom attributes should not be allowed
				this.model.getWorkItemConfig(dojo.hitch(this, function(workItemConfig){
					// check if ID is used in built-in attributes
					var isBuiltInUsed = dojo.some(workItemConfig.builtInAttributes, isAttrIDUsed);
					if(isBuiltInUsed === true){
						alert(messages.errAttrIDUsed);
						return;
					}

					if (isCreateMode) {
						var isActionIdUsed = dojo.some(workItemConfig.builtInAttributes, isActionID);
						if (isActionIdUsed === true){
							alert(messages.errAttrIDUsed);
							return;
						}
					
						// check if ID is used in custom attributes
						var custAttrs= this.model.getAllCustomAttributes();
						if (custAttrs && dojo.some(custAttrs, isAttrIDUsed)) {
							alert(messages.errAttrIDUsed);
							return;
						}
						
						// check if ID is used in archived attributes
						var archivedAttrs= this.model.getArchivedAttributes();
						if (archivedAttrs && dojo.some(archivedAttrs, isAttrIDUsed)) {
							alert(messages.errAttrIDUsed);
							return;
						}
						
						// check if ID is used in synthetic attributes
						var reservedAttributeNames= this.model.getReservedAttributeNames();
						if (reservedAttributeNames && dojo.some(reservedAttributeNames, function(attrIdentifier){return attrIdentifier == id})) {
							alert(messages.errSynAttrIDUsed);
							return;
						}
					}
					// ID is not used, so call callback
					callback();
				}));
			}
		}
	},
	
	getValues: function() {
		if (this._isCreateMode()) {
			var sel = this.typeSelect;
			var opt = sel.options[sel.selectedIndex];
			return {
				id: this.idField.value,
				name: this.nameField.value,
				typeId: opt.value,
				typeName: opt.innerHTML
			};
		}
		return this._existingAttributes[this.attributeSelect.selectedIndex];
	}
});

com.ibm.team.workitem.web.process.ui.internal.view.type.AddAttributeContent.openDialog = function(prop){
	var model = prop.model;
	
	// create dialog content
	var attrId = prop.attrId;
	var contentWidget = new com.ibm.team.workitem.web.process.ui.internal.view.type.AddAttributeContent({
		name: prop.name,
		attrId: attrId,
		type: prop.type,
		model: model,
		workItemType: prop.workItemType
	});
	
	// show dialog
	new com.ibm.team.workitem.web.process.ui.internal.view.common.ButtonDialog({
		primaryTitle: prop.title,
		dialogContent: contentWidget,
		onOpen: function(){
			dijit.focus(contentWidget.nameField);
		},
		validate: dojo.hitch(contentWidget, "validate"),
		onOK: function(values){
			model.getAttributeTypes(function(configData){
				var attrTypeId = values.typeId;
				var enumerationTypes = configData.enumerationTypes || [];
				var isEnum = dojo.some(enumerationTypes, function(enumType){
					return (enumType.id == attrTypeId);
				});
				model.addCustomAttribute(prop.workItemType, {
					id: values.id,
					name: values.name,
					typeId: attrTypeId,
					typeName: values.typeName,
					isEnumeration: isEnum
				});
			});
		}
	});
};

})();