 @process override env(z/OS) machine(zSeries) optimize(ADV,64)
                   storemotion inline(MAX) assert(blockreorder)
          TITLE
 ('DFSYSCP0 - IMS OTMA SYNCHRONOUS CALLOUT PROCESSOR - ')
 EXTEND(1200K) BUFSIZE(40K) ;
 ! MACSOURCE   ;

                                                             @SPACE(1);
 /****************************************************@SCPYRT*********/
 /*                                                                  */
 /*    IBM Confidential                                              */
 /*                                                                  */
 /*    OCO Source Materials                                          */
 /*                                                                  */
 /*    5635-A04                                                      */
 /*                                                                  */
 /*        Copyright IBM Corp. 1991,2008                             */
 /*                                                                  */
 /*    The source code for this program is not published or other-   */
 /*    wise divested of its trade secrets, irrespective of what has  */
 /*    been deposited with the U.S. Copyright Office.                */
 /*                                                                  */
 /****************************************************@ECPYRT*********/
 /*******START OF SPECIFICATIONS***************************************/
 /*                                                                   */
 /* MODULE NAME:  DFSYSCP0                                            */
 /*                                                                   */
 /* DESCRIPTIVE NAME: IMS OTMA SYNCHRONOUS CALLOUT PROCESSOR          */
 /*                                                                   */
 /*                                                                   */
 /* STATUS: RELEASE 10    CHANGE LEVEL - 0                            */
 /*                                                                   */
 /* FUNCTION: PROCESS THE SYNCHRONOUS CALLOUT REQUEST AND PASS        */
 /*           THE RESPONSE MESSAGE BACK TO THE CALLER                 */
 /*                                                                   */
 /* NOTES:                                                            */
 /*                                                                   */
 /* DEPENDENCIES: IMS/ESA 10                                          */
 /*                                                                   */
 /* RESTRICTIONS: NONE                                                */
 /*                                                                   */
 /* REGISTER CONVENTIONS: CODE REGISTERS: GPR12, GPR10                */
 /*                       DATA REGISTERS: GPR13                       */
 /*                       LINK REGISTERS: GPR01, GPR14, GPR15         */
 /*                                                                   */
 /* PATCH_LABEL: PL/X                                                 */
 /*                                                                   */
 /* MODULE TYPE: PROCEDURE                                            */
 /*                                                                   */
 /* PROCESSOR: PL/X                                                   */
 /*                                                                   */
 /* MODULE SIZE: SEE ASSEMBLE LISTING                                 */
 /*                                                                   */
 /* ATTRIBUTES: REENTRANT                                             */
 /*                                                                   */
 /* ENTRY POINT: DFSYSCP0                                             */
 /*                                                                   */
 /* PURPOSE: OTMA SYNCHRONOUS CALLOUT PROCESSOR                       */
 /*                                                                   */
 /*                                                                   */
 /* LINKAGE: DFSYSCM  (INTERFACE MACRO FOR DFSYSCP0)                  */
 /*                                                                   */
 /* CALLED FROM:                                                      */
 /*                                                                   */
 /* INPUT: REGISTER 1 MUST POINT TO A PARAMETER LIST (YSCMPRM),       */
 /*        THE FIRST 2 WORDS IN THE LIST ARE THE ECB AND SCD          */
 /*        ADDRESSES.  SEE DFSYSCDS COPY FOR THE COMPLETE FORMAT.     */
 /*                                                                   */
 /* OUTPUT:                                                           */
 /*        R15 = RETURN CODE                                          */
 /*                                                                   */
 /*   SENDRECV CALL:                                                  */
 /*                                                                   */
 /*    R15 = 0    FUNCTION COMPLETED SUCCESSFULLY                     */
 /*    R15 = 4    FUNCTION COMPLETED WITH ERROR INFORMATION IN THE    */
 /*               RESPONSE AREA                                       */
 /*                                                                   */
 /*    R15 = 8    INVALID INPUT DESCRIPTOR                            */
 /*    R15 = 12   INVALID TIMEOUT VALUE                               */
 /*    R15 = 16   TIMEOUT OCCURRED                                    */
 /*    R15 = 20   OTMA FAILED TO PROCESS OR DELIVER THE SYNCHRONOUS   */
 /*               CALLOUT MESSAGE                                     */
 /*    R15 = 24   OTMA CLIENT REJECTED THE CALLOUT MESSAGE            */
 /*    R15 = 28   OTMA FAILED TO PROCESS THE RESPONSE OF THE          */
 /*               SYNCHRONOUS CALLOUT MESSAGE                         */
 /*                                                                   */
 /*    R15 = 32   SYNCHRONOUS CALLOUT REQUEST CANCELLED BY COMMANDS,  */
 /*               SUCH AS /PSTOP OR /STOP TMEMBER TPIPE COMMANDS      */
 /*                                                                   */
 /*    R15 = 36   Error is reported by an OTMA client, such as ICON,  */
 /*               and the response data is not returned.              */
 /*    R15 = 40   Error is reported by an OTMA client, such as ICON,  */
 /*               and the raw data in the response buffer is returned.*/
 /*                                                                   */
 /*    R15 = 44   Synchronous program switch specific error code      */
 /*               This will result in  RC=X'100', RSN=X'110', and     */
 /*               the extended reason code pointed by YSCMRSN.        */
 /*                                                                   */
 /*                                                                   */
 /*   RELSTG CALL:                                                    */
 /*                                                                   */
 /*    R15 = 0    FUNCTION COMPLETED SUCCESSFULLY                     */
 /*    R15 = 8    FUNCTION FAILED                                     */
 /*                                                                   */
 /*                                                                   */
 /* EXIT-NORMAL: RETURN TO CALLER WITH R15 CONTAINING RETURN CODE     */
 /*              (SEE OUTPUT ABOVE)                                   */
 /*                                                                   */
 /* EXIT-ERROR: NONE                                                  */
 /*                                                                   */
 /* EXTERNAL REFERENCES -                                             */
 /*                                                                   */
 /*    ROUTINES:                                                      */
 /*                                                                   */
 /*    DATA-AREAS:                                                    */
 /*                                                                   */
 /*    CONTROL-BLOCKS:                                                */
 /*                                                                   */
 /*    TABLES:                                                        */
 /*                                                                   */
 /*    COPIES:                                                        */
 /*                                                                   */
 /*    MACROS:                                                        */
 /*                                                                   */
 /* MESSAGES:                                                         */
 /*                                                                   */
 /* CHANGE ACTIVITY:                                                  */
 /*                                                                   */
 /*   DATE   CODER REL  REASON                                APAR#   */
 /* -------- ----- ---- ------------------------------------- ------- */
 /* 02/29/08  JCY  10   Creation                              PK70078 */
 /* 10/11/08  JCY  10   Add large message support             PK71135 */
 /* 09/25/08  DEO  11   Add IMS Monitor calls (SLOG 78 and 79)PK71135 */
 /* 03/06/09  JCY  11   Use V11 descriptor search method      PK81543 */
 /* 10/06/09  JCY  11   Add LCRETOKN into 6701 and 67D0       PK91373 */
 /* 03/18/10  JCY  11   Increment TPIPE_SYNC_OUT_CT           PM12142 */
 /* 08/24/10  MRA  12   Reset PSTSYQAB                        PM27812 */
 /* 08/10/10  JCY  11   Report new errors for ICAL            PM29046 */
 /* 06/15/11  JCY  12   Recompile for AWE size change         PM39562 */
 /* 02/28/12  BRM  12   Add debugging                         PM33681 */
 /* 02/21/11  RS   13   Add security check to sync. ICAL      U13CRS2 */
 /* 02/21/11  JCY  13   Add message processing for sync ICAL  U13CJY2 */
 /* 06/10/11  RS   13   Check for security PROFILE for APPC   KD30003 */
 /* 08/29/11  RS   13   Set PSTSYNO and PSTSYQAB for sy. ICAL KD30016 */
 /* 02/29/12  DC   13   SPS Shrq                              U13CDC8 */
 /* 05/15/12  DC   13   SPS MINVERS                           U13CDCA */
 /* 10/02/12  JCY  13   Add mapname support from PM73135      KD20029 */
 /* 06/10/13  JCY  13   Enhance get trancode processing       PM90557 */
 /* 05/08/13  JCY  13   Add mapname for SPS and bad RC for MAXTPM78756*/
 /* ------------------ V14 -----------------------------------------  */
 /* 11/07/12  DC   14   Control Data                          U14CDC1 */
 /* 12/12/12  DC   14   Remove testing message                U14CDCB */
 /* 04/24/14  AJ   14   IMSMON exit support in DFSSLOG calls  U14JAJ1 */
 /*                                                                   */
 /*******END OF SPECIFICATIONS*****************************************/

 ?DFSYPROC MODULE(DFSYSCP0)  COPYRIGHTYEAR(1991,2008)
 TYPE(OTM)
 ERRNUM(16)
 CODEREG(12)
 STATREG(8,7)
 POOL(LUMC)
 INT(OPTIONS(CODEREG(10)))
 APAR(U14CDC1)
 TRACE(YES)
 ID(9);

 /*-------------------------------------------------------------------*/
 /*  DECLARE SYSTEM VARIABLES, REGISTERS AND CONTROL BLOCKS.          */
 /*-------------------------------------------------------------------*/
 %Include Syslib(Dfsytmab);              /* General OTMA vars/blocks  */

 %V='BASED(PARM_PLAS)';
 %Include Syslib(DFSYSCDS);              /* Input parameter list      */
 %Include Syslib(Dfsypre);               /* LUP  message prefix       */
 %Include Syslib(Dfsymsg);               /* OTMA prefix               */
 %INCLUDE SYSLIB(DFSBCB) ;
 %Include Syslib(Dfsrsrc);               /* Global constants          */
 %Include Syslib(Dfsdawe);               /* Awe dsect                 */
 %Include Syslib(Dfsyawe);
 %Include Syslib(Dfsclmd);               /* Dfsclm interface mapping  */
 %Include Syslib(Dfsslxd);               /* Dfsslx interface mapping  */
 %INCLUDE SYSLIB(IXCYMEPL) ;
 %INCLUDE SYSLIB(IXCYGEPL) ;
 %INCLUDE SYSLIB(DFSYMSGH);
 %include syslib(IMODULED);
 %Include Syslib(DFSP62XT);              !* APPC/OTMA PST extension
 ?DFSRAC6 FUNC(DSECT) PLX(YES);          !* DECLARE RAC6 STRUCTURE
 ?DFSYRAC FUNC(DSECT)                    !* DECLARE OTMA RACF STRUCTURE
          PLX(YES)
          BASED(PSTLOGWA)
          SCD(ADDR(SCD_PLAS)) ;
 %INCLUDE SYSLIB(LCDSECT);                                    !@U14JAJ1

 ?Dfssysrc;                              /* Symbolic return codes     */

 %include syslib(ICLI);                  ! CLB, CTB, CNT, CCB ...
 Rfy CLB based(CLB_Ptr);

 %include syslib(DFSLCRE);               ! LCRE
 Rfy DFSLCRE BASED(PSTCRE);              ! Get LCRE address

 %V = 'AUTOMATIC';                       /* Stat area in out storage  */
 %INCLUDE SYSLIB(DFSTRAN);               /* Get FINDDEST STATAREA Map */


 %V='AUTOMATIC';                         /* No Statarea Stg Needed    */
 %Include Syslib(Dfsydstc);              /* Get 6.2 Finddest Mappings */

 ?ZDFSFNDS FUNC(DSECT);                  /* DFSFNDST Parm list        */

 %Include Syslib(Dfsymte);               /* MTE dsect                 */
 Respecify (Mte) Based(Mte_Ptr);

 %Include Syslib(Dfsymcb);               /* MCB dsect                 */
 Respecify (Mcb) Based(Mtectlbk);

 %Include Syslib(Dfsydes);               /* OTMA Descriptor dsect     */

 %V='Based(CLBlu6wa)';
 %Include Syslib(Dfsyqab);               /* Yqab dsect                */

 %V='Based(Yqab_Tpipe_Ptr)';
 %Include Syslib(Dfsypip);               /* Tpipe dsect               */

 %Include Syslib(Dfsusrxd);
 !Include Syslib(Isap);
 %Include Syslib(Dfsssvpl);

 %V = 'BASED(CLBLU6WA)';
 %INCLUDE SYSLIB(DFSYTIB);

 %INCLUDE SYSLIB(IXCYMEPL) ;             /* XCF msg exit parm list    */
 %INCLUDE SYSLIB(IXCYGEPL) ;             /* XCF group exit parm list  */

 %V='BASED(SCDLSCD)';
 %include syslib(DFSLSCD);          !   LSCD dsect

 ?DFSRCWKB;                         !   DBRC Work block        @U13CDCA
 rfy DFSRCWKB BASED(SCDDBRWB);      !   address it             @U13CDCA

 ?Zdfsuse;
 Gen Data Refs(SCDEPFLN);
         IDSPWRK
 @Endgen;

 ?DFSCMDKW;

 /*-------------------------------------------------------------------*/
 /* Declare module input parameters.                                  */
 /*-------------------------------------------------------------------*/
 Declare
    Dest_Name            Char(8)    Based(YSCMDESC),
    In_Timeout           Fixed(31)  Based(YSCMTO),
    Req_Data_Len         Fixed(32)  Based(YSCMALEN),
    Ctl_Data_Len         Fixed(32)  Based(YSCMCLEN),          !@U14CDC1
    Oause                Fixed(32)  Based(YSCMAUSE),
    Reqarea              Char(*)    Based(YSCMREQA),
    RspOTMA              Ptr(31)    Based(YSCMRSPO),
    User_Id              Char(8)    Based(YSCMUSID),
    EXT_Addr             Ptr(31)    Based(YSCMRET),
    User_Data            Fixed(32)  Based(YSCMUDA),
    Lterm_Override       Char(8)    Based(YSCMLTRM),
    Map_Name             Char(8)    Based(YSCMMAPN),          !@PM73135
    Clt_Code             Fixed(32)  Based(YSCMRSN);


 /*-------------------------------------------------------------------*/
 /* Declare miscellaneous variables.                                  */
 /*-------------------------------------------------------------------*/
 Declare
    Mte_Ptr              Ptr(31) Sinit(0),! Addr(Member Table Entry)
    Tpipe_Ptr            Ptr(31),        /* Addr(Tpipe)               */
    Yqab_Ptr             Ptr(31),        /* Addr(Otma_Yqab)           */
    Clb_Ptr              Ptr(31),
    Ctb_Ptr              Ptr(31),
    Yqab_Cnt_Ptr         Ptr(31),
    Prefix_Length        Fixed(31),      !  OTMA prefix length
    Member_Name          Char(16),       /* Destination member        */
    Super_Member_Name    Char(16);       /* Super member name         */

  Dcl CtlData_num_segs   Fixed(8);       ! # Control Data segs @U14CDC1
  Dcl DRDesc_Ptr         Ptr(31);        !  Dest Routing Descriptor
  Dcl Timer_Token Fixed(32) Sinit(0);    ! Timer token for DFSTIMER

  Dcl dfspool_parmlist char(48) bdy(word); ! For DFSPOOL
  Dcl Resp_Buffer_Len  fixed(31);
  Dcl Resp_Buffer_Ptr  Ptr(31);
  Dcl Resp_Buffer      Char(*) based(Resp_Buffer_Ptr);

  Dcl Input_Timeout    Fixed(31);        ! Validated T/O from DFSYSCM
  Dcl Sync_Timeout     Fixed(31);        ! Final T/O for syncallout
  Dcl Sync_Tpipe       Char(8);          ! Final TPname for syncallout
  Dcl Sync_AWE_Ptr     Ptr(31);          ! AWE pointer for syncallout
  Dcl Seg_Synonym_Ptr  Ptr(31);          ! Point to a synonym block
  Dcl ReqArea_LL       Fixed(15)         ! Length of area if sync-p2p
                       Based(YSCMREQA);

  Dcl SPS_Ytib_Clb     Ptr(31);          ! Point to YTIB CLB
  Dcl SPS_Ytib_Addr    Ptr(31) Sinit(0); ! Point to YTIB

  DCL SMB_Ptr     Ptr(31);              ! SMB pointer (tran)
  DCL TranCode    Char(8);              ! Transaction Code
  DCL Userid_Ptr  Ptr(31) Sinit(0);     ! Ptr to ACEE

  ! My_Area defines the OTMA prefix with 128 bytes of user data



  Dcl 1 Local_Flags1       Bit(32),
        2 DestRoutDesc_fnd   Bit(1),     ! Descriptor entry found
        2 DestRoutDesc_used  Bit(1),     ! Descriptor entry is used
        2 DestRoutDesc_smem  Bit(1),     ! Descriptor entry w/ smem
        2 DestRoutDesc_icon  Bit(1),     ! Type IMSCON is found
        2 DestRoutDesc_SPS   Bit(1),     ! Type IMSTRAN is found
        2 ACEE_pool_used     Bit(1),     ! ACEE is from pool
        2 Control_Data       Bit(1),     ! Control Data passed @U14CDC1
        2 *                  Bit(25);    ! Reserved            @U14CDC1

  Dcl 1 Local_Flags2       Bit(8),
        2 Ical_timeout       Bit(1),     ! Timeout occurred
        2 Ical_sps_reroute   Bit(1),     ! reroute tmem/tpipe specified
        2 Ical_cmd           Bit(1),     ! Command ended the ICAL
        2 Ical_CM0_resp      Bit(1),     ! CM0 output
        2 Ical_free_ytib     Bit(1),     ! Free YTIB when REPLYCHK=NO
        2 SPS_Replychk       Bit(1),     ! Replychk = YES
        2 *                  Bit(1),     ! Reserved
        2 *                  Bit(1);     ! Reserved



 /*-------------------------------------------------------------------*/
 /* Declare input message prefix ptrs, and establish addressability.  */
 /*-------------------------------------------------------------------*/
 Declare
    Tmamctl_Ptr          Ptr(31),
    Tmamhdr_Ptr          Ptr(31),
    Tmamhdr_Txn_Ptr      Ptr(31),
    Tmamsec_Ptr          Ptr(31),
    Tmamsuds_Ptr         Ptr(31),
    Tmamsgds_Ptr         Ptr(31),
    Tmamusr_Ptr          Ptr(31) Sinit(0),
    Tmamapp_Ptr          Ptr(31); ! points to LLZZ

 Respecify (Tmamctl) Based(Tmamctl_Ptr);
 Respecify (Tmamhdr) Based(Tmamhdr_Ptr);
 Respecify (Tmamhdr_Txn) Based(Tmamhdr_Txn_Ptr);
 Respecify (Tmamsec) Based(Tmamsec_Ptr);
 Respecify (Tmamsuds) Based(Tmamsuds_Ptr);
 Respecify (Tmamsgds) Based(Tmamsgds_Ptr);
 Respecify (Tmamusrx) Based(Tmamusr_Ptr);
 Respecify (Tmamapp)  Based(Tmamapp_Ptr);
 Respecify (YDRDESC)  Based(DRDesc_Ptr);
 Respecify (Awe)      Based(Sync_AWE_Ptr);
 Respecify (YSYNSEGM) Based(Seg_Synonym_Ptr);
 Rfy       (PST)      Based(Addr(ECB_PLAS)) ;

 /*-------------------------------------------------------------------*/
 /* Declare external routines                                         */
 /*-------------------------------------------------------------------*/

 dcl   DFSYSTO0   Entry  Based(TSCD_MOD_YSTO0) ValueRange(*) ;

 dcl
   dfsycmr0   entry(char(*),
                    char(*),
                    char(*)) based(tscd_mod_ycmr0)
                             valuerange(*);                  !@PK81543
 Dcl DFSDAUT0   Entry Based(Scddaut0) Valuerange(*);


 /*************************************************@PK81543**/
 /*     Parameter List for dfsycmr0                         */
 /***********************************************************/
    ?dfsycmr;                                                !@PK81543



 /*-------------------------------------------------------------------*/
 /* SUBROUTINE RETURN CODES                                           */
 /*-------------------------------------------------------------------*/
 Declare
    Local_RC             Fixed(31);      /* RC from called procedures */

 /*-------------------------------------------------------------------*/
 /* MODULE RETURN CODES                                               */
 /*-------------------------------------------------------------------*/

 ! ERROR CODE FOR STORAGE RELEASE request

 Declare
   Err_In_STG_FREE       Constant(4)      Fixed(31) ;

 Declare
    RC                   Fixed(31);      /* Module Return Code        */

 ! ERROR CODE FOR SEND/RECEIVE request

 Declare
   Err_In_Response_Data  Constant(4)      Fixed(31) ,
   Err_Input_Descriptor  Constant(8)      Fixed(31) ,
   Err_Input_Timeout     Constant(12)     Fixed(31) ,
   Err_Got_Timeout       Constant(16)     Fixed(31) ,
   Err_in_OTMA_Callout   Constant(20)     Fixed(31) ,
   Err_Client_Reject     Constant(24)     Fixed(31) ,
   Err_in_OTMA_Response  Constant(28)     Fixed(31) ,
   Err_Got_STOP_CMD      Constant(32)     Fixed(31) ,
   Err_found_by_client   Constant(36)     Fixed(31) ,    !@PM29046
   Err_found_by_client_W_data Constant(40) Fixed(31) ,   !@PM29046
   Err_Sync_P2P_Specific Constant(44)     Fixed(31) ,         !@U14CDC1
   Err_Sync_Control_Data Constant(48)     Fixed(31) ;         !@U14CDC1

 ! Severe internal error codes
 Declare
   Err_YQAB_Callout      Constant(4)      Fixed(31) ;

 Declare
   Dummy_char16          Constant('                ') Char(16),
   Dummy_char8           Constant('        ')         Char(8),
   Dummy_char4           Constant('    ')             Char(4),
   Max_Timeout_Value     Constant(999999) Fixed(31) ,
   ICON_UserDate_Length  Constant(256)    Fixed(16) ,
   Sys_Default_Timeout   Constant(1000)   Fixed(31) ;

 ! Define the extended reason code for RC X'0100' & RSN X'0110' for
 ! synchronous program switch
 Declare
   Err_Destination            Fixed(31) Constant(00X),
   Err_Security               Fixed(31) Constant(04X),
   Err_Member_Stopped         Fixed(31) Constant(05X),
   Err_Tpipe_Stopped          Fixed(31) Constant(06X),
   Err_FindDest_failed        Fixed(31) Constant(08X),
   Err_DFSMSCE0_rejected      Fixed(31) Constant(0CX),
   Err_No_DFSYTIB             Fixed(31) Constant(0DX),
   Err_Fail_To_Post_YTIB      Fixed(31) Constant(0EX),
   Err_SPS_Bad_Tmem           Fixed(31) Constant(11X),
   Err_SPS_Missing_Tmem_Tpipe Fixed(31) Constant(12X),
   Err_SPS_Bad_Smem_Synctp    Fixed(31) Constant(13X),
   Err_SPS_Bad_Tpipe          Fixed(31) Constant(14X),
   Err_SPS_MINVERS            Fixed(31) Constant(15X),        !@U13CDCA
   Err_TIB_Flood              Fixed(31) Constant(16X),        !@UP13CJY
   Err_SPS_Data_Length_Error  Fixed(31) Constant(20X);


 ! Define the constant for extended error code for SPS timeout
 Declare
   SPS_Timeout                Fixed(31) Constant(20X);


 ! Define the extended error codes for Err_in_OTMA_Callout
 Declare
   Err_REQ_IMS_Down           Fixed(31) Constant(04X),
   Err_REQ_ICAL_Disabled      Fixed(31) Constant(08X),
   Err_REQ_Bad_Member         Fixed(31) Constant(0CX),
   Err_REQ_Bad_Tpipe          Fixed(31) Constant(10X),
   Err_REQ_No_AWE             Fixed(31) Constant(14X),
   Err_REQ_No_Lump            Fixed(31) Constant(18X),
   Err_REQ_Bad_Ipost          Fixed(31) Constant(1CX),
   Err_REQ_No_Hiop            Fixed(31) Constant(20X);

 ! Extended error codes for Err_Sync_Control_Data (104/102C)  !@U14CDC1
 Declare                                                      !@U14CDC1
   Err_CTLDATA_Not_ICON       Fixed(31) Constant(08X),        !@U14CDC1
   Err_CTLDATA_Bad_RTP        Fixed(31) Constant(0CX),        !@U14CDC1
   Err_CTLDATA_Bad_Length     Fixed(31) Constant(10X),        !@U14CDC1
   Err_CTLDATA_Bad_Tag        Fixed(31) Constant(14X);        !@U14CDC1

 /*********************************************************************/
 /* Subroutine - Check_System_Status                                  */
 /*    Check if system is capable of processing this request          */
 /*********************************************************************/
 Check_System_Status: Procedure
               Internal_Proc_Opt;
  Declare
     Local_RC           Fixed(31);

  Local_RC = 0; ! Pre-set a good return code

  /* Check whether the system shutdown is in progress */

  If ((SCDCQFRE | SCDCQDUQ) | (SCDSTINP & SCDSTOUT)) Then Do ;

    ! Cut a 67D0 log record

    Call Cut_OTMA_67D0('DFSYSCP0',
                       ' SHUT DOWN',
                       LCRETOKN,
                       Dummy_Char8,
                       Dummy_char4,
                       Dummy_char4,
                       20,
                       12);

    ! Pass back an extended error reason code
    If (YSCMRSN) Then Clt_Code = Err_REQ_IMS_Down;
    Return Code(Err_in_OTMA_Callout);

    End ; /* If Then */

  ! Check if the system has the callout activation PTF applied

  If (¬TSCD_CALLOUT) Then Do;

    If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
    Out_Msg = 'DFSYSCP0 SYNC CALLOUT IS DISABLED !!!';
    Call Do_Hard_WTO(Out_Msg);
    End;

    ! Cut a 67D0 log record

    Call Cut_OTMA_67D0('DFSYSCP0',
                       'NO CALLOUT',
                       LCRETOKN,
                       Dummy_Char8,
                       Dummy_char4,
                       Dummy_char4,
                       20,
                       13);

    ! Pass back an extended error reason code
    If (YSCMRSN) Then Clt_Code = Err_REQ_ICAL_Disabled;
    Return Code(Err_in_OTMA_Callout);

    End ; /* If Then */

  Return Code(Local_Rc);

 End Check_System_Status;



 /*===================================================================*/
 /* Verify that Member name is not reserved keywords as well as in    */
 /*   valid character set.                                            */
 /*-------------------------------------------------------------------*/

 Verify_Member_Name: Procedure(MemberName)
    INTERNAL_PROC_OPT;

 Dcl CHAR_TYPE_A Char(39)
       CONSTANT('@$#ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'),
     Bad_Name    Char(61) Constant
      (' FOR TO ON AFTER SECURITY WTOR MSDB SDB BASICEDT ISCEDT DBRC ');

 Dcl MemberName    Char(16)  ,

     Key_Ptr       Ptr(31)   ,

     Name_Len      Fixed(31) ,

     local_rc      Fixed(31) ;


   Respecify Kwdhdr  Based(Key_ptr);
   Respecify Kwdntry Based(Key_ptr);

   ! The My_CheckName is prefixed with 1 blank and suffixed
   ! with 1 blank for input member name verification.
   Dcl 1 My_CheckName Char(18),
         2 First_Blank Char(1),
         2 CheckName   Char(16),
         2 Last_Blank  Char(1);

   local_rc = 0 ;

   My_CheckName = ' ';     ! Init the search argument

   CheckName = MemberName;
   Key_Ptr = Scdckwd0;

   /*-----------------------------------------------------------------*/
   /* VERIFY NAME IS NON-BLANK, AND CONTAINS ALPHANUMERIC CHARS.      */
   /*-----------------------------------------------------------------*/

   Name_Len = Verify(MemberName,Char_Type_A);

   If (Name_Len = 0) Then
     Name_Len = Length(MemberName);
   Else If ((Name_Len = 1) |
            (Verify(MemberName(Name_Len:Length(MemberName)),' ') > 0))
        Then
          local_rc = 4;
        Else
          Name_Len = Name_Len - 1;


   If (local_rc = 0) Then Do;

     /*---------------------------------------------------------------*/
     /* VALIDATE NAME AGAINST RESERVED WORDS - USE CHECKNAME, SINCE   */
     /* IT IS GUARANTEED TO BE PADDED BY AT LEAST 1 BLANK.            */
     /*---------------------------------------------------------------*/
     If (Index(Bad_Name,My_CheckName(1:Name_Len+2)) > 0) Then
       local_rc = 4;

     /*---------------------------------------------------------------*/
     /* VERIFY NAME DOESN'T BEGIN WITH 'DFS' OR 'DBCDM'               */
     /*---------------------------------------------------------------*/
     Else If ((MemberName(1:3) = 'DFS')  |
              (MemberName(1:5) = 'DBCDM')) Then
       local_rc = 4;

     /*---------------------------------------------------------------*/
     /* VALIDATE NAME AGAINST SYSTEM CONSOLE AND MTO NAMES            */
     /*---------------------------------------------------------------*/
     Else If ((MemberName = Tscd_Syscon_Name) |
              (MemberName = Tscd_Mem_Name)    |
              (MemberName = Tscd_Pmto_Name)   |
              (MemberName = Tscd_Smto_Name)) Then
       local_rc = 4;

     /*---------------------------------------------------------------*/
     /* VALIDATE NAME AGAINST KEYWORD NAMES                           */
     /*---------------------------------------------------------------*/
     Else If (Key_Ptr ¬= 0) Then
       Do While ((Kwdhdr(1:Length(Kwdtend)) ¬= Kwdtend) &
                 (local_rc = 0));
         If (Kwdhflag = Kwdhdrf) Then
           Key_Ptr = Key_Ptr + Length(Kwdhdr);
         Else If (MemberName = Kwdname) Then
           local_rc = 4;
         Else
           Key_Ptr = Key_Ptr + Length(Kwdntry);
       End;

   End;


   Return Code(local_rc) ;

   Respecify Kwdhdr  Based;
   Respecify Kwdntry Based;

 End Verify_Member_Name ;
                                                             @EJECT;

 /*===================================================================*/
 /* Verify that Tpipe  name is not reserved keywords as well as in    */
 /*   valid character set.                                            */
 /*-------------------------------------------------------------------*/

 Verify_Tpipe_Name: Procedure(TpipeName)
    INTERNAL_PROC_OPT;

 Dcl CHAR_TYPE_A Char(39)
       CONSTANT('@$#ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'),
     Bad_Name    Char(65) Constant
  (' FOR TO ON AFTER SECURITY WTOR MSDB SDB BASICEDT ISCEDT DBRC ALL ');

 Dcl TpipeName     Char(8)   ,

     Key_Ptr       Ptr(31)   ,

     Name_Len      Fixed(31) ,

     local_rc      Fixed(31) ;


   Respecify Kwdhdr  Based(Key_ptr);
   Respecify Kwdntry Based(Key_ptr);

   ! The My_CheckName is prefixed with 1 blank and suffixed
   ! with 1 blank for input member name verification.
   Dcl 1 My_CheckName Char(10),
         2 First_Blank Char(1),
         2 CheckName   Char(8),
         2 Last_Blank  Char(1);

   local_rc = 0 ;

   My_CheckName = ' ';     ! Init the search argument

   CheckName = TpipeName;
   Key_Ptr = Scdckwd0;

   /*-----------------------------------------------------------------*/
   /* VERIFY NAME IS NON-BLANK, AND CONTAINS ALPHANUMERIC CHARS.      */
   /*-----------------------------------------------------------------*/

   Name_Len = Verify(TpipeName,Char_Type_A);

   If (Name_Len = 0) Then
     Name_Len = Length(TpipeName);
   Else If ((Name_Len = 1) |
            (Verify(TpipeName(Name_Len:Length(TpipeName)),' ') > 0))
        Then
          local_rc = 4;
        Else
          Name_Len = Name_Len - 1;


   If (local_rc = 0) Then Do;

     /*---------------------------------------------------------------*/
     /* VALIDATE NAME AGAINST RESERVED WORDS - USE CHECKNAME, SINCE   */
     /* IT IS GUARANTEED TO BE PADDED BY AT LEAST 1 BLANK.            */
     /*---------------------------------------------------------------*/
     If (Index(Bad_Name,My_CheckName(1:Name_Len+2)) > 0) Then
       local_rc = 4;

     /*---------------------------------------------------------------*/
     /* VERIFY NAME DOESN'T BEGIN WITH 'DFS' OR 'DBCDM'               */
     /*---------------------------------------------------------------*/
     Else If (TpipeName ¬= 'DFS$$TOQ') &
             ((TpipeName(1:3) = 'DFS')  |
              (TpipeName(1:5) = 'DBCDM')) Then
       local_rc = 4;

     /*---------------------------------------------------------------*/
     /* VALIDATE NAME AGAINST SYSTEM CONSOLE AND MTO NAMES            */
     /*---------------------------------------------------------------*/
     Else If ((TpipeName = Tscd_Syscon_Name) |
              (TpipeName = Tscd_Pmto_Name)   |
              (TpipeName = Tscd_Smto_Name)) Then
       local_rc = 4;

     /*---------------------------------------------------------------*/
     /* VALIDATE NAME AGAINST KEYWORD NAMES                           */
     /*---------------------------------------------------------------*/
     Else If (Key_Ptr ¬= 0) Then
       Do While ((Kwdhdr(1:Length(Kwdtend)) ¬= Kwdtend) &
                 (local_rc = 0));
         If (Kwdhflag = Kwdhdrf) Then
           Key_Ptr = Key_Ptr + Length(Kwdhdr);
         Else If (TpipeName = Kwdname) Then
           local_rc = 4;
         Else
           Key_Ptr = Key_Ptr + Length(Kwdntry);
       End;

   End;


   Return Code(local_rc) ;

   Respecify Kwdhdr  Based;
   Respecify Kwdntry Based;

  End Verify_Tpipe_Name ;
                                                             @EJECT;

  /*******************************************************************/
  /* Subroutine - Check_Tpipe_And_Others                             */
  /*    Find or create a tpipe for a synchronous program switch      */
  /*    Also, we validate the tmember and tpipe for the late         */
  /*    message reroute.                                             */
  /*    We check if we have the OTMA global flood condition          */
  /*******************************************************************/
  Check_Tpipe_And_Others: Procedure
                Internal_Proc_Opt;
   Declare
       Local_RC           Fixed(31);

   Dcl Member_Name_for_SPS Char(16);
   Dcl Got_Tmember        Bit(1);
   Dcl Got_Tpipe          Bit(1);

     If (TSCD_GLOBAL_TIB_ACT)&(TSCD_GLOBAL_TIB_100) Then Do;

       ! Need to do some clean-up before exit
       Call Total_Cleanup;

       ! Pass back an extended error reason code
       If (YSCMRSN) Then Clt_Code = Err_TIB_Flood;

       Return Code(Err_Sync_P2P_Specific);

     End; ! End of checking OTMA global flood condition

     If (PSTICALO) Then Do;
       If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
       Out_Msg = 'DFSYSCP0 PSTICALO is NON ZERO....!!!!';
       Call Do_Hard_WTO(Out_Msg);
       End;
     End;

     ! Init with member and tpipe name for synchronous program switch

     Member_Name_for_SPS = 'DFSYICAL        '; ! member name

     Sync_Tpipe          = 'DFSTPIPE'; ! tpipe name


     ?DFSYDST  FUNC(FIND)
               TYPE(TPIPE)
               MEMBER(Addr(Member_Name_for_SPS))
               LTERM(Addr(Sync_Tpipe))
               CREATE(YES)
               CREATEMEM(YES)
               OPTION(SPS)
               RETCODE(local_rc)
               RETADDR(Tpipe_Ptr) ;


     If (Local_RC) Then Do;

       ! Need to do some clean-up before exit
       Call Total_Cleanup;

       ! Pass back an extended error reason code
       If (YSCMRSN) Then Clt_Code = Err_REQ_Bad_Tpipe;

       Return Code(Err_in_OTMA_Callout);

     End;


     If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
     Out_Msg = 'DFSYSCP0 Get Tpipe for SPS       !!!!';
     Call Do_Hard_WTO(Out_Msg);
     End;

     If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
     Out_Msg = '>>DFSYSCP0 REROUTE TMM= ' || Ydrdesc_Tmember||
               ' TPP= ' || Ydrdesc_Tpipe;
     Call Do_Hard_WTO(Out_Msg);
     End;

     If (Tpipe_Ptr->Tpipe_Mte_Ptr->MTEUSTOP) Then Do;

       If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
       Out_Msg = 'DFSYSCP0 MEMBER STOPPED fOR SPS  !!!!';
       Call Do_Hard_WTO(Out_Msg);
       End;

       Call Total_Cleanup;

       ! Pass back an extended error reason code
       If (YSCMRSN) Then Clt_Code = Err_Member_Stopped;

       Return Code(Err_Sync_P2P_Specific);
     End;



     If (Tpipe_Ptr->Tpipe_Stop) Then Do;

       If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
       Out_Msg = 'DFSYSCP0 TPIPE  STOPPED FOR SPS  !!!!';
       Call Do_Hard_WTO(Out_Msg);
       End;

       ! Need to do some clean-up before exit
       Call Total_Cleanup;

       ! Pass back an extended error reason code
       If (YSCMRSN) Then Clt_Code = Err_Tpipe_Stopped;

       Return Code(Err_Sync_P2P_Specific);
     End;


     ! Let's validate the reroute tmember and tpipe name if specified
     Got_Tmember = False; ! Assume we have no tmember
     Got_Tpipe   = False; ! Assume we have no tpipe

     If (ydrdesc_tmember ¬= ''b  &
         ydrdesc_tmember ¬= ' 'c ) then Do;

       ! Verify the tmember name for this SPS call
       Call Verify_Member_Name(ydrdesc_tmember) Retcode(local_rc) ;

       If (Local_RC) Then Do;

         If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
         Out_Msg = '>>DFSYSCP0 REROUTE Bad Tmember Name ';
         Call Do_Hard_WTO(Out_Msg);
         End;


         ! Need to do some clean-up before exit
         Call Total_Cleanup;

         ! Pass back an extended error reason code
         If (YSCMRSN) Then Clt_Code = Err_SPS_Bad_Tmem;

         Return Code(Err_Sync_P2P_Specific);

       End;

       Got_Tmember = True; ! We got a tmember for reroute

     End; ! End of tmember validation


     If (ydrdesc_tpipe   ¬= ''b  &
         ydrdesc_tpipe   ¬= ' 'c ) then Do;

       ! Verify the tpipe name for this SPS call
       Call Verify_Tpipe_Name(ydrdesc_tpipe) Retcode(local_rc) ;

       If (Local_RC) Then Do;

         If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
         Out_Msg = '>>DFSYSCP0 REROUTE Bad Tpipe  Name ';
         Call Do_Hard_WTO(Out_Msg);
         End;

         ! Need to do some clean-up before exit
         Call Total_Cleanup;

         If (YSCMRSN) Then Clt_Code = Err_SPS_Bad_Tpipe;

         Return Code(Err_Sync_P2P_Specific);

       End;

       Got_Tpipe = True; ! We got a tmember for reroute

     End; ! End of tpipe validation

    ! Check if we are missing tmember or tpipe. We need both for reroute

    If (Got_Tmember & (¬Got_Tpipe))   |
       (Got_Tpipe   & (¬Got_Tmember))   Then Do;

       ! Need to do some clean-up before exit
       Call Total_Cleanup;

       ! Pass back an extended error reason code
       If (YSCMRSN) Then Clt_Code = Err_SPS_Missing_Tmem_Tpipe;

       Return Code(Err_Sync_P2P_Specific);

    End;
    Else
      If (Got_Tmember & Got_Tpipe) Then Ical_sps_reroute = True;

    If (Ydrdesc_Smem & Ydrdesc_SyncTP) Then Do;

      Call Total_Cleanup;

      ! Pass back an extended error reason code
      If (YSCMRSN) Then Clt_Code = Err_SPS_Bad_Smem_Synctp;

      Return Code(Err_Sync_P2P_Specific);

    End;

   Return Code(Local_RC);

  End Check_Tpipe_And_Others;



 /*********************************************************************/
 /* Subroutine - Search_DestRoutDesc                                  */
 /*    Search the Destination Routing Descriptors for destination     */
 /* SPECIAL NOTE:                                                     */
 /*    This V11 logic is different from the V10. Mainly we            */
 /*    support type-2 command for type D descriptor in V11. With that,*/
 /*    the descriptor logic has been enhanced. The following code is  */
 /*    based on the similar code in DFSYFND0 to search for the        */
 /*    descriptor.                                                    */
 /*********************************************************************/
 Search_DestRoutDesc: Procedure
               Internal_Proc_Opt;

 Dcl
    Local_RC          Fixed(31);                             !@PK81543

 dcl                                                         !@PK81543
    ndx               fixed(31),
    ydrd_rc           ptr(31),
    ycmr_rc           ptr(31),
    ycmr_ptr          ptr(31),
    ydrdesc_ptr       ptr(31),
    scd_ptr           ptr(31),
    ydrdesc_len       fixed(31);

 dcl
    neg_one              ptr(31) constant('7FFFFFFF'X);

    Local_RC = 0; ! Pre-set a good return code

    ! Need to get some storage before the descriptor searching@PK81543

    ydrdesc_len = length(ydrdesc) + length(ycmr);            !@PK81543

    ?ZDFSPOOL FUNC(GET)                 !issue DFSPOOL call
              POOL(HIOP)
              LEN(ydrdesc_len)
              ADDR(ydrdesc_ptr)
              ECB(addr(ecb_plas))
              SCD(addr(scd_plas))
              WAIT(YES)
              RC(Local_RC);                                  !@PK81543

    If (Local_RC = 0) then
       ?Dfsbfill DEST(ydrdesc_ptr) LEN(ydrdesc_len) FILL(0);
    ELSE
      Do;

         If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
          Out_Msg = 'DFSYSCP0 no stg for descriptor = ' || dest_name;
          Call Do_Hard_WTO(Out_Msg);
         End;                                                !@PK81543

         ! Cut a 67D0 log record

         Call Cut_OTMA_67D0('DFSYSCP0',
                        ' NO HIOP  ',
                        LCRETOKN,
                        Dest_Name,
                        Dummy_char4,
                        Dummy_char4,
                        20,
                       88);
                                                             !@PK81543
         ! Pass back an extended error reason code
         If (YSCMRSN) Then Clt_Code = Err_REQ_No_Hiop;

         Return Code(Err_in_OTMA_Callout);                   !@PK81543

      End;

    ! Need to build the input parameters for calling dfsycmr0 @PK81543
    ! to locate the descriptor                                @PK81543

    ycmr_ptr = ydrdesc_ptr + length(ydrdesc);                !@PK81543

    rfy ycmr based(ycmr_ptr);
    rfy ydrdesc based(ydrdesc_ptr);

    ycmr_func               = ycmr_qrv;                      !@PK81543
    ycmr_retrec             = true;                          !@PK81543
    ycmr_dest_len           = length(dest_name);             !@PK81543
    ycmr_ydrdesc            = ydrdesc_ptr;                   !@PK81543
    ydrdesc_bwdptr          = neg_one;                       !@PK81543
    ydrdesc_name            = dest_name;                     !@PK81543
    ycmr_scd                = addr(scd_plas);                !@PK81543
    ycmr_ecb                = addr(ecb_plas);                !@PK81543
    DestRoutDesc_fnd        = false;                         !@PK81543


  CL: do while (ydrdesc_bwdptr  ¬= 0);                       !@PK81543

       ydrdesc_name         = dest_name;
       ycmr_retcode         = 4;            !continue

       call dfsycmr0(ecb_plas,scd_plas,ycmr) retcode(ycmr_rc);

       if (Ydrdesc_Mask) then                                !@PK81543
        do;
          if ((ydrdesc_name_len = 1)   &
              (ydrdesc_name(1::1) = ydrdesc_star))  then
            do;
               DestRoutDesc_fnd = true;
               leave CL;
            end;
          else
            if (ydrdesc_name(1::ydrdesc_name_len-1) =
                dest_name(1::ydrdesc_name_len-1))   then
              do;
                DestRoutDesc_fnd = true;
                leave CL;
              end;
            else;
        end;
       else
        if (ydrdesc_name = dest_name) then
          do;
            DestRoutDesc_fnd = true;
            leave CL;
          end;
        else;

    end; ! End of searching for descriptor entry

    ! Check the result of the descriptor searching            @PK81543

    If (DestRoutDesc_fnd) & (YDRDESC_IMSCON|YDRDESC_IMSTRAN) Then
      Do;
       drdesc_ptr = ydrdesc_ptr;

       SELECT;

         WHEN (YDRDESC_IMSCON) Do;

          DestRoutDesc_ICON =  True; ! This is for IMS Connect

          If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
           Out_Msg = 'DFSYSCP0 ICON descriptor = ' || dest_name;
           Call Do_Hard_WTO(Out_Msg);
          End;

         End;

         WHEN (YDRDESC_IMSTRAN) Do;

          DestRoutDesc_SPS  =  True; ! This is Sync Prog Switch

          If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
           Out_Msg = 'DFSYSCP0 SPS descriptor = ' || dest_name;
           Call Do_Hard_WTO(Out_Msg);
          End;

         End;

         OTHERWISE;
       END; ! End of Select

      End; ! End of found descriptor
    Else ! it is an error case
      Do;
         If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
          Out_Msg = 'DFSYSCP0 INVALID descriptor = ' || dest_name;
          Call Do_Hard_WTO(Out_Msg);
         End;

         ! Cut a 67D0 log record

         Call Cut_OTMA_67D0('DFSYSCP0',
                        ' BAD DESC ',
                        LCRETOKN,
                        Dest_Name,
                        Dummy_char4,
                        Dummy_char4,
                        8,
                        1);   !@PK81543

         Return Code(Err_Input_Descriptor);  !@PK81543
      End;

    rfy ycmr based;
    rfy (YDRDESC)  based(DRDESC_PTR);

  Return Code(Local_Rc);

 End Search_DestRoutDesc;


 /******************************************************* @PK81543 ****/
 /* Subroutine - Release_Descriptor_STG                               */
 /*    Release the HIOP storage for the descriptor entry              */
 /*********************************************************************/
 Release_Descriptor_STG: Procedure
               Internal_Proc_Opt;                            !@PK81543
  Declare
     Local_RC           Fixed(31);


  If (drdesc_ptr) Then
   Do; ! Let release the storage for the descriptor entry

    ?ZDFSPOOL FUNC(REL)
            POOL(HIOP)
            ADDR(drdesc_ptr)
            ECB(addr(ecb_plas))
            SCD(addr(scd_plas))
            WAIT(YES)
            RC(Local_RC);

    If (Local_RC) Then

     Do; ! Cut a 67D0 log record

         Call Cut_OTMA_67D0('DFSYSCP0',
                        ' BAD RLSE ',
                        ' CANNOT REL STG ',
                        Dest_Name,
                        Dummy_char4,
                        Dummy_char4,
                        20,
                       87);

     End;! End of cutting 67D0

     drdesc_ptr = 0; ! Clear the pointer

   End;! End of storage release

 End Release_Descriptor_STG; !@PK81543

 /*********************************************************************/
 /* Subroutine - Vaslidate_Control_Data                               */
 /*    Check the Sync Callout Control Data for:                       */
 /*        1) Total Length = summ of all items                        */
 /*        2) Items have good tags                                    */
 /*                                                                   */
 /*    Format of Control Data                                         */
 /*    LLLL | <tag1> data1 </tag1> { LLLL | <tag2> data2 </tag2> ...} */
 /*********************************************************************/
 Validate_Control_Data: Procedure                             !@U14CDC1
               Internal_Proc_Opt;                             !@U14CDC1
                                                              !@U14CDC1
  Dcl                                                         !@U14CDC1
     Local_RC          Fixed(31);                             !@U14CDC1
                                                              !@U14CDC1
  rfy (r1, r2, r3, r4, r5, r6, r15)  rstd ;  ! Work registers !@U14CDC1
                                                              !@U14CDC1
  r1 = Ctl_Data_Len;   ! Total length of Control Data         !@U14CDC1
  r3 = YSCMCTLA;       ! Start of Control Data Items          !@U14CDC1
                                                              !@U14CDC1
           GEN CODE SETS(r15);                                !@U14CDC1
           XR    R15,R15          Let's be optimistic         !@U14CDC1
           XR    R2,R2            Total of Item lengths       !@U14CDC1
 ITEM_LOOP DS    0H                                           !@U14CDC1
 * Check Item length compared to Total CD length              !@U14CDC1
           ICM   R4,15,0(R3)      Get item length             !@U14CDC1
           BNP   EXIT_RC4         bad length                  !@U14CDC1
           AR    R2,R4            add to total                !@U14CDC1
           CR    R1,R2            make sure still valid       !@U14CDC1
           BL    EXIT_RC4         too much, bad length        !@U14CDC1
 * Check beginning Tag for this Item                          !@U14CDC1
           CLI   4(R3),C'<'       Proper beginning tag        !@U14CDC1
           BNE   EXIT_RC8         No, tag error               !@U14CDC1
           AHI   R4,-5            Item length less LLLL<      !@U14CDC1
           LA    R5,5(,R3)        first byte of tag           !@U14CDC1
           CLI   0(R3),C'>'       is there a tag name         !@U14CDC1
           BE    EXIT_RC8         No, tag error               !@U14CDC1
 BTAG_LOOP DS    0H                                           !@U14CDC1
           CLI   0(R5),C'>'       End of tag                  !@U14CDC1
           BE    BTAG_FND         Yes, all set                !@U14CDC1
           LA    R5,1(,R5)        No, next byte               !@U14CDC1
           BCT   R4,BTAG_LOOP     Keep looking                !@U14CDC1
           B     EXIT_RC8         Tag error - no end of tag   !@U14CDC1
 BTAG_FND  DS    0H                                           !@U14CDC1
 * Calculate End Tag and validate                             !@U14CDC1
           LA    R4,5(,R3)        account for LLLL<           !@U14CDC1
           SR    R5,R4            length of tag name          !@U14CDC1
           LA    R6,3(,R5)        end tag has </>             !@U14CDC1
           L     R4,0(,R3)        item length                 !@U14CDC1
           SR    R4,R6            offset to end tag           !@U14CDC1
           LA    R6,0(R4,R3)      start of end tag            !@U14CDC1
           CLI   0(R6),C'<'       beginning of end tag        !@U14CDC1
           BNE   EXIT_RC8         no, tag error               !@U14CDC1
           CLI   1(R6),C'/'       it's an ending tag          !@U14CDC1
           BNE   EXIT_RC8         no, tag error               !@U14CDC1
           LA    R6,2(,R6)        start of end tag name       !@U14CDC1
           LA    R4,5(,R3)        start of beginning tag name !@U14CDC1
 TAG_CMPR  DS    0H                                           !@U14CDC1
           CLC   0(1,R4),0(R6)    tags the same               !@U14CDC1
           BNE   EXIT_RC8         no, tag error               !@U14CDC1
           LA    R4,1(,R4)        bump beginning tag          !@U14CDC1
           LA    R6,1(,R6)        bump ending tag             !@U14CDC1
           BCT   R5,TAG_CMPR      keep checking               !@U14CDC1
           CLI   0(R6),C'>'       END OF TAG                  !@U14CDC1
           BNE   EXIT_RC8         NO, TAG ERROR               !@U14CDC1
 * tags match                                                 !@U14CDC1
           CR    R1,R2            All done Items              !@U14CDC1
           BNH   EXIT_VCD         Yes, exit                   !@U14CDC1
           L     R4,0(,R3)        Get Item length             !@U14CDC1
           AR    R3,R4            Point to next Item          !@U14CDC1
           B     ITEM_LOOP        go check it                 !@U14CDC1
 EXIT_RC4  DS    0H                                           !@U14CDC1
           LHI   R15,4            RC = 04, invalid length     !@U14CDC1
           B     EXIT_VCD                                     !@U14CDC1
 EXIT_RC8  DS    0H                                           !@U14CDC1
           LHI   R15,8            RC = 08, invalid tag        !@U14CDC1
 EXIT_VCD  DS    0H                                           !@U14CDC1
           @ENDGEN;                                           !@U14CDC1
                                                              !@U14CDC1
  Local_RC = r15;                                             !@U14CDC1
                                                              !@U14CDC1
  rfy (r1, r2, r3, r4, r5, r6, r15)  unrstd ;                 !@U14CDC1
                                                              !@U14CDC1
  If (Local_RC = 4) Then                                      !@U14CDC1
     If (YSCMRSN) Then Clt_Code = Err_CTLDATA_Bad_Length;     !@U14CDC1
                                                              !@U14CDC1
  If (Local_RC = 8) Then                                      !@U14CDC1
     If (YSCMRSN) Then Clt_Code = Err_CTLDATA_Bad_Tag;        !@U14CDC1
                                                              !@U14CDC1
  Return Code(Local_Rc);                                      !@U14CDC1
                                                              !@U14CDC1
 End Validate_Control_Data;                                   !@U14CDC1

 /*********************************************************************/
 /* Subroutine - Search_OTMA_Member                                   */
 /*    Check if OTMA member is there for handling sync callout msg    */
 /*                                                                   */
 /*    If exit w/o error, MTE_Ptr points to OTMA member or super      */
 /*    member.                                                        */
 /*********************************************************************/
 Search_OTMA_Member: Procedure
               Internal_Proc_Opt;
  Declare
     Local_RC           Fixed(31);

  Local_RC = 0; ! Pre-set a good return code

  ?DFSYFMTE NAME(YDRDESC_TMEMBER)
           RETADDR(MTE_Ptr) RETCODE(Local_RC);


  If (Local_RC = 0) & (MTESPCLT) Then !@PK71135
   Do;  ! Get the super member MTE

    super_member_name = MTESPNM;    ! Get the super member name
    ?DFSYFMTE NAME(super_member_name)
             RETADDR(MTE_Ptr) RETCODE(Local_RC); !@PK71135

   End;

  !!!! TEMP CODE UT BEGIN !!!!!
  !!!! This temporary code is for unit testing with PROTOTYPE tool
  !!!! for sync callout function                           @PK71135
  ! IF (LOCAL_RC = 0) THEN MCBOFFST = 94;
  !!!! TEMP CODE UT END  !!!!!!

  ! If member not found, IMS server, member stop, Just return
  If (Local_RC ¬= 0) | (MTE1IMS) | (MTE1STOP) |
     ((¬MTESPMEM) & (MTESTATE ¬= MTEACTIV)) |
     (MCBOFFST = 0)
    Then Do;

     ! Cut a 67D0 log record

     Call Cut_OTMA_67D0('DFSYSCP0',
                        ' BAD MEMBR',
                        LCRETOKN,
                        DEST_NAME,
                        Dummy_char4,
                        Dummy_char4,
                        20,
                        1);
     ! Pass back an extended error reason code
     If (YSCMRSN) Then Clt_Code = Err_REQ_Bad_Member;

     Return Code(Err_in_OTMA_Callout);

    End;

  ! If descriptor has super member flag and the member name is
  ! not a super member, we need to reject this call.
  if (Local_RC = 0) & (ydrdesc_smem) & (MTESPMEM = False) Then

    Do;

     ! Cut a 67D0 log record

     Call Cut_OTMA_67D0('DFSYSCP0',
                        ' BAD DESC ',
                        LCRETOKN,
                        Dest_Name,
                        Dummy_char4,
                        Dummy_char4,
                        8,
                        2);

     Return Code(Err_Input_Descriptor);

    End;

  Return Code(Local_Rc);

 End Search_OTMA_Member;

 /*********************************************************************/
 /* Subroutine - Get_Timeout_Value                                    */
 /*    Validate timeout value from DFSYSCM/DFSDLA40 and determine     */
 /*    the timeout value used for the sync callout message            */
 /*                                                                   */
 /*    If exit w/o error, Sync_Timeout has the T/O for sync callout   */
 /*********************************************************************/
 Get_Timeout_Value: Procedure
               Internal_Proc_Opt;
  Declare
     Local_RC           Fixed(31);

  Local_RC = 0; ! Pre-set a good return code

  If (YSCMTO = 0) Then           ! Check the input parameter
    Input_Timeout = 0;           ! No T/O specified in DFSYSCM
  Else
    Input_Timeout = In_Timeout;  ! Obtain T/O from DFSYSCM

  ! input timeout value cannot be greater than 999999
  If (Input_Timeout > Max_Timeout_Value) Then
     Input_Timeout = Max_Timeout_Value;

  ! Let's check if T/O in DFSYSCM is negative
  If (Input_Timeout < 0) Then

    Do;

     ! Cut a 67D0 log record

     Call Cut_OTMA_67D0('DFSYSCP0',
                        ' BAD TIMOU',
                        LCRETOKN,
                        Dest_Name,
                        Dummy_char4,
                        Dummy_char4,
                        12,
                        0);

     Return Code(Err_Input_Timeout);

    End;

  Else

    ! Determine timeout value from DFSYSCM, Descriptor, or default
    If (Input_Timeout = 0 ) Then

      Do;
       ! Let's check if there is a T/O in OTMA descriptor
       If (YDRDESC_SYNC_TO = 0) Then
         Sync_Timeout = Sys_Default_Timeout;
       Else
         Sync_Timeout = YDRDESC_SYNC_TO;
      End;

    Else

      ! We have a timeout value specified in ICAL
      Do; !@PK71135

       ! Let's check if there is a timeout in OTMA descriptor
       If (YDRDESC_SYNC_TO = 0) Then
         Sync_Timeout = Input_Timeout;
       Else
         ! Now we have timeout value in both ICAL and descriptor
         ! ICAL can only UNDERRIDE the timeout value in descriptor
         If (Input_Timeout < YDRDESC_SYNC_TO) Then
           Sync_Timeout = Input_Timeout;
         Else
           Sync_Timeout = YDRDESC_SYNC_TO;

      End; !@PK71135

  If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do; !@PM39562

   Out_Msg='((T/O values: AIB=XXXXXXXX DESC=XXXXXXXX PICK=xxxxxxxx))';
   Call C2X(Input_Timeout,4,Out_Msg(19),8);
   Call C2X(YDRDESC_SYNC_TO,4,Out_Msg(33),8);
   Call C2X(Sync_Timeout,4,Out_Msg(47),8);
   Call Do_Hard_WTO(Out_Msg);

  End; !@PM39562

  If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do; !@PM39562

  Out_Msg='((T/O values: AIB=XXXXXXXX DESC=XXXXXXXX PICK=xxxxxxxx))';
  Call C2X(Input_Timeout,4,Out_Msg(19),8);
  Call C2X(YDRDESC_SYNC_TO,4,Out_Msg(33),8);
  Call C2X(Sync_Timeout,4,Out_Msg(47),8);
  Call Do_Hard_WTO(Out_Msg);

  End; !@PM39562

  Return Code(Local_Rc);

 End Get_Timeout_Value;


  /******************************************************************/
  /* Subroutine - Get_YTIB_For_SPS                                  */
  /*    Create an YTIB for this synchronous program switch message  */
  /*    Create OTMA prefix (ctl, state, security, and user data)    */
  /*    If exit w/o error, CLB_Ptr points to the YTIB block         */
  /******************************************************************/
  Get_YTIB_For_SPS: Procedure
                Internal_Proc_Opt;

   Declare
       1  MY_OTMA_AREA BASED,
         3 CTL      CHAR(LENGTH(TMAMCTL))      BDY(HWORD), !Control
         3 HDR      CHAR(2)                    BDY(HWORD), !State
         3 HDR_TXN  CHAR(70)                   BDY(HWORD),
         3 HDR_SEC  CHAR(LENGTH(TMAMSEC))      BDY(HWORD), !Security
         3 HDR_SUDS CHAR(LENGTH(TMAMSUDS))     BDY(HWORD),
         3 HDR_SGDS CHAR(LENGTH(TMAMSGDS))     BDY(HWORD);

   Declare
       Local_RC           Fixed(31),
       My_Udata_Len       Fixed(16) Sinit(0),
       Orig_Tmamhdr_Txn_Ptr Ptr(31) Sinit(0),
       Orig_Tmember_Name  Char(16),
       Orig_Tpipe_Name    Char(8),
       Current_Ptr        Ptr(31);

   DCL Input_Is_Otma      Bit(1); ! Original is from OTMA
   DCL Come_with_Userdata Bit(1); ! Has the user data prefix

   ! If input has no  OTMA userdata and reroute is requested, we will
   ! create a default OTMA userdata with length of 1024
   DCL UserData_Len       Fixed(16) CONSTANT(1024);

   ! Original OTMA security flag for PROFILE
   DCL My_Security_Flag   CHAR(1);


     ! First, let's determine if it is from an OTMA client
     ! And, if yes, get the length of user data prefix

     If (PSTLU62 & PST1OTMA) & (PSTCB62P) &
        (PSTCB62P->Lup_Msg_Is_Otma) Then Do; ! Source is from OTMA

       If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
       Out_Msg = '>>DFSYSCP0 Found an OTMA Message ';
       Call Do_Hard_WTO(Out_Msg);
       End;

       Input_Is_Otma = True;
       Come_with_Userdata = False;

       ! Obtain the original member and tpipe names for ALTPCB
       Orig_Tmember_Name = PSTCB62P->LUP_MEMBER_NAME;
       Orig_Tpipe_Name   = PSTCB62P->LUP_LTERM;

       ! We are trying to get to original user data prefix to find
       ! out the length

       Tmamctl_Ptr = PSTCB62P + Length(Dfsypre);

       Current_Ptr = Tmamctl_Ptr + Length(Tmamctl);

       If (Tmamcstd) Then Do;
          Tmamhdr_Ptr = Current_Ptr;
          Orig_Tmamhdr_Txn_Ptr = Addr(Tmamhorg);
          Current_Ptr = Current_Ptr + Tmamhlen;
       End;

       If (Tmamcsec) Then Do;
          Tmamsec_Ptr = Current_Ptr;
          My_Security_Flag = TMAMSFLG;
          Current_Ptr = Current_Ptr + Tmamslen;
       End;
       Else
          My_Security_Flag = ' ';

       If (Tmamcusr) Then Do;
          Tmamusr_Ptr  = Current_Ptr; ! We have input userdata
          My_Udata_Len = TMAMULEX; ! Obtain the user data length
          Come_with_Userdata = True; ! We do have user data prefix
       End;

     End; ! End of determining the length of user data prefix
     Else Do;

       Input_Is_Otma = False;
       My_Udata_Len  = 0; ! no user data is needed
       Current_ptr   = 0; ! Reset
       My_Security_Flag = ' ';

       If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
       Out_Msg = '>>DFSYSCP0 Found a NON-OTMA Message ';
       Call Do_Hard_WTO(Out_Msg);
       End;

     End;

     ! If no userdata is found so far, we need to check the reroute.
     ! If reroute is requested, then the userdata is set to 1024.
     ! Lator on, DFSYIOE0 can be used to build the user data prefix.

     If (My_Udata_Len = 0) & (Ical_sps_reroute|Ydrdesc_Exit) Then
       My_Udata_Len  = UserData_Len;


     Prefix_Length = (Length(Dfsypre)       +  /* LUP  prefix       */
                      Length(My_OTMA_Area)  +  /* OTMA prefix       */
                      My_Udata_Len          +  /* user data length  */
                      1 ) & 'FFFE'X;           /* make it even      */


     ?DFSYDST  FUNC(FIND)
               TYPE(TIB)
               BLKA(Tpipe_Ptr)
               CREATE(YES)
               OPTION(SPS)
               PREFLEN(Prefix_length)
               RETCODE(local_rc)
               RETADDR(CLB_Ptr) ;

     If (Local_RC) Then Do;

       ! Need to do some clean-up before exit
       Call Total_Cleanup;

       ! Pass back an extended error reason code
       If (YSCMRSN) Then Clt_Code = Err_No_DFSYTIB; ! Cannot create YTIB

       Return Code(Err_Sync_P2P_Specific);

     End;

     If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
     Out_Msg = 'DFSYSCP0 Get a YTIB for SPS     !!!!';
     Call Do_Hard_WTO(Out_Msg);
     End;

     SPS_Ytib_Clb         = CLB_Ptr;
     SPS_Ytib_Addr        = SPS_Ytib_Clb -> CLBlu6wa;
     /*-----------------------------------------------------------*/
     /* BUILD MCI PREFIX SECTION                                  */
     /*-----------------------------------------------------------*/
     Tmamctl_Ptr = YTIB_Prefix_Ptr + Length(Dfsypre);

     Tmamcalv = Tscd_Arch_Level;
     TMAMCTXN = True; ! This is a transaction request
     Tmamsp2p = True; ! This is a synchronous program switch
     Tmamctnm = Sync_tpipe;

     TMAMCFIC = True;
     TMAMCLIC = True;

     Tmamcstd = True;
     Tmamcsec = True;
     If (My_Udata_Len > 0) Then Tmamcusr = True;
     Tmamcapp = True;

     Tmamcssn = 1;
     Tmamcseq = 1 ;
     Tmamcrsq = Sync_Timeout; ! Hold the ICAL timeout value

     /*-----------------------------------------------------------*/
     /* BUILD STATE DATA PREFIX SECTION                           */
     /*-----------------------------------------------------------*/

     Tmamhdr_Ptr = Tmamctl_Ptr + Length(Tmamctl);
     Tmamhdr_Txn_Ptr = Addr(Tmamhorg);

     Tmamhlen = Length(Tmamhdr) + Offset(Tmamhiud);
     Tmamhcm1 = True;
     Tmamhslv = Tmamhsl1;

     If (YSCMMAPN) &
        (Map_Name  ¬= ''b  &
         Map_Name  ¬= ' 'c ) then Do;

       Tmamhmap = Map_Name;

       If (Tscd_Lum_Debug ) then Do;
         Out_Msg = 'DFSYSCP0 AIB Map Name= ' || TMAMHMAP ;
         Call Do_Hard_WTO(Out_Msg);
       End;

     End; ! End of getting map name from AIB
     Else Tmamhmap = ' ';

     If (Input_Is_Otma) Then Do;
       Tmamhcor = Orig_Tmamhdr_Txn_Ptr->Tmamhcor;
       Tmamhcid = Orig_Tmember_Name; ! Save original member name
       Tmamrtod = Orig_Tpipe_Name;   ! Save original tpipe name
     End;
     Else
       Tmamhcor =  LCRETOKN;

     ! Find the lterm overide in AIB or Descriptor
     If (YSCMLTRM) &
        (Lterm_Override  ¬= ''b  &
         Lterm_Override  ¬= ' 'c ) then Do;

       TMAMHLTM = Lterm_Override;

       If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
       Out_Msg = 'DFSYSCP0 AIB LTERM= '||TMAMHLTM ;
       Call Do_Hard_WTO(Out_Msg);
       End;

     End; ! End of locating Lterm override in the AIB
     Else Do; ! Check if we have it in the descriptor

       If (ydrdesc_lterm ¬= ''b  &
           ydrdesc_lterm ¬= ' 'c ) then Do;

         TMAMHLTM = ydrdesc_lterm;

         If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
         Out_Msg = 'DFSYSCP0 Descriptor LTERM= '||TMAMHLTM ;
         Call Do_Hard_WTO(Out_Msg);
         End;

       End;
       Else Do;

         TMAMHLTM = PSTSYMBO;

         If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
         Out_Msg = 'DFSYSCP0 We use PSTSYMBO = '||PSTSYMBO;
         Call Do_Hard_WTO(Out_Msg);
         End;
       End;

     End; ! End of getting lterm override from descriptor


     /*-----------------------------------------------------------*/
     /* BUILD SECURITY PREFIX SECTION                             */
     /*-----------------------------------------------------------*/

     Tmamsec_Ptr = Tmamhdr_Ptr + Tmamhlen;
     Tmamsflg = My_Security_Flag; ! For SECURITY PROFILE is needed
     Tmamslen = Length(Tmamsec) + Length(Tmamsuds) + Length(Tmamsgds);

     ! Userid is from PSTUSID field
     Tmamsuds_Ptr = Tmamsec_Ptr + Length(Tmamsec);
     Tmamsuln = Length(Tmamsuds) - Length(Tmamsuln);
     Tmamsuty = Tmamsuxx;
     Tmamsuid = User_ID;

     ! Userid is from PSTGRPNM field
     Tmamsgds_Ptr = Tmamsuds_Ptr + Length(Tmamsuds);
     Tmamsgln = Length(Tmamsgds) - Length(Tmamsgln);
     Tmamsgty = Tmamsgxx;
     Tmamsgrp = PSTGRPNM;

     /*-----------------------------------------------------------*/
     /* BUILD USER DATA PREFIX SECTION if Needed                  */
     /*-----------------------------------------------------------*/
     If (My_Udata_Len > 0) Then Do;

       Tmamusr_Ptr  = Tmamsec_Ptr + Tmamslen;

       Tmamulex     = My_Udata_Len;

       If (Input_Is_Otma)&(Come_with_Userdata) Then Do;

         ?DFSMOVE SOURCE(Current_Ptr)
                  DEST(Tmamusr_Ptr)
                  LEN(Tmamulex);

         If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
           Out_Msg = '>>DFSYSCP0 Come with UserData Prefix... ';
           Call Do_Hard_WTO(Out_Msg);
         End;

       End; ! End of copying the original user data from OTMA
       Else Do; ! Build default user data prefix

         If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
         Out_Msg = '>>DFSYSCP0 Building default UserData Prefix.. ';
         Call Do_Hard_WTO(Out_Msg);
         End;

       End;

     End; ! end for building the user data prefix

     /*-----------------------------------------------------------*/
     /* BUILD APPLICATION DATA POINTER                            */
     /*-----------------------------------------------------------*/

     Tmamapp_Ptr  = YSCMREQA;

     YTIB_Cntrl_Data_Ptr  = TMAMCTL_Ptr ;
     YTIB_State_Data_Ptr  = TMAMHDR_Ptr ;
     YTIB_Sec_Data_Ptr    = TMAMSEC_Ptr ;
     If (My_Udata_Len) Then YTIB_User_Data_Ptr = TMAMUSR_Ptr ;
     YTIB_Appl_Data_Ptr   = TMAMAPP_Ptr ;
     YTIB_Correlator      = TMAMHCOR;
     YTIB_Wait_ECB        = Addr(ECB_PLAS) ;

     YTIB_SPS_Resp_Code   = 0; !Assume no error code

     YTIB_SPS_Input       = True;

     If (¬Input_Is_Otma) Then
       YTIB_SPS_NONOTMA = True; ! ICAL was NOT from OTMA

     YTIB_Trancode        = TranCode;
     STCK(YTIB_MSG_ARRIVAL_TIME);    ! Store time for timeout
     YTIB_Timeout_ELP     = Sync_Timeout;

     MTE_Ptr              = YTIB_MTE_PTR;

     /* Obtain the info from descriptor */
     If (Ical_sps_reroute) Then Do;
       YTIB_SPS_Reroute  = true;
       YTIB_SPS_RR_MN    = Ydrdesc_Tmember;
       YTIB_SPS_RR_TPIPE = Ydrdesc_Tpipe;

       If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
       Out_Msg = '>>DFSYSCP0 REROUTE TMM= ' || Ydrdesc_Tmember||
                 ' TPP= ' || Ydrdesc_Tpipe;
       Call Do_Hard_WTO(Out_Msg);
       End;

     End;

     If (Ydrdesc_Replychk) Then Do;

       Ytib_SPS_Replychk = True;
       SPS_Replychk      = True;

       If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
       Out_Msg = '>>DFSYSCP0 REPLYCHK is YES !!!!!REPLYCHK=YES!!!';
       Call Do_Hard_WTO(Out_Msg);
       End;

     End;

     If (Ydrdesc_Smem    ) Then Do;

       Ytib_SPS_Smem     = True;

       If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
       Out_Msg = '>>DFSYSCP0 SMEM  is YES    !!!!!!!SMEM=YES!!!!!';
       Call Do_Hard_WTO(Out_Msg);
       End;

     End;


     If (Ydrdesc_SyncTP  ) Then Do;

       Ytib_SPS_SyncTP   = True;

       If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
       Out_Msg = '>>DFSYSCP0 SYNCTP is YES   !!!!!SYNCTP=YES!!!!!';
       Call Do_Hard_WTO(Out_Msg);
       End;

     End;


     If (Ydrdesc_Exit    ) Then Do;

       Ytib_SPS_Exit     = True;

       If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
       Out_Msg = '>>DFSYSCP0 EXIT is YES   !!!!!EXIT=YES!!!!!';
       Call Do_Hard_WTO(Out_Msg);
       End;

     End;


   Return Code(Local_RC);

  End Get_YTIB_For_SPS;


 /*********************************************************************/
 /* Subroutine - Get_OTMA_Tpipe                                       */
 /*    Find the TPIPE for sync callout message and obtain a YQAB      */
 /*                                                                   */
 /*    If exit w/o error, Sync_Tpipe has tpipe name and CLB_PTR has   */
 /*    address of YQAB CLB. Tpipe_Ptr points to the Tpipe.            */
 /*********************************************************************/
 Get_OTMA_Tpipe: Procedure
               Internal_Proc_Opt;
  Declare
     Local_RC           Fixed(31);

  Local_RC = 0; ! Pre-set a good return code

  If (YDRdesc_TPIPE = ' ') | (YDRdesc_TPIPE = ''X) Then

    Sync_Tpipe = Dest_Name;      ! Destination name becomes tpipe name

  Else

    Sync_Tpipe = YDRdesc_TPIPE;  ! Get tpipe name from descriptor entry


  ! Get the YQAB for this sync callout message

  ?DFSYDST FUNC(FIND)
       TYPE(QAB)
       LTERM(Addr(Sync_Tpipe))
       Member(Addr(MTENAME))
       CREATE(YES)
       OPTION(HOLDQUE, SMQTIB)
       RETADDR(Clb_Ptr)
       RETCODE(Local_RC);

  ! If cannot locate or create YQAB, something is wrong
  If (Local_RC ¬= 0) | (TPIPE_STOP) Then
    Do;

     ! Cut a 67D0 log record

     If (Local_RC = 0) & (TPIPE_STOP) Then

      Call Cut_OTMA_67D0('DFSYSCP0',
                        ' STO TPIPE',
                        LCRETOKN,
                        Sync_Tpipe,
                        Dummy_char4,
                        Dummy_char4,
                        20,
                        3);


     Else

      Call Cut_OTMA_67D0('DFSYSCP0',
                        '  NO TPIPE',
                        LCRETOKN,
                        Sync_Tpipe,
                        Dummy_char4,
                        Dummy_char4,
                        20,
                        4);

     ! Pass back an extended error reason code
                                                             !@PM78756
     If (YSCMRSN) Then Do;                                   !@PM78756
       If (local_rc = YDSTRC_REACH_MTP ) Then Clt_Code = 36; !@PM78756
       Else Clt_Code = 16;                                   !@PM78756
     End;                                                    !@PM78756

     Return Code(Err_in_OTMA_Callout);

    End;
                                                              !@U14CDC1
   ! If there is Control Data then Resume TPIPE must be       !@U14CDC1
   ! able to accept it - otherwise fail the ICAL              !@U14CDC1
   If (Control_Data) Then do;                                 !@U14CDC1
      If (YQAB_RESUME_TPIPE) &                                !@U14CDC1
         (¬YQAB_RTP_MODE_CTLDATA) Then do;                    !@U14CDC1
         If (Tscd_Lum_Debug) then Do;                         !@U14CDCB
           Out_Msg = 'DFSYSCP0: *** Control Data but RTP bad';!@U14CDC1
           Call Do_Hard_WTO(Out_Msg);                         !@U14CDC1
         End;                                                 !@U14CDCB
         ! Pass back an extended error reason code            !@U14CDC1
         If (YSCMRSN) Then Clt_Code = Err_CTLDATA_Bad_RTP;    !@U14CDC1
         Return Code(Err_Sync_Control_Data);                  !@U14CDC1
      End;    ! End of RT test                                !@U14CDC1
   End;     ! End of Control Data test                        !@U14CDC1

   Tpipe_Sync_Callout = True;
   Tpipe_Ptr = Yqab_Tpipe_Ptr;


  Return Code(Local_Rc);

 End Get_OTMA_Tpipe;

 /*********************************************************************/
 /* Subroutine - Get_Sync_AWE                                         */
 /*    Create an AWE for this callout message and init the AWE        */
 /*                                                                   */
 /*    If exit w/o error, Sync_AWE_Ptr points to the new AWE          */
 /*                                                                   */
 /*********************************************************************/
 Get_Sync_AWE: Procedure
               Internal_Proc_Opt;
  Declare
     Local_RC           Fixed(31);

  Local_RC = 0; ! Pre-set a good return code


  ?DFSBCB FUNC(GET) BLK(AWE) Retaddr(Sync_AWE_Ptr) Retcode(Local_RC);

  If (Local_RC = 0) Then

   Do;

     AWOY = ""x;        ! Clear the AWE
     awosynco = True;   ! Sync callout AWE
     awocecba = Addr(ECB_PLAS) ; ! ECB address
     awostate = AWO_SYNC_INITIAL; ! Set initial status
     If (DestRoutDesc_icon) Then STCK(aworname); ! AWE token

     ! Increment the AWE count in TPIPE
     ?DFSCS LOC(TPIPE_SYNC_AWE_CT) ADD(1);

   End;

  Else

   Do;
    Call Cut_OTMA_67D0('DFSYSCP0',
                       ' NO AWE   ',
                       LCRETOKN,
                       Dummy_Char8,
                       Dummy_char4,
                       Dummy_char4,
                       20,
                       5);

    ! Pass back an extended error reason code
    If (YSCMRSN) Then Clt_Code = Err_REQ_No_AWE;
    Return Code(Err_in_OTMA_Callout);
   End;

  Return Code(Local_Rc);

 End Get_Sync_AWE;

 /*********************************************************************/
 /* Subroutine - Build_Output_Prefix                                  */
 /*    build OTMA output prefix for the sync callout message          */
 /*                                                                   */
 /*    If exit w/o error, awopfptr points to the output prefix        */
 /*                                                                   */
 /*********************************************************************/
 Build_Output_Prefix : Procedure
               Internal_Proc_Opt;
  Declare
     Token_Ptr          Ptr(31),
     Local_RC           Fixed(31);

  Local_RC = 0; ! Pre-set a good return code

  If (¬YDRDESC_IMSCON) Then
    Do;

     ! Cut a 67D0 log record

     Call Cut_OTMA_67D0('DFSYSCP0',
                        ' BAD DESC ',
                        LCRETOKN,
                        Dummy_Char8,
                        Dummy_char4,
                        Dummy_char4,
                        8,
                        3);

     Return Code(Err_Input_Descriptor);

    End;

  Prefix_Length = (Length(Dfsypre)  +  /* OTMA prefix             */
                 Length(Tmamctl)  +  /* MCI                       */
                 Length(Tmamhdr)  +  /* state data header         */
                 Offset(Tmamhiud) +  /* transaction state dat     */
                 Length(Tmamsec)  +  /* security header           */
                 Length(Tmamsuds) +  /* userid                    */
               ICON_UserDate_Length  + /* user data               */
                    4             +  /* 4 bytes of LLZZ           */
                 1 ) & 'FFFE'X;

  ?ZDFSPOOL FUNC(GET)
            LEN(Prefix_Length)
            LIST(ADDR(DFSPOOL_PARMLIST))
            POOL(LUMP)
            ADDR(Awopfptr)
            RC(Local_RC);

  If (Local_RC ¬= 0) Then
    Do;
      Call Cut_OTMA_67D0('DFSYSCP0',
                         ' NO LUMP  ',
                         LCRETOKN,
                         Dummy_Char8,
                         Dummy_char4,
                         Dummy_char4,
                         20,
                         6);

      ! Pass back an extended error reason code
      If (YSCMRSN) Then Clt_Code = Err_REQ_No_Lump;
      Return Code(Err_in_OTMA_Callout);
    End;

  ?DFSBFILL DEST(Awopfptr) LEN(Prefix_Length) FILL(0);

  Respecify (Dfsypre) Based(Awopfptr);

  Lup_Member_Name = YDRDESC_TMEMBER;
  Lup_Lterm = Sync_Tpipe;
  Lup_Msg_Ver = True;                  /* Support message version   */
  LUP_IMSR    = SSCDIMSR;              /* Obtain IMS rel level      */
  LUP_IMSL    = SSCDIMSL;              /* Obtain IMS mod level      */
  Lup_FixPre_Len = Addr(Lup_OTMA) - Addr(DFSYPRE);
  Lup_Length = Prefix_Length;

  /*----------------------------------------------------------------*/
  /* BUILD MCI PREFIX SECTION                                       */
  /*----------------------------------------------------------------*/
  Tmamctl_Ptr = Awopfptr + Lup_FixPre_Len;
  Tmamcalv = Tscd_Arch_Level;
  Tmamcdta = True;
  Tmamctnm = Sync_tpipe;
  Tmamcrrq = True ;
  Tmamcstd = True;
  Tmamcsec = True;
  Tmamcusr = True;
  Tmamcapp = True;

  /*----------------------------------------------------------------*/
  /* BUILD STATE DATA PREFIX SECTION                                */
  /*----------------------------------------------------------------*/


  Tmamhdr_Ptr = Tmamctl_Ptr + Length(Tmamctl);
  Tmamhdr_Txn_Ptr = Addr(Tmamhorg);
  Tmamhlen = Length(Tmamhdr) + Offset(Tmamhiud);
  Tmamhcm0 = True;
  Tmamhslv = Tmamhsl1;
  If (Control_Data) Then          ! If there is Control Data   @U14CDC1
    TMAMHCTD = True;              !   then set indicator       @U14CDC1

  ! We may have a map name to be passed to IMS Connect         @PM73135
  If (YSCMMAPN) &
     (Map_Name  ¬= ''b  &
      Map_Name  ¬= ' 'c ) then Do;                            !@PM73135

    Tmamhmap = Map_Name;                                      !@PM73135

    If (Tscd_Lum_Debug ) then Do;
      Out_Msg = 'DFSYSCP0 AIB Map Name= ' || TMAMHMAP ;       !@PM73135
      Call Do_Hard_WTO(Out_Msg);                              !@PM73135
    End;

  End; ! End of getting map name from AIB                     !@PM73135
  Else Tmamhmap = ' ';                                        !@PM73135

  Tmamhsyc = True; ! This is a sync callout message
  Tmamosxq = MCBOFFST; ! Save the offset in the state data
  Tmamhcor = LCRETOKN; ! Store recovery token in state data @PK91373

  /*----------------------------------------------------------------*/
  /* BUILD SECURITY DATE PREFIX SECTION                             */
  /*----------------------------------------------------------------*/
  Tmamsec_Ptr = Tmamhdr_Ptr + Tmamhlen;
  Tmamslen = Length(Tmamsec) + Length(Tmamsuds);

  /*----------------------------------------------------------------*/
  /* BUILD SECURITY USERID FIELD                                    */
  /*----------------------------------------------------------------*/
  Tmamsuds_Ptr = Tmamsec_Ptr + Length(Tmamsec);
  Tmamsuln = Length(Tmamsuds) - Length(Tmamsuln);
  Tmamsuty = Tmamsuxx;
  Tmamsuid = User_ID;

  /*----------------------------------------------------------------*/
  /* BUILD USER DATA SECTION OF THE PREFIX                          */
  /*----------------------------------------------------------------*/
  Tmamusr_Ptr  = Tmamsec_Ptr + Tmamslen;
  TMAMULEX = ICON_UserDate_Length;
  TMAMUORG = Sync_Tpipe;
  TMAMUCM0 = True;
  TMAMUAL1 = True;
  ! Check for Adapter and Converter
  If ((YDRDesc_Adapter ¬= ' ' ) &
   (YDRDesc_Adapter ¬= ''X )) Then Do;
     TMAMUADT = YDRDesc_Adapter;
     TMAMUDRV = YDRDesc_Convertr;
  End;


  /*----------------------------------------------------------------*/
  /* Build the CORRELATOR TOKEN in the user data prefix             */
  /*----------------------------------------------------------------*/
  Token_Ptr = Tmamusr_Ptr + MCBOFFST;
  Rfy YSYNTKN based(Token_Ptr);

  YSYNTKN = ""x;
  YSYNTKN_LEN   = Length(YSYNTKN);
  YSYNTKN_IMSNM = SCDIMSNM;
  YSYNTKN_MEMTK = MTETOKEN;
  YSYNTKN_AWETK = Aworname;
  YSYNTKN_TPIPE = Sync_tpipe;
  YSYNTKN_USRID = User_ID;

  /*----------------------------------------------------------------*/
  /* BUILD LLZZ for the application data section                    */
  /*----------------------------------------------------------------*/
  Tmamapp_Ptr  = Tmamusr_Ptr + Tmamulex;
  TMAMALEN     = 0;  ! DFSYQAB0 will insert the length
  TMAMAZZ      = ""x;

  If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
  Out_Msg = 'DFSYSCP0: ICAL TOKEN has MTOKEN= xxxxxxxxxxxxxxxx ' ||
            'TPIPE NAME= xxxxxxxx';

  Call C2X(YSYNTKN_MEMTK,8,Out_Msg(34),16);
  Out_Msg(63::8)  = YSYNTKN_TPIPE ;

  Call Do_Hard_WTO(Out_Msg);
  End;


  Return Code(Local_Rc);

 End Build_Output_Prefix;

 /************************************************************@PK71135*/
 /* Subroutine - Process_Request_Data                                 */
 /*    Prepare the request data from the ICAL. Will segment the data  */
 /*    if it is larger then TSCD_MAX_ICAL_SEG_SIZE.                   */
 /*                                                                   */
 /*    If there is Control Data then we always have multiple  @U14CDC1*/
 /*    segments (synonym chain) with as many blocks as needed @U14CDC1*/
 /*    for the Control Data followed by application data.     @U14CDC1*/
 /*                                                                   */
 /*    If exit w/o error, awocbufa points to the data chain           */
 /*                                                                   */
 /*********************************************************************/
 Process_Request_Data: Procedure
               Internal_Proc_Opt;
  Declare
     Local_RC           Fixed(31);
  Dcl
    Curr_Ptr            Ptr(31),
    Remaining_Len       Fixed(32),
    Seg_Addr            Ptr(31),
    Seg_Len             Fixed(32),
    New_Seg_Ptr         Ptr(31),
    Next_Synonym_ptr    Ptr(31);
  Dcl                                                         !@U14CDC1
    1 ProReq_flags      Bit(8),                               !@U14CDC1
      2 CtlData_done      Bit(1),                             !@U14CDC1
      2 *                 Bit(7);                             !@U14CDC1

  CtlData_num_segs = 0;     ! init to no segs                 !@U14CDC1
  CtlData_done = false;     ! init to not done                !@U14CDC1
  Local_RC = 0; ! Pre-set a good return code

  If (Req_Data_Len > TSCD_MAX_ICAL_SEG_SIZE) |                !@U14CDC1
     (Control_Data) Then                                      !@U14CDC1

   Do; ! Start of multi-segment processing

    If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
    Out_Msg = 'DFSYSCP0 large message processing ......';
    Call Do_Hard_WTO(Out_Msg);
    End;

    If (Control_Data) then do;   ! Start with Control Data    !@U14CDC1
       Curr_Ptr = YSCMCTLA;          ! Point to Control Data  !@U14CDC1
       Remaining_Len = Ctl_Data_Len; ! Control Data length    !@U14CDC1
    End;                                                      !@U14CDC1
    Else Do;   ! No Control Data - do Application Data        !@U14CDC1
       Curr_Ptr = YSCMREQA;          ! Point to data segment  !@U14CDC1
       Remaining_Len = Req_Data_Len; ! Save segment length    !@U14CDC1
    End;                                                      !@U14CDC1

    ! Get a 12-byte message synonym block for the first segment
    ?ZDFSPOOL FUNC(GET)
            LEN(Length(YSYNSEGM))
            LIST(ADDR(DFSPOOL_PARMLIST))
            POOL(LUMP)
            ADDR(New_Seg_Ptr)
            RC(Local_RC);

    If (Local_RC ¬= 0) Then !@PK71135
     Do;
      Call Cut_OTMA_67D0('DFSYSCP0',
                         ' NO LUMP  ',
                         LCRETOKN,
                         Dummy_Char8,
                         Dummy_char4,
                         Dummy_char4,
                         20,
                        14);
      ! Pass back an extended error reason code
      If (YSCMRSN) Then Clt_Code = Err_REQ_No_Lump;

      Return Code(Err_in_OTMA_Callout);
     End;

    Awocbufa = New_Seg_Ptr; ! AWE points to 1st block

 MORE_SEG:

    ! keep count of the number of Control Data segments       !@U14CDC1
    If (Control_Data) & (¬CtlData_done) then                  !@U14CDC1
      CtlData_num_segs = CtlData_num_segs + 1;                !@U14CDC1

    Seg_Synonym_Ptr = New_Seg_Ptr; ! Reference to YSYNSEGM DSECT

    Seg_Addr = Curr_Ptr; ! Prepare the segment pointer

    If (Remaining_Len > TSCD_MAX_ICAL_SEG_SIZE) Then !@PK71135
     Do; ! more segment is needed
      Seg_Len = TSCD_MAX_ICAL_SEG_SIZE; ! Prepare length info
      ! Compute the remaining data after this segment
      Remaining_Len = Remaining_Len - TSCD_MAX_ICAL_SEG_SIZE;
      Curr_Ptr = Curr_Ptr + Seg_Len; ! Prepare for next segment
     End;
    ELSE
     Do; !@PK71135
      Seg_Len = Remaining_Len; ! Prepare the last segment
      ! when done Control Data, then do app data              !@U14CDC1
      If (Control_Data) & (¬CtlData_done) Then do;            !@U14CDC1
        CtlData_done = true;             ! done Control Data  !@U14CDC1
        Remaining_Len = Req_Data_Len;    ! app data length    !@U14CDC1
        Curr_Ptr = YSCMREQA;             ! app data area      !@U14CDC1
      End;                                                    !@U14CDC1
      Else do;           ! All done app data                  !@U14CDC1
        Remaining_Len = 0; ! No more data left                !@U14CDC1
        YSYNSEGM_NEXT = 0; ! Next segment point is 0          !@U14CDC1
      End;                                                    !@U14CDC1
     End; !@PK71135

    ! Now build the synonym block with segment addr and len info
    YSYNSEGM_MSG  = Seg_Addr;
    YSYNSEGM_LEN  = Seg_Len;

    If (Remaining_Len = 0) Then do; !@PK71135
      ! Done multi-seg, set Control Data segs (if present)    !@U14CDC1
      If (Control_Data) Then do;                              !@U14CDC1
        Respecify (Dfsypre) Based(Awopfptr);                  !@U14CDC1
        Tmamctl_Ptr = Awopfptr + Lup_FixPre_Len;              !@U14CDC1
        TMAMCTDN = CtlData_num_segs;                          !@U14CDC1
      End;         ! End of Control Data seg count            !@U14CDC1

     Return Code(Local_Rc);
    End;
    ELSE
     Do; ! We need to get another synonym block for remaining data

      ! Get a 12-byte message synonym block for a new segment
      ?ZDFSPOOL FUNC(GET)
            LEN(Length(YSYNSEGM))
            LIST(ADDR(DFSPOOL_PARMLIST))
            POOL(LUMP)
            ADDR(New_Seg_Ptr)
            RC(Local_RC);

      If (Local_RC ¬= 0) Then !@PK71135
       Do;
         ! Let's release the message synonym blocks

         Local_RC = 0;

         Seg_Synonym_ptr = awocbufa;

         Do while ((Seg_Synonym_ptr ¬= 0) & (Local_RC = 0));

          Next_Synonym_ptr = YSYNSEGM_NEXT;

          ?ZDFSPOOL FUNC(REL)
                    POOL(LUMP)
                    ADDR(Seg_Synonym_ptr)
                    LIST(ADDR(DFSPOOL_PARMLIST))
                    ECB(Addr(ECB_PLAS))
                    SCD(Addr(SCD_PLAS))
                    RC(local_rc);

          IF (Local_RC) THEN
           Do;   !@PK71135
            Call Issue_Severe_Error(06);
           End;  !@PK71135

          Seg_Synonym_ptr = Next_Synonym_ptr;

         End; ! end of do-while for freeing the synonym blocks

       End;
      ELSE !@PK71135
       Do;
        YSYNSEGM_NEXT = New_Seg_Ptr; ! Points to next block
        GOTO MORE_SEG;
       End; !@PK71135

     End; ! End of getting another synonym block

   End; ! End of multi-segment processing

  ELSE

  Do;   ! Start of single segment processing  @PK71135

  ! Get a 12-byte message synonym block for each data segment
  ?ZDFSPOOL FUNC(GET)
            LEN(Length(YSYNSEGM))
            LIST(ADDR(DFSPOOL_PARMLIST))
            POOL(LUMP)
            ADDR(Seg_Synonym_Ptr)
            RC(Local_RC);

  If (Local_RC ¬= 0) Then
    Do;
      Call Cut_OTMA_67D0('DFSYSCP0',
                         ' NO LUMP  ',
                         LCRETOKN,
                         Dummy_Char8,
                         Dummy_char4,
                         Dummy_char4,
                         20,
                         7);

      ! Pass back an extended error reason code
      If (YSCMRSN) Then Clt_Code = Err_REQ_No_Lump;
      Return Code(Err_in_OTMA_Callout);
    End;

  YSYNSEGM_NEXT = 0;
  YSYNSEGM_MSG  = YSCMREQA; ! Point to data segment
  YSYNSEGM_LEN  = Req_Data_Len; ! Save the segment length

  Awocbufa = Seg_Synonym_Ptr;

  End;  ! End of single segment processing

  Return Code(Local_Rc);

 End Process_Request_Data;

 /********************************************************************/
 /*                                                                  */
 /* Get_LUBT_Shr  - GET LUBT LATCH SHARE FOR Sync callout Tpipe      */
 /*                                                                  */
 /********************************************************************/
 Get_Latch_Shr: Procedure
                Internal_Proc_Opt;

  Respecify (Awe) Based(Addr(Dfssvc_Msg_Awe));

  ?DFSCLM FUNC(GET) TYPE(LUBT) MODE(SHR) WAIT(YES)
            PARM(Awe) ECB(Ecb_Plas) ADDR(Tpipe_Latch_Hdr);

  Respecify (Awe) Based(Sync_AWE_Ptr);

 End Get_Latch_Shr;


 /********************************************************************/
 /*                                                                  */
 /* Get_LUBT_Excl - GET LUBT LATCH EXCLUSIVE FOR Sync callout Tpipe  */
 /*                                                                  */
 /********************************************************************/
 Get_Latch_Excl: Procedure
                Internal_Proc_Opt;

  Respecify (Awe) Based(Addr(Dfssvc_Msg_Awe));

  ?DFSCLM FUNC(GET) TYPE(LUBT) MODE(EXCL) WAIT(YES)
            PARM(Awe) ECB(Ecb_Plas) ADDR(Tpipe_Latch_Hdr);

  Respecify (Awe) Based(Sync_AWE_Ptr);

 End Get_Latch_Excl;

 /********************************************************************/
 /*                                                                  */
 /* Rel_LUBT_Excl - Release LUBT LATCH EXCLUSIVE FOR Sync callout Tp */
 /*                                                                  */
 /********************************************************************/
 Rel_Latch: Procedure
                Internal_Proc_Opt;

  Respecify (Awe) Based(Addr(Dfssvc_Msg_Awe));

  ?DFSCLM FUNC(REL) TYPE(LUBT) MODE(ANY) WAIT(YES)
            PARM(Awe) ECB(Ecb_Plas) ADDR(Tpipe_Latch_Hdr);

  Respecify (Awe) Based(Sync_AWE_Ptr);

 End Rel_Latch;


 /*********************************************************************/
 /* Subroutine - Chain_Sync_AWE_to_YQAB                              */
 /*                                                                   */
 /*    Insert the AWE, which represent this sync callout message, to  */
 /*    YQAB_CALLOUT chain. This chain is FIFO.                        */
 /*                                                                   */
 /*********************************************************************/
 Chain_Sync_AWE_to_YQAB: Procedure
               Internal_Proc_Opt;
  Declare
     Done               Bit(1),
     Curr_AWE_Ptr       Ptr(31),
     Next_AWE_ptr       Ptr(31),
     Local_RC           Fixed(31);


  Done = False;

  Awostate = AWO_SYNC_TO_BE_PROCESSED; ! ready to be processed by YQAB0

  ! Get the latch in order to insert a sync AWE to YQAB
  Call Get_Latch_Excl;

  ! Remember that Sync_AWE_Ptr points to our sync callout AWE

  Curr_AWE_Ptr = YQAB_Callout;

  ! If there is no Sync AWE in the chain
  If (Curr_AWE_Ptr = 0) Then

   Do;

    ! Add it into the head of the chain
    ?Dfscs Old(Curr_AWE_Ptr) New(Sync_AWE_Ptr)
        Loc(Yqab_Callout) Swapped(Done) Wait(NO);

    If (Done) Then
      Do;
A      ! increment the sync callout awe count                 !@PM27812
A      ?DFSCS LOC(TPIPE_SYNC_AWE_CTX) ADD(1);                 !@PM27812
       ! Release the latch
       Call Rel_Latch     ;
       Return;
      End;

   End;

  ! There are other Sync AWE in the chain

 AddRetry:

  ! We need to find the last AWE in the chain, then add a new one

  ! First, let's get the next AWE in the chain
  Next_AWE_Ptr = Curr_AWE_Ptr->awenext;

  ! if Curr_AWE_Ptr points to the last AWE in the chain
  If (Next_AWE_Ptr = 0) Then

   Do;

    ! Add it into the head of the chain
    ?Dfscs Old(Next_AWE_Ptr) New(Sync_AWE_Ptr)
        Loc(Curr_AWE_Ptr->awenext) Swapped(Done) Wait(NO);

    If (Done) Then
     Do;
A     ! increment the sync callout awe count                  !@PM27812
A     ?DFSCS LOC(TPIPE_SYNC_AWE_CTX) ADD(1);                  !@PM27812
      ! Release the latch
      Call Rel_Latch     ;
      Return;
     End;

    GoTo AddRetry; ! If Not Done, then retry it

   End;

  Else

   ! The Curr_AWE_Ptr is not the last one
   Do;

    ! Get the next AWE in the chain
    Curr_AWE_Ptr = Next_AWE_Ptr;

    GoTo AddRetry; ! If Not Done, then retry it

   End;


 End Chain_Sync_AWE_to_YQAB;

 /*********************************************************************/
 /* Subroutine - Deliver_Sync_Callout_MSg                             */
 /*    Let's wait up DFSYQAB0 to deliver this sync callout msg        */
 /*                                                                   */
 /*    If exit with error, simply return back to caller.              */
 /*                                                                   */
 /*********************************************************************/
 Deliver_Sync_Callout_Msg:Procedure
               Internal_Proc_Opt;
  Declare
     Local_RC           Fixed(31);

  Local_RC = 0; ! Pre-set a good return code

  ! Check if we have an input EXT routine address
  If (YSCMRET  = 0) Then
      Do;
        If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
        Out_Msg = 'DFSYSCP0 NO EXT ROUTINE ........   !!';
        Call Do_Hard_WTO(Out_Msg);
        End;
      End;
  Else
      Do; ! Need to save EXT routine info for DFSYSND0
        If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
        Out_Msg = 'DFSYSCP0 HAS EXT ROUTINE ..........!!';
        Call Do_Hard_WTO(Out_Msg);
        End;

        AWEENQER = YSCMRET ;  ! SAVE THE EXIT ROUTINE ADDRESS @PK71135
        AWEQH    = YSCMUDA ;  ! SAVE THE DATA FOR EXIT ROUTINE@PK71135


      End;   ! End of saving exit routine info

  ! increment the sync callout message count

  Do;                                                         !@PM12142
    DCL Done                Bit(1);                           !@PM12142
    DCL TmpCt               FIXED(32);                        !@PM12142
    DCL NewCT               FIXED(32);                        !@PM12142

    TmpCt =TPIPE_SYNC_OUT_CT;                                 !@PM12142

    do until(Done);
      NewCt   = TmpCt;               !Copy current value @PM12142
      If (NewCt >= 65535) Then
        NewCt = 1;                   !Reset to 1 if >= 65535
      Else
        NewCt = NewCt + 1;           !Increment by 1  @PM09695

      ?Dfscs Old(TmpCt)
             New(NewCt)
             Loc(TPIPE_SYNC_OUT_CT)
             Wait(NO)
             Swapped(Done);
    end; !do until Done                                        @PM12142

  End;                                                        !@PM12142


  If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
  Out_Msg = 'DFSYSCP0 calls YQAB w/ TMEM= ' || TPIPE_MEMBER_NAME ||
            ' TPIPE= ' || TPIPE_LTERM_NAME; !@PK81543
  Call Do_Hard_WTO(Out_Msg);
  End;

  ! Set the PST flag
  PSTSYNO = True;

  ! Set the YQAB address
  PSTSYQAB = CLBlu6wa;

  Call Activate_OTMA_Block RetCode(local_rc) ;

  If (Local_RC ¬= 0) Then
   Do;

    AWOSTATE = awo_sync_initial; ! back to basic state

    Call Cut_OTMA_67D0('DFSYSCP0',
                       ' POST FAIL',
                       LCRETOKN,
                       Dummy_Char8,
                       Dummy_char4,
                       Dummy_char4,
                       20,
                       11);

    ! Pass back an extended error reason code
    If (YSCMRSN) Then Clt_Code = Err_REQ_Bad_Ipost;
    Return Code(Err_in_OTMA_Callout);

   End;

  Return Code(Local_Rc);

 End Deliver_Sync_Callout_Msg;

 /*********************************************************************/
 /* Subroutine - Release_Prefix_, Sync_Blocks, and YTIB               */
 /*                                                                   */
 /*********************************************************************/
 Release_Prefix_and_Syn_Blocks:Procedure
               Internal_Proc_Opt;
  Declare
     Next_Synonym_ptr   Ptr(31),
     Local_RC           Fixed(31);

  Local_RC = 0; ! Pre-set a good return code

  SELECT;

  WHEN (DestRoutDesc_icon) Do;

  If (Sync_AWE_Ptr = 0) Then LEAVE;

  ! If a severe error exists, keep this AWE for problem analysis
  ! YQAB_CALLOUT should have the AWE in the chain,
  ! DFS1269 will be issued to the console, and
  ! 67D0 log record should have the address of the problem AWE
  If (Awosyerr) Then Return;

  ! Let's first release the prefix buffer
  If (Awopfptr) THEN
  ?ZDFSPOOL FUNC(REL)
            POOL(LUMP)
            ADDR(Awopfptr)
            LIST(ADDR(DFSPOOL_PARMLIST))
            ECB(Addr(ECB_PLAS))
            SCD(Addr(SCD_PLAS))
            RC(local_rc);


  IF (Local_RC) THEN
    Do;
     Call Issue_Severe_Error(01);
    End;

  ! Let's release the message synonym blocks

  Local_RC = 0;
  Seg_Synonym_ptr = awocbufa;

  Do while ((Seg_Synonym_ptr ¬= 0) & (Local_RC = 0));

   Next_Synonym_ptr = YSYNSEGM_NEXT;

   ?ZDFSPOOL FUNC(REL)
             POOL(LUMP)
             ADDR(Seg_Synonym_ptr)
             LIST(ADDR(DFSPOOL_PARMLIST))
             ECB(Addr(ECB_PLAS))
             SCD(Addr(SCD_PLAS))
             RC(local_rc);

   IF (Local_RC) THEN
    Do;
     Call Issue_Severe_Error(02);
    End;

   Seg_Synonym_ptr = Next_Synonym_ptr;

  End; ! end of free the synonym blocks

  End; ! End of cleanup for DestRoutDesc_icon

  WHEN (DestRoutDesc_SPS) Do; ! ICAL for synchronous program switch

   If (Seg_Synonym_ptr) Then Do;

    Do while ((Seg_Synonym_ptr ¬= 0) & (Local_RC = 0));

    Next_Synonym_ptr = YSYNSEGM_NEXT;

    ?ZDFSPOOL FUNC(REL)
             POOL(LUMP)
             ADDR(Seg_Synonym_ptr)
             LIST(ADDR(DFSPOOL_PARMLIST))
             ECB(Addr(ECB_PLAS))
             SCD(Addr(SCD_PLAS))
             RC(local_rc);

    IF (Local_RC) THEN
     Do;
      Call Issue_Severe_Error(07);
     End;

    Seg_Synonym_ptr = Next_Synonym_ptr;

    End; ! end of do-while

   End; ! End of message synonym blocks

   If (SPS_YTIB_Clb) Then Do; ! Release YTIB

     If (Ical_free_ytib) Then; ! Let's free the YTIB
     Else Do; ! Check if we need to avoid releasing YTIB

       ! If special flag is set, let DFSYLUS0/DFSYTIB0 release YTIB
       If ((Ical_timeout) & (awocltcd = SPS_Timeout)) |
          (ICAL_CM0_resp) |
          (Ical_CMD) Then Do;

         If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
           Out_Msg = 'DFSYSCP0 DOES NOT release YTIB...!!';
           Call Do_Hard_WTO(Out_Msg);
         End;

         RETURN;

       End; ! end of NOT releasing YTIB

     End; ! End of checking other flags for NOT releasing YTIB

     ?DFSYDST Func(RELEASE)
              Type(TIB)
              BLKA(SPS_YTIB_Clb)
              RetCode(local_rc);

     If (local_rc) Then
      Call Issue_Severe_Error(08);
     ELSE Do;

       If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
         Out_Msg = 'DFSYSCP0 FREE the YTIB............';
         Call Do_Hard_WTO(Out_Msg);
       End;

     End; ! End of issuing WTO msg

   End; ! End of YTIB cleanup

  End; ! End of cleanup for DestRoutDesc_SPS

  OTHERWISE;

  END; ! End of SELECT



 End Release_Prefix_and_Syn_Blocks;

 /*********************************************************************/
 /* Centralize the severe error processing                            */
 /*********************************************************************/

 Issue_Severe_Error: Procedure(Error_Num)
    INTERNAL_PROC_OPT;

 dcl  Error_Num      Fixed(32) ;
 dcl  Result         Fixed(31) ;

   ! Turn on a flag in AWE for this severe error
   ! This AWE is pointed by Sync_AWE_ptr
   Awosyerr = True;
   Awostate = AWO_SYNC_SEVERE_ERROR;
   YQAB_SYNC_ERROR = True; ! Internal error found in this YQAB

   ?DFSYSER  Err(Error_Num)
             Trace(Sync_AWE_Ptr)
             Tpipe(YQAB_LTERM)
             Result(Result);


 End Issue_Severe_Error;

 /*********************************************************************/
 /* Release SYNC AWE storage and decrement the AWE count in TPIPE     */
 /*********************************************************************/

 Rel_Sync_AWE_Stg: Procedure
    INTERNAL_PROC_OPT;

  Dcl  Local_RC       Fixed(31) ;

  ! If a severe error exists, keep this AWE for problem analysis
  ! YQAB_CALLOUT should have the AWE in the chain,
  ! DFS1269 will be issued to the console, and
  ! 67D0 log record should have the address of the problem AWE
  If (Awosyerr) Then Return;

  ?DFSBCB  FUNC(REL) BLK(AWE) ITASK(YES) QSAV(NO)
           BLKA(Sync_AWE_Ptr->awoy) RetCode(Local_RC);

  If (Local_RC) then
    Call Issue_Severe_Error(03);
  ELSE
    ?DFSCS LOC(TPIPE_SYNC_AWE_CT) ADD(-1);

 End Rel_Sync_AWE_Stg;

 /*********************************************************************/
 /* Subroutine - Total_Cleanup                                        */
 /*    Release all the unused stoarges for this callout request       */
 /*                                                                   */
 /*    Can be called ONLY when a valid descriptor entry found         */
 /*********************************************************************/

 Total_Cleanup: Procedure
    INTERNAL_PROC_OPT;

  Dcl  Local_RC       Fixed(31) ;

  ! Release the descriptor storage if needed
  If (drdesc_ptr) Then Call Release_Descriptor_STG;

  ! First release the prefix, synonym blocks, YTIB
  Call Release_Prefix_and_Syn_Blocks;

  ! Second, release the AWE storage
  If (Sync_AWE_Ptr) Then Call Rel_Sync_AWE_Stg;

 End Total_Cleanup;

 /*********************************************************************/
 /* Subroutine - Delete_Sync_AWE_from_YQAB                            */
 /*    Release the AWE block that represents this sync callout msg    */
 /*    from YQAB chain.                                               */
 /*    Note: Sync_AWE_Ptr points to the AWE to be removed             */
 /*                                                                   */
 /*********************************************************************/
 Delete_Sync_AWE_from_YQAB:Procedure
               Internal_Proc_Opt;

  Declare
     Done               Bit(1),
     FOUND              Bit(1),
     Prev_AWE_Ptr       Ptr(31),
     Curr_AWE_Ptr       Ptr(31),
     Next_AWE_ptr       Ptr(31),
     Local_RC           Fixed(31);


  Done  = False;
  FOUND = False;

  ! If a severe error exists, keep this AWE for problem analysis
  ! YQAB_CALLOUT should have the AWE in the chain,
  ! DFS1269 will be issued to the console, and
  ! 67D0 log record should have the address of the problem AWE
  If (Awosyerr) Then Return;

  ! Get the latch in order to delete a sync AWE to YQAB
  Call Get_Latch_Excl;

  ! Remember that Sync_AWE_Ptr points to our sync callout AWE

  Curr_AWE_Ptr = YQAB_Callout;

 DEL1001:

  ! If there is no Sync AWE in the chain
  If (Curr_AWE_Ptr = 0) Then
   Do;
    Call Rel_Latch     ;
    Call Issue_Severe_Error(04);
    Return;
   End;

  Next_AWE_Ptr = Curr_AWE_Ptr->awenext;

  ! Check if the first AWE in the chain is to be removed
  If (Curr_AWE_Ptr->aworname = aworname) Then
   Do;

    ! Del it into the head of the chain
    ?Dfscs Old(Curr_AWE_Ptr) New(Next_AWE_Ptr)
        Loc(Yqab_Callout) Swapped(Done) Wait(NO);

    If (Done) Then
      Do;
       Call Rel_Latch     ;
       Return;
      End;
    ELSE
      Goto DEL1001;

   End;

  ELSE

   ! The AWE to be removed is NOT the first AWE in YQAB_CALLOUT chain

   Do While (¬FOUND);

    Prev_AWE_Ptr = Curr_AWE_Ptr;
    Curr_AWE_Ptr = Prev_AWE_Ptr->awenext;

 DEL1002:

    If (Curr_AWE_Ptr = 0) & (¬FOUND) Then
      Do;
       Call Rel_Latch     ;
       Call Issue_Severe_Error(05);
       Return;
      End;

    Next_AWE_Ptr = Curr_AWE_Ptr->awenext;

    If (Curr_AWE_Ptr->aworname = aworname) Then

     Do;

      ! Del it from the chain
      ?Dfscs Old(Curr_AWE_Ptr) New(Next_AWE_Ptr)
          Loc(Prev_AWE_Ptr->awenext) Swapped(Done) Wait(NO);

      If (Done) Then
        Do;
         FOUND= True;
         Call Rel_Latch     ;
        End;
      ELSE
        Goto DEL1002;

     End;

   End; ! End of Do while


 End Delete_Sync_AWE_from_YQAB;

 /*********************************************************************/
 /* Subroutine - Proc_Sync_Callout                                    */
 /*    Let's process a sync callout request                           */
 /*                                                                   */
 /*    If exit w/o error, call IWAIT for response message             */
 /*    If exit w/ error, then we will return back to DFSDLA40         */
 /*********************************************************************/
 Proc_Sync_Callout:Procedure
               Internal_Proc_Opt;
  Declare
     Local_RC           Fixed(31);

  Local_RC = 0; ! Pre-set a good return code

  If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
  Out_Msg = 'DFSYSCP0 PROCESSING Sync Callout !!';
  Call Do_Hard_WTO(Out_Msg);
  End;

  /*******************************************************************/
  /* Search the OTMA tmember (this could be a super member.)         */
  /*******************************************************************/

  Call Search_OTMA_Member  Retcode(Local_RC);

  ! Exit if error
  If (Local_RC ¬= 0) Then
   Do;
     Call Release_Descriptor_STG;
     Return Code(Local_RC);
   End;

  /*******************************************************************/
  /* Find or Create a TPIPE for this Sync callout message            */
  /*******************************************************************/

  Call Get_OTMA_Tpipe Retcode(Local_RC);

  ! Exit if error
  If (Local_RC ¬= 0) Then
   Do;
     Call Release_Descriptor_STG;
     Return Code(Local_RC);
   End;

  /*******************************************************************/
  /* Obtain an AWE to represent this sync callout request            */
  /*******************************************************************/

  Call Get_Sync_AWE  Retcode(Local_RC);

  ! Exit if error
  If (Local_RC ¬= 0) Then
   Do;
     Call Release_Descriptor_STG;
     Return Code(Local_RC);
   End;

  /*******************************************************************/
  /* Build OTMA output prefix for this sync callout message          */
  /*******************************************************************/

  Call Build_Output_Prefix  Retcode(Local_RC);

  ! Exit if error
  If (Local_RC ¬= 0) Then
   Do;
    ! Need to do some clean-up before exit
    Call Total_Cleanup;
    Return Code(Local_RC);
   End;

  /*******************************************************************/
  /* Prepare the request data for this sync callout message          */
  /*******************************************************************/

  Call Process_Request_Data Retcode(Local_RC);

  ! Exit if error
  If (Local_RC ¬= 0) Then
   Do;
    ! Need to do some clean-up before exit
    Call Total_Cleanup;
    Return Code(Local_RC);
   End;

  /****************************************************** @PK81543 ***/
  /* Release the storage for descriptor entry                        */
  /*******************************************************************/
  Call Release_Descriptor_STG;

  /*******************************************************************/
  /* Cut SLOG code 78 if DC monitor is on                            */
  /*******************************************************************/
  Call Cut_SLOG_78;

  /*******************************************************************/
  /* Call timer routine to check for timeout                         */
  /*******************************************************************/
  Call Start_Sync_Timer;

  /*******************************************************************/
  /* Insert the AWE into YQAB. ICAL can run immediatelly if DFSYQAB0 */
  /* is active. This is why we need to call timer before.            */
  /*******************************************************************/
A ! Set the PST flag                                          !@PM27812
A PSTSYNO = True;                                             !@PM27812
A ! Set the YQAB address                                      !@PM27812
A PSTSYQAB = Clblu6wa;                                        !@PM27812

  ! Cut OTMT trace entry
  If (SCDTRAOA) Then ! @PM29046
   Do;
   Call OTMT_Trace(trace_tmamod_awe,
             11x,
             PSTFLAG2,
             Addr(ECB_PLAS),
             PSTSYQAB,
             AWORNAME);
   End;

  Call Chain_Sync_AWE_to_YQAB;

  /*******************************************************************/
  /* Time to activate DFSYQAB0 to deliver this sync callout msg      */
  /*******************************************************************/

  Call Deliver_Sync_Callout_MSg Retcode(Local_RC);

  ! Exit if error
  If (Local_RC ¬= 0) Then
   Do;

    ! Cut OTMT trace entry
    If (SCDTRAOA) Then ! @PM29046
    Call OTMT_Trace(trace_tmamod_awe,
               12x,
               PSTFLAG2,
               Addr(ECB_PLAS),
               PSTSYQAB,
               AWORNAME);

    ! Reset the PST flag
    PSTSYNO = False;
A   PSTSYQAB = 0;                                             !@PM27812

    Call Cut_SLOG_79;

    ! Delete the AWE from the YQAB chain
    Call Delete_Sync_AWE_from_YQAB;

    ! Need to do some clean-up before exit
    Call Total_Cleanup;

   End;


  Return Code(Local_Rc);

 End Proc_Sync_Callout;


 /*********************************************************************/
 /* Subroutine - Cut_SLOG_78                                          */
 /*    Call IMS Monitor to cut a DC monitor record before ICAL sends  */
 /*    to OTMA client                                                 */
 /*********************************************************************/
 Cut_SLOG_78:Procedure
               Internal_Proc_Opt;


  Dcl save_pstwrk1 fixed(31),     !Temp storage for PST area
      save_pstwrk2 fixed(31);     !Temp storage for PST area

  /**************************************************** @PK71135******/
  /* Call IMS monitor for sync callout start (SLOG code 78)          */
  /*******************************************************************/

   save_pstwrk1 = pstwrk1;        !Save existing value
   save_pstwrk2 = pstwrk2;        !Save existing value
   pstwrk1 = dest_name(1::4);     !Descriptor to PST work area
   pstwrk2 = dest_name(5::4);     !Descriptor to PST work area
   rfy (R0,R1,R5,R6,R14,R15) rstd;
   r5 = addr(ecb_plas);
   r6 = addr(scd_plas);
   gen refs(SCDMNTR0,SCDMONWK,SCDSTMN1,LMONARRY,              !@U14JAJ1
            LMONSLNT,USRXPMLN,R5,R6);                         !@U14JAJ1
   DFSSLOG CODE=78,PST=(R5),SCD=(R6),INCLCD=NO                 @U14JAJ1
   @endgen;
   rfy (R0,R1,R5,R6,R14,R15) unrstd;
   pstwrk1 = save_pstwrk1;        !Restore existing value
   pstwrk2 = save_pstwrk2;        !Restore existing value

 End Cut_SLOG_78;

 /*********************************************************************/
 /* Subroutine -   Cut_SLOG_79                                        */
 /*    Call IMS Monitor to cut a DC monitor record after ICAL receives*/
 /*    a response back from OTMA client                               */
 /*********************************************************************/
 Cut_SLOG_79:Procedure
               Internal_Proc_Opt;

  /*****************************************************  @PK71135****/
  /* Call IMS monitor for sync callout end (SLOG code 79)            */
  /*******************************************************************/

   rfy (R0,R1,R5,R6,R14,R15) rstd;
   r5 = addr(ecb_plas);
   r6 = addr(scd_plas);
   gen refs(SCDMNTR0,SCDMONWK,SCDSTMN1,R5,R6);                !@U14JAJ1
   DFSSLOG CODE=79,PST=(R5),SCD=(R6),INCLCD=NO                 @U14JAJ1
   @endgen;
   rfy (R0,R1,R5,R6,R14,R15) unrstd;


 End Cut_SLOG_79;



 /*********************************************************************/
 /* Subroutine - Proc_NAK_From_YQAB                                   */
 /*    Let's process a NAK from DFSYQAB0                              */
 /*                                                                   */
 /*    If exit w/o error, simply return back to caller.               */
 /*                                                                   */
 /*********************************************************************/
 Proc_NAK_From_YQAB:Procedure
               Internal_Proc_Opt;
  Declare
     Local_RC           Fixed(31);

  Local_RC = 0; ! Pre-set a good return code
  If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
  Out_Msg = 'DFSYSCP0 PROCESSING CLIENT NAK   !!';
  Call Do_Hard_WTO(Out_Msg);
  End;
  ! Pass back any client error code
  If (YSCMRSN) Then Clt_Code = awocltcd;

  SELECT (Awostate);

   WHEN (AWO_SYNC_NAK_IN)
     Do;

      !  See if rejected for RT not supporting Control Data   !@U14CDC1
      If (AWOCLTCD = Err_CTLDATA_Bad_RTP) Then                !@U14CDC1
         Local_RC = Err_Sync_Control_Data;                    !@U14CDC1
      Else                                                    !@U14CDC1
         Local_RC = Err_Client_Reject;                        !@U14CDC1

      Call Cut_OTMA_67D0('DFSYSCP0',
                         ' GOT NACK ',
                         LCRETOKN,
                         ' SIMPLE ',
                         Dummy_char4,
                         Dummy_char4,
                         20,
                         8);


     End;

   WHEN (AWO_SYNC_NAK_RR_IN)
     Do;

      Local_RC = Err_Client_Reject;
      Call Cut_OTMA_67D0('DFSYSCP0',
                         ' GOT NACK ',
                         LCRETOKN,
                         'CONTINUE',
                         Dummy_char4,
                         Dummy_char4,
                         20,
                         9);


     End;


   OTHERWISE

     ! unknown status code from DFSYQAB0
     Do;

       !  See if rejected for RT not supporting Control Data  !@U14CDC1
       If (AWOCLTCD = Err_CTLDATA_Bad_RTP) Then               !@U14CDC1
          Local_RC = Err_Sync_Control_Data;                   !@U14CDC1
       Else                                                   !@U14CDC1
          Local_RC = Err_Client_Reject;                       !@U14CDC1

       Call Cut_OTMA_67D0('DFSYSCP0',
                          ' GOT NACK ',
                          LCRETOKN,
                          'UNKNOWN ',
                          Dummy_char4,
                          Dummy_char4,
                          20,
                          10);

     End;

  END; ! End of SELECT

  ! Delete the AWE from the YQAB chain
  Call Delete_Sync_AWE_from_YQAB;

  ! We need to free the prefix, synonym blocks, and AWE
  Call Total_Cleanup;

  Return Code(Local_Rc);

 End Proc_NAK_From_YQAB;

 /********************************************************* @PK81543 **/
 /*                                                                   */
 /* Subroutine - Proc_ACK_Timeout                                     */
 /*    Let's process the ACK timeout from DFSYQAB0                    */
 /*                                                                   */
 /*********************************************************************/
 Proc_ACK_Timeout:Procedure
               Internal_Proc_Opt; !@PK81543
  Declare
     Local_RC           Fixed(31);

  Local_RC = 0; ! Pre-set a good return code

  If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
  Out_Msg = 'DFSYSCP0 processing ACK Timeout  !!';
  Call Do_Hard_WTO(Out_Msg);
  End;

  If (YSCMRSN) Then Clt_Code = 8; ! set extended reason code PM29046

  Local_RC = Err_Got_Timeout;

  Call Cut_OTMA_67D0('DFSYSCP0',
                     ' ACK T/O  ',
                     ' UNKNOWN STATUS ',
                     Dest_Name,
                     Dummy_char4,
                     Dummy_char4,
                     16,
                     20);


  ! Delete the AWE from the YQAB chain
  Call Delete_Sync_AWE_from_YQAB;

  ! We need to free the prefix, synonym blocks, and AWE
  Call Total_Cleanup;

  Return Code(Local_Rc);

 End Proc_ACK_Timeout; !@PK81543

 /*********************************************************************/
 /* Subroutine - Proc_STOP_from_CMD                                   */
 /* Process the stop command to cancel the sync callout message       */
 /*********************************************************************/

 Proc_STOP_from_CMD:Procedure
    INTERNAL_PROC_OPT;

  Dcl  Local_RC       Fixed(31) ;
  If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
  Out_Msg = 'DFSYSCP0 PROCESSING CMD NAK      !!!!';
  Call Do_Hard_WTO(Out_Msg);
  End;

  Local_RC = Err_Got_STOP_CMD;

  Call Cut_OTMA_67D0('DFSYSCP0',
                     'CMD ISSUED',
                     LCRETOKN,
                     Dummy_Char8,
                     Dummy_char4,
                     Dummy_char4,
                     32,
                     2);


  ! Delete the AWE from the YQAB chain
  If (DestRoutDesc_icon) Then Call Delete_Sync_AWE_from_YQAB;

  ! We need to free the prefix, synonym blocks, and AWE
  Call Total_Cleanup;

  Return Code(Local_Rc);

 End Proc_STOP_from_CMD;

 /*********************************************************************/
 /* Subroutine - Proc_RSP_From_YMEM                                   */
 /*    Let's process the response from DFSYMEM0                       */
 /*                                                                   */
 /*                                                                   */
 /*********************************************************************/
 Proc_RSP_From_YMEM:Procedure
               Internal_Proc_Opt;
  Declare
     Local_RC           Fixed(31);

  Local_RC = 0; ! Pre-set a good return code
  If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
  Out_Msg = 'DFSYSCP0 PROCESSING RESPONSE     !!!!';
  Call Do_Hard_WTO(Out_Msg);
  End;

  ! Pass back response data only if the length is > 0     @PM29046
  If (AWOSTLEN > 0) Then Do;                             !@PM29046
    ! Pass back the resp len
    OAuse   = AWOSTLEN;
    ! Pass back the response buf address
    RspOTMA = Awosyrsp;
  End; !@PM29046

  ! Pass back any client error code
  If (YSCMRSN) & (awocltcd ¬= 0) Then

    Do;
     If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
     Out_Msg = 'DFSYSCP0 PASSES BACK A SENSE CODE!!!!';
     Call Do_Hard_WTO(Out_Msg);
     End;
     Clt_Code = awocltcd;

     Local_RC = Err_In_Response_Data;

    End;

   ! Choose the bad RC to return to DFSDLA40 for ICAL          @PM29046
   If (AWORSERR) Then Local_RC = Err_In_OTMA_Response;        !@PM29046
   Else Do;
     If (AWOICERR) Then Do;                                   !@PM29046

       If (AWOSTLEN > 0) Then ! If we have raw data to return  @PM29046
         Local_RC = Err_found_by_client_W_data;               !@PM29046
       Else
         Local_RC = Err_found_by_client;                      !@PM29046

     End;
   End; ! end of non-AWORSERR                                  @PM29046

  If (SCDTRAOA) Then Do;

      Call OTMT_Trace(trace_tmamod_awe,
             16x,
             PSTFLAG2,
             Local_RC,
             Clt_Code,
             AWORNAME);

  End;

  ! Delete the AWE from the YQAB chain
  Call Delete_Sync_AWE_from_YQAB;

  ! Need to do some clean-up before exit
  Call Total_Cleanup;


  Return Code(Local_Rc);

 End Proc_RSP_From_YMEM;

 /*********************************************************************/
 /*********************************************************************/
 /*                                                                   */
 /* Subroutine : Proc_Send_Receive                                    */
 /*                                                                   */
 /*********************************************************************/
 /*********************************************************************/

 Proc_Send_Receive: Procedure
                    Internal_Proc_Opt;

  Dcl Local_RC fixed(31);      ! Local return code for this routine


  If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
  Out_Msg = 'DFSYSCP0 PROCESSING SENDRCV REQUEST!!';
  Call Do_Hard_WTO(Out_Msg);
  End;

  /*******************************************************************/
  /* Initialization                                                  */
  /*******************************************************************/

  Local_RC = 0;                ! Assume everything is OK
  drdesc_ptr = 0;              ! Assume there is no descriptor
  SPS_Ytib_Clb = 0;            ! Assume there is no YTIB CLB for SPS
  Seg_Synonym_Ptr = 0;         ! Assume there is no syn block
  Sync_AWE_Ptr = 0;            ! Assume no sync callout AWE

  /*******************************************************************/
  /* Check the system status first                                   */
  /*******************************************************************/

  Call Check_System_Status Retcode(Local_RC);

  ! Exit if error
E If (Local_RC ¬= 0) Then                                     !@PM33681
E   Do;                                                       !@PM33681
E     If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;      !@PM33681
E       Out_Msg = 'DFSYSCP0 Check_System_Status RC=XXXXXXXX'; !@PM33681
E       Call C2X(Local_RC,4,Out_Msg(33),8);                   !@PM33681
E       Call Do_Hard_WTO(Out_Msg);                            !@PM33681
E     End;                                                    !@PM33681
E     Return Code(Local_RC);                                  !@PM33681
E   End;                                                      !@PM33681

  /*******************************************************************/
  /* Search Destination Routing Descriptors for a match              */
  /*******************************************************************/

  Call Search_DestRoutDesc Retcode(Local_RC);

  ! Exit if error
  If (Local_RC ¬= 0) Then
   Do;
E    If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;      !@PM33681
E      Out_Msg = 'DFSYSCP0 Search_DestRoutDesc RC=XXXXXXXX'; !@PM33681
E      Call C2X(Local_RC,4,Out_Msg(33),8);                   !@PM33681
E      Call Do_Hard_WTO(Out_Msg);                            !@PM33681
E    End;                                                    !@PM33681

     Call Release_Descriptor_STG;
     Return Code(Local_RC);
   End;
                                                              !@U14CDC1
  ! ********************************************************* !@U14CDC1
  !  Check if Control Data passed                             !@U14CDC1
  !  - then descriptor must be IMSCON                         !@U14CDC1
  ! ********************************************************* !@U14CDC1
  If (YSCMCTLA) Then do;          ! Does Control Data exist ? !@U14CDC1
     Control_Data = True;                                     !@U14CDC1
                                                              !@U14CDC1
     If Tscd_Lum_Debug then Do;                               !@U14CDC1
       Out_Msg = 'DFSYSCP0: Control Data passed ';            !@U14CDC1
       Call Do_WTO(Out_Msg);                                  !@U14CDC1
     end;                                                     !@U14CDC1
                                                              !@U14CDC1
     If (¬DestRoutDesc_ICON) Then do;                         !@U14CDC1
        If Tscd_Lum_Debug then Do;                            !@U14CDC1
          Out_Msg = 'DFSYSCP0: Control Data but not ICON ';   !@U14CDC1
          Call Do_WTO(Out_Msg);                               !@U14CDC1
        end;                                                  !@U14CDC1
                                                              !@U14CDC1
        ! Pass back an extended error reason code             !@U14CDC1
        If (YSCMRSN) Then Clt_Code = Err_CTLDATA_Not_ICON;    !@U14CDC1
        Return Code(Err_Sync_Control_Data);                   !@U14CDC1
     End;    ! End of ICON test                               !@U14CDC1
                                                              !@U14CDC1
     Call Validate_Control_Data Retcode(Local_RC);            !@U14CDC1
                                                              !@U14CDC1
     ! Validate will set specific errors if found             !@U14CDC1
     If (Local_RC) Then                                       !@U14CDC1
        Return Code(Err_Sync_Control_Data);                   !@U14CDC1
                                                              !@U14CDC1
  End;    ! End of Control Data                               !@U14CDC1

  /*******************************************************************/
  /* If SPS (IMSTRAN) and SHRQ, THEN MUST BE MINVERS >= 13.1         */
  /*******************************************************************/
  If (DestRoutDesc_SPS) & (Scdshrdq) &  ! IMSTRAN SHRQ        !@U13CDCA
     (RCWBMVER < RCWBMV13) Then do;     ! with bad MINVERS    !@U13CDCA
                                                              !@U13CDCA
      Call Release_Descriptor_STG;                            !@U13CDCA
                                                              !@U13CDCA
      If Tscd_Lum_Debug then Do;                              !@U13CDCA
        Out_Msg = 'DFSYSCP0: SPS SHRQ NOT MINVERS 13.1 ';     !@U13CDCA
        Call Do_WTO(Out_Msg);                                 !@U13CDCA
      end;                                                    !@U13CDCA
                                                              !@U13CDCA
      ! Pass back an extended error reason code               !@U13CDCA
      If (YSCMRSN) Then Clt_Code = Err_SPS_MINVERS;           !@U13CDCA
      Return Code(Err_Sync_P2P_Specific);                     !@U13CDCA
                                                              !@U13CDCA
  end;      ! done bad MINVERS                                !@U13CDCA
                                                              !@U13CDCA

  /*******************************************************************/
  /* Validate the input timeout value and determine the timeout      */
  /*  value for this sync callout message                            */
  /*******************************************************************/

  Call Get_Timeout_Value  Retcode(Local_RC);

  ! Exit if error
  If (Local_RC ¬= 0) Then
   Do;
     Call Release_Descriptor_STG;
     Return Code(Local_RC);
   End;

  ! Perform action based on type of descriptor found
  SELECT;

    WHEN (DestRoutDesc_icon) Do; ! ICAL to IMS Connect

      Call Proc_Sync_Callout Retcode(Local_RC);
      If (Local_RC ¬= 0) Then Return Code(Local_RC); ! Exit if error

    End;

    WHEN (DestRoutDesc_SPS) Do; ! ICAL for synchronous program switch

      Call Proc_Sync_Prog_Switch Retcode(Local_RC);
      If (Local_RC ¬= 0) Then Return Code(Local_RC); ! Exit if error

    End;

    OTHERWISE;

  END; ! End of SELECT


  /*******************************************************************/
  /* WAIT HERE                                                       */
  /*******************************************************************/

  ?IWAIT ECB(ECB_PLAS) ;

  /*****************************************************  @PK71135****/
  /* Call IMS monitor for sync callout end (SLOG code 79)            */
  /*******************************************************************/

   Call Cut_SLOG_79;

  /*******************************************************************/
  /* WAKE UP HERE!!                                                  */
  /* Perform action based on the post code                           */
  /*******************************************************************/

  If (SCDTRAOA) Then ! @PM29046
  Call OTMT_Trace(trace_tmamod_awe,
             13x,
             PSTFLAG2,
             Addr(ECB_PLAS),
             PSTSYQAB,
             AWORNAME);


  If (DestRoutDesc_icon) |
     (DESTROUTDESC_SPS) then do;
    ! Reset the PST flag
    PSTSYNO = False;
A   PSTSYQAB = 0;                                             !@PM27812
  End;

  Select;

     ! Check if a NAK is returned from DFSYTIB0 for SPS
     When (ECB_PLAS(2::3) = 'PTB') Do;

       CALL Cancel_Sync_Timer;

       CALL Proc_NAK_From_YTIB Retcode(Local_RC);

     End;

     ! Check if a response is returned from DFSYSLM0 for SPS
     When (ECB_PLAS(2::3) = 'PSL') Do;

       CALL Cancel_Sync_Timer;

       CALL Proc_RSP_From_YSLM Retcode(Local_RC);

     End;

     ! Check if an abort has occurred from DFSYLUS0 for SPS
     When (ECB_PLAS(2::3) = 'PLU') Do;

       CALL Cancel_Sync_Timer;

       CALL Proc_Abort_From_YLUS Retcode(Local_RC);

     End;



     ! Pass back a CM0 DFS message

     When (ECB_PLAS(2::3) = 'PLT') Do;

       CALL Cancel_Sync_Timer;

       If (SCDTRAOA) Then ! Cut a table trace if needed
       Call OTMT_Trace(trace_tmamod_p2p,
                  33x,
                  PSTFLAG2,
                  Addr(ECB_PLAS),
                  AWOCLTCD,
                  AWORNAME);


       ! if YTIB needs to be freed, set a flag
       If (AWOIFTIB) Then ICAL_Free_YTIB = True;

       ICAL_CM0_resp = True; ! This is a CM0 output
       CALL Proc_Abort_From_YLUS Retcode(Local_RC);

     End;

     ! Pass back a CM0 response message

     When (ECB_PLAS(2::3) = 'PSK') Do;

       CALL Cancel_Sync_Timer;

       If (SCDTRAOA) Then ! Cut a table trace if needed
       Call OTMT_Trace(trace_tmamod_p2p,
                  34x,
                  PSTFLAG2,
                  Addr(ECB_PLAS),
                  AWOCLTCD,
                  AWORNAME);


       ! if YTIB needs to be freed, set a flag
       If (AWOIFTIB) Then ICAL_Free_YTIB = True;

       ICAL_CM0_resp = True; ! This is a CM0 output
       CALL Proc_RSP_From_YSLM Retcode(Local_RC);

     End;

     ! Check if a NAK is returned from DFSYQAB0
     When (ECB_PLAS(2::3) = 'QAB') Do;

       If (SCDTRAOA) Then ! @PM29046
       Call OTMT_Trace(trace_tmamod_awe,
                  14x,
                  PSTFLAG2,
                  Addr(ECB_PLAS),
                  PSTSYQAB,
                  AWORNAME);

       CALL Cancel_Sync_Timer;
       CALL Proc_NAK_From_YQAB Retcode(Local_RC);
E      If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;     !@PM33681
E        Out_Msg = 'DFSYSCP0 Proc_NAK_From_YQAB RC=XXXXXXXX'; !@PM33681
E        Call C2X(Local_RC,4,Out_Msg(32),8);                  !@PM33681
E        Call Do_Hard_WTO(Out_Msg);                           !@PM33681
E      End;                                                   !@PM33681


     End;

     ! Check if a ACK Timeout is returned from DFSYQAB0
     When (ECB_PLAS(2::3) = 'ATO') Do;                       !@PK81543

       If (SCDTRAOA) Then ! @PM29046
       Call OTMT_Trace(trace_tmamod_awe,
                  19x,
                  PSTFLAG2,
                  Addr(ECB_PLAS),
                  PSTSYQAB,
                  AWORNAME);

       CALL Cancel_Sync_Timer;
       CALL Proc_ACK_Timeout Retcode(Local_RC);
E      If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;     !@PM33681
E        Out_Msg = 'DFSYSCP0 Proc_ACK_Timeout RC=XXXXXXXX';   !@PM33681
E        Call C2X(Local_RC,4,Out_Msg(30),8);                  !@PM33681
E        Call Do_Hard_WTO(Out_Msg);                           !@PM33681
E      End;                                                   !@PM33681

     End;                                                    !@PK81543

     ! Check if command module cancelled the sync callout msg
     When ((ECB_PLAS(2::3) = 'CMD') |
           (ECB_PLAS(2::3) = 'ABD') ) Do;

       If (SCDTRAOA) Then Do;

         If (DESTROUTDESC_SPS) Then
           Call OTMT_Trace(trace_tmamod_p2p,
                      35x,
                      PSTFLAG2,
                      Err_Got_STOP_CMD,
                      AWOCLTCD,
                      AWORNAME);
         Else
           Call OTMT_Trace(trace_tmamod_awe,
                  15x,
                  PSTFLAG2,
                  Addr(ECB_PLAS),
                  PSTSYQAB,
                  AWORNAME);
       End;

       CALL Cancel_Sync_Timer;

       ! if YTIB needs to be freed for a SPS msg, set a flag
       If (DestRoutDesc_SPS)&(¬SPS_Replychk)&(AWOIFTIB) Then
        ICAL_Free_YTIB = True;

       Ical_Cmd     = True; ! CMD issued to end ICAL
       CALL Proc_STOP_from_CMD Retcode(Local_RC);
E      If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;     !@PM33681
E        Out_Msg = 'DFSYSCP0 Proc_STOP_from_CMD RC=XXXXXXXX'; !@PM33681
E        Call C2X(Local_RC,4,Out_Msg(32),8);                  !@PM33681
E        Call Do_Hard_WTO(Out_Msg);                           !@PM33681
E      End;                                                   !@PM33681

     End;

     ! Check if the response is returned from DFSYMEM0
     When (ECB_PLAS(2::3) = 'MEM') Do;

       If (SCDTRAOA) Then Do;

           Call OTMT_Trace(trace_tmamod_awe,
                  16x,
                  PSTFLAG2,
                  Addr(ECB_PLAS),
                  PSTSYQAB,
                  AWORNAME);

       End;

       CALL Cancel_Sync_Timer;
       CALL Proc_RSP_From_YMEM Retcode(Local_RC);
E      If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;     !@PM33681
E        Out_Msg = 'DFSYSCP0 Proc_RSPP_From_YMEM RC=XXXXXXXX';!@PM33681
E        Call C2X(Local_RC,4,Out_Msg(33),8);                  !@PM33681
E        Call Do_Hard_WTO(Out_Msg);                           !@PM33681
E      End;                                                   !@PM33681

     End;

     ! Check if this is a timeout
     When (ECB_PLAS(2::3) = 'TMO') Do;


       ! if YTIB needs to be freed for a SPS msg, set a flag
       If (DestRoutDesc_SPS)&(¬SPS_Replychk)&(AWOIFTIB) Then
        ICAL_Free_YTIB = True;

       Ical_Timeout = True; ! Timeout is reported
       CALL Proc_TimeOut Retcode(Local_RC);
E      If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;     !@PM33681
E        Out_Msg = 'DFSYSCP0 Proc_TimeOut RC=XXXXXXXX';       !@PM33681
E        Call C2X(Local_RC,4,Out_Msg(26),8);                  !@PM33681
E        Call Do_Hard_WTO(Out_Msg);                           !@PM33681
E      End;                                                   !@PM33681

     End;


     ! Unknown cases
     Otherwise Do;

       If (SCDTRAOA) Then ! @PM29046
       Call OTMT_Trace(trace_tmamod_awe,
                  18x,
                  PSTFLAG2,
                  Addr(ECB_PLAS),
                  PSTSYQAB,
                  AWORNAME);

       Call Cut_OTMA_67D0('DFSYSCP0',
                               'POSTERROR ',
                               LCRETOKN,
                               Dummy_Char8,
                               Dummy_char4,
                               ECB_PLAS(1::4),
                               20,
                               15);


       ! And let timeout handles this case.

     End;


  End; ! End of Select


   Return Code(Local_RC);


 End Proc_Send_Receive;

  /*******************************************************************/
  /* Subroutine - Prepare_Segments_For_SPS                           */
  /*    Build input segments with OTMA prefix for request data       */
  /*                                                                 */
  /*    If exit w/o error, Seg_Synonym_Ptr points to chain of        */
  /*    message synonym blocks, which point to the segments          */
  /*******************************************************************/
  Prepare_Segments_For_SPS: Procedure
                Internal_Proc_Opt;
   Declare
       Local_RC           Fixed(31);
   Dcl Curr_Pos           Ptr(31);
   Dcl Curr_Len           Fixed(31);
   Dcl Last_Pos           Ptr(31);
   Dcl Next_pos           Ptr(31);
   Dcl First_Seg          Bit(1);
   Dcl More               Bit(1);
   Dcl Temp_Synonym_Ptr   Ptr(31);
   Dcl Prev_Synonym_Ptr   Ptr(31) Sinit(0);



     ! Data cannot be less than or equal to 4
     ! Valid data format would be LLZZ + Trancode + Data
     If (Req_Data_Len <= 4) Then Do;

       If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
       Out_Msg = 'DFSYSCP0 data length < = 4   ..... ';
       Call Do_Hard_WTO(Out_Msg);
       End;

       ! Need to do some clean-up before exit
       Call Total_Cleanup;

       ! Pass back an extended error reason code
       If (YSCMRSN) Then Clt_Code = Err_SPS_Data_Length_Error;

       Return Code(Err_Sync_P2P_Specific);

     End;


     ! Process multi-segment or single segment
     If (Req_Data_Len > YSCMREQA->TMAMALEN) Then Do;

      /*-----------------------------------------------------------*/
      /* Process multi-segment message for SPS                     */
      /*-----------------------------------------------------------*/
      Local_Rc  = 0;
      Curr_Pos  = YSCMREQA;
      Last_Pos  = YSCMREQA + Req_Data_Len;
      First_Seg = True;
      More      = True;

      Do WHILE (More);

        ! First check if we have data following LLZZ
        If (Last_Pos - Curr_Pos) < 5 Then Do;

         Local_Rc = Err_Sync_P2P_Specific;
         ! Pass back an extended error reason code
         If (YSCMRSN) Then Clt_Code = Err_SPS_Data_Length_Error;
         More = False;

        End;
        ELSE Do;

         Curr_Len = Curr_Pos->TMAMALEN;
         Next_Pos = Curr_Pos + Curr_Len;

         ! Check if we are out of boundary
         If (Next_Pos > Last_Pos) Then Do;

          Local_Rc = Err_Sync_P2P_Specific;
          ! Pass back an extended error reason code
          If (YSCMRSN) Then Clt_Code = Err_SPS_Data_Length_Error;
          More = False;

         End;
         Else Do; ! Within the boundary

          If (Next_Pos = Last_Pos) Then More = False; ! Last segment

          ! Get a 12-byte message synonym block for this data segment
          ?ZDFSPOOL FUNC(GET)
                  LEN(Length(YSYNSEGM))
                  LIST(ADDR(DFSPOOL_PARMLIST))
                  POOL(LUMP)
                  ADDR(Temp_Synonym_Ptr)
                  RC(Local_RC);

          If (Local_RC ¬= 0) Then Do;

           ! Pass back an extended error reason code
           If (YSCMRSN) Then Clt_Code = Err_REQ_No_Lump;
           Local_Rc = Err_in_OTMA_Callout;
           More = False;

          End;
          Else Do;

           If (First_Seg) Then Do;

            ! Process the first segment
            Seg_Synonym_Ptr = Temp_Synonym_Ptr;
            First_Seg = False;

           End;
           Else Do;

            ! Process non-first segment and chain
            ! to the previous synonym block
            Prev_Synonym_Ptr->YSYNSEGM_NEXT = Temp_Synonym_Ptr;

           End;

           Temp_Synonym_Ptr->YSYNSEGM_MSG  = Curr_Pos;
           Temp_Synonym_Ptr->YSYNSEGM_LEN  = Curr_Len;
           Temp_Synonym_Ptr->YSYNSEGM_NEXT = 0;

           Prev_Synonym_Ptr = Temp_Synonym_Ptr;
           Curr_Pos = Next_Pos;

          End; ! end of getting the storage for synonym block

         End; ! end of within the boundary

        End;

      End; ! End of DO WHILE

      If (Local_Rc) Then Do; ! Exit if error found
       ! Need to do some clean-up before exit
       Call Total_Cleanup;
       Return Code(Local_Rc);
      End;

     End; ! End of multi-segment processing
     Else Do;

      /*-----------------------------------------------------------*/
      /* CHECK IF THIS IS A SINGLE SEGMENT MESSAGE                 */
      /*-----------------------------------------------------------*/
      If (Req_Data_Len = YSCMREQA->TMAMALEN) Then Do;


        ! Get a 12-byte message synonym block for this data segment
        ?ZDFSPOOL FUNC(GET)
                  LEN(Length(YSYNSEGM))
                  LIST(ADDR(DFSPOOL_PARMLIST))
                  POOL(LUMP)
                  ADDR(Seg_Synonym_Ptr)
                  RC(Local_RC);

        If (Local_RC ¬= 0) Then
          Do;

            ! Need to do some clean-up before exit
            Call Total_Cleanup;
            ! Pass back an extended error reason code
            If (YSCMRSN) Then Clt_Code = Err_REQ_No_Lump;
            Return Code(Err_in_OTMA_Callout);

          End;

        YSYNSEGM_NEXT = 0;
        YSYNSEGM_MSG  = YSCMREQA; ! Point to data segment
        YSYNSEGM_LEN  = Req_Data_Len; ! Save the segment length

      End;
      Else Do; ! This is an error condition

        If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
        Out_Msg = 'DFSYSCP0 data length incorrect ... ';
        Call Do_Hard_WTO(Out_Msg);
        End;

        ! Need to do some clean-up before exit
        Call Total_Cleanup;

        ! Pass back an extended error reason code
        If (YSCMRSN) Then Clt_Code = Err_SPS_Data_Length_Error;

        Return Code(Err_Sync_P2P_Specific);

      End;

     End; ! End of single segment message processing


     If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
     Out_Msg = 'DFSYSCP0 Build Input OTMA segments!!!';
     Call Do_Hard_WTO(Out_Msg);
     End;

   Return Code(Local_RC);

  End Prepare_Segments_For_SPS;


 /********************************************************************/
 /*                                                                  */
 /* Subroutine : Proc_Sync_Prog_Switch (SPS)                         */
 /*                                                                  */
 /********************************************************************/

 Proc_Sync_Prog_Switch: Procedure
               Internal_Proc_Opt;

   DCL Local_RC    Fixed(32);
   If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
   Out_Msg = 'DFSYSCP0 PROCESSING A SYNC PROG SWITCH REQUEST !!';
   Call Do_Hard_WTO(Out_Msg);
   End;

   /*-----------------------------------------------------------*/
   /* CHECK SECURITY                                            */
   /*-----------------------------------------------------------*/

   Call Do_Finddest_Security Retcode(Local_RC);
   If Local_RC ¬= 0 then Return Code(Local_RC);

   /*-----------------------------------------------------------*/
   /* BUILD OR FIND OTMA MEMBER/TPIPE FOR THIS SPS              */
   /*-----------------------------------------------------------*/

   Call Check_Tpipe_And_Others Retcode(Local_RC);
   If (Local_RC ¬= 0) Then Return Code(Local_RC); ! Exit if error

   /*-----------------------------------------------------------*/
   /* PREPARE REQUEST DATA                                      */
   /*-----------------------------------------------------------*/

   Call Prepare_Segments_For_SPS Retcode(Local_RC);
   If (Local_RC ¬= 0) Then Return Code(Local_RC); ! Exit if error

   /*-----------------------------------------------------------*/
   /* GET AN YTIB FOR THIS SPS AND PREPARE OTMA PREFIX          */
   /*-----------------------------------------------------------*/

   Call Get_YTIB_For_SPS Retcode(Local_RC);
   If (Local_RC ¬= 0) Then Return Code(Local_RC); ! Exit if error

   /*-----------------------------------------------------------*/
   /* RELEASE THE DESCRIPTOR STORAGE                            */
   /*-----------------------------------------------------------*/

   Call Release_Descriptor_STG;

   /*-----------------------------------------------------------*/
   /* GET AN AWE IN ORDER TO ACTIVATE DFSYTIB0 FOR PROCESSING   */
   /*-----------------------------------------------------------*/

   Call Get_AWE_For_SPS Retcode(Local_RC);
   If (Local_RC ¬= 0) Then Return Code(Local_RC); ! Exit if error

   /*-----------------------------------------------------------*/
   /* Cut SLOG code 78 if DC monitor is on                      */
   /*-----------------------------------------------------------*/
   Call Cut_SLOG_78;

   /*-----------------------------------------------------------*/
   /* START THE TIMEOUT TIMER                                   */
   /*-----------------------------------------------------------*/

   Call Start_Sync_Timer;

   /*-----------------------------------------------------------*/
   /* ACTIVATE DFSYTIB0 SERVTASK TO PROCESS THIS ICAL           */
   /*-----------------------------------------------------------*/

   Call Act_TIB0_For_SPS Retcode(Local_RC);
   If (Local_RC ¬= 0) Then Return Code(Local_RC); ! Exit if error



   Return Code(Local_RC);

  End Proc_Sync_Prog_Switch;





  /******************************************************************/
  /* Subroutine - Get_AWE_For_SPS                                   */
  /*    Create an AWE for this synchronous program switch message   */
  /*                                                                */
  /*    If exit w/o error, Sync_AWE_Ptr points to the new AWE       */
  /*    and TPIPE_SYNC_AWE_CT will be incremented by 1.             */
  /******************************************************************/
  Get_AWE_For_SPS: Procedure
                Internal_Proc_Opt;
   Declare
       Local_RC           Fixed(31);

     Call Get_Sync_AWE  Retcode(Local_RC);

     If (Local_RC) Then Do;

       ! Need to do some clean-up before exit
       Call Total_Cleanup;

       Return Code(Local_RC);

     End;

     Sync_AWE_Ptr->aworname = YTIB_TOKEN     ; ! Tib token
     Sync_AWE_Ptr->awocecba = Addr(ECB_PLAS) ; ! ECB address
     Sync_AWE_Ptr->awotnotf = True; ! DFSYSTO0 does not free this AWE
     Sync_AWE_Ptr->awocbufa = Seg_Synonym_Ptr;
     Sync_AWE_Ptr->awostate = AWO_SYNC_PROG_SWITCH;
     Sync_AWE_Ptr->AWOFUNC  = AWO_SCP_PROCESS_INPUT;
     Sync_AWE_Ptr->awotpipe = CLBLU6WA; ! For serviceability

     If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
     Out_Msg = 'DFSYSCP0 Get an AWE for SPS     !!!!';
     Call Do_Hard_WTO(Out_Msg);
     End;

   Return Code(Local_RC);

  End Get_AWE_For_SPS;

  /******************************************************************/
  /* Subroutine - Get_TIB0_For_SPS                                  */
  /*    Activate DFSYTIB0 for synchronous program switch message    */
  /*                                                                */
  /*    If exit w/o error, DFSYTIB0 will process this message       */
  /******************************************************************/
  Act_TIB0_For_SPS: Procedure
                Internal_Proc_Opt;
   Declare
       Local_RC         Fixed(31);

   Dcl
       YTIB_ECB         Char(4) Based(CLB_Ptr) ; ! YTIB CLB ECB

   Dcl Done             Bit(1);
   Dcl TmpCt            FIXED(32);
   Dcl NewCT            FIXED(32);

     PSTSYQAB = SPS_Ytib_Addr; ! Save YTIB address in PST
     PSTSYNO  = True;          ! ICAL is active

     ! Cut OTMT trace entry for sending a SPS message
     If (SCDTRAOA) Then
      Do;
      Call OTMT_Trace(trace_tmamod_p2p,
                21x,
                PSTFLAG2,
                Addr(ECB_PLAS),
                CLB_Ptr,
                AWORNAME);
      End;

     Ytib_AWE_Ptr = Sync_AWE_Ptr; ! Store AWE Ptr to YTIB block
     Ytib_SPS_AWE_Ptr = Sync_AWE_Ptr; ! copy for shrq BE resp  @U13CDC8

     ! Activate DFSYTIB0 for message processing
     Call Activate_OTMA_Block RetCode(local_rc) ;

     If (Local_RC) Then Do;

       ! Cut a DC monitor record if needed
       Call Cut_SLOG_79;

       ! Need to do some clean-up before exit
       Call Total_Cleanup;

       ! Pass back an extended error reason code
       If (YSCMRSN) Then Clt_Code = Err_Fail_To_Post_YTIB;

       Return Code(Err_Sync_P2P_Specific);

     End;

     ! increment the sync callout message count

     TmpCt =TPIPE_SYNC_OUT_CT;

     Do until(Done);

         NewCt   = TmpCt;               !Copy current value
         If (NewCt >= 65535) Then
           NewCt = 1;                   !Reset to 1 if >= 65535
         Else
           NewCt = NewCt + 1;           !Increment by 1

         ?Dfscs Old(TmpCt)
                New(NewCt)
                Loc(TPIPE_SYNC_OUT_CT)
                Wait(NO)
                Swapped(Done);
     End; !do until Done


     If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
     Out_Msg = 'DFSYSCP0 Activate DFSYTIB0      !!!!';
     Call Do_Hard_WTO(Out_Msg);
     End;

   Return Code(Local_RC);

  End Act_TIB0_For_SPS;

  /******************************************************************/
  /* Subroutine - Activate_OTMA_Block                               */
  /*  This function will post the YTIB itask or YQAB itask          */
  /******************************************************************/

  Activate_OTMA_Block: Procedure
     INTERNAL_PROC_OPT;

  dcl  local_rc         Fixed(31) ;

    If Tscd_Lum_Debug then Do;
     Out_Msg = 'DFSYSCP0: Activate YTIB/YQAB ' ;
     Call Do_WTO(Out_Msg);
    End;

    ?DFSSERVR FUNC(ACTIVATE)
              POST(' SCP')
              ECB(CLB_PTR->I001C)
              RETCODE(local_rc)
              ASMSCD(Addr(SCD_PLAS));


    Return Code(local_rc) ;

  End Activate_OTMA_Block ;


  /*******************************************************************/
  /* Subroutine - Proc_NAK_From_YTIB                                 */
  /*    Let's process a NAK from DFSYTIB0                            */
  /*                                                                 */
  /*******************************************************************/
  Proc_NAK_From_YTIB:Procedure
                Internal_Proc_Opt;
   Declare
      Local_RC           Fixed(31);

   If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
   Out_Msg = 'DFSYSCP0 PROCESSING YTIB   NAK   !!';
   Call Do_Hard_WTO(Out_Msg);
   End;

   ! Set the RC back to DFSDLA40
   Local_RC = Err_Sync_P2P_Specific;

   ! Set the extended reason code for the error
   If (YSCMRSN) Then Clt_Code = awocltcd;

   If (SCDTRAOA) Then ! Cut a table trace if needed
   Call OTMT_Trace(trace_tmamod_p2p,
              30x,
              PSTFLAG2,
              Local_RC,
              Clt_Code,
              AWORNAME);

   Call Cut_OTMA_67D0('DFSYSCP0',
                           ' TIB NACK ',
                           LCRETOKN,
                           Dest_Name,
                           Dummy_char4,
                           Dummy_char4,
                           Clt_Code,
                           Sync_Timeout);


   ! We need to free the prefix, YTIB, and AWE
   Call Total_Cleanup;

    Return Code(Local_Rc);

  End Proc_NAK_From_YTIB;

  /*******************************************************************/
  /* Subroutine - Proc_RSP_From_YSLM                                 */
  /*    Let's process the response from DFSYSLM0                     */
  /*                                                                 */
  /*                                                                 */
  /*******************************************************************/
  Proc_RSP_From_YSLM:Procedure
                Internal_Proc_Opt;
   Declare
      Local_RC           Fixed(31);

   Local_RC = 0; ! Set a good return code

   ! Pass back any client error code
   If (YSCMRSN) Then Do;

    If (awocltcd ¬= 0) Then Do;

      If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
      Out_Msg = 'DFSYSCP0: This is an error message !!!!';
      Call Do_Hard_WTO(Out_Msg);
      End;

      Clt_Code = awocltcd;
      Local_RC = Err_In_Response_Data;

    End;
    Else
      Clt_Code = 0;

   End;

   If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
   Out_Msg = 'DFSYSCP0 Passes a resp back for SPS  !!!!';
   Call Do_Hard_WTO(Out_Msg);
   End;

   ! Pass back response data only if the length is > 0
   If (AWOSTLEN > 0) Then Do;
     ! Pass back the resp len
     OAuse   = AWOSTLEN;
     ! Pass back the response buf address
     RspOTMA = Awosyrsp;
   End;

   If (SCDTRAOA) Then ! Cut a table trace if needed
   Call OTMT_Trace(trace_tmamod_p2p,
              31x,
              PSTFLAG2,
              Local_RC,
              Clt_Code,
              AWORNAME);

   ! Need to do some clean-up before exit
   Call Total_Cleanup;

   Return Code(Local_Rc);

  End Proc_RSP_From_YSLM;


 /*********************************************************************/
 /*********************************************************************/
 /*                                                                   */
 /* Subroutine : Proc_TimeOut                                         */
 /*                                                                   */
 /*********************************************************************/
 /*********************************************************************/

 Proc_TimeOut: Procedure
               Internal_Proc_Opt;

   DCL Local_RC    Fixed(32);

   ! Let's set the extended reason code
   If (YSCMRSN) Then Do;                                      !@PM29046

     If (DestRoutDesc_SPS) Then Clt_Code = 32;
     Else Do; ! Callout to ICON

     If (AWOISND0) Then Clt_Code = 4;        ! No XCF send     @PM29046
     Else If (AWOISND1) Then Clt_Code = 8;   ! No ACK          @PM29046
          Else If (AWOISND2) Then Clt_Code = 12;! No resp      @PM29046
               Else If (AWOISND3) Then Clt_Code = 16; ! error  @PM29046

     End; ! End of callout to ICON

   End;                                                       !@PM29046

   If (SCDTRAOA) Then Do;

     If (DESTROUTDESC_SPS) Then
       Call OTMT_Trace(trace_tmamod_p2p,
                  36x,
                  PSTFLAG2,
                  Err_Got_Timeout,
                  Clt_Code,
                  AWORNAME);
     Else
       Call OTMT_Trace(trace_tmamod_awe,
              17x,
              PSTFLAG2,
              Addr(ECB_PLAS),
              PSTSYQAB,
              AWORNAME);

   End;


   ! Delete the AWE from the YQAB chain
   If (DestRoutDesc_icon) Then Call Delete_Sync_AWE_from_YQAB;

   ! We need to free the prefix, synonym blocks, and AWE
   Call Total_Cleanup;

   Call Cut_OTMA_67D0('DFSYSCP0',
                           ' TIMEOUT  ',
                           LCRETOKN,
                           Dest_Name,
                           Dummy_char4,
                           Dummy_char4,
                           Clt_Code,
                           Sync_Timeout);

   If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
   Out_Msg = 'DFSYSCP0 End of Timeout Processing! ';
   Call Do_Hard_WTO(Out_Msg);
   End;

   Local_RC = Err_Got_Timeout;

   Return Code(Local_Rc);

 End Proc_TimeOut;


 /*******************************************************************/
 /* Subroutine - Proc_Abort_From_YLUS                               */
 /*    Let's process an abort reported from DFSYLUS0                */
 /*                                                                 */
 /*******************************************************************/
 Proc_Abort_From_YLUS:Procedure
               Internal_Proc_Opt;
  Declare
     Local_RC           Fixed(31);

  Local_RC = 0; ! Set a good return code

  If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
  Out_Msg = 'DFSYSCP0 passed a bad RC/RSN/ERSN code  !!';
  Call Do_Hard_WTO(Out_Msg);
  End;

  ! Set the RC back to DFSDLA40
  Local_RC = Err_Sync_P2P_Specific;

  ! Set the extended reason code for the error
  If (YSCMRSN) Then Clt_Code = awocltcd;

  If (SCDTRAOA) Then ! Cut a table trace if needed
  Call OTMT_Trace(trace_tmamod_p2p,
             32x,
             PSTFLAG2,
             Local_RC,
             Clt_Code,
             AWORNAME);

  ! We need to free the prefix, YTIB, and AWE
  Call Total_Cleanup;

  Return Code(Local_Rc);

 End Proc_Abort_From_YLUS;


  /********************************************************* @PM90557**/
  /*                                                                  */
  /*  Subroutine Get_Trancode                                         */
  /*                                                                  */
  /********************************************************************/

  Get_Trancode: Procedure
                Internal_Proc_Opt;                           !@PM90557

    Dcl
       1  Segment               Based(YSCMREQA),
          2  Segment_Length     Fixed(15),
          2  Segment_ZZ         Fixed(15),
          2  Segment_Data       Char(*);                     !@PM90557

    Dcl  I            Fixed(31) ;                           !@PM90557
    Dcl  Data_Length  Fixed(31) ;                           !@PM90557
    Dcl  Dest         Char(8);                              !@PM90557
    Dcl  Max_Tran_Len Constant(8) Fixed(31);                !@PM90557
    Dcl  Local_RC     Fixed(31);                            !@PM90557

    Local_RC      = 0;                   ! Assume OK         @PM90557

    If (Segment_Length > 4) Then Do;                        !@PM90557

      Data_Length   = Segment_Length - 4 ; ! Decrement the LLZZ
      Dest      = ' ' ;  ! Reset the temp destination
      TranCode  = ' ' ;  ! Reset the tran code to all blanks

      If (Data_Length < Max_Tran_Len) Then
        Dest = Segment_Data(1::Data_Length) ;               !@PM90557
      Else
        Dest = Segment_Data(1::Max_Tran_Len) ;              !@PM90557

      I = INDEX(Dest,' '); ! Locate the first blank in Dest(8)

      If I = 0 Then
        TranCode = Dest;       ! No blank, take everything
      Else
        TranCode = Dest(1::I); ! Found blank(s), take upto 1st blank

    End; ! end of valid LLZZ + trancode + data               @PM90557
    Else
      Local_RC = 4; ! No trancode                            @PM90557

    Return Code(local_rc); !@PM90557

  End Get_Trancode; !@PM90557


  /********************************************************************/
  /*                                                                  */
  /*  Subroutine Do_Finddest_Security                                 */
  /*             Process DFSMSCE0 exit then call finddest and then    */
  /*                     check if user is authorized to issue the     */
  /*                     ICAL for the transaction.                    */
  /********************************************************************/

  Do_Finddest_security: Procedure
                          Internal_Proc_Opt;

   DCL SMB_Ptr     Ptr(31);              ! SMB pointer (tran)
   DCL I           Fixed(15);            ! Offset for TranCode
   DCL Temp_R7     Fixed(31);
   DCL Local_RC    FIXED(31);            ! local return code

   DCL Search_Type Char(4) constant('SMBS');  ! Search SMBs first


   /*  Get the Transaction code from the input                 */

   Call Get_Trancode RetCode(Local_RC);                     !@PM90557

   If (Local_RC = 0) Then Do; ! Process the trancode         @PM90557

     /*  Call DFSMSCE0 user exit -- Deferred to future support   */

     /*  Call finddest to get the SMB                            */

     DFSTRAN = ''B;                        ! Clear statistic area

     ?ZDFSFNDS FUNC(FIND)
             NAME(Addr(TranCode))
             SEARCH(Addr(Search_Type))
             DESC(NO)
             CREATEU(NO)
             INUSE(NO)
             STATAREA(Addr(DFSTRAN))
             ITASK(YES)
             REFS(SCD,SCDFNDS2)
             ADDR(SMB_Ptr)
             RETCODE(Local_RC);

     If Local_RC = 0 Then Do;        ! Finddest worked
      If TRANCPIC = True |           ! Cpic not allowed
         PFND1SMB = False |          ! Destination is no tran
         TRANCONV = True then Do;    ! Conversational tran
         Local_RC = 8;               ! Set Error
      End; ! End of checking
     End; ! End of Findest work case

   End; ! End of getting a trancode                        @PM90557

   If (Local_RC ¬= 0) Then Do;                    ! Exit if error
     If YSCMRSN ¬= 0 then Clt_Code = Err_Destination;
     If (SCDTRAOA) Then Do;
      Call OTMT_Trace(trace_tmamod_p2p,
                41x,
                PSTFLAG2,
                Err_Sync_P2P_Specific,
                Clt_Code,
                LCRETOKN(1:8) );
      End;
     Return Code(Err_Sync_P2P_Specific);
     End;

   ! Check security

   !* If there is a ACEE in the dependent region then just use it
   !* That's for all cases: APPC, OTMA, or legacy with DFSBSEX0
   If PSTDACEE = 0 then Do;             ! Do we have ACEE in dependent
      !* No ACEE in dependent region
      If PSTLU62 = True then Do;        ! APPC or OTMA

         !* If security for APPC/OTMA is FULL then PSTDACEE is not
         !* zero.  For APPC we need to check for security PROFILE.
         !* If not PROFILE then we need to get it from the LSCD.

         If PST1OTMA = True | LSCD_RACF_OPTION = LSCD_RACF_PROFILE
         then
            !* If security none has been specified then just exit
            If P62RACOP = 'N' Then Return Code(0);
            Else;                  ! not 'N'
         Else
            !* This is APPC and not PROFILE option
            If LSCD_RACF_OPTION = LSCD_RACF_NONE then Return Code(0);


         /**** APPC case    ******/
         !* For APPC we need to create the ACEE
         If PST1OTMA = False then Do;    ! It is APPC

            ! We need to be out of xmem mode to create the ACEE so
            ! switch back to dependent region.

            ?ZISWITCH to(UNSTACK) ecb(addr(ECB_PLAS))
                                  scd(addr(SCD_PLAS));

            ?DFSRAC6 FUNC(RACINIT)
                  TOKENIN(Addr(P62UTOKN))     !*RACF USER TOKEN
                  INVER(YES)                  !*Input Verification Check
                  ACEE(YES)                   !*REQUEST A(ACEE) RETURNED
                  RACOPT(P62RACOP)            !*TP Profile RACF Option
                  PLUNAME(ADDR(P62LUNM))      !*LUNAME
                  LOCALLU(ADDR(P62LOCLU))     !*Local-LU
                  PLX(YES)
                  RC(Local_RC);

                  PSTDACEE = RAC6ACEO;    !* Assign ACEE to PST
            !* Back to xmem mode with CTL as primary and DEP as home.
            ?ZISWITCH to(XMCTL) stack(YES)
                      ecb(addr(ECB_PLAS)) scd(addr(SCD_PLAS));

            If (Local_RC ¬= 0) Then Do;       ! Exit if error
               If YSCMRSN ¬= 0 then Clt_Code = Err_Security;
               If (SCDTRAOA) Then Do;
                Call OTMT_Trace(trace_tmamod_p2p,
                          42x,
                          PSTFLAG2,
                          Err_Sync_P2P_Specific,
                          Clt_Code,
                          LCRETOKN(1:8) );
               End;
               Return Code(Err_Sync_P2P_Specific);
               End;
            End;                    ! End PSTCBOTM=False (APPC)

         /**** OTMA case    ******/
         Else Do;

            !* OTMA might have already an ACEE in the ACEE pool
            !* If it has then we can use it but the ACEE will be
            !* in the control region.

            If (PSTORGID = SCDSMQNM) then
               Call Find_UserID_Block;    ! Try to find ACEE in pool

            !* If ACEE found then the ACEE is in the control region
            !* and we need to put it into PSTCACEE

            If Userid_Ptr = 0 then do;
               !* No ACEE in pool we need to create one in the
               !* dependent region

               ! We need to be out of xmem mode to create the ACEE so
               ! switch back to dependent region.

               ?ZISWITCH to(UNSTACK) ecb(addr(ECB_PLAS))
                                     scd(addr(SCD_PLAS));

               !* Now call DFSYRAC0 to create the ACEE.  Note that
               !* we don't pass the YTIB so DFSYRAC0 will not try
               !* to find the ACEE in the pool.

               ?DFSYRAC FUNC(RACINIT)
                     TOKENIN(Addr(P62UTOKN))     !*RACF USER TOKEN
                     INVER(YES)                  !*Input Verification
                     ACEE(YES)                   !*REQUEST A(ACEE)
                     RACOPT(P62RACOP)            !*TP Profile RACF Opt
                     PLX(YES)
                     RC(Local_RC);

                     PSTDACEE = YRACACEO;    !* Assign ACEE to PST

               !* Back to xmem mode with CTL as primary and DEP as home.
               ?ZISWITCH to(XMCTL) stack(YES)
                         ecb(addr(ECB_PLAS)) scd(addr(SCD_PLAS));

               If (Local_RC ¬= 0) Then Do;       ! Exit if error
                  If YSCMRSN ¬= 0 then Clt_Code = Err_Security;
                  If (SCDTRAOA) Then Do;
                   Call OTMT_Trace(trace_tmamod_p2p,
                             43x,
                             PSTFLAG2,
                             Err_Sync_P2P_Specific,
                             Clt_Code,
                             LCRETOKN(1:8) );
                  End;
                  Return Code(Err_Sync_P2P_Specific);
                  End;
               End;                 ! End Userid_Ptr = 0

            Else do;                ! ACEE not in hash table
               Rfy UserID_Block based(Userid_Ptr);
               PSTCACEE = USRB_ACEE_PTR;      ! Get ACEE pointer
               ACEE_pool_used = True;         ! Mark it
               End;                 ! End Else (Userid_Ptr)

            End;                    ! End Else (OTMA case)
          End;                      ! End PSTLU62 = True
      End;                          ! End PSTDACEE=0



   Rfy  (R1,R2,R7,R10,R11) Restricted;
   Temp_R7 = R7;

   !*  Registers for DFSDAUT0:
   !*   R1  = PST
   !*   R2  = Caller-id = 44
   !*   R7  = CTB Zero if BMP
   !*   R10 = SMB
   !*   R11 = SCD

   R1 = ADDR(Ecb_Plas);                  ! PST
   R2 = '44'X;                           ! Caller-id function
   If PSTCNT = 0 Then R7 = 0;            ! No CTB (BMP)
   Else R7 = PSTCNT -> CNTCTBPT;         ! Move in CTB
   R10 = SMB_Ptr;                        ! SMB
   R11 = ADDR(Scd_Plas);                 ! SCD

   Call DFSDAUT0 Retcode(Local_RC);      ! Call security (RACF)

   R7 = Temp_R7;
   Rfy  (R1,R2,R7,R10,R11) UnRestricted;

   If ACEE_pool_used = True then         ! ACEE from pool could timeout
      PSTCACEE = 0;                      ! clear it so no longer used

   If (Local_RC ¬= 0) Then Do;           ! Exit if error
      If YSCMRSN ¬= 0 then Clt_Code = Err_Security;  ! extended reason
      If (SCDTRAOA) Then Do;
       Call OTMT_Trace(trace_tmamod_p2p,
                 44x,
                 PSTFLAG2,
                 Err_Sync_P2P_Specific,
                 Clt_Code,
                 LCRETOKN(1:8) );
      End;
      Return Code(Err_Sync_P2P_Specific);
      End;

   Return Code(0);                       ! All worked fine

 End Do_Finddest_Security;

 /*********************************************************************/
 /*********************************************************************/
 /*                                                                   */
 /* Subroutine : Proc_Rel_Storage                                     */
 /*                                                                   */
 /*********************************************************************/
 /*********************************************************************/
 Proc_Rel_Storage:  Procedure
                    Internal_Proc_Opt;

   Dcl Local_RC fixed(31);      ! Local return code for this routine
   Dcl Tmp_AWE_Ptr Ptr(31);     ! Temporary AWE address

   Local_RC = 0;                ! preset a good return code

   If (YSCMRSPO = 0) | (YSCMAUSE = 0) Then

     Do;
      If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
      Out_Msg = 'DFSYSCP0 RELSTG W/O BUFFER ADDRESS !!';
      Call Do_Hard_WTO(Out_Msg);
      End;
      Call Cut_OTMA_67D0('DFSYSCP0',
                            ' RELST FAL',
                            LCRETOKN,
                            Dummy_Char8,
                            Dummy_char4,
                            Dummy_char4,
                            20,
                            16);

      Local_RC = Err_In_STG_FREE;

     End;

   Else

     Do;

      Call RelStgBuf(OAUSE, RspOTMA) RetCode(local_rc);

      If (Local_RC) Then

        Do;
         If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
         Out_Msg = 'DFSYSCP0 RELSTG FAILED   ........  !!';
         Call Do_Hard_WTO(Out_Msg);
         End;
         Call Cut_OTMA_67D0('DFSYSCP0',
                            ' RELST FAL',
                            LCRETOKN,
                            Dummy_Char8,
                            Dummy_char4,
                            Dummy_char4,
                            99,
                            Local_RC);

         Local_RC = Err_In_STG_FREE;

        End;

      Else Do;

         If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
         Out_Msg = 'DFSYSCP0 RELEASE STORAGE WORKS  !!!!!';
         Call Do_Hard_WTO(Out_Msg);
         End;

      End;


     End;

   Return Code(Local_RC);

 End Proc_Rel_Storage;

 /*==================================================================*/
 /* Subroutine to call IMODULE to release storage                    */
 /*==================================================================*/

 RelStgBuf: Procedure(stglen,stgptr)
    INTERNAL_PROC_OPT;


 dcl stglen              Fixed(31);    !Arg: Stg buf len
 dcl stgptr              ptr(31);      !Arg: Stg buf ptr
 dcl Local_rc            Fixed(31);    !Res: Return code
 dcl imodprm(4) FIXED(31);
 dcl buff char(*)        based(Real_ptr);

 dcl Real_Len            Fixed(31);
 dcl Real_ptr            Ptr(31);

 Real_Len = stglen + Length(YI_OPFX);
 Real_Ptr = stgptr - Length(YI_OPFX);

 ?imodule func(FREESTOR)
          epad(buff)
          key(7)
          lv(Real_Len)
          sp(0)
          branch(YES)
          parm(imodprm)
          retcode(Local_rc);



 return code(Local_rc);

 End RelStgBuf;


 /*********************************************************************/
 /* Subroutine - Start_Sync_Timer                                     */
 /*                                                                   */
 /*********************************************************************/
 Start_Sync_Timer:Procedure
               Internal_Proc_Opt;

  Dcl Local_RC      Fixed(31);
  Dcl Timer_AWE_Ptr Ptr(31);

  Respecify (Awe)      Based(Timer_AWE_Ptr);

  Local_RC = 0; ! Pre-set a good return code

  ?DFSBCB FUNC(GET) BLK(AWE) Retaddr(Timer_AWE_Ptr) Retcode(Local_RC);

  ! Exit if cannot obtain an AWE for the DFSTIMER call
  If (Local_RC) Then do;

    ! Let's issue an DFS1269E error msg. Customers need to be
    ! informed to cancel a long running ICAL if needed.

    ?DFSYSER  Err(11)
              Trace(Sync_AWE_Ptr)
              Tpipe(YQAB_LTERM)
              Result(Local_RC);

    RETURN;

  End;

  SELECT;

    WHEN (DestRoutDesc_icon) Do; ! ICAL to IMS Connect

     aweqh    = Addr(TSCD_QOIC) ;          ! DFSYMOM0 que header
     awofunc  = AWO_OIC_SYNC_TIMER_POP;    ! AWE function for DFSYMOM0
     awotbcb  = True;                      ! BCB type of AWE
     awocbufa = 0;                         ! No additional stg attached
     awocecba = Addr(ECB_PLAS) ;           ! Caller's ecb
     awotpipe = CLBlu6wa;                  ! Address of hold queue YQAB
     aworname = Sync_AWE_Ptr->aworname;    ! ICAL token
     awotnotf = False;                     ! Free this AWE at DFSYMOM0
     awocwait = False;                     ! No need to post back

    End; ! End of preparation for building AWE for callout to ICON

    WHEN (DestRoutDesc_SPS) Do; ! ICAL for synchronous program switch

     aweqh    = Addr(MCBAWANC ) ;          ! DFSYMEM0 que header
     awofunc  = awo_mem_sps_timeout   ;    ! AWE function for DFSYMEM0
     awotbcb  = True;                      ! BCB type of AWE
     awocbufa = 0;                         ! No additional stg attached
     awocecba = Addr(ECB_PLAS) ;           ! Caller's ecb
     awotname = Sync_Tpipe;                ! Tpipe name
     aworname = YTIB_TOKEN;                ! YTIB token
     awotnotf = False;                     ! Free this AWE at DFSYMEM0
     awocwait = False;                     ! No need to post back

    End; ! End of preparation for building AWE for sps

    Otherwise;

  End; ! End of SELECT


  /*******************************************************************/
  /* Need to call DFSTIMER with the timeout value specified          */
  /*******************************************************************/

  ?Dfstimer func(TIMER)
      bintvl(Sync_Timeout)
      token(Timer_Token)
      awe(awe)
      retcode(Local_RC);

  Respecify (Awe)      Based(Sync_AWE_Ptr);


 End Start_Sync_Timer;


 /*********************************************************************/
 /*********************************************************************/
 /*                                                                   */
 /* Subroutine : Cancel_Sync_Timer                                    */
 /*                                                                   */
 /*********************************************************************/
 /*********************************************************************/

 Cancel_Sync_Timer: Procedure
                    Internal_Proc_Opt;

  Dcl Timer_RC  Fixed(31);

   ?dfstimer func(CANCEL)
             token(Timer_Token)
             retcode(timer_rc);


 End Cancel_Sync_Timer;

 /*********************************************************************/
 /*                                                                   */
 /* Subroutine : Find_UserID_Block                                    */
 /*                                                                   */
 /* Function:    Find ACEE in ACEE pool                               */
 /*                                                                   */
 /*********************************************************************/

 Find_UserID_Block: Procedure
                    Internal_Proc_Opt;

 Dcl  I             Fixed(15);


 !
 ! Defind the UserID hash table pointed by TSCD_USER_HASH
 !

 Declare
  UserID_table(0:TSCD_USERID_HASH_ENTNUM-1)
                                    PTR(31) based(TSCD_USER_HASH);

 /*-------------------------------------------------------------------*/
 /* LOCATE UserID                                                     */
 /*-------------------------------------------------------------------*/
 Rfy UserID_Block based(Userid_Ptr);

 ?DFSHASH HASHVAL(I) KEY(PSTUSID) KEYNUMWORDS(2)
          RANGE(TSCD_USERID_HASH_ENTNUM);

 Userid_Ptr = Userid_Table(I);

 Do While ((Userid_Ptr ¬= 0) & (USRB_Name ¬= PSTUSID));
    Userid_Ptr = USRB_Next_Userid;
 End;


 End Find_Userid_block;





 /*********************************************************************/
 /*********************************************************************/
 /*                                                                   */
 /* MAIN PROCEDURE                                                    */
 /*                                                                   */
 /*********************************************************************/
 /*********************************************************************/
 If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
 Out_Msg = 'DFSYSCP0 STARTING - SAY .. XXXXXXXX';
 Out_Msg(28:35) = 'HELLO   ';
 Call Do_Hard_WTO(Out_Msg);
 End;
 /*-------------------------------------------------------------------*/
 /* Initialize module return codes and output fields.                 */
 /*-------------------------------------------------------------------*/
 RC = 0;                                 !* Init Module Return Code
 Local_Flags1 = ''X;                     !* Init Processing flags
 Local_Flags2 = ''X;                     !* Init SPS Processing flags


 Select;


    ! Process a SEND/RECEIVE request
    When (YSCMSNDR) Do;

      CALL Proc_Send_Receive Retcode(RC);

    End;

    ! Process a STORAGE RELEASE request
    When (YSCMRLSG) Do;

      CALL Proc_Rel_Storage Retcode(RC);

    End;

    Otherwise;

 End; ! End of Select

 If (Tscd_Lum_Debug | Tscd_Out_Sync_Trace) then Do;
 Out_Msg = 'DFSYSCP0 ENDING   - SAY .. XXXXXXXX';
 Out_Msg(28:35) = 'GOOD BYE';
 Call Do_Hard_WTO(Out_Msg);
 End;

 ?Dfsyrtrn Code(Rc) Trace(Yes);
 ?Epilog;

 End DFSYSCP0;
