#pragma csect(code,"PKQ2550C")
#pragma csect(static,"PKQ2550S")

/*-------------------------------------------------------------------*/
/*  Sprung zu  Programmanfang: 'F PKQANF'                            */
/*             Includes      : 'F PKQINC'                            */
/*             globalen Vars : 'F PKQVAR'                            */
/*             Typedef's     : 'F PKQTYP'                            */
/*-------------------------------------------------------------------*/

/*-------------------------------------------------------------------*/
/*- Testschalter ----------------------------------------------------*/
/*-------------------------------------------------------------------*/
#define ROLLBACK

/*===PKQ2550===========================================================

   Folgesoll  (vorerst nur fr LEBEN)
  ====================================

  --------------------------------------------------------------------
   Eingabe

    1.) DB2-Tabelle TBKO Ablaufplan,... (TABSYS)
    2.) Parameter fr Restart (sind noch genauer zu spezifizieren)

  --------------------------------------------------------------------
   Returnwerte

     0       Ok
     12      Fehler im Vorlau / Abarbeitung Ablaufplan
     16      Fehler in Verarbeitung

  --------------------------------------------------------------------

   Beschreibung (siehe auch Dokument "neues FS Leben")

    1.) Steuerung (Tabsys-Tabellen)

    2.) Programmablauf (fachlich)

    3.) Checkpoint/Restart

 ====================================================================*/

/*-------------------------------------------------------------------*/
/*  nderungs-Historie                                               */
/*  Autor          Datum        Modifikation                 AEND-Kz */
/* ----------------------------------------------------------------- */
/*  JOBARDEN/      Maerz 1997   Ersterstellung                       */
/*  NOHUNGER                                                         */
/*  NOHUNGER       26.07.1999   Steuerung ueber             AEND0001 */
/*                              TBKQ_VTRNR_PORT                      */
/*  NOHUNGER       09.08.1999   Cursor-Tuning               AEND0002 */
/*                              Zugriffspfad ueber                   */
/*                              Clustered-Index der Tabelle          */
/*                              TBKO_VTRJOURNAL zwingen, um          */
/*                              Sortierung zu vermeiden              */
/*                                                                   */
/*  NOHUNGER       10.08.1999   Werte fuer Laufstatistik    AEND0003 */
/*                              in Tabelle OTKQ_BATCH_STAT           */
/*                              bei jedem Commit sichern,            */
/*                              bei einem Restart werden             */
/*                              die Werte ermittelt                  */
/*                                                                   */
/*  NOHUNGER       17.08.1999   Beim fuellen der Struktur   AEND0004 */
/*                              TABVJ werden die Daten-              */
/*                              felder Stornodatum, Wikdatum,        */
/*                              Ruhedatum und Btrfreidatum           */
/*                              initialisiert                        */
/*                              Desweiteren werden Fehler            */
/*                              in der Tabelle TBKQ_BATCHFEHLER      */
/*                              protokolliert                        */
/*                              (Funktion insertBatchfehler)         */
/*                                                                   */
/*                                                                   */
/*  NOHUNGER       23.08.1999   Sonderversion nur fuer ST   AEND0005 */
/*                              NEUE ABRECHNUNG:                     */
/*                              KZ FBKZ wird auf Z abgefragt         */
/*                              !!!  Diese Version darf nicht in die */
/*                              Praxis uebergeben werden             */
/*  NOHUNGER       29.09.1999                               AEND0006 */
/*                              MKQ2550 wird nicht mehr aufgerufen,  */
/*                              da die Inkassoversorgung bereits im  */
/*                              MKO1580 erfolgt                      */
/*                              Abrechnungstabellen zuruecksetzen,   */
/*                              wenn MKO1580 fehlerhaft              */
/*  NOHUNGER       22.10.1999   AEND0005 rueckgaengig       AEND0007 */
/*                              gemacht                              */
/*  NOHUNGER       09.11.1999   In der Funktion LockTable   AEND0008 */
/*                              wurden die Tabellen TBKO_ABR_        */
/*                              BETRAG und -BUCHUNG entfernt         */
/*  NOHUNGER       16.12.1999   printf-Ausgaben der         AEND0009 */
/*                              MKO1500-Schnittstelle in der Praxis  */
/*                              ausschalten                          */
/*  Wagner         04.11.2002                               AEND0010 */
/*                              Vertraege mit Vormerkung ueberlesen. */
/*                                                                   */
/*  Wagner         04.05.2006                               AEND0011 */
/*                              SQLCODE -811 beim Pruefen der        */
/*                              Vormerkung vermeiden.                */
/*                                                                   */
/*  NOHUNGER       08.06.2006                               AEND0012 */
/*                              Bei schwebenden Vertrgen wird die   */
/*                              Bearbeitungsnummer eines vorhandenen */
/*                              Tabelleneintrags korrigiert          */
/*                                                                   */
/*  Wagner         07.05.2007                               AEND0013 */
/*                              Abfrage auf Vormerkung durch den     */
/*                              TVG 2875 ergaenzt.                   */
/*                                                                   */
/*  NOHUNGER       15.05.2007                               AEND0014 */
/*                              Einschraenkung der Anzahl der        */
/*                              VTRJOURNAL-Eintrge                  */
/*                                                                   */
/*  M. Knig       16.10.2007                               AEND0015 */
/*                              technStatus = 'I' aussteuern         */
/*                                                                   */
/*  H. Hoffmann    02.03.2009                               AEND0016 */
/*                              TBKO_KORREP update aufgenommen       */
/*                                                                   */
/*  N.Hungerecker  26.06.2012   Hinweismeldung erhebenBisDat AEND0017*/
/*                              == vtrWerte.erstattenBisDat,         */
/*                              die bisher als Fehler gezaehlt       */
/*                              wurde, herausgenommen                */
/*-------------------------------------------------------------------*/

/*===KO-MODULE=========================================================
 bentigte Kompass-Module (wenn mglich mit Untermodulbaum!!)
   MKO2500 Lesen TBKO_ABLAUFPLAN,...
   MKO1580 Abrechnung
   MKO1500 Ermittlung erhebenBisDatum

   MKO2000 Inkasso-Bestckung
           Untermodule: MKO2002,...

=====================================================================*/

/*===INCLUDE===========================================================
 Includes fr die internen Strukturen:
   XBPTABAP: Ablaufplan. Hier werden die Werte aus TBKO_ABLAUFPLAN
             und der dazugehrigen Tabellen abgelegt
   XBPTABAW:
   XBPTABAB: Aktuelle Bearbeitung. Daten zur VSNR / Police (Kavis)
   XBPTABVJ: Vertragsjournal. Zu TAB_AB gehrende Eintrge des
             TBKO_VTRJOURNAL/VERTRAG (alle mit Vollhistorie)

 Includes fr den Aufruf von Kompass-Modulen:
   XKO2500C fr MKO2500: lesen TBKO_ABLAUFPLAN
   MKO1580  Abrechnung
   MKO2000  Inkasso-Versorgung

=====================================================================*/

/*-------------------------------------------------------------------*/
/*-- include -------------------------------------------------PKQINC-*/
/*-------------------------------------------------------------------*/
#include <errno>
#include <decimal>

/*#include <XKQ2550C>     Inkasso schreiben RW006/004 AEND0006 ***/

#include "XAG0002"
EXEC SQL INCLUDE SQLCA;
#include "XAG0SQL"


#include <XKO2500C>        // MKO2500 lesen TBKO_ABLAUFPLAN
#include <XKO1500C>        // MKO1500 Ermitteln erhebenBisDatum
#include <XKO3130C>        // MKO3130 Update BearbNr

#include <XKO1518C>        // MKO1518 Abrechnung
#include <XKO1571C>        // MKO1571 Abrechnung
#include <XKO1580C>        // MKO1580 Abrechnung

#include <XKO5010C>        // MKO5010 Ermitteln Mandant/Sprachcode

#include <XBPTABVJ>        // definiert TABVJ und VJ_SATZ
#include <XBPTABAB>        // definiert AKT_BEARB
#include <XBPTABAR>        // definiert TABABR und ABR_SATZ
#include <XBPTABVK>        // definiert TABABR_VK

/*-------------------------------------------------------------------*/
/*-- defines --------------------------------------------------------*/
/*-------------------------------------------------------------------*/

#define EC_Vlk_Unvollstaendig 4041
#define EC_OpenAusgabedatei   4042
#define EC_OpenListdatei      4043
#define EC_TABVJ_OVERFLOW     4044
#define EC_ErhebenEqErstatten 4045
#define EC_SelektionFehler    4046
#define EC_2000_Fehler        4047

#define EC_MKO2500_Fehler     4500
#define EC_MKO1500_Fehler     4501
#define EC_MKO1518_Fehler     4502
#define EC_MKO1571_Fehler     4503
#define EC_MKO1580_Fehler     4504
#define EC_MKO5010_Fehler     4505

/* #define EC_MKQ2550_FETCH_ERR  4700 AEND0006 ***/

#define EC_OPEN_VtrJ          4600

#define PROT_DDNAME "DD:PROTOKOL"
#define LIST_DDNAME "DD:LISTE"


#define SZ_FOLGESOLL "3001"
#define SZ_ANPSOLL   "3005"
#define SZ_DIFFSOLL  "3006"
#define MAX_APTAB 50

#define C_DATE(dec, date) { \
   int tt, mm, jjjj, tmp; \
   tmp = dec; \
   tt = tmp % 100; \
   mm = tmp % 10000; mm /= 100;  \
   jjjj = tmp / 10000; \
   sprintf(date, "%02d.%02d.%04d", tt, mm, jjjj); }

#define C_TIME(dec, time) { \
   int ss, mm, hh, tmp; \
   tmp = dec; \
   ss = tmp % 100; \
   mm = tmp % 10000; mm /= 100;  \
   hh = tmp / 10000; \
   sprintf(time, "%02d:%02d:%02d", hh, mm, ss); }

#define PLS(str) fprintf(LIST, str);
#define PLP(str) fprintf(PROT, str);

#define PLD { char fs[5], abr[5], ink[5], txt[30]; \
              decimal(7,0) pliZeit; \
              char tmpTime[9]; \
   now(tmpTime); pliZeit = pliTime(tmpTime); \
   fprintf(LIST, "%D(7,0) %10.10s %D(15,0) Storno in Schwebe \n", \
      pliZeit, aktB.vsnr, tabVJ[0].vtrnr); \
  }


#define PLT { char fs[5], abr[5], ink[5], txt[30];              \
              decimal(7,0) pliZeit;                             \
              char tmpTime[9];                                  \
   now(tmpTime); pliZeit = pliTime(tmpTime);                    \
   if(folgesollAnford && fsRC) strcpy(fs,"ja");                 \
      else strcpy(fs,"nein");                                   \
   if(aw1571 && abrRC)          strcpy(abr,"ja");               \
      else strcpy(abr,"nein");                                  \
  fprintf(LIST, "%D(7,0) %10.10s %D(15,0) fs: %-4s abr: %-4s \
     ink: %-4s %s",          \
      pliZeit, aktB.vsnr, tabVJ[0].vtrnr, fs, abr, ink, txt);   \
  }
#define PLT_newline fprintf(LIST, "\n");

/*#define PROTANZVSNR { \

  decimal(7,0) pliZeit; \
  char tmpTime[9]; \

 now(tmpTime); pliZeit = pliTime(tmpTime); \
 if (anzVSNR == 5) {   \
 fprintf(LIST, "%D(7,0) %10.10s %D(15,0)  %d Vertraege verarbeitet\n",\
     pliZeit, aktB.vsnr, tabVJ[0].vtrnr, gesVSNR); \
  anzVSNR = 0; \
 }
} */
/*-------------------------------------------------------------------*/
/*-- Strukturen / typedef's ----------------------------------PKQTYP-*/
/*-------------------------------------------------------------------*/
typedef _Packed struct {
   VJ_SATZ   vj;            // Eintrge aus TBKO_VTR-Journal/Vertrag
   AKT_BEARB ab;            // Eintrge aus TBKI/KO_VERTRAG
} GWSATZ;

typedef _Packed struct {     // --- fr vtrDict (Key ist Vtrnr)
   int       posVon;         // Idx: Beginn in tabVJ
   int       posBis;         // Idx: Ende in tabVJ
   int       posAbr;         // letzter abgerechneter Stand in tabVJ
   int       posVordatV;     // Idx: V-Stand Vordatierung
   int       posVordatU[20]; // Idx: U-Stand Vordatierung
   int       vordat;         // bool Vtr mit Vordatierung
   int       tv3001;         // bool tv3001 (aus MKO2501)
   int       tvSchwebe;      // bool TV_SCHWEBE (aus MKO2501)
   int       bearbNrHochsetzen;   // bool aus MKO2501
   decimal(9,0) erstattenBisDat;  // tv3001 erstattenBis Datum
   decimal(9,0) erhebenBisDat;    // tv3001 erstattenBis Datum
   decimal(9,0) hauptFaelligDat;  // Hauptfaelligkeitsdatum
   decimal(9,0) vordatStornoDat;  // Datum vordat. Storno
} VTRWERTE;

typedef int (*funcPtr)();  /* pointer to a function returning an int */

/*-------------------------------------------------------------------*/
/*-- forward declarations -------------------------------------------*/
/*-------------------------------------------------------------------*/
  static void   vorlauf(void);          // Vorverarbeitung
  static void   verarbeitung(void);     // Hauptverarbeitung
  static void   nachlauf(void);         // Nachverarbeitung
  static void   vorlaufkarteAuswerten(void);
  static void   lockTable(void);        // Exclusiv-Lock DB2-Tabellen
  static void   getmandantsprach(void); // Erm. Mandant/Sprachcode
  static void   ablaufplan_vtrnr_port(void); // Ausw. TBKO_ABLAUFPLAN
                                        //        und TBKQ_VTRNR_PORT
  static void   pruef_vormerk(void);    // Prfen auf Vormerkung
  static void   getHostvars(void);      // copy Hostvar -> structs
  static void   makeAktBearb(void);     // aktBearb aufbauen
  static void   addTabVJ(void);         // tabVJ aufbauen
  static void   initAktBearb(void);     // aktBearb initialisieren
  static void   initTabVJ(void);        // tabVJ initialisieren

  static int    folgesoll(void);            // Folgesollverarb.
  static int    abrechnung(void);           //
  /* static int    inkasso(void);           AEND0006 ***/
  static void   diffSollPruefung(void);     // ehemals MKV3016
  static void   fuellenAbrTab(void);        //
  static int    vordatierungPruefen(void);  //
  static int    genNewTabVJ(void);          //
  static int    insertVJ(void);             //
  static int    updateBearbNr(void);        //
  static int    selektierung(void);         //
  static void   schwebeBehandlung(void);    //
  static void   aufbauAbrGrundlagen(void);  //
  static int    ermittlungErhebenBis(void); //
  static int    resetAbrtab(void);          //
  static int    ProtFehler(char *, char *, char *);
  static int    PKQ2550Fehler(char *, char *, int, int);
  static int    updateAbrechFehler(void);
  static void   sqlFehler(char *, char *,long int);
  static void   checkpoint(void);           //Checkpoint und Commit
  static void   Batch_Statistik(void);      //Laufstatistik sichern
  static void   restart(void);              //Aufsetzpunkt ermitteln
  static int    updateAblaufplan(void);     //Ablaufstatus -> E
  static int    insertBatchfehler(void);    //
/*-------------------------------------------------------------------*/
/*-- globale Variablen ---------------------------------------PKQVAR-*/
/*-------------------------------------------------------------------*/
  static ENV    Env;
  static DICT   vlkDict;         // Dict mit Eintrgen der Vorlaufkarte
  static GW     gw;              // GruppenWechsel-Instanz
  static GWSATZ gwSatz;          // GruppenWechsel-Satz
  static char   datumJetzt[11];  // Lauf-Datum (today()) Beginn
  static char   zeitJetzt[9];    // Lauf-Zeit (now()) Beginn
  int           testlauf;        // Kennung PX oder TEST
  int           KzVormerkung;    // AEND0010
  int           hSQLCODE;        // AEND0010
  char cDate[11],cTime[9];
  /*                                      AEND0006-A
  funcPtr       putInkasso;      // Funktion putInkasso / MKQ2550
                                          AEND0006-E  ***/

  static FILE   *PROT;           // Protokoll
  static FILE   *LIST;           // Liste
  static FILE   *SYSP;           // SYSPRINT auf SYSTERM ummappen
                                 // wg. Konflikten mit PLI !!

  static TABAP  tabAP;           // IFL (Informationsfluss) TAB_AP,
                                 // Werte der Ablaufplaung.
  static int    anzTabVJ;        // aktuelle Anzahl der Eintrge
                                 // in der Tabelle tabVJ
  static TABVJ  tabVJ;           // IFL (Informationsfluss) TAB_VJ,
                                 // Werte Vertragsjournal/Vertrag
  static AKT_BEARB aktB;         // IFL (Informationsfluss) AktBearb,
                                 // Werte akt. Bearbeitung KO/KI

  static int    insPos;          // pos innerhalb der tabVJ zum
                                 // Insert in TBKO_VTRJOURNAL
  static int    selPos;          // pos innerhalb der tabVJ zur
                                 // berprfung der Selktions-
                                 // kriterien

  static int    maxVorgLfdnr;    // max VorgLfdNr der tabVJ
                                 // funktioniert nur fr LBS,
                                 // da VSNR zu VTRNR eindeutig

  static int    maxHistnr;       // max Histnr der tabVJ

  static int    maxBearbNr;      // max BearbNr der tabVJ
                                 // aktB.nummer = maxBearbNr + 1

                                 //----- MKV3016 Anfang
  decimal(9,0)  stichtag3001Von; // 'Stichtag von' fr Folgesoll
  decimal(9,0)  stichtag3001Bis; // 'Stichtag bis' fr Folgesoll
  static int    nurDiffsoll;     // Boolean (aus MKV3016)
  static int    kzAnfo3016;      // Boolean (aus MKV3016)
  static int    folgesollAnford; // retcode als Bool aus MKV3016
  int           fsRC;            // RC folgesoll()
  int           commit_count;    // Zaehler fr Commit-Rate
  decimal(15,0) aktVtrnr;        // DiffSollPrfung bentigt diese Var
                                 // GrpWechsel innerhalb tabVJ
  decimal(11,0) newAktBearbNr;   // wird fast immer von DiffsollPrfung
                                 // gesetzt (ehemals MKV3016)
                                 //----- MKV3016 Ende

  static int    rollback;        // Rollback durchfuehren ?

  static int    tv3001;          // Boolean Anforderung Abrechnung
  static int    aw1571;          // Boolean Anforderung Abrechnung
  static int    aw2000;          // Boolean Anforderung Inkasso

  DICT          cDict;           // counterDict
  VTRWERTE      vtrWerte;        // fr vtrDict
  DICT          vtrDict;         // dict fr versch. Werte der
                                 // untersch. Vtrnrs des akt. VJ

/*-------------------------------------------------------------------*/
/*-- Protokoll-Zaehler ----------------------------------------------*/
/*-------------------------------------------------------------------*/

  /*decimal(15,2) RW006Wert;          AEND99999 */
  decimal(15,0) gesVSNR;         // Anzahl der vom Cursor ermittelten
                                 // VSNRs
  decimal(15,0) gesFetch;        // Anzahl eingelesener Journalein-
                                 // traege
  decimal(15,0) gesErh;          // aktuelle Anzahl der bereits erfolg-
                                 // reich verarbeiteten VSNRs
  static int    anzErh;          // Zaehler der verarbeiteten VSNR
                                 // innerhalb einen Zaehlintervalls
  decimal(15,0) gesStoSchwebe;   // Anzahl Storno-in-Schwebe-Vertraege
  decimal(15,0) anzFehler;       // Anzahl Fehlerhafter Vertrge
                                 // VSNRs
  static int    firstCommit;     // Erster Commit ? -> CHKPT_KZ setzen
  static int    Seite;           // Seitenzaehler
  static int    Zeile;           // Zeilenzaehler



  /*----- Tabellen, Variablen, Pointer, ... fr Abrechnung ----------*/
  S_MKO5010   smko5010;
  S_MKO5010*  pmko5010;
  S_MKO1518   smko1518;
  S_MKO1518*  pmko1518;
  S_MKO1571   smko1571;
  S_MKO1571*  pmko1571;
  S_MKO1580   smko1580;
  S_MKO1580*  pmko1580;

  static int  anzTabVG;             // aktuelle Anzahl der Eintrge
  TABVG       tabVG;

  TAB_ABR_ZEITRBER   tabAbrZeitR;   // TBKO_ZEITRBER
  TAB_ABR_RATENZAHL  tabAbrRaten;   // TBKO_RATENZAHL
  TAB_ABR_STEUERSATZ tabAbrSteuer;  // TBKO_RATENZAHL
  TAB_ABR_KUMULIER   tabAbrKum;     // TBKO_RATENZAHL

  int abrRC;                        // RC der Abrechnung (Kompass)
  char amodul[8];                   // Aufgerufenes Modul
  char ftext[60];                   // Fehlertext
  char ffunktion[30];               // Funktion, in der Fehler auftrat

  /*----- Tabellen, Variablen, Pointer, ... fr Inkasso    ----------*/
 /*M_MKQ2550   kq2550;                   AEND0006 ***

  static int  anzTabAbr;               aktuelle Anzahl der Eintrge
  TABABR      tabAbr;                  fuer MKQ2550

  int inkRC;                           RC mkq2550 TRUE/FALSE
                                         AEND00006 ***/

EXEC SQL BEGIN DECLARE SECTION;
 char           hMANDANT;
 decimal(15,0)  hVTRNR_VON;           // VtrJ-Cursor WHERE
 decimal(15,0)  hVTRNR_BIS;           // VtrJ-Cursor WHERE
 char           hVSNR_VON[11];        // VtrJ-Cursor WHERE
 char           hVSNR_BIS[11];        // VtrJ-Cursor WHERE
 decimal(3,0)   hVKAT_VON;            // VtrJ-Cursor WHERE
 decimal(3,0)   hVKAT_BIS;            // VtrJ-Cursor WHERE
 decimal(15,0)  hVTRNR;               // Kompass TBKO_VTRJOURNAL
 decimal(5,0)   hHISTNR;              // Kompass TBKO_VTRJOURNAL
 decimal(5,0)   hVORG_LFDNR;          // Kompass TBKO_VTRJOURNAL
 char           hVORG_SCHL[5];        // Kompass TBKO_VTRJOURNAL
 char           hVSNR[11];            // Kompass TBKO_VTRJOURNAL
 char           hPERSNR[8];           // Kompass TBKO_VTRJOURNAL
 char           hBEARBDT[11];         // Kompass TBKO_VTRJOURNAL
 char           hBEARBZEIT[9];        // Kompass TBKO_VTRJOURNAL
 char           hAEND_STATUS;         // Kompass TBKO_VTRJOURNAL
 char           hAENDDT[11];          // Kompass TBKO_VTRJOURNAL
 char           hABR_STATUS;          // Kompass TBKO_VTRJOURNAL
 char           hERHBISDT[11];        // Kompass TBKO_VTRJOURNAL
 char           hERSBISDT[11];        // Kompass TBKO_VTRJOURNAL
 short int      ivERHBISDT;           // Kompass TBKO_VTRJOURNAL Ind
 short int      ivERSBISDT;           // Kompass TBKO_VTRJOURNAL Ind
 char           hABRMODUS[3];         // Kompass TBKO_VTRJOURNAL
 decimal(15,2)  hVTGNBT;              // Kompass TBKO_VTRJOURNAL
 long int       hWHGNR;               // Kompass TBKO_VTRJOURNAL
 char           hFBKZ;                // Kompass TBKO_VTRJOURNAL
 long int       hBEARBNR;             // Kompass TBKO_VTRJOURNAL
 short int      hTWAEND;              // Kompass TBKO_VTRJOURNAL
 long int       hGSNRVG;              // Kompass TBKO_VTRJOURNAL
 char           hERHBISDT_KZ;         // Kompass TBKO_VTRJOURNAL
 decimal(3,0)   hVTRKATKZ;            // Kompass TBKO_VTRJOURNAL
 decimal(5,0)   hFOB_POL_HISTNR;      // Kompass TBKO_VTRJOURNAL
 char           hTEVORG_SCHL;         // Kompass TBKO_VTRJOURNAL
 char           hSCHW_AEND_STATUS;    // Kompass TBKO_VTRJOURNAL
 char           hRZ_PROZTYP;          // Kompass TBKO_VERTRAG
 decimal(5,2)   hRZ_PROZ;             // Kompass TBKO_VERTRAG
 short int      ivRZ_PROZ;            // Kompass TBKO_VERTRAG Ind
 long int       hCODEXNR;             // Kompass TBKO_VERTRAG
                                      // fr LBS: Datum abw. Faell.
 short int      ivCODEXNR;            // Kompass TBKO_VERTRAG Ind
 char           hBESTUEBKZ;           // Kompass TBKO_VERTRAG
 char           hVTGABL[11];          // Kompass TBKO_VERTRAG
 char           hABLKNDG_BIS[5];      // Kompass TBKO_VERTRAG
 char           hZAHLW[3];            // Kompass TBKO_VERTRAG
 char           hVTGBEGDT[11];        // Kompass TBKO_VERTRAG
 char           hTECHN_STATUS;        // Kompass TBKO_VERTRAG
 char           hABRTECH_STATUS;      // Kompass TBKO_VERTRAG
 decimal(5,0)   hVARTKZ;              // Kompass TBKO_VERTRAG
 char           hABW_STEUERKZ;        // Kompass TBKO_VERTRAG
 long int       hGSNRVGKI;            // Kavis TBKI_VERTRAG
 char           hZWEIG[5];            // Kavis TBKI_VERTRAG
 short int      hTWAENDKI;            // Kavis TBKI_VERTRAG
 char           hVSTATDAT[11];        // Kavis TBKI_VERTRAG
 char           hVSTAT;               // Kavis TBKI_VERTRAG
 char           hANTRART[7];          // Kavis TBKI_VERTRAG
 char           hANTRAUD[5];          // Kavis TBKI_VERTRAG
 char           hBATCHID[9];
 char           hSPRACHCODE;
 char           hGEPLLAUF[11];
 decimal(3,0)   hVSNRPORT;
 char           hSTICHTAGVON[11];
 char           hSTICHTAGBIS[11];
 char           hRESTART_VSNR[11];    // TBKQ_BATCH_COMMIT
 char           hDGEFLAUF[11];

 decimal(5,0)   hLFDNR;         // ID-Nummer, die den entspr. Wert in
                                // OTKQ_BATCH_STAT identifiziert
 decimal(15,2)  hWERT;          // Wert der Batch-Statistik fuer
                                // Wertesicherung
 long int       hRC;            // TBKQ_BATCHFEHLER
 char           hFEHLMELD[80];  // TBKQ_BATCHFEHLER
 char           hhAENDDT[11];
                                      /* AEND9999 Anfang

 char          hCOD_MANDANT;
 long int      hCOD_GESELLSCHAFT;
 char          hNR_POLICE[11];
 char          hCOD_SYSTEM[3];
 char          hID_EXTERN[16];
 long int      hNR_SAMMEL_EXTERN;
 decimal(15,0) hCOD_LEISTUNGSART;
 short int     hCOD_BUCH_TYP;
 short int     hCOD_VORGANG;
 long int      hNR_LFD;
 decimal(5,0)  hCOD_VTRART_KO;
 short int     hCOD_VORG_DETAIL;
 short int     hCOD_BUCH_DETAIL;
 short int     hCOD_ZAHLART;
 char          hDAT_FAELLIG[11];
 char          hDAT_GUELTIG_AB[11];
 char          hDAT_GUELTIG_BIS[11];
 char          hDAT_ABGABE[11];
 char          hCOD_ISOWHG[4];
 decimal(15,2) hBTR_BUCHUNG;
 decimal(15,2) hBTR_STEUER;
 decimal(5,2)  hPRO_STEUERSATZ;
 short int     hCOD_VERRECHNUNG;
 char          hKEZ_FEHLER;
 char          hKEZ_VERARB_INKASSO;

 char  testVSNR[11];
                                         AEND9999 Ende */


EXEC SQL END DECLARE SECTION;

/*-------------------------------------------------------------------*/
/*-- SQL Declare Cursor ---------------------------------------------*/
/*-------------------------------------------------------------------*/
/*        A C H T U N G - Wichtig fuer Cursor                        */
/*                                                                   */
/*  AEND0001 - Die Steuerung erfolgt nicht mehr ueber die Tabelle    */
/*             TBKO_VSNR_PORT sondern ueber TBKQ_VTRNR_PORT          */
/*                                                                   */
/*             Die folgende Where-Bedingung wurde ersetzt:           */
/*             WHERE J.VSNR BETWEEN :hVSNR_VON AND :hVSNR_BIS        */
/*                                                                   */
/*  AEND0002 - Cursor-Tuning -so unsinnig diese Massnahme erscheint, */
/*             sie zwingt den Zugriffspfad ueber den Clustered-Index */
/*             der Tabelle TBKO_VTRJOURNAL. Der Cursor wird dadurch  */
/*             sehr schnell geoeffnet und es erfolgt keine Sortierung*/
/*             -> Zeitersparnis                                      */
/*                                                                   */
/*             Die folgende Where-Bedingung wurde ersetzt:           */
/*             AND  V.VTRNR = J.VTRNR                                */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*********************************************************************/

EXEC SQL DECLARE VtrJ CURSOR WITH HOLD FOR
     SELECT J.VTRNR,          J.HISTNR,     J.VORG_LFDNR, J.VORG_SCHL,
            J.VSNR,           J.PERSNR,     J.BEARBDT,    J.BEARBZEIT,
            J.AEND_STATUS,    J.AENDDT,     J.ABR_STATUS, J.ERHBISDT,
            J.ERSBISDT,       J.ABRMODUS,   J.VTGNBT,     J.WHGNR,
            J.FBKZ,           J.BEARBNR,    J.TWAEND,     J.GSNRVG,
            J.ERHBISDT_KZ,    J.VTRKATKZ,
            J.FOB_POL_HISTNR, J.TEVORG_SCHL,
            V.RZ_PROZTYP,     V.RZ_PROZ,    V.CODEXNR,    V.BESTUEBKZ,
            V.VTGABL,         V.ABLKNDG_BIS, V.ZAHLW,     V.VTGBEGDT,
            V.TECHN_STATUS,   V.ABRTECH_STATUS,           V.VARTKZ,
            V.ABW_STEUERKZ,
            I.GSNRVG,         I.ZWEIG,      I.TWAEND,     I.VSTATDAT,
            I.VSTAT,          I.ANTRART,    I.ANTRAUD
     FROM   TBKO_VTRJOURNAL J, TBKO_VERTRAG V, TBKI_VERTRAG I
                                         /*   AEND0001 - Anfang    */
     WHERE  J.VTRNR    BETWEEN :hVTRNR_VON AND :hVTRNR_BIS
                                         /*   AEND0001 - Ende      */
       AND  J.VTRKATKZ BETWEEN :hVKAT_VON AND :hVKAT_BIS
                                         /*   AEND0002 - Anfang    */
       AND  V.VTRNR = J.VTRNR + 0        /* ==> siehe Hinweis      */
                                         /*   AEND0002 - Ende      */
     /*AND  J.BEARBDT > '30.11.2004'          AEND0014             */
       AND  J.BEARBDT > CURRENT DATE - 15 MONTH        /* AEND0014 */
       AND  V.VSNR  = I.VSNR
       AND  V.HISTNR = ( SELECT MAX(V1.HISTNR)
                         FROM   TBKO_VERTRAG V1
                         WHERE  V1.VTRNR = J.VTRNR
                         AND    V1.HISTNR <= J.HISTNR )
       AND  J.VSNR = I.VSNR
       AND  I.MANDANT = :hMANDANT
     /*AND  J.FBKZ    = 'Z'                AEND0005/AEND0007 */
     ORDER BY  J.VTRNR ASC,
               J.HISTNR DESC, J.VORG_LFDNR DESC;
        /*     fr Folgesoll Leben wird nach                       */
        /*     J.VTRNR ASC, J.HISTNR DESC, J.VORG_LFDNR DESC;      */
        /*     vorheriger Cursor: J.VSNR ASC, J.VTRNR DESC,        */
        /*                        J.HISTNR DESC, J.VORG_LFDNR DESC;*/


                                   /* AEND9999 Anfang

EXEC SQL DECLARE testRW CURSOR FOR
     SELECT
     COD_MANDANT,         COD_GESELLSCHAFT,    NR_POLICE,
     NR_SAMMEL_EXTERN,    COD_SYSTEM,          ID_EXTERN,
     COD_VTRART_KO,       COD_LEISTUNGSART,    COD_BUCH_TYP,
     COD_VORGANG,         COD_VORG_DETAIL,     COD_BUCH_DETAIL,
     COD_ZAHLART,         DAT_FAELLIG,         DAT_GUELTIG_AB,
     DAT_GUELTIG_BIS,     DAT_ABGABE,          COD_ISOWHG,
     BTR_BUCHUNG ,        BTR_STEUER,          PRO_STEUERSATZ,
     COD_VERRECHNUNG,     KEZ_FEHLER,          KEZ_VERARB_INKASSO
     FROM OTIN_RW006
     WHERE NR_POLICE = :testVSNR
       AND COD_MANDANT = '1'
       AND COD_GESELLSCHAFT = 1
     ORDER BY BTR_BUCHUNG
     ;

    typedef _Packed struct _s_test_rw006
      { char       COD_MANDANT;
        long int   COD_GESELLSCHAFT;
        char       NR_POLICE[11];
        char       COD_SYSTEM[3];
        char       ID_EXTERN[16];
        long int   NR_SAMMEL_EXTERN;
        decimal(15,0)  COD_LEISTUNGSART;
        short int  COD_BUCH_TYP;
        short int  COD_VORGANG;
        decimal(5,0)   COD_VTRART_KO;
        short int  COD_VORG_DETAIL;
        short int  COD_BUCH_DETAIL;
        short int  COD_ZAHLART;
        char       DAT_FAELLIG[11];
        char       DAT_GUELTIG_AB[11];
        char       DAT_GUELTIG_BIS[11];
        char       DAT_ABGABE[11];
        char       COD_ISOWHG[4];
        decimal(15,2)  BTR_BUCHUNG;
        decimal(15,2)  BTR_STEUER;
        decimal(5,2)   PRO_STEUERSATZ;
        short int  COD_VERRECHNUNG;
        char       KEZ_FEHLER;
        char       KEZ_VERARB_INKASSO;
     } S_TEST_RW006;

   S_TEST_RW006    rw006;
   S_TEST_RW006    * prw006;

                                      AEND9999 ENDE   */


/*-------------------------------------------------------------------*/
/*-- Programmbeginn ------------------------------------------PKQANF-*/
/*-------------------------------------------------------------------*/

/*===main()============================================================
  Synopsis:      int main(void)
  Parameter:     keine
  Return:        0  OK
                 16 Fehler und Programmabbruch
  Beschreibung:  Hauptfunktion
  Modifikation:  03.03.1998 JoBarden/pdv  Ersterstellung
=====================================================================*/
int main(void) {
 int rc;

 /*prw006 = &rw006;      AEND9999 */

   SYSP = fopen("dd:SYSTERM","w");    // SYSPRINT auf SYSTERM ummappen
   stdout = SYSP;                     // wg. Konflikten mit KO-PLI !!


   Env = initAG0002(Env);
   printf("PKQ2550: main() Beginn\n");

   cDict = openDict("zaehler", 10, 0, sizeof(int), UNIQUE);

   TRY
      vorlauf();
   RECOVER
      printf("PKQ2550: main() RECOVER ---> Fehler im Vorlauf\n");
      printf("PKQ2550: main() RECOVER ---> errmsg()\n%s\n", errmsg());
      exit(12);
   END

   TRY
      verarbeitung();
      today(cDate); now(cTime);
      printf("PKQ2550: main() nach verarbeitung() OK: %s %s\n",
         cDate, cTime);
   RECOVER
      today(cDate); now(cTime);
      printf("PKQ2550: main() RECOVER ---> nach verarbeitung() %s %s\n",
         cDate, cTime);
      printf(
         "PKQ2550: main() RECOVER ---> Fehler in der Verarbeitung\n");
      printf("PKQ2550: main() RECOVER ---> ROLLBACK performed\n");
      printf("PKQ2550: main() RECOVER ---> errmsg()\n%s\n", errmsg());
      EXEC SQL ROLLBACK;
      exit(16);
   END

   TRY
      nachlauf();

   RECOVER
      printf("PKQ2550: main() RECOVER ---> Fehler im Nachlauf\n");
      printf("PKQ2550: main() RECOVER ---> ROLLBACK performed\n");
      EXEC SQL ROLLBACK;
      exit(16);
   END

   if (rollback) {
   today(cDate); now(cTime);
   printf("PKQ2550: main() vor  rollback() OK: %s %s\n",
      cDate, cTime);
   EXEC SQL ROLLBACK;
   today(cDate); now(cTime);
   printf("PKQ2550: main() nach rollback() OK: %s %s\n",
      cDate, cTime);
   }

   return(0);
}

/*===vorlauf()=========================================================
  Synopsis:      void vorlauf(void);
  Beschreibung:  Vorlauf
                 ffnen der Protokoll-Datei(en), Lesen von TBKO_
                 ABLAUFPLAN, ...
  Modifikation:  03.03.1998 JoBarden/pdv  Ersterstellung
=====================================================================*/
static void vorlauf(void) {

 int mkq2550RC = 0;

   gesVSNR= 0;
   gesFetch = 0;
   gesErh= 0;
   anzErh= 0;
   gesStoSchwebe = 0;
   anzFehler = 0;
   firstCommit = 0;
   Seite= 1;
   Zeile= 0;
   /*RW006Wert  = 0D;        AEND99999 */
   /*kq2550.anzabr    = 0;      AEND0006
   kq2550.vtrbtg    = 0D;
   kq2550.vtrinkbtg = 0D;
   kq2550.btg_verr  = 0D;
   kq2550.anzRW006  = 0D;
   kq2550.WertRW006 = 0D;
   kq2550.anzRW004  = 0D;
   kq2550.WertRW004 = 0D;       AEND0006 ***/

   /*--- Eroeffnen der Protokolldatei -------------------------------*/
   Assert (((PROT = fopen(PROT_DDNAME,"w")) != NULL),
            ABBRUCH, EC_OpenAusgabedatei,
            "PKQ2550: vorlauf() Open Ausgabedatei PROT failed" );

   /*--- Eroeffnen der Listen-Datei   -------------------------------*/
   Assert (((LIST = fopen(LIST_DDNAME,"w")) != NULL),
            ABBRUCH, EC_OpenListdatei,
            "PKQ2550: vorlauf() Open Listen-Datei LIST failed" );

   now(zeitJetzt);
   today(datumJetzt);
   printf(".....today/now: Datum %s Zeit %s\n",datumJetzt,zeitJetzt);


   PLS(".---------------------------------------------------------.\n");
   PLS("!  Folgesoll LBS                                 PKQ2550  !\n");
   PLS("!  STORNO in SCHWEBE - Liste                              !\n");
   PLS("!---------------------------------------------------------'\n");
   PLS(" \n");
   PLS("  Zeit   VSNR       VtrNr           Bemerkung             !\n")
 /*"Zeit   VSNR       VtrNr           Fsl  Abr  Ink  Info/Fehler\n");*/
 /*PLS(                                                              */
   PLS("-----------------------------------------------------------\n");

   fprintf(PROT,"****************************************************");
   fprintf(PROT,"*********************\n");
   fprintf(PROT,
    "*  Folgesoll LBS  Laufprotokoll vom %s             Seite %5d  *\n",
        datumJetzt,Seite);
   fprintf(PROT,"*  Programm PKQ2550                                 ");
   fprintf(PROT,"                    *\n");
   fprintf(PROT,"                    *\n");
   fprintf(PROT,"****************************************************");
   fprintf(PROT,"*********************\n");

   printf("****************************************************");
   printf("*********************\n");
   printf(
    "*  Folgesoll LBS  Laufprotokoll vom %s              Seite %d *\n",
        datumJetzt,Seite);
   printf("*  Programm PKQ2550                                 ");
   printf("                    *\n");
   printf("****************************************************");
   printf("*********************\n");

   vorlaufkarteAuswerten();

   getmandantsprach();

   ablaufplan_vtrnr_port();

   aufbauAbrGrundlagen();

   /*--- Fetch und Initialisierung vom MKQ2550 ----AEND0006----------
   putInkasso = (funcPtr) fetch("mkq2550");

   if(putInkasso == NULL) {
      escape(ABBRUCH, EC_MKQ2550_FETCH_ERR,
         "PKQ2550: vorlauf() MKQ2550-Fetch failed" );
   } else {
      printf("PKQ2550: Erstaufruf von Modul MKQ2550 OK\n");
   }

     -- fllen der bergabestruktur fpr MKQ2550 --AEND0006-E---------*/
   memcpy(hDGEFLAUF,datumJetzt,11);
   /*kq2550.pVJ       = &tabVJ;                   AEND0006-A
   kq2550.anzvj     = 0;
   kq2550.pAB       = &aktB;
   kq2550.pABR      = &tabAbr;
   memcpy(kq2550.laufdatum,datumJetzt,11);
   kq2550.mandant    = smko5010.mandant;
   kq2550.sprachcode = smko5010.sprachcode;

   mkq2550RC = (*putInkasso)(&kq2550, Env);
   if (mkq2550RC == 0) {
     printf("PKQ2550: Initialisierung MKQ2550 OK\n");
   } else {
     printf("PKQ2550: Initialisierung MKQ2550 fehlerhaft\n");
   }                                               AEND0006-E ***/
}

/*===nachlauf()========================================================
  Synopsis:      void nachlauf(void)
  Beschreibung:  Nachlauf
  Modifikation:  03.03.1998 JoBarden/pdv  Ersterstellung
=====================================================================*/
static void nachlauf(void) {
int rc;

   updateAblaufplan();
   Batch_Statistik();                       /***  AEND0003  ***/



   /*--- Protokollieren der Verarbeitungszhler ---------------------*/
   fprintf(PROT,"***********************************\n");
   fprintf(PROT,"********    Laufstatistik    ******\n");
   fprintf(PROT,"***********************************\n");
   fprintf(PROT,"Eingelesene Vertraege                : %15D(15,0)\n",
           gesVSNR);
   fprintf(PROT,"Eingelesene Journaleintraege         : %15D(15,0)\n",
           gesFetch);
   fprintf(PROT,"Erhobene Vertraege                   : %15D(15,0)\n",
           gesErh);
   fprintf(PROT,"Anzahl Storno in Schwebe             : %15D(15,0)\n",
           gesStoSchwebe);
 /*fprintf(PROT,"Wert eingef. Inkassosaetze (RW006)   : %15D(15,2)\n",
           RW006Wert); */
                                        /*** AEND0006-A
   fprintf(PROT,"Anzahl eingef. Inkassosaetze (RW006) : %15D(15,0)\n",
           kq2550.anzRW006);
   fprintf(PROT,"Wert eingef. Inkassosaetze (RW006)   : %15D(15,2)\n",
           kq2550.WertRW006);
   fprintf(PROT,"Anzahl Updates auf RW004             : %15D(15,0)\n",
           kq2550.anzRW004);
   fprintf(PROT,"Wert Beitragsvorwegabzug (RW004)     : %15D(15,2)\n",
           kq2550.WertRW004);
                                         *** AEND0006-E ***/
   fprintf(PROT,"Anzahl fehlerhafter Vertraege        : %15D(15,0)\n",
           anzFehler);
   fprintf(PROT," \n");
   /*iterDict(cDict, 0);*/
   sortDict(cDict, 'K', 0, 0, "A", 0);
   while(nextDict(cDict)) {
    int *d = dataDict(cDict);
      fprintf(PROT,"** Z{hler: %-40.40s: %8d\n",
         keyDict(cDict), *d);
   }

   /*--- Schlieen der PROT-Ausgabedatei ----------------------------*/

   fclose(PROT);

}

/*===vorlaufkarteAuswerten()===========================================
  Synopsis:      void vorlaufkarteAuswerten(void)
  Beschreibung:  Auswertung der Vorlaufkarte und Lock-Table-Steuerung
  Modifikation:  03.03.1998 NoHunger  Ersterstellung
=====================================================================*/
static void vorlaufkarteAuswerten(void) {

 char *pTemp;

   vlkDict = openDict("VlkDict",10, 0, 0, "U");
   getParm(vlkDict,"READER");

   pTemp=getDict(vlkDict, "LOCK");
   printf("Eingestellte Werte in der Vorlaufkarte:\n");
   printf(" \n");

   printf("LOCK:     %.5s   ",pTemp);
   if (pTemp[0] == 'N') {
      printf("==> es wird kein LOCK-TABLE durchgefuehrt\n");
   } else {
      printf("==> LOCK-TABLE wird durchgefuehrt\n");
      lockTable();
   }

   pTemp=getDict(vlkDict, "ROLLBACK");
   printf("ROLLBACK: %.5s   ",pTemp);

   if (pTemp[0] == 'N') {
      printf("  ==> es wird kein ROLLBACK durchgefuert\n");
      rollback = FALSE;
   } else {
      printf("  ==> ROLLBACK wird durchgefuert\n");
      rollback = TRUE;
   }

   pTemp=getDict(vlkDict, "COMMIT");
   printf("COMMIT:   %.5s ",pTemp);

   commit_count   = (atoi(pTemp));
   printf(" ==> COMMIT-Frequenz: %d \n",commit_count);
   printf(" \n");
   printf(" \n");

}

/*===lockTable()=====================================================
  Synopsis:      void lockTable(void)
  Beschreibung:  Exclusives Locken der DB2-Kompass-Tabellen
  Modifikation:  03.03.1998 NoHunger  Ersterstellung
=====================================================================*/
static void lockTable(void) {
  printf("Es folgt LOCK-TABLE\n");
  EXEC SQL
     LOCK TABLE  TBKO_VTRJOURNAL    IN EXCLUSIVE MODE;
     CHECK0(ABBRUCH, "PKQ2550: LOCK TABLE TBKO_VTRJOURNAL  ");
  EXEC SQL
     LOCK TABLE  TBKO_VTRJOUTEIL    IN EXCLUSIVE MODE;
     CHECK0(ABBRUCH, "PKQ2550: LOCK TABLE TBKO_VTRJOUTEIL  ");
  EXEC SQL
     LOCK TABLE  TBKO_LVB           IN EXCLUSIVE MODE;
     CHECK0(ABBRUCH, "PKQ2550: LOCK TABLE TBKO_LVB         ");
  EXEC SQL
     LOCK TABLE  TBKO_ABR_VORGANG   IN EXCLUSIVE MODE;
     CHECK0(ABBRUCH, "PKQ2550: LOCK TABLE TBKO_ABR_VORGANG ");
 /*EXEC SQL                                           AEND0008
     LOCK TABLE  TBKO_ABR_BETRAG    IN EXCLUSIVE MODE;
     CHECK0(ABBRUCH, "PKQ2550: LOCK TABLE TBKO_ABR_BETRAG ");
  EXEC SQL
     LOCK TABLE  TBKO_ABR_BUCHUNG   IN EXCLUSIVE MODE;
     CHECK0(ABBRUCH, "PKQ2550: LOCK TABLE TBKO_ABR_BUCHUNG ");*/
  EXEC SQL
     LOCK TABLE  TBKO_ABR_VST       IN EXCLUSIVE MODE;
     CHECK0(ABBRUCH, "PKQ2550: LOCK TABLE TBKO_ABR_VST     ");
  EXEC SQL
     LOCK TABLE  TBKO_ABR_GEBUEHR   IN EXCLUSIVE MODE;
     CHECK0(ABBRUCH, "PKQ2550: LOCK TABLE TBKO_ABR_GEBUEHR ");
  EXEC SQL
     LOCK TABLE  TBKO_SCHNITTSTELLE IN EXCLUSIVE MODE;
     CHECK0(ABBRUCH, "PKQ2550: LOCK TABLE TBKO_SCHNITTSTELLE");
  EXEC SQL
     LOCK TABLE  TBKO_VTRKORRESP    IN EXCLUSIVE MODE;
     CHECK0(ABBRUCH, "PKQ2550: LOCK TABLE TBKO_VTRKORRESP   ");
  EXEC SQL
     LOCK TABLE  TBKO_KORRESP       IN EXCLUSIVE MODE;
     CHECK0(ABBRUCH, "PKQ2550: LOCK TABLE TBKO_KORRESP      ");
  EXEC SQL
     LOCK TABLE  TBKO_ABR_FOB_VG    IN EXCLUSIVE MODE;
     CHECK0(ABBRUCH, "PKQ2550: LOCK TABLE TBKO_ABR_FOB_VG   ");
  EXEC SQL
     LOCK TABLE  TBKO_ABR_FOB_BU    IN EXCLUSIVE MODE;
     CHECK0(ABBRUCH, "PKQ2550: LOCK TABLE TBKO_ABR_FOB_BU   ");
  EXEC SQL
     LOCK TABLE  TBKO_ABR_FOB_BG    IN EXCLUSIVE MODE;
     CHECK0(ABBRUCH, "PKQ2550: LOCK TABLE TBKO_ABR_FOB_BG   ");
  EXEC SQL
     LOCK TABLE  TBKO_ABR_PROV_VG   IN EXCLUSIVE MODE;
     CHECK0(ABBRUCH, "PKQ2550: LOCK TABLE TBKO_ABR_PROV_VG  ");
  EXEC SQL
     LOCK TABLE  TBKO_ABR_PROV_AP   IN EXCLUSIVE MODE;
     CHECK0(ABBRUCH, "PKQ2550: LOCK TABLE TBKO_ABR_PROV_AP  ");
  EXEC SQL
     LOCK TABLE  TBKO_ABR_PROV_IP   IN EXCLUSIVE MODE;
     CHECK0(ABBRUCH, "PKQ2550: LOCK TABLE TBKO_ABR_PROV_IP  ");
  EXEC SQL
     LOCK TABLE  TBKO_VERTRAG       IN EXCLUSIVE MODE;
     CHECK0(ABBRUCH, "PKQ2550: LOCK TABLE TBKO_VERTRAG      ");
  EXEC SQL
     LOCK TABLE  TBKO_BVB           IN EXCLUSIVE MODE;
     CHECK0(ABBRUCH, "PKQ2550: LOCK TABLE TBKO_BVB          ");
  EXEC SQL
     LOCK TABLE  TBKF_LVB_SFR       IN EXCLUSIVE MODE;
     CHECK0(ABBRUCH, "PKQ2550: LOCK TABLE TBKO_LVB_SFR      ");
  EXEC SQL
     LOCK TABLE  TBKO_LEIBED        IN EXCLUSIVE MODE;
     CHECK0(ABBRUCH, "PKQ2550: LOCK TABLE TBKO_LEIBED       ");
  EXEC SQL
     LOCK TABLE  TBKO_VTRBED        IN EXCLUSIVE MODE;
     CHECK0(ABBRUCH, "PKQ2550: LOCK TABLE TBKO_VTRBED       ");
  EXEC SQL
     LOCK TABLE  TBKF_K_BEITR_VB    IN EXCLUSIVE MODE;
     CHECK0(ABBRUCH, "PKQ2550: LOCK TABLE TBKF_K_BEITR_VB   ");
  EXEC SQL
     LOCK TABLE  TBKO_VTR_SCHWEBE   IN EXCLUSIVE MODE;
     CHECK0(ABBRUCH, "PKQ2550: LOCK TABLE TBKO_VTR_SCHWEBE  ");
  EXEC SQL
     LOCK TABLE  TBKO_POL_SCHWEBE   IN EXCLUSIVE MODE;
     CHECK0(ABBRUCH, "PKQ2550: LOCK TABLE TBKO_POL_SCHWEBE  ");
  printf("LOCK-TABLE erfolgreich beendet\n");
}


/*===getmandantsprach() ===============================================
  Synopsis:      getmandandsprach(void)
  Beschreibung:  Mit dem Kompass-Modul MKO5010 wird der Name des
                 ausfuehrenden MVS-Jobs ermittelt. Mit dem Jobnamen
                 wird auf die Tabelle TBKO_KOMP_BATCH zugegriffen
                 und die Spalten MANDANT und SPRACHCODE gelesen.
                 Diese Werte werden an das rufende Programm zurueck-
                 gegeben.
  Modifikation:  10.06.98 N.Hungerecker  Ersterstellung
=====================================================================*/
static void getmandantsprach(void) {

   /*--- Struktur fr Aufruf MKO5010 fllen -------------------------*/
     memcpy(smko5010.strukturName, "XKO5010 ", 8);
   memcpy(smko5010.version, "0001", 4);

   pmko5010 = &smko5010;        // wg. Doppel-Ptr PLI

   MKO5010(&pmko5010);

   printf("Ermittelte Werte aus MK5010:\n");
   printf(" \n");

   if(memcmp(smko5010.retcode, "0000", 4) == 0) {
      printf("Mandant:     %c\n",smko5010.mandant);
      printf("Sprachcode   %c\n",smko5010.sprachcode);
      printf("Batch-ID     %.8s\n",smko5010.batchid);
   } else {
      fprintf(PROT,"Fehler aus MKO5010 - RC: %.4s\n",smko5010.retcode);
      fprintf(PROT,"                 FMODUL: %s\n",smko5010.fmodul);
      printf("Fehler aus MKO5010 - RC: %.4s\n",smko5010.retcode);
      printf("                 FMODUL: %s\n",smko5010.fmodul);
      escape(ABBRUCH, EC_MKO5010_Fehler,
         "PKQ2550: getmandantsprach() MKO5010 failed" );
   }
   printf(" \n");

   if (smko5010.batchid[0] == 'P') {
      printf("Job laeuft in Praxisumgebung \n");
      testlauf = FALSE;
   } else {
      printf("Job laeuft in Test/Systemtestumgebung \n");
      testlauf = TRUE;
   }
   printf(" \n");
   printf(" \n");

}
/*===ablaufplan_vtrnr_port=============================================
  Synopsis:      int ablaufplan_vtrnr_port(void)
  Beschreibung:  Mit dem Kompass-Modul MKO2500 wird die Struktur
                 tabAP gefllt. MKO2500 macht Zugriffe auf die
                 Tabsystabellen TBKO_ABLAUFPLAN und abhngige.
                 MKO2500 gibt den nchsten noch nicht bearbeiteten
                 Lauf (min VSNR-Portion & min geplant_Datum) zurck.
                 Beim Aufruf wird der Status in TBKO_ABLAUFPLAN
                 auf 'B' (in Bearbeitung gesetzt).
  Modifikation:  03.03.1998 JoBarden/pdv  Ersterstellung
=====================================================================*/
static void ablaufplan_vtrnr_port(void) {
 S_MKO2500  smko2500;
 S_MKO2500* pmko2500;
 int i;
 int t3001, t3005, t3006;
 char tDate[11];

   /*--- Struktur fr Aufruf MKO2500 fllen -------------------------*/
   memcpy(smko2500.ende, "ENDE", 4);
   memcpy(smko2500.strukturName, "SMKO2500", 8);
   memcpy(smko2500.version, "01.00", 5);
   memcpy(smko2500.batchID, smko5010.batchid, 8);
   smko2500.aufrufKz = 'A';
   smko2500.updateKz = 'N'; // Original
   smko2500.mandant = smko5010.mandant;
   smko2500.sprachcode = smko5010.sprachcode;
   smko2500.pAP = &tabAP;

   pmko2500 = &smko2500;        // wg. Doppel-Ptr PLI

   MKO2500(&pmko2500);

   fprintf(PROT,"Ermittelte Werte aus Ablaufplan:\n");
   fprintf(PROT," \n");
   printf("Ermittelte Werte aus Ablaufplan:\n");
   printf(" \n");

   if(memcmp(smko2500.retcode, "0000", 4) == 0) {
      fprintf(PROT,"BATCH_ID:         %.8s\n",
         smko2500.pAP->ap.batchID);
      fprintf(PROT,"GEPL_LAUF:        %D(9,0)\n",
         smko2500.pAP->ap.datGeplant);
      fprintf(PROT,"VSNR_PORTION:     %D(3,0)\n",
         smko2500.pAP->ap.vsnrPortionNr);
   /* fprintf(PROT,"VSNR   Von - Bis: %.10s - %.10s\n",
         smko2500.pAP->ap.vonVSNR, smko2500.pAP->ap.bisVSNR);*/
      fprintf(PROT,"VTRKAT Von - Bis: %D(3,0) - %D(3,0)\n",
         smko2500.pAP->ap.vonVtrKatKz, smko2500.pAP->ap.bisVtrKatKz);
      fprintf(PROT,"TVSCHL:           %.4s    Stichtag: %D(9,0)\n",
         smko2500.pAP->bis[0].tvschl,smko2500.pAP->bis[0].stichtag);
      fprintf(PROT,"TVSCHL:           %.4s    Stichtag: %D(9,0)\n",
         smko2500.pAP->bis[1].tvschl,smko2500.pAP->bis[1].stichtag);
      printf("BATCH_ID:         %.8s\n",
         smko2500.pAP->ap.batchID);
      printf("GEPL_LAUF:        %D(9,0)\n",
         smko2500.pAP->ap.datGeplant);
      printf("VSNR_PORTION:     %D(3,0)\n",
         smko2500.pAP->ap.vsnrPortionNr);
   /* printf("VSNR   Von - Bis: %.10s - %.10s\n",
         smko2500.pAP->ap.vonVSNR, smko2500.pAP->ap.bisVSNR); */
      printf("VtrKat Von - Bis: %D(3,0) - %D(3,0)\n",
         smko2500.pAP->ap.vonVtrKatKz, smko2500.pAP->ap.bisVtrKatKz);
      printf("TVSCHL:           %.4s    Stichtag: %D(9,0)\n",
         smko2500.pAP->bis[0].tvschl,smko2500.pAP->bis[0].stichtag);
      printf("TVSCHL:           %.4s    Stichtag: %D(9,0)\n",
         smko2500.pAP->bis[1].tvschl,smko2500.pAP->bis[1].stichtag);


   // Feste Werte fuer Checkpointschreibung einstellen
      memcpy(hBATCHID,smko2500.pAP->ap.batchID,8);
             hBATCHID[8] = '\0';
      C_DATE(smko2500.pAP->ap.datGeplant,tDate);
      memcpy(hGEPLLAUF,tDate,10);
             hGEPLLAUF[10] = '\0';
      hMANDANT     = smko5010.mandant;
      hSPRACHCODE  = smko5010.sprachcode;
      hVSNRPORT    = smko2500.pAP->ap.vsnrPortionNr;

   } else {
      fprintf(PROT,"Fehler aus MKO2500  RC : %.4s\n",smko2500.retcode);
      fprintf(PROT,"                SqlCode: %d     fmodul: %.7s\n",
              smko2500.sqlcode, smko2500.fmodul);
      printf("Fehler aus MKO2500  RC : %.4s\n",smko2500.retcode);
      printf("                SqlCode: %d     fmodul: %.7s\n",
              smko2500.sqlcode, smko2500.fmodul);
      escape(ABBRUCH, EC_MKO2500_Fehler,
         "PKQ2550: ablaufplan() MKO2500 failed" );
   }


                                /*****   AEND0001 - Anfang    ***/
/*-------------------------------------------------------------------*/
/*   TBKQ_VTRNR_PORT  Nummernkreis ermitteln                         */
/*-------------------------------------------------------------------*/
   EXEC SQL
      SELECT VON_VTRNR
            ,BIS_VTRNR
      INTO
            :hVTRNR_VON
           ,:hVTRNR_BIS

      FROM  TBKQ_VTRNR_PORT
      WHERE VSNR_PORTION = :hVSNRPORT
        AND MANDANT      = :hMANDANT
        AND SPRACHCODE   = :hSPRACHCODE
   ;

   CHECK(ABBRUCH,
     "PKQ2550: Fehler bei Select in Tabelle TBKQ_VTRNR_PORT");
   CHECK0(ABBRUCH,
     "PKQ2550: In TBKQ_VTRNR_PORT kein Eintrag zu VSNR-Portion vorh.");
   fprintf(PROT,"VTRNR   Von - Bis:  %D(15,0) - %D(15,0)\n",
                        hVTRNR_VON,hVTRNR_BIS);
   printf("VTRNR   Von - Bis:  %D(15,0) - %D(15,0)\n",
                        hVTRNR_VON,hVTRNR_BIS);
                                /*****   AEND0001 - Ende      ***/
   fprintf(PROT," \n");


   /****************************************************************/
   /*  Fuer die Checkpointschreibung werden die Hostvariablen      */
   /*  versorgt. Anschliessend wird auf RESTART geprueft und die   */
   /*  einige Hostvariablen fuer den VTRJ-Cursor versorgt          */
   /*                                                              */
   /*  Bei Restart (Checkpoint-KZ in TBKO_ABLAUFPLAN = J) wird     */
   /*  die Tabelle TBKQ_BATCH_COMMIT gelesen und die letzte ein-   */
   /*  getragene VTRNr in die Hostvariable hVTRNR_VON eingestellt. */
   /*  Somit laeuft der Cursor nicht ueber die gesamte Datenmenge. */
   /*  Wird kein Eintrag in der Tabelle gefunden, so erfolgt ein   */
   /*  Programmabbruch.                                            */
   /****************************************************************/


   if (tabAP.ap.checkpoint == 'J') {
      restart();
      fprintf(PROT,"!!!!!   ACHTUNG  R E S T A R T   !!!!!\n");
      printf("!!!!!   ACHTUNG  R E S T A R T   !!!!!\n");
      fprintf(PROT,"Cursorverarbeitung beginnt bei:");
      fprintf(PROT,"VTRNR: %D(15,0), VSNR  %.10s\n",
                    hVTRNR_VON,hRESTART_VSNR);
                                    /*** AEND0003 - ANFANG  ***/
   /*--- Protokollieren der ermittelten Restartwerte ----------------*/
   fprintf(PROT,"***********************************\n");
   fprintf(PROT,"** Ermittelte Werte bei Restart ***\n");
   fprintf(PROT,"***********************************\n");
   fprintf(PROT,"Eingelesene Vertraege                : %15D(15,0)\n",
           gesVSNR);
   fprintf(PROT,"Eingelesene Journaleintraege         : %15D(15,0)\n",
           gesFetch);
   fprintf(PROT,"Erhobene Vertraege                   : %15D(15,0)\n",
           gesErh);
   fprintf(PROT,"Anzahl Storno in Schwebe             : %15D(15,0)\n",
           gesStoSchwebe);
                                          /*** AEND0006-A
   fprintf(PROT,"Anzahl eingef. Inkassosaetze (RW006) : %15D(15,0)\n",
           kq2550.anzRW006);
   fprintf(PROT,"Wert eingef. Inkassosaetze (RW006)   : %15D(15,2)\n",
           kq2550.WertRW006);
   fprintf(PROT,"Anzahl Updates auf RW004             : %15D(15,0)\n",
           kq2550.anzRW004);
   fprintf(PROT,"Wert Beitragsvorwegabzug (RW004)     : %15D(15,2)\n",
           kq2550.WertRW004);
                                           *** AEND0006-E ***/
   fprintf(PROT,"Anzahl fehlerhafter Vertraege        : %15D(15,0)\n",
           anzFehler);
   fprintf(PROT," \n");
                                           /*** AEND0003 - ENDE ***/
   }
   else {
 /*hVTRNR_VON = 400000000000000D; */
   strncpy(hVSNR_VON, tabAP.ap.vonVSNR, 10);
   }


   strncpy(hVSNR_BIS, tabAP.ap.bisVSNR, 10);
   hVKAT_VON = tabAP.ap.vonVtrKatKz;
   hVKAT_BIS = tabAP.ap.bisVtrKatKz;


   /*---- merken von einigen Werten. Diese werden fr die Prfung ---*/
   /*---- auf Differenzfolgesoll bentigt (ehem. MKV3016) -----------*/

   t3001=t3005=t3006 = FALSE;
   for(i=0; i < MAX_APTAB; i++) {
      if(memcmp(tabAP.bis[i].tvschl, SZ_FOLGESOLL, 4) == 0) {
         stichtag3001Von = tabAP.von[i].stichtag;
         stichtag3001Bis = tabAP.bis[i].stichtag;
         C_DATE(tabAP.von[i].stichtag, tDate);
         memcpy(hSTICHTAGVON,tDate,10);
                hSTICHTAGVON[10] = '\0';
         C_DATE(tabAP.bis[i].stichtag, tDate);
         memcpy(hSTICHTAGBIS,tDate,10);
                hSTICHTAGBIS[10] = '\0';
    //   hSTICHTAGVON = tabAP.von[i].stichtag;
    //   hSTICHTAGBIS = tabAP.bis[i].stichtag;
         t3001 = TRUE;
      }
      if(memcmp(tabAP.bis[i].tvschl, SZ_ANPSOLL, 4) == 0) {
         t3005 = TRUE;
      }
      if(memcmp(tabAP.bis[i].tvschl, SZ_DIFFSOLL, 4) == 0) {
         t3006 = TRUE;
      }
   }
   if(t3001 && t3005) {
      kzAnfo3016 = TRUE;         //
   } else {
      kzAnfo3016 = FALSE;        // Folgesoll durchfhren MKV3016
   }                             // muss nicht ausgefhrt werden

   if(t3006) {
      nurDiffsoll = TRUE;     // Folgesoll nicht durchfhren
   } else {
      nurDiffsoll = FALSE;
   }

   if(t3001) {
     tv3001 = TRUE;
   } else {
     tv3001 = FALSE;
   }

}

/*===verarbeitung()====================================================
  Synopsis:      int verarbeitung(void)
  Beschreibung:  Hauptverarbeitung
                 Lesen des Steuercursors VtrJ mit Gruppenwechsel
                 ber VSNR. Bei Gruppenwechsel wird die Funktion
                 folgesoll() aufgerufen. Zu diesem Zeitpunkt sind
                 die entsprechenden Strukturen gefllt (tabVJ,aktBearb)
  Modifikation:  04.03.1998 JoBarden/pdv  Ersterstellung
=====================================================================*/
static void verarbeitung(void) {
 char cTime[9], cDate[11];
 int rc;


   /*---  OPEN Cursor -----------------------------------------------*/

   today(cDate); now(cTime);
   fprintf(PROT,"Vor  OPEN Cursor: %s %s\n", cDate, cTime);
   printf("Vor  OPEN Cursor: %s %s\n", cDate, cTime);
   EXEC SQL OPEN VtrJ;
   today(cDate); now(cTime);
   fprintf(PROT,"Nach OPEN Cursor: %s %s\n", cDate, cTime);
   printf("Nach OPEN Cursor: %s %s\n", cDate, cTime);

   CHECK0(ABBRUCH, "PKQ2550: verarbeitung() Open VtrJ-Cursor");

   /*--- vtrDictionary nach erfolgreichen OPEN bereitstellen. -------*/
   /*--- ENTFAELLT, da bei LBS zu einer VTRNR genau eine VSNR -------*/
   /*--- gehrt!!                                  f 'XVTR'   -------*/
   /* vtrDict = openDict("vtrDict", 7, sizeof(decimal(15,0)),    */
   /*                   sizeof(VTRWERTE), UNIQUE);               */

   /*--- Gruppenwechselobjekt nach erfolgreichen OPEN bereitstellen -*/
   gw = openGW(&gwSatz, sizeof(gwSatz),
               &(gwSatz.vj.vsnr),
               sizeof(gwSatz.vj.vsnr),
               &(gwSatz.vj.vtrnr),
               sizeof(gwSatz.vj.vtrnr),0);

   while(!eofGW(gw)) {
     GWEVENT gwEvent;

     EXEC SQL FETCH VtrJ
       INTO :hVTRNR,          :hHISTNR,     :hVORG_LFDNR, :hVORG_SCHL,
            :hVSNR,           :hPERSNR,     :hBEARBDT,    :hBEARBZEIT,
            :hAEND_STATUS,    :hAENDDT,     :hABR_STATUS,
            :hERHBISDT :ivERHBISDT ,
            :hERSBISDT :ivERSBISDT ,
            :hABRMODUS,   :hVTGNBT,     :hWHGNR,
            :hFBKZ,           :hBEARBNR,    :hTWAEND,     :hGSNRVG,
            :hERHBISDT_KZ,    :hVTRKATKZ,
            :hFOB_POL_HISTNR, :hTEVORG_SCHL,
            :hRZ_PROZTYP,     :hRZ_PROZ :ivRZ_PROZ,
            :hCODEXNR :ivCODEXNR,       :hBESTUEBKZ,
            :hVTGABL,         :hABLKNDG_BIS, :hZAHLW,   :hVTGBEGDT,
            :hTECHN_STATUS,   :hABRTECH_STATUS,         :hVARTKZ,
            :hABW_STEUERKZ,
            :hGSNRVGKI,       :hZWEIG,     :hTWAENDKI,  :hVSTATDAT,
            :hVSTAT,          :hANTRART,   :hANTRAUD;

     CHECK(ABBRUCH, "PKQ2550: verarbeitung() Fetch VtrJ-Cursor");

     getHostvars();              // Hostvars in Strukturen kopieren
     gesFetch++;
     addGW(gw, !SQLCODE);

     while(gwEvent = eventGW(gw)) {
        switch(gwEvent) {
            case BEGIN_OF_1:      /*---- Bei Gruppenbeginn (VSNR) --*/

               if (testlauf)
                  printf("Gruppe 1: %D(15,0) %D(5,0) %.10s \n",
                  gwSatz.vj.vtrnr,gwSatz.vj.histnr, gwSatz.vj.vsnr);

               pruef_vormerk();   /* AEND0010 */

               initAktBearb();     // initialisieren
               initTabVJ();        // initialisieren
               fsRC=abrRC=TRUE;    // init
               //earDict(vtrDict); // initialisieren vtrDict
               break;
            case BEGIN_OF_2:      /*---- Bei Gruppenbeginn (VTRNR) -*/
               vtrWerte.tv3001 = FALSE;
               vtrWerte.posAbr = 0;
               vtrWerte.posVon = anzTabVJ;
               break;
            case END_OF_2:        /*---- Bei Gruppenende (VTRNR) ---*/
               vtrWerte.posBis = anzTabVJ;
               incrDict(cDict,"X: GruppenEnde 2");
               /* addDict(vtrDict,&gwSatz.vj.vtrnr,&vtrWerte); XVTR */
               break;
            case AT_RECORD:       /*---- Bei jedem Record ----------*/
               addTabVJ();        // Struktur tabVJ mit den aktuellen
                                  // Werten aus TBKO_VERTRAG/VTRJournal
                                  // fllen.
               break;
            case END_OF_1:        /*---- Bei Gruppenende (VSNR) ----*/
               incrDict(cDict,"X: GruppenEnde 1");
               gesVSNR++;

              if (!KzVormerkung)  /* AEND0010 */
              {

               makeAktBearb();    // Struktur aktuelle Bearb.
                                   // mit den Werten aus TBKI_VERTRAG
                                   // fllen.

               if(folgesollAnford) {
                   incrDict(cDict,"X: folgesollAnford is TRUE");
                   fsRC=folgesoll(); // Folgesollverarbeitung ausfhren
                                    // Selektion,.. MKO2501
               }

               if(aw1571 && fsRC) {
                   incrDict(cDict,"X: aw1571 is TRUE");
                   abrRC = abrechnung();   // Abrechnung durchfhren
                                           // ! TabAW simulieren !
               }
                                          /*** AEND0006-A
               if(aw2000 && fsRC && abrRC) {
                   incrDict(cDict,"X: aw2000 & abrRC is TRUE");
                   inkRC = inkasso();      // bergabe an RW006
                                           // ! TabAW simulieren !
               }
                                           *** AEND0006-E ***/
               if (testlauf) {
                  PLT;
                  PLT_newline;
               }
              }   /* AEND0010 */

              break;
         }
      }

      if (anzFehler > 2000){
        fprintf(PROT,"!!!!!!!!!!! A C H T U N G  !!!!!!!!!!!!!!!\n");
        fprintf(PROT,"PKQ2550: Notbremse -> mehr als 2000 fehler");
        fprintf(PROT,"hafte Vertraege\n");
        escape(ABBRUCH,EC_2000_Fehler,
        "PKQ2550: Notbremse -> mehr als 2000 fehlerhafte Vertraege");
      }
   } /*--- end while(eof(gw) ----------------------------------------*/
}

/*=== pruef_vormerk()=========================== AEND0010 ===========
  Synopsis:      pruef_vormerk(void)
  Beschreibung:  Diese Funktion ermittelt, ob Vormerkung vorhanden ist
                 Aufgerufen wird diese Funktion aus der Funktion
                 verarbeitung
=====================================================================*/
static void pruef_vormerk(void)
{
  hSQLCODE = SQLCODE;

  EXEC SQL
     SELECT   J.AENDDT
     INTO    :hhAENDDT
     FROM TBKO_VTRJOUTEIL T
         ,TBKO_VTRJOURNAL J
          WHERE T.VTRNR         = :hVTRNR
            AND J.VTRNR         = T.VTRNR
            AND J.HISTNR        = T.HISTNR
            AND J.VORG_LFDNR    =  1
            AND J.AEND_STATUS   = 'E'
            AND T.HISTNR  =
               ( SELECT MAX(T1.HISTNR)
                    FROM TBKO_VTRJOUTEIL T1
                    WHERE T1.VTRNR         =  T.VTRNR
                      AND T1.TVSCHL        IN ('2866'
                                              ,'2867'
                                              ,'2875')
               )                        /* AEND0013 */
  FETCH FIRST ROW ONLY;             /* AEND0011 */

  CHECK(ABBRUCH,
     "PKQ2550: Fehler beim SELECT in Funktion pruef_vormerk");

  if (SQLCODE == 100)          /* keine Vormerkung vorhanden */
    KzVormerkung = FALSE;
  else                         /*       Vormerkung vorhanden */
    KzVormerkung = TRUE;

  SQLCODE = hSQLCODE;
}

/*===getHostvars()=====================================================
  Synopsis:      void getHostvars(void)
  Beschreibung:  Die Hostvariablen des Hauptcursors VtrJ
                 werden in die Gruppenwechselstruktur berfhrt
  Modifikation:  04.03.1998 JoBarden/pdv  Ersterstellung
=====================================================================*/
#define PLIDATE(dec,str) { \
  if(strlen(str) > 0) dec = pliDate(str); else dec = 0D;  }

static void getHostvars(void) {
 char cMM[3], cTT[3];
   gwSatz.vj.vtrnr           = hVTRNR;
   gwSatz.vj.histnr          = hHISTNR;
   gwSatz.vj.vorgLfdnr       = hVORG_LFDNR;
   gwSatz.vj.bearbNr         = hBEARBNR;
   if (testlauf) {
     printf("gethostvar-->gwSatz.vj.vtrnr    : %D(15,0)\n",hVTRNR);
     printf("gethostvar-->gwSatz.vj.histnr   : %D(5,0)\n",hHISTNR);
     printf("gethostvar-->gwSatz.vj.vorgLfdnr: %D(5,0)\n",hVORG_LFDNR);
     printf("gethostvar-->gwSatz.vj.bearbNr: %D(5,0)\n",hBEARBNR);
   }
   gwSatz.vj.bearbDatum      = pliDate(hBEARBDT);
   gwSatz.vj.bearbZeit       = pliTime(hBEARBZEIT);
   pliChar(gwSatz.vj.persnr   , 8 , hPERSNR);
   pliChar(gwSatz.vj.vorgSchl , 4 , hVORG_SCHL);
   gwSatz.vj.techVorgSch     = hTEVORG_SCHL;
   gwSatz.vj.aendStatus      = hAEND_STATUS;
   gwSatz.vj.aendDatum       = pliDate(hAENDDT);
   gwSatz.vj.abrStatus       = hABR_STATUS;
   if(ivERHBISDT == 0)
      gwSatz.vj.erhebenBis   = pliDate(hERHBISDT);
   else
      gwSatz.vj.erhebenBis   = 0D;


   gwSatz.vj.erhebenBisKz    = hERHBISDT_KZ;
   if(ivERSBISDT == 0)
      gwSatz.vj.erstattenBis = pliDate(hERSBISDT);
   else
      gwSatz.vj.erstattenBis = 0D;
   pliChar(gwSatz.vj.abrModus , 4 , hABRMODUS);
   pliChar(gwSatz.vj.vsnr     , 10, hVSNR);
   gwSatz.vj.twaend          = hTWAEND;
   gwSatz.vj.fbkz            = hFBKZ;
   gwSatz.vj.fobPolHistnr    = hFOB_POL_HISTNR;
   gwSatz.vj.gsnrvg          = hGSNRVG;
   gwSatz.vj.bdnr            = 0D;       /*-- unntig QV --*/
   gwSatz.vj.agenturnr       = 0D;       /*-- unntig QV --*/
   gwSatz.vj.vtgnbt          = hVTGNBT;            // VJ
   gwSatz.vj.whgnr           = hWHGNR;             // VJ
   gwSatz.vj.technStatus     = hTECHN_STATUS;      // VT
   gwSatz.vj.abrtechStatus   = hABRTECH_STATUS;    // VT
   gwSatz.vj.vtrkatkz        = hVTRKATKZ;          // VJ
   gwSatz.vj.vtrartkz        = hVARTKZ;            // VT
   gwSatz.vj.beginnDatum     = pliDate(hVTGBEGDT); // VT
   gwSatz.vj.ablaufDatum     = pliDate(hVTGABL);   // VT
   if(strlen(hABLKNDG_BIS) > 0) {
     memset(cTT, '\0', 3); memcpy(cTT, hABLKNDG_BIS, 2);
     memset(cMM, '\0', 3); memcpy(cMM, hABLKNDG_BIS+3, 2);
     gwSatz.vj.ablaufKndgBis   = (atoi(cMM) * 100 + atoi(cTT));
   } else {
     gwSatz.vj.ablaufKndgBis   = 0D;               // VT
   }
   pliChar(gwSatz.vj.zahlweise, 2 , hZAHLW);
                                               /*** AEND0004  ***/
   gwSatz.vj.stornoDatum  = 0D;
   gwSatz.vj.btrFreiDatum = 0D;
   gwSatz.vj.ruheDatum    = 0D;
   gwSatz.vj.wikDatum     = 0D;
                                               /*** AEND0004  ***/
   gwSatz.vj.rzProztyp       = hRZ_PROZTYP;        // VT
   gwSatz.vj.rzProz          = hRZ_PROZ;           // VT
   gwSatz.vj.abwSteuerkz     = hABW_STEUERKZ;      // VT
   gwSatz.vj.bestuebkz       = hBESTUEBKZ;         // VT
   if(ivCODEXNR == 0) {
      incrDict(cDict,"Abw. Faelligkeit");
      gwSatz.vj.abwFaellig  = hCODEXNR;           // in LBS ist CODEXNR
   } else {                                       // ein Datum
      gwSatz.vj.abwFaellig  = 0D;
   }

   /*--- strnoDaten etc knnen erst bei vorhandensein des gesamt VJ -*/
   /*--- ermittelt werden! ------------------------------------------*/

   /*--- vorbelegung Struktur aktuelle Bearbeitung ----*/
   pliChar(gwSatz.ab.vsnr     , 10, hVSNR);
   gwSatz.ab.twaend           = hTWAENDKI;         // KI_VT
   pliChar(gwSatz.ab.zweig    , 4 , hZWEIG);       // KI_VT
   gwSatz.ab.kavisgesnr       = hGSNRVGKI;         // KI_VT
   pliChar(gwSatz.ab.antrart  , 6 , hANTRART);     // KI_VT
   pliChar(gwSatz.ab.antraud  , 4 , hANTRAUD);     // KI_VT
   gwSatz.ab.vstat            = hVSTAT;            // KI_VT
   pliChar(gwSatz.ab.vstatdat , 10, hVSTATDAT);    // KI_VT
}

/*===initAktBearb()====================================================
  Synopsis:      void initAktBearb(void)
  Beschreibung:  Grundinit. der Struktur: aktB (aktuelle Bearbeitung)
  Modifikation:  09.03.1998 JoBarden/pdv  Ersterstellung
=====================================================================*/
static void initAktBearb(void) {
   aktB.inkassoart = ' ';      /*-- wird im PKO2502 nicht gesetzt ---*/
   aktB.perdat = 0D;           /*-- wird im PKO2502 nicht gesetzt ---*/

   memset(aktB.reserve, ' ', sizeof(aktB.reserve));

   maxBearbNr = 0D;
   newAktBearbNr = 0D;         /*-- wird u.U. von der DiffSollPrf. -*/
                               /*-- gesetzt. (ehem. MKV3016) --------*/

   aktB.datum  = pliDate(datumJetzt);   // wird in vorlauf() ermittelt
   aktB.zeit   = pliTime(zeitJetzt);    // wird in vorlauf() ermittelt

   /*-- alle anderen Werte werden in makeAktBearb geschrieben,    ---*/
   /*-- daher ist keine weitere initialisierung notwendig.        ---*/
}

/*===initTabVJ()=======================================================
  Synopsis:      void initTabVJ(void)
  Beschreibung:  Grundinit. der Struktur: tabVJ bzw. des VJ-Zhlers
  Modifikation:  09.03.1998 JoBarden/pdv  Ersterstellung
=====================================================================*/
static void initTabVJ(void) {
   anzTabVJ = 0;
   aktVtrnr = 0D;
   maxVorgLfdnr = 0;   // initialisieren
   maxHistnr = 0;      // initialisieren
   folgesollAnford = TRUE;   /*-- MKV3016.RETCODE = 050 --*/
   incrDict(cDict,"MKV3016 Ret50 Defaultinitialisierung");
}

/*===makeAktBearb()====================================================
  Synopsis:      void makeAktBearb(void)
  Beschreibung:  Die Fuktion wird am Gruppenende aufgerufen.
  Modifikation:  04.03.1998 JoBarden/pdv  Ersterstellung
=====================================================================*/
static void makeAktBearb(void) {

   if(newAktBearbNr == 0D) {
      aktB.nummer = maxBearbNr+1;       // wird in addTabVJ() ermittelt
   } else {
      incrDict(cDict,"BearbNr-Korrektur");
      aktB.nummer = newAktBearbNr;      // Korrektur durch DiffSoll-
   }                                    // Prfung (ehem. MKV3016)

   memcpy(aktB.persnr, "PKO2500 ", 8);

   memcpy(aktB.bdnr  , "136"     , 3);    /*-- Dummy --*/
   memcpy(aktB.agtnr , "700001"  , 6);    /*-- Dummy --*/

   memcpy(aktB.vsnr       , gwSatz.ab.vsnr    , 10);
          aktB.twaend     = gwSatz.ab.twaend;
   memcpy(aktB.zweig      , gwSatz.ab.zweig   , 4);
          aktB.kavisgesnr = gwSatz.ab.kavisgesnr;
   memcpy(aktB.antrart    , gwSatz.ab.antrart , 6);
   memcpy(aktB.antraud    , gwSatz.ab.antraud , 4);
          aktB.vstat      = gwSatz.ab.vstat;
   memcpy(aktB.vstatdat   , gwSatz.ab.vstatdat, 10);

}

/*===addTabVJ()========================================================
  Synopsis:      void addTabVJ(void)
  Beschreibung:  Die Fuktion wird nach jedem Fetch aufgerufen.
                 Der VJ-Satz aus dem Gruppenwechsel-Satz wird
                 in die tabVJ-struktur (Array) kopiert und der
                 Zhler anzTabVJ wird um eins erhht.
                 Gleichzeitg werden bestimmte Datumswerte ermittelt
                 (stornoDatum, ruheDatum, wikDatum,...) und die
                 maximale Bearbeitungsnummer ermittelt.
  Modifikation:  04.03.1998 JoBarden/pdv  Ersterstellung
=====================================================================*/
static void addTabVJ(void) {

   /*--- VJ-Satz in tabVJ bernehmen --------------------------------*/
   anzTabVJ++;
   memcpy(&tabVJ[anzTabVJ-1], &gwSatz.vj, sizeof(VJ_SATZ));

   /* stornoDatum setzen, unter gewissen Umstnden... */
   if( (gwSatz.vj.technStatus == 'S') &&
       (gwSatz.vj.vorgLfdnr == 1) &&
       ( ( memcmp(gwSatz.vj.vorgSchl, "0000", 4) == 0) ||
         ( memcmp(gwSatz.vj.vorgSchl, "1100", 4) == 0) ||
         ( memcmp(gwSatz.vj.vorgSchl, "5000", 4) == 0) ||
         ( memcmp(gwSatz.vj.vorgSchl, "9000", 4) == 0) ) ) {
     gwSatz.vj.stornoDatum = gwSatz.vj.aendDatum;
   } else {
     gwSatz.vj.stornoDatum = 0D;
   }


   /* ruheDatum setzen, unter gewissen Umstnden... */
   if( (gwSatz.vj.technStatus == 'R') &&
       (gwSatz.vj.vorgLfdnr == 1) &&
       (memcmp(gwSatz.vj.vorgSchl, "2900", 4) == 0) ) {
     gwSatz.vj.ruheDatum = gwSatz.vj.aendDatum;
   } else {
     gwSatz.vj.ruheDatum = 0D;
   }

   /* wikDatum setzen, unter gewissen Umstnden... */
   if((memcmp(gwSatz.vj.vorgSchl, "2950", 4) == 0)) {
     gwSatz.vj.wikDatum = gwSatz.vj.aendDatum;
   } else {
     gwSatz.vj.wikDatum = 0D;
   }

   /* btrFreiDatum setzen, unter gewissen Umstnden... */
   /* ???????????????????????????????????????????????? */
   if( (gwSatz.vj.technStatus == 'B') &&
       (gwSatz.vj.vorgLfdnr == 1) &&
       (memcmp(gwSatz.vj.vorgSchl, "????", 4)==0) ) {
     gwSatz.vj.btrFreiDatum = gwSatz.vj.aendDatum;
   } else {
     gwSatz.vj.btrFreiDatum = 0D;
   }

   /*--- Werte fr makeAktBearb merken ------------------------------*/
   if(gwSatz.vj.bearbNr > maxBearbNr) {
      maxBearbNr = gwSatz.vj.bearbNr;              // fr aktBearb
   }

   /*****************************************************************/
   /* Es muss die groesste VorgLfdnr der hoechsten Histnr ermittelt */
   /* werden. Da der Cursor VTRJ nach Histnr und VorgLfdnr absteiend*/
   /* sortiert ist, ist dies in der Regel der erste eingelesene     */
   /* Satz. Die max. VorgLfdnr wird fuer den neu zu generierenden   */
   /* Eintrag um 1 erhoeht                                          */
   /*****************************************************************/

   if(gwSatz.vj.vorgLfdnr > maxVorgLfdnr) {
     if(gwSatz.vj.histnr >= maxHistnr) {
        maxHistnr    = gwSatz.vj.histnr;
        maxVorgLfdnr = gwSatz.vj.vorgLfdnr;
        if (testlauf) {
          printf("Uebernahme der max VorgLfdnr\n");
          printf("maxHistnr: %d\n",maxHistnr);
          printf("maxVorgLfdnr: %d\n",maxVorgLfdnr);
        }
     }
   }

   /*--- evtl. Prfung auf Differenzfolgesoll -----------------------*/

   if(kzAnfo3016) {    /*-- Differenzsollprfung wurde angefordert --*/
      diffSollPruefung();
   }
}

/*===diffSollPruefung()================================================
  Synopsis:      void addTabVJ(void)
  Beschreibung:  Die Fuktion ...
                 Prfung auf Differenzfolgesoll (die hierzu ntigen
                 Werte aus tabAP wurden in ablaufplan() in globale
                 Varaiblen gestellt. (ehemals MKV3016)

  Modifikation:  14.04.1998 JoBarden/pdv  Ersterstellung
=====================================================================*/
static void diffSollPruefung(void) {

   if(nurDiffsoll) {
      folgesollAnford = FALSE;
   } else {
      folgesollAnford = TRUE;
   }

   if(  (gwSatz.vj.vorgLfdnr == 1)     &&
        (gwSatz.vj.aendStatus == 'E')  &&     // erledigt
        (gwSatz.vj.technStatus == 'A') &&     // aktiv
        (gwSatz.vj.vtrnr != aktVtrnr)  &&
        (gwSatz.vj.abrStatus == 'W')   &&     // wartend
        (memcmp(gwSatz.vj.abrModus,"DB",2)==0) ) {

      aktVtrnr = gwSatz.vj.vtrnr;  // umsetzen fr nachsten GrpWechsel

      gwSatz.vj.bearbDatum = aktB.datum; /*- Faelligkeitsdat setzen -*/
      newAktBearbNr = gwSatz.vj.bearbNr; /*- BearbNr korrigieren ----*/

      if( (stichtag3001Von <= gwSatz.vj.erhebenBis) &&
          (stichtag3001Bis >= gwSatz.vj.erhebenBis) ) {
         folgesollAnford = TRUE;   /*-- MKV3016.RETCODE = 050 --*/
         incrDict(cDict,"MKV3016 Ret50 Set");
      } else {
         folgesollAnford = FALSE;  /*-- MKV3016.RETCODE = 000 --*/
         incrDict(cDict,"MKV3016 Ret00 Set (TAB_AW mit 3005)");
          /*-- Hier noch TAB_AW fr aktuellen Satz erstellen ---*/
      printf("hier noch TAB_AW fuer aktuellen Satz erstellen\n");
      }
   }
}

/*===folgesoll()=======================================================
  Synopsis:      void folgesoll(void)
  Return:        OK 0
                 Err -1
  Beschreibung:
  Modifikation:  17.03.1998 JoBarden/pdv  Ersterstellung
=====================================================================*/
static int  folgesoll(void) {
 int iVJ;
 int rc;

      rc=selektierung();       // Auswerten der Selektionsbedingungen
                               // aus MKO2501
         if(rc == -1) {
            return FALSE;
         }

      if(vtrWerte.tv3001) {    // Selektion hat Flligkeit ergeben
         rc = genNewTabVJ();
         if(rc == -1) {
            return FALSE;
         }
         rc=insertVJ();
         if(rc == -1) {
            return FALSE;
         }
         aw1571 = TRUE;
         aw2000 = TRUE;
         if(vtrWerte.vordat) {
            rc = updateBearbNr();
            if(rc == -1) {
               return FALSE;
            }
         }
         schwebeBehandlung();

      } else {
         aw1571 = FALSE;
         aw2000 = FALSE;
      }

      return TRUE;

}

/*===selektierung()====================================================
  Synopsis:      int selektierung(void)
  Beschreibung:
  Modifikation:  11.03.1998 JoBarden/pdv  Ersterstellung
=====================================================================*/
static int selektierung(void) {
 int iVJ;
 int rc;

   if(!tv3001) return;         // Abbruch, wenn in tabAP der teilVorgSl
                               // 3001 nicht enthalten ist

   if(aktB.nummer > maxBearbNr) {
      maxBearbNr = aktB.nummer;
   }


   for(iVJ = vtrWerte.posVon; iVJ < vtrWerte.posBis; iVJ++) {
      if( ( tabVJ[iVJ].aendStatus == 'E' ) &&
          ( ( tabVJ[iVJ].abrStatus == 'E' ) ||
            ( tabVJ[iVJ].abrStatus == 'W' ) &&
            ( tabVJ[iVJ].bearbNr == aktB.nummer ) ) ) {
         break;
      }
   }
   if(iVJ > (anzTabVJ-1)) {
      /* nicht faellig, da nicht gefunden => Neugeschft */
      incrDict(cDict, "NF: Neugesch{ft vordatiert/schwebend");
      goto NICHT_FAELLIG;
   } else {
      vtrWerte.posAbr = iVJ;       // merken letzter abgerech. Stand
   }
   vtrWerte.erstattenBisDat = tabVJ[vtrWerte.posAbr].erhebenBis;

   if(vtrWerte.erstattenBisDat >= stichtag3001Bis) {
                                                    // nur LBS: '>='
      incrDict(cDict, "NF: StichtagBis");           // sonst:   '>'
      goto NICHT_FAELLIG;
   }
  /* AEND0017 */
   if(vtrWerte.erstattenBisDat >=
               tabVJ[vtrWerte.posAbr].ablaufDatum) {

      incrDict(cDict, "NF: Vertragsablaeufer ");
      goto NICHT_FAELLIG;
   }

   /*-- Storno in Schwebe bei LBS nicht abrechnen --------------*/
   for(iVJ = vtrWerte.posVon; iVJ < vtrWerte.posAbr; iVJ++) {
      if( ( tabVJ[iVJ].abrStatus    == 'W'        ) &&
          ( tabVJ[iVJ].erhebenBis   !=  0          ) &&
          ( tabVJ[iVJ].erstattenBis !=  0          ) &&
          ( tabVJ[iVJ].aendStatus   != 'V'         ) &&
          ( tabVJ[iVJ].bearbNr      != aktB.nummer ) &&
          ( tabVJ[iVJ].technStatus  == 'S'         ) ) {
          PLD;
         gesStoSchwebe++;
         incrDict(cDict, "NF: Storno in Schwebe (nur LBS)");
         goto NICHT_FAELLIG;
      }
   }

   rc = vordatierungPruefen();
   if (rc == (-1))
     return (-1);

   /*---- Hauptflligkeit ermitteln: Monat und Tag vom -----------*/
   /*---- Ablaufdatum und Jahr aus erstattenBisDatum -------------*/

   if(tabVJ[vtrWerte.posAbr].abwFaellig > 0D) {
      vtrWerte.hauptFaelligDat = tabVJ[vtrWerte.posAbr].abwFaellig;
   } else {
     int ablDat,ersDat,hfDat;
      ablDat = tabVJ[vtrWerte.posAbr].ablaufDatum;
      ersDat = vtrWerte.erstattenBisDat;
      ersDat /= 10000;
      ablDat %= 10000;

      hfDat = (ersDat * 10000 + ablDat);
      vtrWerte.hauptFaelligDat = hfDat;
   }
   if(vtrWerte.hauptFaelligDat < vtrWerte.erstattenBisDat) {
      vtrWerte.hauptFaelligDat += 10000;    // Jahr um eins erhhen
   }

   /*---- nur fr LBS:   -----------------------------------------*/
   /*---- Position des Vertragsstandes merken, der fr die -------*/
   /*---- Selektion herangezogen wird.                     -------*/
   selPos = vtrWerte.posAbr;
    if (testlauf) {
     printf(".....selPos: %d\n", selPos);
     printf("aktB.nummer: %d\n", aktB.nummer);
    }

   /*---- jetzt folgen die Selektionen ---------------------------*/

   if((tabVJ[selPos].fbkz=='F')||(tabVJ[selPos].fbkz=='B')) {
      incrDict(cDict, "NF: F}hrungs- / Beteiligungsgesch{ft");
      goto NICHT_FAELLIG;      // Fhrung-/Beteiligungsgeschft
   }

   if(tabVJ[selPos].technStatus == 'S') {
      incrDict(cDict, "NF: stornierter Vertrag");
      goto NICHT_FAELLIG;      // Stornierte Vertrge
   }

   if(tabVJ[selPos].technStatus == 'R') {
      incrDict(cDict, "NF: ruhender Vertrag");
      goto NICHT_FAELLIG;      // Ruhende Vertrge
   }

   if(tabVJ[selPos].technStatus == 'B') {
      incrDict(cDict, "NF: beitragsfreier Vertrag");
      goto NICHT_FAELLIG;      // Beitragsfreie Vertrge
   }

   if(tabVJ[selPos].technStatus == 'I') {
      incrDict(cDict, "NF: Invitatio Vertrag     ");
      goto NICHT_FAELLIG;      // Invitatio Vertrge   AEND0015
   }

   if(tabVJ[selPos].technStatus == 'D') {
      incrDict(cDict, "NF: Deaktiver Invitatio Vertrag");
      goto NICHT_FAELLIG;      // Invitatio Vertrge   AEND0015
   }

   if(memcmp(tabVJ[selPos].zahlweise,"03",2) == 0) {
      /*-- kz_nicht_faellig = 'ZW ' --*/
      incrDict(cDict, "NF: kurzfristiger Vertrag");
      goto NICHT_FAELLIG;      // Kurzfristige Vertrge
   }

   if(vtrWerte.vordat) {
      if(memcmp(tabVJ[vtrWerte.posVordatV].vorgSchl,"0000",4) == 0) {
         if(tabVJ[vtrWerte.posVordatV].aendDatum >
                                       vtrWerte.erstattenBisDat) {
            vtrWerte.bearbNrHochsetzen = TRUE;
            vtrWerte.vordatStornoDat =
               tabVJ[vtrWerte.posVordatV].aendDatum;
            printf("------> vordat. Storno > ErhBisAlt\n");
            incrDict(cDict,"f{lliger vordatierter Vertrag");
            goto FAELLIG;
         } else {
           rc=PKQ2550Fehler("selekierung",
                            "VordatStorno <= erhBisAlt",
                            1,0);
            incrDict(cDict,"Err: selektierung() vordatStorn <= x");
            return (-1);
         }
      } else {
         if(tabVJ[vtrWerte.posVordatV].aendDatum <
                                       vtrWerte.erstattenBisDat) {
   printf("aenddt: %D(9,0)\n",tabVJ[vtrWerte.posVordatV].aendDatum);
   printf("erst. bis: %D(9,0)\n",vtrWerte.erstattenBisDat);

            rc=PKQ2550Fehler("selekierung",
                             "verstrichende VDAT",
                             2,0);
            incrDict(cDict,"Err: selektierung() verstrichn. VDat");
            return (-1);
         } else {
            incrDict(cDict,"f{lliger vordatierter Vertrag");
            goto FAELLIG;
         }
      }
   }




   FAELLIG:
      incrDict(cDict,"X: GESAMT f{llige Vertr{ge");
      vtrWerte.tv3001 = TRUE;
      return;

   NICHT_FAELLIG:
      incrDict(cDict,"X: GESAMT nicht f{llige Vertr{ge");
      if (testlauf) {
        printf("          nicht f{llig.\n");
      }
      vtrWerte.tv3001 = FALSE;
      return ;
}
/*==int vordatierungPruefen()==========================================
  Synopsis:      int vordatierungPrfen(void)
  Beschreibung:
  Modifikation:  11.03.1998 JoBarden/pdv  Ersterstellung
=====================================================================*/
static int vordatierungPruefen(void) {
 int iVJ, i, iUU;
 int rc;

   vtrWerte.vordat = FALSE;
   vtrWerte.posVordatV = 0D;
   for(i=0;i<20;i++) vtrWerte.posVordatU[i] = 0D;

   for(iVJ = vtrWerte.posVon; iVJ < vtrWerte.posAbr; iVJ++) {
      if( ( tabVJ[iVJ].vorgLfdnr == 1 ) &&
          ( tabVJ[iVJ].aendStatus == 'V' ) ) {
         vtrWerte.vordat = TRUE;
         vtrWerte.posVordatV = iVJ;
         /*--- der vorhergehende Stand mu 'U' sein und zur ----*/
         /*--- aktuellen Vtrnr gehren -------------------------*/
         for(iUU = 0; iUU < 20; iUU++) {
            if( (tabVJ[iVJ+iUU+1].aendStatus == 'U') &&
                ( (iVJ+iUU+1) <= vtrWerte.posBis)    ) {
               vtrWerte.posVordatU[iUU] = iVJ+iUU+1;
            } else {
               break;
            }
         }
         if(iUU > 0) {
            rc=PKQ2550Fehler("vordatPruef",
                             "mehr als ein U-Stand",
                             3,0);
            incrDict(cDict,"Err: vordatPruef mehr als ein U-Stand");
            return  (-1);
         }
         if(iUU >= 20) {
            rc=PKQ2550Fehler("vordatPruef",
                             "U-Stand nicht gefunden",
                             4,0);
            incrDict(cDict,"Err: vordatPruef U-Stand not found");
            return  (-1);
         }
         printf("Vordat-Positionen: V:%d U:%d Abr:%d\n",
           vtrWerte.posVordatV,vtrWerte.posVordatU[0],vtrWerte.posAbr);

         /*---- nicht mehr weitersuchen, wenn 'V' gefunden wurde ----*/
         break;
      }
   }
}

/*===genNewTabVJ()=====================================================
  Synopsis:     int genNewTabVJ(void)
  Return:       ok 0
                err -1
  Beschreibung: Es wird ein neuer Eintrag fr die tabVJ auf Grundlage
                der letzten Abrechnung generiert.
                Um die Sortierung beizubehalten (LBS hat nur eine
                VtrNr pro VSNR) werden die Eintrge der tabVJ nach
                hinten verschoben und der neue Eintrag an Pos 0
                eingefgt.
  Modifikation:  17.03.1998 JoBarden/pdv  Ersterstellung
=====================================================================*/
static int genNewTabVJ(void) {
 int i;
 int rc;

   if ((anzTabVJ+1) > MAX_TAB_VJ) {
      rc=PKQ2550Fehler("genNewTabVJ",
                       "tabVJ overflow",
                       5,0);
      return (-1);
   }

   /*--- tabVJ um eine pos nach hinten kopieren -------------------*/
   for(i=anzTabVJ; i > 0; i--) {
      memcpy(&tabVJ[i], &tabVJ[i-1], sizeof(VJ_SATZ));
   }
   /*--- wichtig: selPos anpassen ---------------------------------*/
   /*--- achtung: posVon und posBis sind jetzt auch nicht mehr ----*/
   /*---          aktuell!!!!!!                                ----*/
   selPos++;
   vtrWerte.posVon++;        // Idx: Beginn in tabVJ
   vtrWerte.posBis++;        // Idx: Ende in tabVJ
   vtrWerte.posAbr++;        // letzter abgerechneter Stand in tabVJ
   vtrWerte.posVordatV++;    // Idx: V-Stand Vordatierung
   for(i=0; i < 20; i++) {
      vtrWerte.posVordatU[i]++;   // Idx: U-Stand Vordatierung
   }
   if (testlauf) {
     printf(".....selPos (angepasst): %d\n", selPos);
   }
   /*--- erstmal kopieren des Abrechnungssatzes (tabVJ[selPos]) ---*/
   /*--- als neuen Satz an den Anfang der tabVJ.                ---*/
   memcpy(&tabVJ[0], &tabVJ[selPos], sizeof(VJ_SATZ));
   /*
   printf("genNew-->tabVJ[0].histnr: %D(5,0)\n",tabVJ[0].histnr);
   */
   /*--- jetzt teilweise die Werte der aktuellen Bearbeitung    ---*/
   /*--- bernehmen  ----------------------------------------------*/
   tabVJ[0].bearbNr = aktB.nummer;
   tabVJ[0].bearbDatum = aktB.datum;
   tabVJ[0].bearbZeit = aktB.zeit;
   memcpy(tabVJ[0].persnr, aktB.persnr, 8);

   /*--- restliche Werte setzen  ----------------------------------*/
   tabVJ[0].vorgLfdnr = tabVJ[selPos].vorgLfdnr +1;
   memcpy(tabVJ[0].vorgSchl, "3000", 4);
   tabVJ[0].techVorgSch = ' ';
   tabVJ[0].abrStatus = 'W';
   tabVJ[0].erhebenBisKz = 'F';
   memcpy(tabVJ[0].abrModus, "FB", 2);
   tabVJ[0].erstattenBis = vtrWerte.erstattenBisDat;

   TRY
      rc = ermittlungErhebenBis();         /*--- call MKO1500 ---*/
   RECOVER
      printf("++++PKQ2550: MKO1500 failed\n");
      return (-1);
   END

   tabVJ[0].erhebenBis = vtrWerte.erhebenBisDat;
   tabVJ[0].twaend = aktB.twaend;


   if(vtrWerte.erhebenBisDat == vtrWerte.erstattenBisDat) {
      rc=PKQ2550Fehler("genNewTabVJ",
                       "ErhebenBisDatum = ErstattenBisDatum",
                       6,0);
      incrDict(cDict,"Err: genNewTabVJ ErhBisDat = ErsBisNeu");
      return (-1);

   }

   if(vtrWerte.vordatStornoDat != 0) {
  /*  if(vtrWerte.vordatStornoDat > 0) { */
      if(vtrWerte.erhebenBisDat > vtrWerte.vordatStornoDat) {
         printf("VSNR mit vordat. Storno < erhBisNeu\n");
      } else {
         rc=PKQ2550Fehler("genNewTabVJ",
                          "vordatStornoDat >= ErhBisNeu",
                          7,0);
         incrDict(cDict,"Err: genNewTabVJ vordtStornoDt >= ErhBisNeu");
         return (-1);
      }
   }

   insPos = 0;
   anzTabVJ++;
   return 0;
}

/*===ermittelungErhebenBis()===========================================
  Synopsis:      int ermittelungErhebenBis(void)
  Beschreibung:  Mit dem Kompass-Modul MKO1500 wird das neue
                 erhebenBisDatum ermittelt.
  Modifikation:  18.03.1998 JoBarden/pdv  Ersterstellung
=====================================================================*/
static int  ermittlungErhebenBis(void) {
 int rc;
 S_MKO1500  smko1500;
 S_MKO1500* pmko1500;

   /*--- Struktur fr Aufruf MKO1500 fllen -------------------------*/
   memcpy(smko1500.ende, "ENDE", 4);
   memcpy(smko1500.strukturName, "SMKO1500", 8);
   memcpy(smko1500.version, "01.00", 5);
   memcpy(smko1500.vorgang, "3000", 4);
   memcpy(smko1500.teilvorgang, "3001", 4);
   memcpy(smko1500.sparte, "KL ", 3);

   smko1500.funktion = 'F';
   memcpy(smko1500.zahlweise, tabVJ[selPos].zahlweise, 2);
   smko1500.kzZahlweise = ' ';
   smko1500.aendDatum = 0;
   smko1500.beginnDatum = tabVJ[selPos].beginnDatum;
   smko1500.ablaufDatum = tabVJ[selPos].ablaufDatum; // nur LBS, sonst
                                                     // mit MKO4028
   smko1500.kzHauptfaelligkeit = ' ';
   smko1500.stornoDatum = tabVJ[selPos].stornoDatum;
   smko1500.ruheDatum = tabVJ[selPos].ruheDatum;
   smko1500.wikDatum = tabVJ[selPos].wikDatum;
   smko1500.grenzeFaellig = 0;
   smko1500.inkDatum = stichtag3001Bis;
   smko1500.inkPeriode = 'M';
   smko1500.inkStichtag = 0;

   smko1500.mandant = smko5010.mandant;
   smko1500.sprachcode = smko5010.sprachcode;

   smko1500.erstattenBisDatum = vtrWerte.erstattenBisDat;


   smko1500.abwFaellig = tabVJ[selPos].abwFaellig; // nur LBS, sonst 0
                  /** AEND0006 - neue Felder fuer MKO1500 fuellen ***/
   smko1500.bearbDatum       = tabVJ[selPos].bearbDatum;
   smko1500.gsnrvg           = tabVJ[selPos].gsnrvg;
   smko1500.vtrkatkz         = tabVJ[selPos].vtrkatkz;
 /*                                                   AEND0009 Anfang
   printf("mko1500.bearbDatum: %D(9,0)\n",smko1500.bearbDatum);
   printf("mko1500.aendDatum: %D(9,0)\n",smko1500.aendDatum);
   printf("mko1500.beginnDatum: %D(9,0)\n",smko1500.beginnDatum);
   printf("mko1500.ablaufDatum: %D(9,0)\n",smko1500.ablaufDatum);
   printf("mko1500.stornoDatum: %D(9,0)\n",smko1500.stornoDatum);
   printf("mko1500.ruheDatum: %D(9,0)\n",smko1500.ruheDatum);
   printf("mko1500.wikDatum: %D(9,0)\n",smko1500.wikDatum);
   printf("mko1500.inkDatum: %D(9,0)\n",smko1500.inkDatum);
   printf("mko1500.ERSTBISDT: %D(9,0)\n",smko1500.erstattenBisDatum);
   printf("mko1500.ERHBISDT: %D(9,0)\n",smko1500.erhbenBisDatum); */





   //smko1500.erhebenBisDatum  ... wird nicht gefuellt, sondern
   //                              vom MKO1500 ermittelt

   pmko1500 = &smko1500;        // wg. Doppel-Ptr PLI

   MKO1500(&pmko1500);

   if(memcmp(smko1500.retcode, "0000", 4) == 0) {
       if (testlauf) {
         printf("-<GET_ERH_BIS>- AusgabeDatum: %D(9,0)|\n",
         smko1500.ausgabeDatum);
       }

      vtrWerte.erhebenBisDat = smko1500.ausgabeDatum;
   } else {
      rc=ProtFehler("MKO1500",smko1500.fmodul,smko1500.retcode);
      incrDict(cDict,"Err: ermittlungErhebenBis MKO1500");
      return -1;
   }
   return 0;
}

/*===insertVJ()========================================================
  Synopsis:      int insertVJ(void)
  Beschreibung:
  Modifikation:  17.03.1998 JoBarden/pdv  Ersterstellung
=====================================================================*/
static int insertVJ(void) {
 char tTime[9];
 char tDate[11];
 EXEC SQL BEGIN DECLARE SECTION;
   struct {
       decimal(15,0)  VTRNR;
       decimal(5,0)   HISTNR;
       decimal(5,0)   VORG_LFDNR;
       char       VORG_SCHL[5];
       char       VSNR[11];
       char       PERSNR[9];
       char       BEARBDT[11];
       char       BEARBZEIT[9];
       char       AEND_STATUS;
       char       AENDDT[11];
       char       ABR_STATUS;
       char       ERHBISDT[11];
       char       ERSBISDT[11];
       char       ABRMODUS[3];
       decimal(15,2)  VTGNBT;
       long int   WHGNR;
       char       FBKZ;
       long int   BEARBNR;
       long int   GSNRVG;
       short int  TWAEND;
       char       ERHBISDT_KZ;
       decimal(5,0)   FOB_POL_HISTNR;
       char       TEVORG_SCHL;
       decimal(3,0)   VTRKATKZ;
       char       SCHW_AEND_STATUS;
   } hvj;
 EXEC SQL END   DECLARE SECTION;

   hvj.VTRNR = tabVJ[insPos].vtrnr;
   hvj.HISTNR = tabVJ[insPos].histnr;
   hvj.VORG_LFDNR = tabVJ[insPos].vorgLfdnr;

   if (testlauf) {
      printf("Werte fuer INSERT\n");
      printf("tabVJ(inspos)-->hvj.vtrnr: %D(15,0)\n",hvj.VTRNR);
      printf("tabVJ(inspos)-->hvj.histnr: %D(5,0)\n",hvj.HISTNR);
      printf("tabVJ(inspos)-->hvj.vorg_lfdnr: %D(5,0)\n"
             ,hvj.VORG_LFDNR);
   }
   memcpy(hvj.VORG_SCHL,tabVJ[insPos].vorgSchl,4);
          hvj.VORG_SCHL[4] = '\0';
   memcpy(hvj.VSNR,tabVJ[insPos].vsnr,10);
          hvj.VSNR[10] = '\0';
   memcpy(hvj.PERSNR,tabVJ[insPos].persnr,8);
          hvj.PERSNR[8] = '\0';
   C_DATE(tabVJ[insPos].bearbDatum, tDate);
   memcpy(hvj.BEARBDT,tDate,10);
          hvj.BEARBDT[10] = '\0';
   C_TIME(tabVJ[insPos].bearbZeit, tTime);
   memcpy(hvj.BEARBZEIT,tTime,8);
          hvj.BEARBZEIT[8] = '\0';
   hvj.AEND_STATUS = tabVJ[insPos].aendStatus;
   C_DATE(tabVJ[insPos].aendDatum, tDate);
   memcpy(hvj.AENDDT,tDate,10);
          hvj.AENDDT[10] = '\0';
   hvj.ABR_STATUS = tabVJ[insPos].abrStatus;
   C_DATE(tabVJ[insPos].erhebenBis, tDate);
   memcpy(hvj.ERHBISDT,tDate,10);
          hvj.ERHBISDT[10] = '\0';
   C_DATE(tabVJ[insPos].erstattenBis, tDate);
   memcpy(hvj.ERSBISDT,tDate,10);
          hvj.ERSBISDT[10] = '\0';
   memcpy(hvj.ABRMODUS,tabVJ[insPos].abrModus,2);
          hvj.ABRMODUS[2] = '\0';
   hvj.VTGNBT = tabVJ[insPos].vtgnbt;
   hvj.WHGNR = tabVJ[insPos].whgnr;
   hvj.FBKZ = tabVJ[insPos].fbkz;
   hvj.BEARBNR = tabVJ[insPos].bearbNr;
   if (testlauf) {
     printf("tabVJ(inspos)-->hvj.vsnr: %s\n",hvj.VSNR);
     printf("tabVJ(inspos)-->hvj.bearbnr: %d\n",hvj.BEARBNR);
   }
   hvj.GSNRVG = tabVJ[insPos].gsnrvg;
   hvj.TWAEND = tabVJ[insPos].twaend;
   hvj.ERHBISDT_KZ = tabVJ[insPos].erhebenBisKz;
   hvj.FOB_POL_HISTNR = tabVJ[insPos].fobPolHistnr;
   hvj.TEVORG_SCHL = tabVJ[insPos].techVorgSch;
   hvj.VTRKATKZ = tabVJ[insPos].vtrkatkz;
   hvj.SCHW_AEND_STATUS = ' ';   /* default-Wert */

   EXEC SQL INSERT INTO TBKO_VTRJOURNAL
          ( VTRNR,               HISTNR,           VORG_LFDNR,
            VORG_SCHL,           VSNR,             PERSNR,
            BEARBDT,             BEARBZEIT,        AEND_STATUS,
            AENDDT,              ABR_STATUS,       ERHBISDT,
            ERSBISDT,            ABRMODUS,         VTGNBT,
            WHGNR,               FBKZ,             BEARBNR,
            TWAEND,              GSNRVG,           ERHBISDT_KZ,
            FOB_POL_HISTNR,      TEVORG_SCHL,      VTRKATKZ )
   VALUES ( :hvj.VTRNR,          :hvj.HISTNR,      :hvj.VORG_LFDNR,
            :hvj.VORG_SCHL,      :hvj.VSNR,        :hvj.PERSNR,
            :hvj.BEARBDT,        :hvj.BEARBZEIT,   :hvj.AEND_STATUS,
            :hvj.AENDDT,         :hvj.ABR_STATUS,  :hvj.ERHBISDT,
            :hvj.ERSBISDT,       :hvj.ABRMODUS,    :hvj.VTGNBT,
            :hvj.WHGNR,          :hvj.FBKZ,        :hvj.BEARBNR,
            :hvj.TWAEND,         :hvj.GSNRVG,      :hvj.ERHBISDT_KZ,
            :hvj.FOB_POL_HISTNR, :hvj.TEVORG_SCHL, :hvj.VTRKATKZ );

  //CHECK0(ABBRUCH, "PKQ2550: insertVJ() INSERT TBKO_VTRJOURNAL");
    CHECK0(WARNUNG,
        "PKQ2550: insertVJ - Fehler bei Insert TBKO_VTRJOURNAL");
   if (sqlca.sqlcode != 0) {
      sqlFehler("insertVJ","Fehler bei Insert TBKO_VTRJOURNAL",
                sqlca.sqlcode);
//fprintf(PROT,"***************************************");
//fprintf(PROT,"***************************************\n");
//fprintf(PROT,"Fehler in Modul        :  PKQ2550\n");
//fprintf(PROT,"          VSNR         :  %.10s\n",aktB.vsnr);
//fprintf(PROT,"          VTRNR        :  %D(15,0)\n",hvj.VTRNR);
//fprintf(PROT,"          HISTNR       :  %D(5,0)\n",hvj.HISTNR);
//fprintf(PROT,"          VORG_LFDNR   :  %D(5,0)\n",hvj.VORG_LFDNR);
//fprintf(PROT,"          Funktion     :  insertVJ\n");
//fprintf(PROT,"          Fehlermeldung:  Fehler bei Insert ");
//fprintf(PROT,                          "TBKO_VTRJOURNAL\n");
//fprintf(PROT,"          SqlCode      :  %d   \n",sqlca.sqlcode);
  fprintf(PROT,"***************************************");
  fprintf(PROT,"***************************************\n");
//printf("***************************************");
//printf("***************************************\n");
//printf("Fehler in Modul        :  PKQ2550\n");
//printf("          VSNR         :  %.10s\n",aktB.vsnr);
//printf("          VTRNR        :  %D(15,0)\n",hvj.VTRNR);
//printf("          HISTNR       :  %D(5,0)\n",hvj.HISTNR);
//printf("          VORG_LFDNR   :  %D(5,0)\n",hvj.VORG_LFDNR);
//printf("          Funktion     :  insertVJ\n");
//printf("          Fehlermeldung:  Fehler bei Insert ");
//printf(                          "TBKO_VTRJOURNAL\n");
//printf("          SqlCode      :  %d   \n",sqlca.sqlcode);
//printf("***************************************");
//printf("***************************************\n");
  return (-1);
  }



   if (testlauf) {
     printf("PKQ2550: Insert in die TBKO_VTRJOURNAL\n");
     printf("HISTNR:       %D(5,0)\n",hvj.HISTNR);
     printf("VORG_LFDNR:   %D(5,0)\n",hvj.VORG_LFDNR);
     printf("VORG_SCHL:    %s\n",hvj.VORG_SCHL);
     printf("PERSNR:       %s\n",hvj.PERSNR);
     printf("BEARBDT:      %s\n",hvj.BEARBDT);
     printf("AENDSTATUS:   %c\n",hvj.AEND_STATUS);
     printf("ABRSTATUS:    %c\n",hvj.ABR_STATUS);
     printf("ERHBISDT:     %s\n",hvj.ERHBISDT);
     printf("ERSBISDT:     %s\n",hvj.ERSBISDT);
     printf("ABRMODUS:     %s\n",hvj.ABRMODUS);
     printf("VTGNBTR:      %D(15,2)\n",hvj.VTGNBT);
     printf("BEARBNR:      %ld\n",hvj.BEARBNR);
   }
 return (0);
}

/*===schwebeBehandlung()===============================================
  Synopsis:      void schwebeBehandlung(void)
  Beschreibung:
  Modifikation:  24.03.1998 JoBarden/pdv  Ersterstellung
=====================================================================*/
static void schwebeBehandlung(void) {
 int iVJ;
 int schwebePos;
 char tDate[11];
 decimal(9,0) erhBisAlt;
 decimal(9,0) ersBisAlt;
 decimal(9,0) ersBisNeu;
 decimal(9,0) erhBisNeu;
 EXEC SQL BEGIN DECLARE SECTION;
  char           ERHBISDT[11];
  char           ERSBISDT[11];
  long int       BEARBNR;
  long int       HV_BEARBNR;
  decimal(15,0)  HV_VTRNR;
  decimal(5,0)   HV_HISTNR;
  decimal(5,0)   HV_VORG_LFDNR;
  char           HV_VSNR[11];
 EXEC SQL END DECLARE SECTION;

 //printf("Funktion schwebeBehandlung\n");
   for(iVJ = vtrWerte.posVon; iVJ < vtrWerte.posBis; iVJ++) {
      if( ( tabVJ[iVJ].abrStatus    == 'W' ) &&
          ( tabVJ[iVJ].aendStatus   != 'V' ) &&
          ( tabVJ[iVJ].erhebenBis   != 0   ) &&
          ( tabVJ[iVJ].erstattenBis != 0   ) &&
          ( tabVJ[iVJ].bearbNr != aktB.nummer  ) ) {

         incrDict(cDict, "schwebe gefunden");
         vtrWerte.tvSchwebe = TRUE;
         schwebePos = iVJ;

         /*-- merken der alten Daten --*/
         erhBisAlt = tabVJ[schwebePos].erhebenBis;
         ersBisAlt = tabVJ[schwebePos].erstattenBis;

         /*-- neue Daten speichern --*/
         if(tabVJ[schwebePos].erhebenBis < vtrWerte.erhebenBisDat) {
            tabVJ[schwebePos].erhebenBis = vtrWerte.erhebenBisDat;
         }
         tabVJ[schwebePos].erstattenBis = vtrWerte.erhebenBisDat;

         /*-- merken der neuen Daten --   WOZU ?? */
         erhBisNeu = tabVJ[schwebePos].erhebenBis;
         ersBisNeu = tabVJ[schwebePos].erstattenBis;

         /*-- Set HostVars fr Update TBKO_VTRJOURNAL --*/
         HV_VTRNR      = tabVJ[schwebePos].vtrnr;
         HV_HISTNR     = tabVJ[schwebePos].histnr;
         HV_BEARBNR    = tabVJ[schwebePos].bearbNr;   /* AEND0016 */
         HV_VORG_LFDNR = tabVJ[schwebePos].vorgLfdnr;

         strncpy(HV_VSNR,aktB.vsnr,10);       /* AEND0016 */
         HV_VSNR[10] = '\0';

         C_DATE(tabVJ[schwebePos].erhebenBis, tDate);
         memcpy(ERHBISDT,tDate,10);
                ERHBISDT[10] = '\0';
         C_DATE(tabVJ[schwebePos].erstattenBis, tDate);
         memcpy(ERSBISDT,tDate,10);
                ERSBISDT[10] = '\0';

         /*-- Behandlung von Vordatierungen --*/
         if(vtrWerte.vordat) {
            EXEC SQL UPDATE TBKO_VTRJOURNAL
                     SET    ERHBISDT   = :ERHBISDT,
                            ERSBISDT   = :ERSBISDT
                     WHERE  VTRNR      = :HV_VTRNR
                     AND    HISTNR     = :HV_HISTNR
                     AND    VORG_LFDNR = :HV_VORG_LFDNR;
            CHECK0(ABBRUCH,
              "PKQ2550: schwebeBehandlung() UPD-1 TBKO_VTRJOURNAL");
         } else {
            /*-- diese Mimik nur fr LBS --*/
            /*-- !!noch zu kontrollieren!! X9KL0508 --*/
            incrDict(cDict, "schwebe gefunden (maxBearbNr++)");
            maxBearbNr++;
            BEARBNR = maxBearbNr;
            if (testlauf) {
               printf("PKQ2550: UPDATE auf BEARBNR\n");
               printf("BEARBNR:      %ld\n",BEARBNR);
               printf("HISTNR:       %D(5,0)\n",HV_HISTNR);
               printf("VORG_LFDNR:   %D(5,0)\n",HV_VORG_LFDNR);
            }
            EXEC SQL UPDATE TBKO_VTRJOURNAL
                     SET    ERHBISDT   = :ERHBISDT,
                            ERSBISDT   = :ERSBISDT,
                            BEARBNR    = :BEARBNR
                     WHERE  VTRNR      = :HV_VTRNR
                     AND    HISTNR     = :HV_HISTNR
                     AND    VORG_LFDNR = :HV_VORG_LFDNR;
            CHECK0(ABBRUCH,
              "PKQ2550: schwebeBehandlung() UPD-2 TBKO_VTRJOURNAL");
                                             /* aend0012 */
            if (testlauf) {
               printf("PKQ2550: UPDATE der TBKO_VTRKORRESP\n");
               printf("BEARBNR:      %ld\n",BEARBNR);
               printf("HISTNR:       %D(5,0)\n",HV_HISTNR);
               printf("VORG_LFDNR:   %D(5,0)\n",HV_VORG_LFDNR);
            }
                                             /* aend0016 */
            EXEC SQL UPDATE TBKO_VTRKORRESP
                     SET    BEARBNR    = :BEARBNR
                     WHERE  VTRNR      = :HV_VTRNR
                     AND    HISTNR     = :HV_HISTNR
                     AND    VORG_LFDNR = :HV_VORG_LFDNR;
            CHECK(ABBRUCH,
              "PKQ2550: schwebeBehandlung() UPD TBKO_VTRKORRESP");

            if (testlauf) {
               printf("PKQ2550: UPDATE der TBKO_KORRESP\n");
               printf("BEARBNR:      %ld\n",BEARBNR);
               printf("VSNR:         %s\n",HV_VSNR);
               printf("VORG_LFDNR:   %D(5,0)\n",HV_VORG_LFDNR);
            }
            EXEC SQL UPDATE TBKO_KORRESP
                     SET    BEARBNR    = :BEARBNR
                     WHERE  VSNR       = :HV_VSNR
                     AND    BEARBNR    = :HV_BEARBNR;
            CHECK(ABBRUCH,
              "PKQ2550: schwebeBehandlung() UPD TBKO_KORRESP");


         }



      }
   }

}

/*===updateBearbNr()===================================================
  Synopsis:      void updateBearbNr(void)
  Beschreibung:  fachlich ist diese Funktion noch zu klren!!
                 sie ist aus dem Programm MKO2501 komplett bernommen.
  Modifikation:  24.03.1998 JoBarden/pdv  Ersterstellung
=====================================================================*/
static int updateBearbNr(void) {
 struct {
   decimal(11,0)  alt;
   decimal(11,0)  neu;
 } tabBearbNr[20];

 int           i,k,posU,posV;
 decimal(11,0) neuBearbNr;
 S_MKO3130     smko3130;
 S_MKO3130    *pmko3130;

   /*---- Initialisierung -------------------------------------------*/
   for(i=0; i<20; i++) {
     tabBearbNr[i].alt = 0D;
     tabBearbNr[i].neu = 0D;
   }
   pmko3130 = &smko3130;
   memcpy(smko3130.strukturName, "XKO3130 ", 8);
   memcpy(smko3130.version     , "01.00"   , 5);
   memcpy(smko3130.ende        , "ENDE"    , 4);

   /*---- als erstes die U-Stnde der Vordatieurng behandeln --------*/
   for(k=0; k<20; k++) {
      posU = vtrWerte.posVordatU[k];
      neuBearbNr = 0D;
      if(posU > 0) {
         /*--- nur weiter, wenn ein U-Stand abgelegt ist ---*/
         for(i=0; i<20; i++) {
            if(tabBearbNr[i].alt == tabVJ[posU].bearbNr) {
               neuBearbNr = tabBearbNr[i].neu;
               break;
            }
         } /* ende for */
         if(neuBearbNr == 0) {
            maxBearbNr++;
            neuBearbNr = maxBearbNr;
            for(i=0; i<20; i++) {
               if(tabBearbNr[i].alt == 0D) {
                  /*--- alte und neue BearbNr an nchstem freien --*/
                  /*--- Tabellenplatz eintragen.                 --*/
                  tabBearbNr[i].alt == tabVJ[posU].bearbNr;
                  tabBearbNr[i].neu = neuBearbNr;
                  break;
               }
               if(i=20) {
                  printf("Fehler tabBearbNr zu klein!\n");
                  return(-1);
               }

               memcpy(smko3130.vsnr, aktB.vsnr, 10);
               smko3130.vtrnr      = tabVJ[posU].vtrnr;
               smko3130.histnr     = tabVJ[posU].histnr;
               smko3130.vorgLfdnr  = tabVJ[posU].vorgLfdnr;
               smko3130.status     = 'U';
               smko3130.bearbNrAlt = tabVJ[posU].bearbNr;
               smko3130.bearbNrNeu = neuBearbNr;
               mko3130(&pmko3130);
               if(smko3130.retcode != 0) {
                  printf("Fehler bei MKO3130: %d SQL: %d %d %.8s \n",
                     smko3130.retcode, smko3130.sqlcode,
                     smko3130.sqlCallNr, smko3130.fmodul);
                  return(-1);
               }
            } /* ende for */
         } /* ende if */
      } /* ende if */
   } /* ende for (fr jeden U-Stand -> Zhler: k) */

   /*---- jetzt den V-Stand der Vordatieurng behandeln --------------*/
   posV = vtrWerte.posVordatV;
   neuBearbNr = 0D;
   if(posV > 0) {
      /*--- nur weiter, wenn ein V-Stand existiert ---*/
      for(i=0; i<20; i++) {
         if(tabBearbNr[i].alt == tabVJ[posV].bearbNr) {
            neuBearbNr = tabBearbNr[i].neu;
            break;
         }
      } /* ende for */
      if(neuBearbNr == 0) {
         maxBearbNr++;
         neuBearbNr = maxBearbNr;
         for(i=0; i<20; i++) {
            if(tabBearbNr[i].alt == 0D) {
               /*--- alte und neue BearbNr an nchstem freien --*/
               /*--- Tabellenplatz eintragen.                 --*/
               tabBearbNr[i].alt == tabVJ[posV].bearbNr;
               tabBearbNr[i].neu = neuBearbNr;
               break;
            }
            if(i=20) {
               printf("Fehler tabBearbNr zu klein!\n");
               return(-1);
            }

            memcpy(smko3130.vsnr, aktB.vsnr, 10);
            smko3130.vtrnr      = tabVJ[posV].vtrnr;
            smko3130.histnr     = tabVJ[posV].histnr;
            smko3130.vorgLfdnr  = tabVJ[posV].vorgLfdnr;
            smko3130.status     = 'U';
            smko3130.bearbNrAlt = tabVJ[posV].bearbNr;
            smko3130.bearbNrNeu = neuBearbNr;
            mko3130(&pmko3130);
            if(smko3130.retcode != 0) {
               printf("Fehler bei MKO3130: %d SQL: %d %d %.8s \n",
                  smko3130.retcode, smko3130.sqlcode,
                  smko3130.sqlCallNr, smko3130.fmodul);
               return(-1);
            }
         } /* ende for */
      } /* ende if */
   } /* ende if */

}

/*===aufbauAbrGrundlagen===============================================
  Synopsis:      void aufbauAbrGrundlagen(void)
  Beschreibung:
  Modifikation:  11.03.1998 JoBarden/pdv  Ersterstellung
=====================================================================*/
static void aufbauAbrGrundlagen(void) {
 char          tempTabName[30];
 int           i;
 decimal(5,0)  anzAbrKum, anzSteuer, anzRaten, anzZeitR = 0D;

   /*---- Grundeinstellungen fr MKO1571 ---------------------------*/
   memcpy(smko1571.strukturName, "SMKO1571", 8);
   memcpy(smko1571.version, "01.00", 5);
   memcpy(smko1571.ende, "ENDE", 4);
          smko1571.mandant = smko5010.mandant;
          smko1571.sprachcode = smko5010.sprachcode;
          smko1571.erstaufruf = 'J';
          smko1571.kzVerarb   = 'B';
          smko1571.abrVgKz    = '2';
          smko1571.pTabVG     = &tabVG;
          pmko1571 = &smko1571;

   /*---- Grundeinstellungen fr MKO1580 ---------------------------*/
   memcpy(smko1580.strukturName, "SMKO1580", 8);
   memcpy(smko1580.version, "01.00", 5);
   memcpy(smko1580.ende, "ENDE", 4);
          smko1580.mandant = smko5010.mandant;
          smko1580.sprachcode = smko5010.sprachcode;
     /*
          smko1580.dko530_ja_nein = 'N';
          smko1580.vtb_gb_pkz = 'D';
          smko1580.anzTabVtbGb= 0;
          smko1580.abr_bu_pkz = 'P';
          smko1580.zeitrber_pkz = 'P';
          smko1580.ratenzahl_pkz = 'P';
          smko1580.steuersatz_pkz = 'P';
          smko1580.abrkumulier_pkz = 'P';
          smko1580.vtb_bz_pkz = 'P';
          smko1580.anzVtbBz = 1;
          memset(smko1580.kum_bn_var,' ',sizeof(smko1580.kum_bn_var));
          smko1580.vtb_ss_ja_nein = 'N';
     */

          pmko1580 = &smko1580;

   /*---- Grundeinstellungen fr MKO1518 ---------------------------*/
   memcpy(smko1518.strukturName, "SMKO1518", 8);
   memcpy(smko1518.version, "01.00", 5);
   memcpy(smko1518.ende, "ENDE", 4);
          smko1518.mandant = smko5010.mandant;
          smko1518.sprachcode = smko5010.sprachcode;
          pmko1518 = &smko1518;

   /*---- TBKO_ABRKUMULIER -----------------------------------------*/
   strcpy(     tempTabName, "TBKO_ABRKUMULIER");
   memset(smko1518.tabName, ' ', sizeof(smko1518.tabName));
   memcpy(smko1518.tabName, tempTabName, strlen(tempTabName));
          smko1518.tabAnzahl = MAX_ABR_KUMULIER;
          smko1518.abrKum.pTabAbrKum    = &tabAbrKum;
          smko1518.abrKum.kzVtrKatKz    = 'A';
          smko1518.abrKum.kzGsnrvg      = 'A';
          smko1518.abrKum.kzAbrKumEbene = 'A';
   memset(smko1518.abrKum.abrKumEbene, ' ', 20); // 10  2 Stellen
   for(i = 0; i < 10; i++) {
          smko1518.abrKum.gsnrvg[i]       = 0D;
          smko1518.abrKum.vtrKatKz[i]     = 0D;
          smko1518.abrKum.vtrKatKz[i+10]  = 0D;
   }

   MKO1518(&pmko1518);
   if(memcmp(smko1518.retcode, "0000", 4) == 0) {
      anzAbrKum = smko1518.anzAbrKum;
   // printf("-<ABR_GRUND>- AbrKum Anzahl: %D(5,0)\n", anzAbrKum);
   // printf("-<ABR_GRUND>- AbrKum[0]: gs:%D(5,0), vKatKz:%D(3,0)\n",
   //     tabAbrKum[0].gsnrvg, tabAbrKum[0].vtrKatKz);
   } else {
      printf("-<ABR_GRUND>- Retcode: |%.4s|\n",smko1518.retcode);
      printf("-<ABR_GRUND>- fmodul:  |%.7s|\n", smko1518.fmodul);
      escape(ABBRUCH, EC_MKO1518_Fehler,
        "PKQ2550: aufbauAbrGrundlagen() MKO1518 failed (AbrKumulier)");
   }

   /*---- TBKO_ZEITRBER --------------------------------------------*/
   strcpy(     tempTabName, "TBKO_ZEITRBER");
   memset(smko1518.tabName, ' ', sizeof(smko1518.tabName));
   memcpy(smko1518.tabName, tempTabName, strlen(tempTabName));
          smko1518.tabAnzahl = MAX_ABR_ZEITRBER;
          smko1518.zeitr.pTabAbrZeitR = &tabAbrZeitR;
          smko1518.zeitr.kzGsnrvg     = 'A';
          smko1518.zeitr.kzVtrKatKz   = 'A';
   for(i = 0; i < 10; i++) {
          smko1518.zeitr.gsnrvg[i]       = 0D;
          smko1518.zeitr.vtrKatKz[i]     = 0D;
          smko1518.zeitr.vtrKatKz[i+10]  = 0D;
   }

   pmko1518 = &smko1518;
   MKO1518(&pmko1518);
   if(memcmp(smko1518.retcode, "0000", 4) == 0) {
      anzZeitR = smko1518.anzZeitrber;
   // printf("-<ABR_GRUND>- Zeitrber Anzahl: %D(5,0)\n", anzZeitR);
   // printf("-<ABR_GRUND>- Zeitrber[0]: gs:%D(5,0), vKatKz:%D(3,0)\n",
   //     tabAbrZeitR[0].gsnrvg, tabAbrZeitR[0].vtrKatKz);
   } else {
      printf("-<ABR_GRUND>- Retcode: |%.4s|\n",smko1518.retcode);
      printf("-<ABR_GRUND>- fmodul:  |%.7s|\n", smko1518.fmodul);
      escape(ABBRUCH, EC_MKO1518_Fehler,
         "PKQ2550: aufbauAbrGrundlagen() MKO1518 failed (Zeitrber)" );
   }

   /*---- TBKO_RATENZAHL -------------------------------------------*/
   strcpy(     tempTabName, "TBKO_RATENZAHL");
   memset(smko1518.tabName, ' ', sizeof(smko1518.tabName));
   memcpy(smko1518.tabName, tempTabName, strlen(tempTabName));
          smko1518.tabAnzahl = MAX_ABR_RATENZAHL;
          smko1518.raten.pTabAbrRaten = &tabAbrRaten;
          smko1518.raten.kzGsnrvg     = 'A';
          smko1518.raten.kzVtrKatKz   = 'A';
          smko1518.raten.kzZahlweise  = 'A';
   memset(&smko1518.raten.zahlweise, ' ', 10);   // 5  2 Stellen
   for(i = 0; i < 10; i++) {
          smko1518.raten.gsnrvg[i]       = 0D;
          smko1518.raten.vtrKatKz[i]     = 0D;
          smko1518.raten.vtrKatKz[i+10]  = 0D;
   }

   pmko1518 = &smko1518;
   MKO1518(&pmko1518);
   if(memcmp(smko1518.retcode, "0000", 4) == 0) {
      anzRaten = smko1518.anzRaten;
   // printf("-<ABR_GRUND>- Raten Anzahl: %D(5,0)\n", anzRaten);
   // printf("-<ABR_GRUND>- Raten[0]: gs:%D(5,0), vKatKz:%D(3,0)\n",
   //     tabAbrRaten[0].gsnrvg, tabAbrRaten[0].vtrKatKz);
   } else {
      printf("-<ABR_GRUND>- Retcode: |%.4s|\n",smko1518.retcode);
      printf("-<ABR_GRUND>- fmodul:  |%.7s|\n", smko1518.fmodul);
      escape(ABBRUCH, EC_MKO1518_Fehler,
         "PKQ2550: aufbauAbrGrundlagen() MKO1518 failed (Ratenzahl)" );
   }

   /*---- TBKO_STEUERSARTZ -----------------------------------------*/
   strcpy(     tempTabName, "TBKO_STEUERSATZ");
   memset(smko1518.tabName, ' ', sizeof(smko1518.tabName));
   memcpy(smko1518.tabName, tempTabName, strlen(tempTabName));
          smko1518.tabAnzahl = MAX_ABR_STEUERSATZ;
          smko1518.steuer.pTabAbrSteuer = &tabAbrSteuer;
          smko1518.steuer.kzVtrKatKz   = 'A';
          smko1518.steuer.kzIntKfz     = 'A';
          smko1518.steuer.kzSteuerart  = 'A';
   memset(&smko1518.steuer.intKfz   , ' ', 30);   // 10  3 Stellen
   memset(&smko1518.steuer.steuerart, ' ', 10);   // 5   2 Stellen
   for(i = 0; i < 10; i++) {
          smko1518.steuer.vtrKatKz[i]     = 0D;
          smko1518.steuer.vtrKatKz[i+10]  = 0D;
   }

   pmko1518 = &smko1518;
   MKO1518(&pmko1518);
   if(memcmp(smko1518.retcode, "0000", 4) == 0) {
      anzSteuer = smko1518.anzSteuer;
   // printf("-<ABR_GRUND>- Steuer Anzahl: %D(5,0)\n", anzSteuer);
   // printf("-<ABR_GRUND>- Steuer[0]: intKfz:%.3s, vKatKz:%D(3,0)\n",
   //     tabAbrSteuer[0].intKfz, tabAbrSteuer[0].vtrKatKz);
   } else {
      printf("-<ABR_GRUND>- Retcode: |%.4s|\n",smko1518.retcode);
      printf("-<ABR_GRUND>- fmodul:  |%.7s|\n", smko1518.fmodul);
      escape(ABBRUCH, EC_MKO1518_Fehler,
         "PKQ2550: aufbauAbrGrundlagen() MKO1518 failed (Steuersatz)");
   }

   /*---- Tabellen aus MKO1518 in MKO1580 bernehmen (Pointer) -----*/
   memset(smko1580.abrBuTxt, ' ', sizeof(smko1580.abrBuTxt));
          smko1580.pZeitR      = &tabAbrZeitR;
          smko1580.anzZeitR    = anzZeitR;
          smko1580.pRaten      = &tabAbrRaten;
          smko1580.anzRaten    = anzRaten;
          smko1580.pSteuer     = &tabAbrSteuer;
          smko1580.anzSteuer   = anzSteuer;
          smko1580.pAbrKum     = &tabAbrKum;
          smko1580.anzAbrKum   = anzAbrKum;
          smko1580.kzUpdAbr    = 'J';

   /*--- in KO ber include X9000434 gefllt ------------------------*/
   memcpy(smko1580.kobaIntKfz, "D  ", 3);      /*--- Deutschland ----*/
          smko1580.kobaRundung = -1;           /*--- auf Zehntel ----*/
          smko1580.kobaSteuer[0].steuerKuerzel[0]='V';  /* VS:   ----*/
          smko1580.kobaSteuer[0].steuerKuerzel[1]='S';  /* %-Steuer -*/
          smko1580.kobaSteuer[0].steuerCodeNr = 1700D;

   for(i = 1; i < 5; i++) {        // die nchsten 4 initialisieren
          smko1580.kobaSteuer[i].steuerKuerzel[0] = ' ';
          smko1580.kobaSteuer[i].steuerKuerzel[1] = ' ';
          smko1580.kobaSteuer[i].steuerCodeNr = 0D;
   }

}

/*===abrechnung()======================================================
  Synopsis:      void abrechnung(void)
  Beschreibung:
  Modifikation:  27.03.1998 JoBarden/pdv  Ersterstellung
=====================================================================*/
static int abrechnung(void) {
 int    i;
 int    rc = 0;

   /*--- interne Tabelle TabVG anhand tabVJ aufbauen ----------------*/
   for(i = 0; i < anzTabVJ; i++) {
      memset(tabVG[i].modus,   ' ', 2);
      memset(tabVG[i].reserve, ' ', 8);
      memset(tabVG[i].frei,    ' ', 13);
             tabVG[i].vtrnr        = tabVJ[i].vtrnr;
             tabVG[i].histnr       = tabVJ[i].histnr;
             tabVG[i].vorgLfdnr    = tabVJ[i].vorgLfdnr;
      memcpy(tabVG[i].vorgSchl,      tabVJ[i].vorgSchl,4);
      memcpy(tabVG[i].vsnr,          tabVJ[i].vsnr, 10);
             tabVG[i].polhistnr    = tabVJ[i].twaend;
             tabVG[i].gsnrvg       = tabVJ[i].gsnrvg;
             tabVG[i].whgnr        = tabVJ[i].whgnr;
             tabVG[i].bearbDatum   = tabVJ[i].bearbDatum;
             tabVG[i].bearbNr      = tabVJ[i].bearbNr;
             tabVG[i].aendDatum    = tabVJ[i].aendDatum;
             tabVG[i].aendStatus   = tabVJ[i].aendStatus;
             tabVG[i].erhebenVon   = 0D;
             tabVG[i].erhebenBis   = tabVJ[i].erhebenBis;
             tabVG[i].erstattenVon = 0D;
             tabVG[i].erstattenBis = tabVJ[i].erstattenBis;
      memcpy(tabVG[i].abrModus,      tabVJ[i].abrModus,2);
             tabVG[i].abrStatus    = tabVJ[i].abrStatus;
             tabVG[i].vtrkatkz     = tabVJ[i].vtrkatkz;
             tabVG[i].fbkz         = tabVJ[i].fbkz;
   }
   anzTabVG = i;


   /*---- Parameter fr MKO1571 ------------------------------------*/
   // DAS MODUL MKO1571 ERMITTELT DIE SATZARTEN
   // "ABRGE" UND STELLT SIE FUER DIE FOLGE-
   // BEARBEITUNG IN EINER INTERNEN TABELLE ZUR
   // VERFUEGUNG.                  -- Prog-Beschreibung aus Kompass

   memcpy(smko1571.vsnr,       aktB.vsnr, 10);
          smko1571.bearbNr   = aktB.nummer;
          smko1571.anzTabVG  = anzTabVG;
   memcpy(smko1571.zweig,      aktB.zweig, 4);
   memcpy(smko1571.kobaIntKfz, "D  ", 3);

   pmko1571 = &smko1571;
   MKO1571(&pmko1571);
   if(memcmp(smko1571.retcode, "0000", 4) == 0) {
   /* printf("-<ABR_1571>- OK\n"); */
      smko1571.erstaufruf = ' ';
      incrDict(cDict,"X: abrechnung MKO1571 OK");
   } else {
      printf("-<ABR_1571>- Retcode: |%.4s|\n",smko1571.retcode);
      printf("-<ABR_1571>- fmodul:  |%.7s|\n", smko1571.fmodul);
      incrDict(cDict,"Err: abrechnung MKO1571");
      rc=ProtFehler("MKO1571",smko1571.fmodul,smko1571.retcode);
      rc=updateAbrechFehler();
      return(FALSE);
   }

   /*---- Parameter fr MKO1580 ------------------------------------*/
   memcpy(smko1580.zweig, aktB.zweig, 4);
   smko1580.pTabAbrGE     = smko1571.pTabAbrGE;
   smko1580.anzTabAbrGE   = smko1571.anzTabAbrGE;
   smko1580.pTabAbrVG     = smko1571.pTabAbrVG;
   smko1580.anzTabAbrVG   = smko1571.anzTabAbrVG;
   smko1580.pTabAbrSV     = smko1571.pTabAbrSV;
   smko1580.anzTabAbrSV   = smko1571.anzTabAbrSV;
   /*
      smko1580.pTabVtrKat    = smko1571.pTabVtrKat;
      smko1580.anzTabVtrKat  = smko1571.anzTabVtrKat;
   */
   smko1580.pTabFobHist   = smko1571.pTabFobHist;
   smko1580.anzTabFobHist = smko1571.anzTabFobHist;
   smko1580.kzAuslagernGE = smko1571.kzAuslagernGE;

   pmko1580 = &smko1580;
   MKO1580(&pmko1580);
   if(memcmp(smko1580.retcode, "0000", 4) == 0) {
   /* printf("-<ABR_1580>- OK\n"); */
      incrDict(cDict,"X: abrechnung MKO1580 OK");
   } else {
      printf("-<ABR_1580>- Retcode: |%.4s|\n",smko1580.retcode);
      printf("-<ABR_1580>- fmodul:  |%.7s|\n", smko1580.fmodul);
      incrDict(cDict,"Err: abrechnung MKO1580");
      rc=ProtFehler("MKO1580",smko1580.fmodul,smko1580.retcode);
      rc=updateAbrechFehler();
      rc=resetAbrtab();       /*Abrechnungstabellen zuruecksetzen  */
      return(FALSE);
   }
                   /* AEND0006-A Abrechnung war OK -> evtl. Commit */
                                   /* AEND9999 Anfang

      memcpy(testVSNR, aktB.vsnr, 10);

      EXEC SQL OPEN testRW;
      while(SQLOK) {
      EXEC SQL FETCH testRW
       INTO
       :hCOD_MANDANT,         :hCOD_GESELLSCHAFT,    :hNR_POLICE,
       :hNR_SAMMEL_EXTERN,    :hCOD_SYSTEM,          :hID_EXTERN,
       :hCOD_VTRART_KO,       :hCOD_LEISTUNGSART,    :hCOD_BUCH_TYP,
       :hCOD_VORGANG,         :hCOD_VORG_DETAIL,     :hCOD_BUCH_DETAIL,
       :hCOD_ZAHLART,         :hDAT_FAELLIG,         :hDAT_GUELTIG_AB,
       :hDAT_GUELTIG_BIS,     :hDAT_ABGABE,          :hCOD_ISOWHG,
       :hBTR_BUCHUNG ,        :hBTR_STEUER,          :hPRO_STEUERSATZ,
       :hCOD_VERRECHNUNG,     :hKEZ_FEHLER,
       :hKEZ_VERARB_INKASSO;
      CHECK(ABBRUCH, "PKQ2550: verarbeitung() Fetch testRW-Cursor");

        if (SQLCODE == 100) {
          printf("kein RW006-Satz gefunden\n");
        } else {
        RW006Wert = RW006Wert + hBTR_BUCHUNG;

        printf("RW006_Satz wurde gefunden\n");
        printf("VSNR %.10s RW006-Wert: %D(15,2) Ges-Wert: %D(15,2)\n",
           aktB.vsnr, hBTR_BUCHUNG, RW006Wert);
        errno = 0;
        }    Ende else --> Satz wurde gefunden
      }    ende while SQLOK

    EXEC SQL CLOSE testRW;

                                      AEND9999 ENDE   */

      anzErh++;
      gesErh++;
      today(cDate); now(cTime);
      if (anzErh >= commit_count) {


        checkpoint();

      /*fprintf(PROT,"Commit durchgefuehrt, Checkpoint gesetzt\n");*/
        printf("Commit durchgefuehrt, Checkpoint gesetzt \n");

      /*fprintf(PROT,"%s  %s  Verarbeitete Vertraege: %D(15,0)\n",
           cDate, cTime, gesErh); */

        printf("%s  %s  Verarbeitete Vertraege: %D(15,0)\n",
               cDate, cTime, gesErh);
        anzErh= 0;

      }
    return TRUE;
   return(TRUE);

}

 /*===fuellenAbrTab()===================================AEND0006======
  Synopsis:      void fuellenAbrTab(void)
  Beschreibung:  vorlufig, bis die Daten aus MKO1580 zurckgegeben
                 werden.
  Modifikation:  01.04.1998 JoBarden/pdv  Ersterstellung
 ===================================================================
 static void fuellenAbrTab(void) {
 EXEC SQL BEGIN DECLARE SECTION;
   char             ABR_IN_VSNR[11];
   decimal(11,0)    ABR_IN_BEARBNR;
   long int         ABR_IN_BEARBNR_H;
   decimal(15,0)    ABR_IN_VTRNR;
   decimal(15,0)    ABR_VTRNR;
   decimal(5,0)     ABR_VORGANG_HISTNR;
   decimal(5,0)     ABR_VORG_LFDNR;
   decimal(5,0)     ABR_LVNR;
   long int         ABR_BUNR;
   decimal(5,0)     ABR_BUCONR;
   char             ABR_BUCH_ART[3];
   decimal(15,3)    ABR_BUBETRAG;
   char             ABR_BUZBEGDT[11];
   char             ABR_BUZENDDT[11];
   char             ABR_FAELL_DATUM[11];
 EXEC SQL END DECLARE SECTION;

 EXEC SQL DECLARE abr CURSOR FOR
      SELECT VTRNR, VORGANG_HISTNR, VORG_LFDNR,
             A.LVNR, A.BUNR, BUCONR, BUCH_ART, BUBETRAG,
             BUZBEGDT, BUZENDDT, FAELL_DATUM
      FROM   TBKO_ABR_BUCHUNG A,
             TBKO_ABR_BETRAG B
      WHERE  A.VSNR     =  B.VSNR
      AND    A.BEARBNR  =  B.BEARBNR
      AND    A.BUNR     =  B.BUNR
      AND    A.ABRKUMNR =  ' '
      AND    A.VSNR     =  :ABR_IN_VSNR
      AND    A.BEARBNR  =  :ABR_IN_BEARBNR_H
      AND    VTRNR      =  :ABR_IN_VTRNR;
   // ohne ORDER BY ??????

   memcpy(ABR_IN_VSNR, aktB.vsnr, 10);ABR_IN_VSNR[10] = '\0';
   ABR_IN_BEARBNR = aktB.nummer;
   ABR_IN_BEARBNR_H = ABR_IN_BEARBNR;
   ABR_IN_VTRNR   = tabVJ[0].vtrnr;
   anzTabAbr = 0;

   EXEC SQL OPEN abr;
   CHECK0(ABBRUCH, "PKQ2550: fuellenAbrTab() Open abr-Cursor");
   while(SQLOK) {
      EXEC SQL FETCH abr INTO
             :ABR_VTRNR, :ABR_VORGANG_HISTNR, :ABR_VORG_LFDNR,
             :ABR_LVNR,  :ABR_BUNR, :ABR_BUCONR,
             :ABR_BUCH_ART, :ABR_BUBETRAG,
             :ABR_BUZBEGDT, :ABR_BUZENDDT, :ABR_FAELL_DATUM;
      if(SQLOK) {
         tabAbr[anzTabAbr].vtrnr   = ABR_VTRNR;
         tabAbr[anzTabAbr].vorghistnr = ABR_VORGANG_HISTNR;
         tabAbr[anzTabAbr].vorglfdnr = ABR_VORG_LFDNR;
         tabAbr[anzTabAbr].lvnr    = ABR_LVNR;
         tabAbr[anzTabAbr].bunr    = ABR_BUNR;
         tabAbr[anzTabAbr].buconr  = ABR_BUCONR;
         tabAbr[anzTabAbr].bubetrag = ABR_BUBETRAG;
         strcpy(tabAbr[anzTabAbr].buchart, ABR_BUCH_ART);
         strcpy(tabAbr[anzTabAbr].buzbegdt, ABR_BUZBEGDT);
         strcpy(tabAbr[anzTabAbr].buzenddt, ABR_BUZENDDT);
         strcpy(tabAbr[anzTabAbr].faelldatum, ABR_FAELL_DATUM);
         anzTabAbr++;
      }
   }
   EXEC SQL CLOSE abr;

   printf("--+--> fuellenAbrTab() - vtrnr: %D(15,0), anz: %d\n",
      ABR_IN_VTRNR, anzTabAbr); *

 }                                       AEND0006-E ***/

 /*==inkasso()=====================================AEND0006-A========
  Synopsis:      int inkasso(void)
  Beschreibung:
  Modifikation:  01.04.1998 JoBarden/pdv  Ersterstellung
 ====================================================================
 static int inkasso(void) {
 char cTime[9], cDate[11];

 int mkq2550RC = 0;
 int rc        = 0;
 EXEC SQL BEGIN DECLARE SECTION;
   decimal(15,0)    KL_VTRNR;
   decimal(11,0)    KL_BEARBNR;
   long int         KL_BEARBNR_H;
   decimal(15,2)    KL_VTRBTG;
   decimal(15,2)    KL_VTRINKBTG;
   decimal(15,2)    KL_BTG_VERR;
   char             KL_VERR_ART[5];
 EXEC SQL END DECLARE SECTION;
 ABR_SATZ *pAbrSatz;

    *-- temporr: fllen ABR-Tabelle (die Daten sollen spter vom  --*
    *--           MKO1580 zurckgegeben werden).                   --*
   fuellenAbrTab();

    *-- temporr: Zugriff auf die Tabelle TBKL_VERTRAG ( die Daten --*
    *--           sollen spter vom Hauptcursor ermittelt werden!? --*
    *-- TEST:     der SELECT ist noch zu verifizieren !!!!!!!!!!!! --*

   KL_VTRNR   = tabVJ[0].vtrnr;
   KL_BEARBNR = aktB.nummer;
   KL_BEARBNR_H = KL_BEARBNR;

   EXEC SQL SELECT  V.VTRBTG, V.VTRINKBTG, V.BTG_VERR, V.VERR_ART
            INTO    :KL_VTRBTG, :KL_VTRINKBTG,
                    :KL_BTG_VERR, :KL_VERR_ART
            FROM    TBKL_VERTRAG V
            WHERE   V.VTRNR  = :KL_VTRNR
            AND     V.HISTNR =
                      (SELECT MAX(V1.HISTNR)
                       FROM   TBKL_VERTRAG V1
                       WHERE  V1.VTRNR = V.VTRNR
                       AND    V1.HISTNR <= (
                                 SELECT MAX(J.HISTNR)
                                 FROM   TBKO_VTRJOURNAL J
                                 WHERE J.VTRNR = :KL_VTRNR
                                 AND J.AEND_STATUS = 'E'
                                 AND J.BEARBNR = :KL_BEARBNR_H)) ;

   CHECK(ABBRUCH, "PKQ2550: inkasso() Select TBKL_VERTRAG");

   if(OK) {
   *  printf("--+--> inkasso() vtrnr:%D(15,0), bearbnr:%D(11,0) %s\n",
         KL_VTRNR, KL_BEARBNR, " - found (OK)");   *
   } else {
      printf("--+--> inkasso() vtrnr:%D(15,0), bearbnr:%D(11,0) %s\n",
         KL_VTRNR, KL_BEARBNR, " - not found (SQL100)");
      return(FALSE);
   }

    *-- fllen der bergabestruktur fpr MKQ2550 ---------------------*
   kq2550.pVJ        = &tabVJ;
   kq2550.anzvj      = anzTabVJ;
   kq2550.pAB        = &aktB;
   kq2550.pABR       = &tabAbr;
   kq2550.anzabr     = anzTabAbr;
   kq2550.vtrbtg     = KL_VTRBTG;
   kq2550.vtrinkbtg  = KL_VTRINKBTG;
   kq2550.btg_verr   = KL_BTG_VERR;
   kq2550.mandant    = smko5010.mandant;
   kq2550.sprachcode = smko5010.sprachcode;
   kq2550.sqlcode    = 0;
   strcpy(kq2550.verr_art, KL_VERR_ART);
    * printf("%s %d, anzabr: %d bNr: %D(11,0), vtrbtg: %D(15,2)\n",
         "--+--> inkasso() vor mkq2550 anzvj:",
         kq2550.anzvj, kq2550.anzabr, aktB.nummer, kq2550.vtrbtg);*

   mkq2550RC = (*putInkasso)(&kq2550, Env);

    *printf("mkq2550RC = %d\n",mkq2550RC); *
   if (mkq2550RC == 0) { /* Verarbeitung VSNR war OK           *
      anzErh++;
      gesErh++;
      today(cDate); now(cTime);
      if (anzErh >= commit_count) {


        checkpoint();

        fprintf(PROT,"Commit durchgefuehrt, Checkpoint gesetzt\n");
        printf("Commit durchgefuehrt, Checkpoint gesetzt \n");

        fprintf(PROT,"%s  %s  Verarbeitete Vertraege: %D(15,0)\n",
           cDate, cTime, gesErh);

        printf("%s  %s  Verarbeitete Vertraege: %D(15,0)\n",
               cDate, cTime, gesErh);
        anzErh= 0;

      }
      return TRUE;
   } else {                     * Bei der Verarbeitung der VSNR      *
     if (mkq2550RC > 0) {       * ist ein Fehler aufgetreten         *
                                * --> bereits vorgenommene UPDATES   *
                                * und INSERTS zuruecksetzen          *

       rc=PKQ2550Fehler("mkq2550",
                         kq2550.Fehlertext,
                         mkq2550RC,
                         kq2550.sqlcode);

      * rc=resetAbrtab();         Abrechnungstabellen zuruecksetzen  *


     } else {   * mkq2550RC < 0                      *
       rc=PKQ2550Fehler("mkq2550",
                         kq2550.Fehlertext,
                         mkq2550RC,
                         kq2550.sqlcode);


     }
   }
} AEND0006-E */

/*===resetAbrtab()===================================================
  Synopsis:      int resetAbrtab(void)
  Beschreibung:  Zuruecksetzen aller Abrechnungstabellen, wenn
                 in der Inkassobestueckung (MKQ2550) ein Fehler
                 fuer diese VSNR aufgetreten ist
  Modifikation:  18.03.1998 JoBarden/pdv  Ersterstellung
=====================================================================*/
static int  resetAbrtab(void) {

 EXEC SQL BEGIN DECLARE SECTION;
  char           HV_VSNR[11];
  long int       HV_BEARBNR;
  decimal(15,0)  HV_VTRNR;
 EXEC SQL END DECLARE SECTION;

 printf("Funktion resetAbrtab nach fehlerhaften MKO1580\n");
 strncpy(HV_VSNR,aktB.vsnr,10);
         HV_VSNR[10] = '\0';
 HV_BEARBNR = aktB.nummer;
 HV_VTRNR = tabVJ[0].vtrnr;

  EXEC SQL
  DELETE FROM TBKO_ABR_VORGANG
  WHERE       VSNR    = :HV_VSNR
    AND       BEARBNR = :HV_BEARBNR
  ;
  CHECK(WARNUNG ,"PKQ2550--+--> resetAbrtab: Fehler bei ABR_VORGANG");
  printf("DELETE TBKO_ABR_VORGANG war OK\n");

  EXEC SQL
  DELETE FROM TBKO_ABR_BUCHUNGN
  WHERE       VSNR    = :HV_VSNR
    AND       BEARBNR = :HV_BEARBNR
  ;
  CHECK(WARNUNG, "PKQ2550--+-> resetAbrtab: Fehler bei ABR_BUCHUNGN");
  printf("DELETE TBKO_ABR_BUCHUNGN war OK\n");

  EXEC SQL
  DELETE FROM TBKO_ABR_VST
  WHERE       VSNR    = :HV_VSNR
    AND       BEARBNR = :HV_BEARBNR
  ;
  CHECK(WARNUNG, "PKQ2550--+--> resetAbrtab: Fehler bei ABR_VST");
  printf("DELETE TBKO_ABR_VST war OK\n");

  EXEC SQL
  DELETE FROM TBKO_ABR_GEBUEHR
  WHERE       VSNR    = :HV_VSNR
    AND       BEARBNR = :HV_BEARBNR
  ;
  CHECK(WARNUNG, "PKQ2550--+--> resetAbrtab: Fehler bei ABR_GEBUEHR");
  printf("DELETE TBKO_ABR_GEBUEHR war OK\n");
  return (0);
}
/*===ProtFehler()====================================================
  Synopsis:      int ProtFehler(void)
  Beschreibung:  Fehlerprotokoll ausgeben
  Modifikation:  26.05.1999 NoHunger  Ersterstellung
=====================================================================*/
static int  ProtFehler(char * amodul, char * fmodul, char * retcode)
{
 EXEC SQL BEGIN DECLARE SECTION;
  char           HV_VSNR[11];
  long int       HV_BEARBNR;
  decimal(15,0)  HV_VTRNR;
 EXEC SQL END DECLARE SECTION;

 int rc;

 /*char amodul[8];
 char fmodul[8];
 char retcode[5]; */

  anzFehler++;

  fprintf(PROT,"***************************************");
  fprintf(PROT,"***************************************\n");
  fprintf(PROT,"Fehler in Modul       :   %s\n",amodul);
  fprintf(PROT,"          VSNR        :   %.10s\n",aktB.vsnr);
  fprintf(PROT,"          VTRNR       :   %D(15,0)\n",tabVJ[0].vtrnr);
  fprintf(PROT,"          BEARBNR     :   %D(11,0)\n",aktB.nummer);

  fprintf(PROT,"          Returncode  :   %.4s\n",retcode);
  fprintf(PROT,"          Fehlermodul :   %.7s\n", fmodul);
  fprintf(PROT,"***************************************\n");

                                      /*** AEND0004 - ANFANG ***/
  memset(hFEHLMELD, ' ', sizeof(hFEHLMELD));
  hRC = (atoi(retcode));
  strncpy(hFEHLMELD,ftext,60);


  rc=insertBatchfehler();
                                      /*** AEND0004 - ENDE   ***/
  return (0);
}
/*===PKQ2550Fehler()====================================================
  Synopsis:      int PKQ2550Fehler(void)
  Beschreibung:  Fehlerprotokoll ausgeben
  Modifikation:  26.05.1999 NoHunger  Ersterstellung
=====================================================================*/
static int PKQ2550Fehler(char * ffunk,char * ftext,int frc,int fsql)
{
  int rc;

  anzFehler++;

  fprintf(PROT,"***************************************");
  fprintf(PROT,"***************************************\n");
  fprintf(PROT,"Fehler in Modul        :  PKQ2550\n");
  fprintf(PROT,"          VSNR         :  %.10s\n",aktB.vsnr);
  fprintf(PROT,"          VTRNR        :  %D(15,0)\n",tabVJ[0].vtrnr);
  fprintf(PROT,"          BEARBNR      :  %D(11,0)\n",aktB.nummer);

  fprintf(PROT,"          Funktion     :  %.30s\n",ffunk);
  fprintf(PROT,"          Fehlermeldung:  %.60s\n",ftext);
  fprintf(PROT,"          Returncode   :  %.d\n",frc);
  fprintf(PROT,"          SQLCODE      :  %.d\n",fsql);

                                      /*** AEND0004 - ANFANG ***/
  memset(hFEHLMELD, ' ', sizeof(hFEHLMELD));
  hRC = frc;
  strcpy(hFEHLMELD,ftext);


  rc=insertBatchfehler();
                                      /*** AEND0004 - ENDE   ***/


  return (0);
}
/*===updateAbrechFehler()===============================================
  Synopsis:      void updateAbrechFehler(void)
  Beschreibung:  Diese Funktion kennzeichnet den aktuellen
                 Journaleintrag als Fehlerhaft (PERSNR = 'PKO2500F')
                 Aufgerufen wird diese Funktion, wenn ein Fehler in
                 der Abrechnung aufgetreten ist, also nach Aufruf der
                 Module MKO1571 oder MKO1580
  Modifikation:  01.03.1999 NoHunger  Ersterstellung
=====================================================================*/
static int updateAbrechFehler(void) {
 int iVJ;
 EXEC SQL BEGIN DECLARE SECTION;
  decimal(15,0)  HV_VTRNR;
  decimal(5,0)   HV_HISTNR;
  decimal(5,0)   HV_VORG_LFDNR;
 EXEC SQL END DECLARE SECTION;

  /*-- Hostvariablen fr Update TBKO_VTRJOURNAL --*/
  HV_VTRNR      = tabVJ[insPos].vtrnr;
  HV_HISTNR     = tabVJ[insPos].histnr;
  HV_VORG_LFDNR = tabVJ[insPos].vorgLfdnr;

   printf("updateAbrechFehler: Update-Werte\n");
   printf("tabVJ(aktuell)->HV_VTRNR: %D(15,0)\n",HV_VTRNR);
   printf("tabVJ(aktuell)->HV_HISTNR: %D(5,0)\n",HV_HISTNR);
   printf("tabVJ(aktuell)->HV_VORG_LFDNR: %D(5,0)\n",HV_VORG_LFDNR);
  EXEC SQL UPDATE TBKO_VTRJOURNAL
           SET    PERSNR   = 'PKO2500F'
           WHERE  VTRNR      = :HV_VTRNR
           AND    HISTNR     = :HV_HISTNR
           AND    VORG_LFDNR = :HV_VORG_LFDNR;
        CHECK0(WARNUNG,
        "PKQ2550: updateAbrechFehler fehlerhaft");
  if (sqlca.sqlcode == 0) {
   fprintf(PROT,"updateAbrechFehler: obige VSNR wurde als ");
   fprintf(PROT,"fehlerhaft gekennzeichnet (PKO2550F) \n");
  fprintf(PROT,"***************************************");
  fprintf(PROT,"***************************************\n");
   fprintf(PROT," \n");
     return (0);
  } else {
   fprintf(PROT,"updateAbrechFehler: Kennz. PKO2500F fehlerh. bei\n");
   fprintf(PROT,"   VTRNR: %D(15,0)\n",HV_VTRNR);
   fprintf(PROT,"   HISTNR: %D(5,0)\n",HV_HISTNR);
   fprintf(PROT,"   VORG_LFDNR: %D(5,0)\n",HV_VORG_LFDNR);
   printf(" updateAbrechFehler: Kennz. PKO2500F fehlerh. bei\n");
   printf("   VTRNR: %D(15,0)\n",HV_VTRNR);
   printf("   HISTNR: %D(5,0)\n",HV_HISTNR);
   printf("   VORG_LFDNR: %D(5,0)\n",HV_VORG_LFDNR);
     return (-1);
  }


}
/*===sqlFehler()===============================================
  Synopsis:      void sqlFehler(void)
  Beschreibung:  Diese Funktion weist SQL-Fehler aus
  Modifikation:  01.03.1999 NoHunger  Ersterstellung
=====================================================================*/
static void sqlFehler(char * ffunktion,char * ftext,long int retcode)
  {
  int rc;
  anzFehler++;

  fprintf(PROT,"***************************************");
  fprintf(PROT,"***************************************\n");
  fprintf(PROT,"Fehler in Modul        :  PKQ2550\n");
  fprintf(PROT,"          VSNR         :  %.10s\n",aktB.vsnr);
  fprintf(PROT,"          VTRNR        :  %D(15,0)\n",tabVJ[0].vtrnr);
  fprintf(PROT,"          HISTNR       :  %D(5,0)\n",tabVJ[0].histnr);
  fprintf(PROT,"          VORG_LFDNR   :  %D(5,0)\n",
                                                tabVJ[0].vorgLfdnr);
  fprintf(PROT,"          BEARBNR      :  %D(11,0)\n",aktB.nummer);
  fprintf(PROT,"          Funktion     :  %.30s\n",ffunktion);
  fprintf(PROT,"          Fehlermeldung:  %.60s\n",ftext);
  fprintf(PROT,"          SqlCode      :  %ld   \n",retcode);
  fprintf(PROT,"***************************************");
  fprintf(PROT,"***************************************\n");
  printf("***************************************");
  printf("***************************************\n");
  printf("Fehler in Modul        :  PKQ2550\n");
  printf("          VSNR         :  %.10s\n",aktB.vsnr);
  printf("          VTRNR        :  %D(15,0)\n",tabVJ[0].vtrnr);
  printf("          HISTNR       :  %D(5,0)\n",tabVJ[0].histnr);
  printf("          VORG_LFDNR   :  %D(5,0)\n",tabVJ[0].vorgLfdnr);
  printf("          BEARBNR      :  %D(11,0)\n",aktB.nummer);
  printf("          Funktion     :  %.30s\n",ffunktion);
  printf("          Fehlermeldung:  %.60s\n",ftext);
  printf("          SqlCode      :  %ld   \n",retcode);
  printf("***************************************");
  printf("***************************************\n");
                                      /*** AEND0004 - ANFANG ***/
  memset(hFEHLMELD, ' ', sizeof(hFEHLMELD));
  hRC = -999;
  strcpy(hFEHLMELD,ftext);


  rc=insertBatchfehler();
                                      /*** AEND0004 - ENDE   ***/
  }
/*===checkpoint()======================================================
  Synopsis:      void checkpoint(void)
  Beschreibung: Commit durchfuehren und anschliessend den Checkpoint
                in die Tabelle TBKQ_BATCH_COMMIT eintragen.
  Modifikation:  10.08.1999 HUN
=====================================================================*/
static void checkpoint(void) {

  EXEC SQL COMMIT WORK;

  CHECK0(ABBRUCH,
        "PKQ2550: Fehler bei COMMIT WORK");


   EXEC SQL
   UPDATE TBKQ_BATCH_COMMIT
      SET VSNR  = :hVSNR
         ,VTRNR = :hVTRNR
   WHERE
         BATCH_ID        = :hBATCHID
     AND GEPL_LAUF       = :hGEPLLAUF
     AND VSNR_PORTION    = :hVSNRPORT
     AND MANDANT         = :hMANDANT
     AND COD_SPRACHCODE  = :hSPRACHCODE
   ;

  CHECK(ABBRUCH,
       "PKQ2550: Fehler UPDATE in Tabelle TBKQ_BATCH_COMMIT");



   if (SQLCODE == 0) {       /*  Update ok  */
     printf("PKQ2550: Update in TBKQ_BATCH_COMMIT erfolgreich\n");
   }

   if (SQLCODE == 100) {       /*  Insert durchfuehren  */
 printf("PKQ2550: Update in TBKQ_BATCH_COMMIT nicht erfolgreich\n");
 printf("PKQ2550: Insert in TBKQ_BATCH_COMMIT wird durchgefuehrt\n");
     EXEC SQL
     INSERT INTO  TBKQ_BATCH_COMMIT
          (
             BATCH_ID
            ,GEPL_LAUF
            ,VSNR_PORTION
            ,MANDANT
            ,COD_SPRACHCODE
            ,VSNR
            ,VTRNR
            ,STICHTAG_VON
            ,STICHTAG_BIS
          )
     VALUES (
            :hBATCHID
           ,:hGEPLLAUF
           ,:hVSNRPORT
           ,:hMANDANT
           ,:hSPRACHCODE
           ,:hVSNR
           ,:hVTRNR
           ,:hSTICHTAGVON
           ,:hSTICHTAGBIS
            )
     ;

   if (SQLCODE == 0) {       /*  Insert ok  */
     printf("PKQ2550: Insert in TBKQ_BATCH_COMMIT erfolgreich\n");
   }

     CHECK0(ABBRUCH,
          "PKQ2550: Fehler bei Insert TBKQ_BATCH_COMMIT");

   } /* End-If SQL-CODE == 100 */

   if (firstCommit == 0) {
     printf("Checkpoint-KZ in TBKO_ABLAUFPLAN setzen\n");
     EXEC SQL
     UPDATE TBKO_ABLAUFPLAN
        SET CHKPT_KZ =   'J'
           ,ABL_STATUS = 'W'
     WHERE
           BATCH_ID        = :hBATCHID
       AND GEPL_LAUF       = :hGEPLLAUF
       AND VSNR_PORTION    = :hVSNRPORT
       AND MANDANT         = :hMANDANT
       AND SPRACHCODE      = :hSPRACHCODE
     ;


    CHECK(ABBRUCH,
         "PKQ2550: Fehler UPDATE in Tabelle TBKO_ABLAUFPLAN");

    firstCommit = 1;

    }

                                /***  AEND0003 - ANFANG  ***/
    Batch_Statistik();    // Zwischenwerte des Programms sichern
                                /***  AEND0003 - ENDE    ***/
  EXEC SQL COMMIT WORK;

}

/*===restart()=========================================================
  Synopsis:      void restart(void)
  Beschreibung: Anhand der Werte aus der TBKO_ABLAUFPLAN wird die
                zuletzt eingetragene VSNR ermittelt (letzter Commit)
                Diese Funktion wird nur nach einem vorherigen
                Programmabbruch aufgerufen
  Modifikation:  22.06.1999 HUN
=====================================================================*/
static void restart(void) {

 int i;

  EXEC SQL
  SELECT VTRNR, VSNR
    INTO :hVTRNR_VON
        ,:hRESTART_VSNR
    FROM TBKQ_BATCH_COMMIT
  WHERE
        BATCH_ID        = :hBATCHID
    AND GEPL_LAUF       = :hGEPLLAUF
    AND VSNR_PORTION    = :hVSNRPORT
    AND MANDANT         = :hMANDANT
    AND COD_SPRACHCODE  = :hSPRACHCODE
   ;

  CHECK(ABBRUCH,
       "PKQ2550: Fehler bei Select in Tabelle TBKQ_BATCH_COMMIT");
  CHECK0(ABBRUCH,
       "PKQ2550: Kein Eintrag in Tabelle TBKQ_BATCH_COMMIT gefunden");


                                     /*** AEND0003 - ANFANG  ***/
  for (hLFDNR=1;hLFDNR<10;hLFDNR++)
   {
     printf("restart HLFDNR= %D(5,0)\n",hLFDNR);

     EXEC SQL
     SELECT WERT
       INTO :hWERT
       FROM OTKQ_BATCH_STAT
     WHERE
           BATCH_ID        = :hBATCHID
       AND GEPL_LAUF       = :hGEPLLAUF
       AND VSNR_PORTION    = :hVSNRPORT
       AND LFDNR           = :hLFDNR
       AND MANDANT         = :hMANDANT
       AND SPRACHCODE  = :hSPRACHCODE
      ;

     CHECK(ABBRUCH,
          "PKQ2550: Fehler bei Select in Tabelle OTKQ_BATCH_STAT");

     if (hLFDNR == 1)
        gesVSNR = hWERT;         // Anz. der ermittelten VSNRs
     if (hLFDNR == 2)
        gesFetch = hWERT;        // Anz. der eingeles. Journaleintraege
     if (hLFDNR == 3)
        gesErh = hWERT;          // Anz. der erhobenen VSNRs
   /*if (hLFDNR == 4)                     AEND0006-A***
        kq2550.anzRW006 = hWERT;
     if (hLFDNR == 5)
        kq2550.WertRW006 = hWERT;
     if (hLFDNR == 6)
        kq2550.anzRW004 = hWERT;
     if (hLFDNR == 7)
        kq2550.WertRW004 = hWERT;         AEND0006-E ***/
     if (hLFDNR == 8)
        gesStoSchwebe = hWERT;   //Anz. Storno-in-Schwebe-Vertraege
     if (hLFDNR == 9)
        anzFehler = hWERT;       //Anz. fehlerhafter Vertraege

   } //for-Schleife
                                     /*** AEND0003 - ENDE  ***/

}

/*=== updateAblaufplan()=============================================
  Synopsis:      updateAblaufplan(void)
  Beschreibung:  Diese Funktion setzt in der Tabelle TBKO_ABLAUFPLAN
                 den Ablaufstatus auf 'E', das aktuelle Laufdatum
                 ein und das CHKPT_KZ auf 'N'
                 Aufgerufen wird diese Funktion aus der Funktion
                 nachlauf
  Modifikation:  01.03.1999 NoHunger  Ersterstellung
=====================================================================*/
static updateAblaufplan(void) {

     printf("Funktion updateAblaufplan\n");
     EXEC SQL
     UPDATE TBKO_ABLAUFPLAN
        SET ABL_STATUS = 'E'
           ,DGEF_LAUF =  :hDGEFLAUF
           ,CHKPT_KZ   = 'N'
     WHERE
           BATCH_ID        = :hBATCHID
       AND GEPL_LAUF       = :hGEPLLAUF
       AND VSNR_PORTION    = :hVSNRPORT
       AND MANDANT         = :hMANDANT
       AND SPRACHCODE      = :hSPRACHCODE
     ;


    CHECK(ABBRUCH,
         "PKQ2550: Fehler UPDATE in Tabelle TBKO_ABLAUFPLAN");


}
/*===Batch_Statistik()====================== AEND0003 - ANFANG ======
  Synopsis:      void Batch_Statistik(void)
  Beschreibung: Die Werte der Laufstatitik werden in die Tabelle
                OTKQ_BATCH_STAT eingetragen.Die Werte werden durch
                eine Nummer (LFDNR) identifiziert.

                Folgende Zuordnung gelten in diesem Programm:
                LFDNR   Bedeutung des Wertes
                  1     Anz. eingelesener Vertrge
                  2     Anz. eingelesener Journaleintraege
                  3     Anz. erhobener / abgerechneter Vertraege
                  4     Anz. eingefuegter Inkassosaetze (OTIN_RW006)
                  5     Wert eingefuegter Inkassosaetze (OTIN_RW006)
                  6     Anz. Buchungen Beitragsvorwegabzug (OTIN_RW004)
                  7     Wert Buchungen Beitragsvorwegabzug (OTIN_RW004)
                  8     Anz. Vertraege 'Storno in Schwebe'
                  9     Anz. fehlerhafter Vertraege

  Modifikation:  10.08.1999 HUN
=====================================================================*/
static void Batch_Statistik(void) {

int i;

  for (i=1;i<10;i++)
   {
    if (testlauf)
        printf("LFDNR = %d\n",i);
    switch (i)
     {
     case 1: hLFDNR = 1;
             hWERT  = gesVSNR;  // Anz. der ermittelten VSNRs
             break;
     case 2: hLFDNR = 2;
             hWERT  = gesFetch; // Anz. der eingeles. Journaleintraege
             break;
     case 3: hLFDNR = 3;
             hWERT  = gesErh;   // Anz. der erhobenen VSNRs
             break;
   /*case 4: hLFDNR = 4;                  AEND0006-A
             hWERT  = kq2550.anzRW006;
             break;
     case 5: hLFDNR = 5;
             hWERT  = kq2550.WertRW006;
             break;
     case 6: hLFDNR = 6;
             hWERT  = kq2550.anzRW004;
             break;
     case 7: hLFDNR = 7;
             hWERT  = kq2550.WertRW004;
             break;                       AEND0006-E ***/
     case 4: hLFDNR = 4;
             hWERT  = 0D;
             break;
     case 5: hLFDNR = 5;
             hWERT  = 0D;
             break;
     case 6: hLFDNR = 6;
             hWERT  = 0D;
             break;
     case 7: hLFDNR = 7;
             hWERT  = 0D;
             break;
     case 8: hLFDNR = 8;
             hWERT  = gesStoSchwebe; //Anz. Storno-in-Schwebe-Vertraege
             break;
     case 9: hLFDNR = 9;
             hWERT  = anzFehler;     //Anz. fehlerhafter Vertraege
             break;
     }

     EXEC SQL
     UPDATE OTKQ_BATCH_STAT
        SET WERT  = :hWERT
     WHERE BATCH_ID        = :hBATCHID
       AND GEPL_LAUF       = :hGEPLLAUF
       AND VSNR_PORTION    = :hVSNRPORT
       AND LFDNR           = :hLFDNR
       AND MANDANT         = :hMANDANT
       AND SPRACHCODE      = :hSPRACHCODE
     ;

     CHECK(ABBRUCH,
       "PKQ2550: Fehler UPDATE OTKQ_BATCH_STAT");

     if (SQLCODE == 0) {       /*  Update ok  */
       if (testlauf)
          printf("PKQ2550: Update OTKQ_BATCH_STAT erfolgreich\n");
     }

     if (SQLCODE == 100) {       /*  Insert durchfuehren  */
      if (testlauf) {
      printf("PKQ2550: Update in OTKQ_BATCH_STAT nicht erfolgreich\n");
      printf("PKQ2550: Ins in OTKQ_BATCH_STAT wird durchgefuehrt\n");
      }
      EXEC SQL
      INSERT INTO  OTKQ_BATCH_STAT
           (
             BATCH_ID
            ,GEPL_LAUF
            ,VSNR_PORTION
            ,LFDNR
            ,MANDANT
            ,SPRACHCODE
            ,WERT
          )
      VALUES (
             :hBATCHID
            ,:hGEPLLAUF
            ,:hVSNRPORT
            ,:hLFDNR
            ,:hMANDANT
            ,:hSPRACHCODE
            ,:hWERT
             )
      ;

        if (SQLCODE == 0) {       /*  Insert ok  */
          if (testlauf)
          printf("PKQ2550: Insert in OTKQ_BATCH_STAT erfolgreich\n");
        }

        CHECK0(ABBRUCH,"PKQ2550: Fehler bei Insert OTKQ_BATCH_STAT");
     }


   } //for


}
                                       /***  AEND0003 - ENDE ***/
                                       /***  AEND0004 - ANFANG ***/
/*===insertBatchfehler()===============================================
  Synopsis:      int insertBatchfehler(void)
  Beschreibung:
  Modifikation:  17.08.1999 NoHunger  Ersterstellung
=====================================================================*/
static int insertBatchfehler(void) {

 EXEC SQL BEGIN DECLARE SECTION;
  decimal(15,0)  HV_VTRNR;
  char           HV_VSNR[11];
 EXEC SQL END DECLARE SECTION;

  /*-- Hostvariablen fr Update TBKO_VTRJOURNAL --*/
  HV_VTRNR      = tabVJ[0].vtrnr;
  strncpy(HV_VSNR,aktB.vsnr,10);
          HV_VSNR[10] = '\0';

   printf("PKQ2550: Funktion insertBatchfehler\n");

   EXEC SQL
     INSERT INTO TBKQ_BATCHFEHLER
            ( ID_BATCH
             ,GEPL_LAUF
             ,VSNR_PORTION
             ,MANDANT
             ,SPRACHCODE
             ,VTRNR
             ,VSNR
             ,RC
             ,FEHLMELD)
     VALUES ( :hBATCHID
             ,:hGEPLLAUF
             ,:hVSNRPORT
             ,:hMANDANT
             ,:hSPRACHCODE
             ,:HV_VTRNR
             ,:HV_VSNR
             ,:hRC
             ,:hFEHLMELD
            )
     ;
   if (sqlca.sqlcode == 0) {
     printf("Neuen Eintrag hinzugefuegt - Insert\n");
   } else {
     if (sqlca.sqlcode == -803) {
       printf("Key schon vorhanden -> Update\n");
        EXEC SQL
        UPDATE TBKQ_BATCHFEHLER
           SET VSNR     = :HV_VSNR
              ,RC       = :hRC
              ,FEHLMELD = :hFEHLMELD
         WHERE ID_BATCH     = :hBATCHID
           AND GEPL_LAUF    = :hGEPLLAUF
           AND VSNR_PORTION = :hVSNRPORT
           AND MANDANT      = :hMANDANT
           AND SPRACHCODE   = :hSPRACHCODE
           AND VTRNR        = :HV_VTRNR
         ;
     }
   }

 return (0);
}